using System;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using System.Text;

namespace uCalcSoftware {
#if WIN64 // Note: The WIN64 symbol must first be added to Project / Properties / Conditional compilation symbols
   public enum Handle_Callback : Int64 { Ignore }
   public enum Handle_DataType : Int64 { Ignore }
   public enum Handle_Expression : Int64 { Ignore }
   public enum Handle_Item : Int64 { Ignore }
   public enum Handle_Matches : Int64 { Ignore }
   public enum Handle_Rule : Int64 { Ignore }
   public enum Handle_String : Int64 { Ignore }
   public enum Handle_Tokens : Int64 { Ignore }
   public enum Handle_Transforms : Int64 { Ignore }
   public enum Handle_uCalc : Int64 { Ignore }
#else
   public enum Handle_Callback : Int32 { Ignore }
   public enum Handle_DataType : Int32 { Ignore }
   public enum Handle_Expression : Int32 { Ignore }
   public enum Handle_Item : Int32 { Ignore }
   public enum Handle_Matches : Int32 { Ignore }
   public enum Handle_Rule : Int32 { Ignore }
   public enum Handle_String : Int32 { Ignore }
   public enum Handle_Tokens : Int32 { Ignore }
   public enum Handle_Transforms : Int32 { Ignore }
   public enum Handle_uCalc : Int32 { Ignore }
#endif


   /// <summary>Data types that come with uCalc</summary>
   /// <remarks>These are built-in data types that come with uCalc</remarks>
   public enum BuiltInType : Int32 {
      Unspecified,
      Auto,
      /// <summary>Boolean</summary>
      Boolean,
      Size_t,
      /// <summary>Int8</summary>
      Integer_8,
      /// <summary>Byte</summary>
      Integer_8u,
      /// <summary>Int16</summary>
      Integer_16,
      /// <summary>Int16u</summary>
      Integer_16u,
      /// <summary>Int32</summary>
      Integer_32,
      /// <summary>Int32u</summary>
      Integer_32u,
      /// <summary>Int64</summary>
      Integer_64,
      /// <summary>Int64u</summary>
      Integer_64u,
      /// <summary>Int128</summary>
      Integer_128,
      /// <summary>Int128u</summary>
      Integer_128u,
      /// <summary>Single</summary>
      Float_Single,
      /// <summary>Double</summary>
      Float_Double,
      /// <summary>Extended</summary>
      Float_Extended,
      Complex,
      Decimal,
      BigNum,
      Date,
      String,
      StringView,
      /// <summary>LPCSTR</summary>
      String_LPCSTR,
      /// <summary>FixedStr</summary>
      String_Fixed,
      /// <summary>WideStr</summary>
      String_Wide,
      /// <summary>StackList</summary>
      List_Stack,
      /// <summary>TableList</summary>
      List_Table,
      /// <summary>SortedList</summary>
      List_Sorted,
      Null,
      Void,
      Any,
      Pointer,
      /// <summary>// +++ Maybe remove; doesn't appear to be used</summary>
      Omni,
      UserDefined,
      DataType,
      Namespace,
      IntPtr,
      ADDR,
      uCalcPtr,
      Item,
      ParsedExpr,
      uCalcCallback,
      Object,
      Intrinsic,
      SameTypeAs,
      DiagAll,
      DiagAllClasses,
      DiagAllTypes,
      DiaguCalc,
      DiagDataType,
      DiagItem,
      DiagDataFormat,
      DiagMatchList,
      DiagParsedExpression,
      DiagStringItem,
      DiagTable
   }

   public enum ChildStringOptions : Int32 {
      ClearAllOnNewObj,
      ClearOverlapOnRead,
      ClearOverlapOnWrite,
      DetachAllOnNewObj,
      DetachOverlapOnRead,
      DetachOverlapOnWrite
   }

   public enum Comparison : Int32 {
      LessThan = -1,
      Equal = 0,
      GreaterThan = 1
   }

   /// <summary>Data type constructors</summary>
   public enum DataTypeConstruction : Int32 {
      ByteSize,
      AllocArray,
      AllocScalar,
      AllocByPtr,
      ReleaseArray,
      ReleaseScalar,
      ResetScalar,
      ResetArray,
      SetArrayValue,
      SetScalarValue,
      SwapScalarValues
   }

   public enum DiagMemCount : Int32 {
      UnitSize,
      Bytes,
      Elements,
      Released
   }

   public enum DiagMemOption : Int32 {
      Default,
      Peak,
      Active,
      PeruCalcObj
   }

   /// <summary>Error hander responses</summary>
   public enum ErrorHandlerResponse : Int32 {
      /// <summary>Immediately stops parsing</summary>
      Abort,
      /// <summary>Continues parsing (after the error is resolved)</summary>
      Resume,
      /// <summary>Raises the same error again for the next error handler to take care of</summary>
      ReRaise
   }

   /// <summary>Error numbers that uCalc may return</summary>
   public enum ErrorMsg : Int32 {
      /// <summary>No error</summary>
      None,
      /// <summary>Inexact result</summary>
      FloatInexact = 1,
      /// <summary>Floating point underflow</summary>
      FloatUnderflow = 2,
      /// <summary>Floating point overflow</summary>
      FloatOverflow = 4,
      /// <summary>Division by 0</summary>
      FloatDivisionByZero = 8,
      /// <summary>Invalid floating point value</summary>
      FloatInvalid = 16,
      /// <summary>Dynamically defined user error</summary>
      Dynamically_Defined,
      /// <summary>Syntax error</summary>
      Syntax_Error,
      /// <summary>Undefined identifier</summary>
      Undefined_Identifier,
      /// <summary>File not found</summary>
      FileNotFound,
      /// <summary>Unrecognized token</summary>
      Unrecognized_Token,
      /// <summary>Unrecognized command</summary>
      Unrecognized_Command,
      /// <summary>Data type mismatch</summary>
      Datatype_Mismatch,
      /// <summary>Invalid number of arguments</summary>
      Invalid_Argument_Count,
      /// <summary>Invalid definition</summary>
      Invalid_Definition,
      /// <summary>Bracket delimiter error</summary>
      Bracket_Error,
      /// <summary>Undefined callback</summary>
      Undefined_Callback,
      /// <summary>Error message already defined</summary>
      ErrorMessageAlreadyDefined,
      /// <summary>This item cannot be modified</summary>
      ItemCannotBeModified,
      /// <summary>Unrecognized class member</summary>
      Unrecognized_Class_Member,
      /// <summary>Unrecognized namespace member</summary>
      Unrecognized_Namespace_Member,
      /// <summary>Unbalanced quote</summary>
      Unbalanced_Quote,
      /// <summary>Value cannot be assigned here</summary>
      ValueCannotBeAssigned,
      /// <summary>Reparsing has passed the specified limit</summary>
      ReParseOverflow,
      /// <summary>Array or list bound exceeded</summary>
      Array_Bounds_Exceeded,
      Customized1 = 100,
      Customized2,
      Customized3,
      Customized4,
      Customized5,
      Customized6,
      Customized7,
      Customized8,
      Customized9,
      Customized10,
      Customized11,
      Customized12,
      Customized13,
      Customized14,
      Customized15,
      Customized16,
      Customized17,
      Customized18,
      Customized19,
      Customized20
   }

   /// <summary>Operator grouping associativity</summary>
   public enum Grouping : Int32 {
      LeftToRight,
      RightToLeft
   }

   /// <summary>Properties that a uCalc item may have</summary>
   /// <remarks>
   /// <para>Items defined with uCalc typically have a combination of properties.  A variable defined with DefineVariable for instance
   /// will have the ItemIs[Variable] property set.  An operator will have the ItemIs[Operator] property set.  If it&apos;s an infix
   /// operator, in addition to having the ItemIs[Operator] property set, ItemIs[Infix] will also be set.</para>
   /// 
   /// <para>The IsProperty function can determine if a given uCalc.Item has a certain property.</para>
   /// 
   /// <para>You can use the IsProperty function to cycle through the list of all items that share certain properties, or to disambiguate
   /// between items that share the same name but have different properties (for instance the + operator with the Prefix property
   /// vs another one with the Infix property).</para>
   /// 
   /// <para>Some uCalc enum elements are keywords in VB.NET.  To avoid a naming conflict, enclose the member name in brackets. For
   /// instance: [Function], [Operator], [Optional], [ByVal], etc.</para>
   /// </remarks>
   public enum ItemIs : Int32 {
      SelectAny,
      SelectAll,
      RightToLeft,
      DataType,
      Array,
      Class,
      Namespace,
      Variable,
      Function,
      Operator,
      FunctionOrOperator,
      Prefix,
      Infix,
      Postfix,
      AddressOwner,
      /// <summary>// +++ This is the default; doesn't appear to be used explicitly</summary>
      ByVal,
      ByRef,
      ByCopy,
      ByExpr,
      /// <summary>// +++ Rename to ByItem for clarity; will match PassBy.ByItem</summary>
      ByHandle,
      Optional,
      VariableNumberOfArgs,
      Alias,
      LiteralItem,
      Locked,
      Exclude,
      Transform,
      Search,
      FinishedSearch,
      ReturnSelf,
      ErrorHandler,
      ApiCall,
      CaseSensitive,
      QuotedText,
      RewindOnChange,
      TransformArg,
      Active,
      WhitespaceCounts,
      IgnoreQuote,
      IgnoreStatementSep,
      IgnoreBracket,
      SkipEvalTransform,
      ExprUnit,
      EagerArg,
      SkipPatternMatching,
      SkipOver,
      Modified,
      UpdatePending,
      ParentUpdatePending,
      UpToEndOrSeparator,
      UpToCompleted,
      SkipReduce,
      PatternCheck,
      EvalStrOutput,
      NotFound,
      ListReturned,
      Focusable,
      DisconnectedParent,
      Child,
      Parent,
      Range,
      HasByExpr
   }

   /// <summary>Licensing options for uCalc products</summary>
   public enum LicenseType : Int32 {
      Demo,
      FMP_Free,
      FMP_Standard_32,
      FMP_Standard_64,
      FMP_32_and_64,
      LanguageBuilder,
      GeneralParser,
      Interpreter,
      ConsoleCalc,
      WindowsCalc,
      StringLib,
      Transform,
      TransformLib,
      Search,
      SearchLib,
      Library1,
      Library2,
      Beta,
      Debugger,
      uCalc_All,
      Developer,
      EndUser
   }

   public enum MatchesOption : Int32 {
      All,
      SameAsBefore,
      EmptyList,
      MainLevelOnly,
      InnerMostOnly,
      FocusableOnly,
      NoOffset
   }

   public enum MemoryTable : Int32 {
      Invalid,
      uCalc,
      Callback,
      DataType,
      Expression,
      Item,
      Matches,
      Rule,
      String,
      Tokens,
      Transforms,
      Format,
      Table,
      uCalcBase = uCalc
   }

   /// <summary>RegEx grammar option for defining tokens; based on C++ standard library regex flags</summary>
   public enum RegExGrammar : Int32 {
      /// <summary>Default grammar; ECMAScript is the default</summary>
      Default,
      /// <summary>JavaScript-like regex grammar</summary>
      ECMAScript,
      /// <summary>Basic POSIX grammar (BRE)</summary>
      basic,
      /// <summary>Extended POSIX grammar (ERE)</summary>
      extended,
      /// <summary>awk POSIX grammar</summary>
      awk,
      /// <summary>grep POSIX grammar</summary>
      grep,
      /// <summary>egrep POXIX grammar</summary>
      egrep
   }

   public enum StringListType : Int32 {
      None,
      Chars,
      Tokens,
      Elements,
      Statements,
      Lines,
      Matches
   }

   /// <summary>Types of tokens</summary>
   public enum TokenType : Int32 {
      None,
      /// <summary>A token defined without any specified property is generic (default)</summary>
      Generic,
      /// <summary>Represents the absence of any character (token not found)</summary>
      Empty,
      /// <summary>A token that behaves as alphanuric (works with word boundaries)</summary>
      AlphaNumeric,
      /// <summary>A token that serves as argument separator (typically a comma)</summary>
      ArgSeparator,
      /// <summary>A token that starts a bracketed group; typically strings like {, [, or (</summary>
      Bracket,
      /// <summary>The closing token that matches a Bracket token. Ex: }, ], or )</summary>
      BracketMatch,
      /// <summary>A token that separates an object from its member, typically a . (a dot)</summary>
      MemberAccess,
      /// <summary>A token that represents a literal value, typically a quoted string or number</summary>
      Literal,
      /// <summary>A token that represents a statement separator, typically a ; (semicolon)</summary>
      StatementSep,
      /// <summary>A token that represents whitespace, typically tab or space characters</summary>
      Whitespace,
      /// <summary>A token that is not delimited by word boundaries (+, -, *, etc), consecutive characters are reduced until they match a value that exists in the symbol table or until 1 character</summary>
      Reducible,
      /// <summary>Non-character that represents the end of a stream of text</summary>
      Eof,
      LocalVar,
      StaticVar,
      /// <summary>Item in parenthesis (or quotes) following this token will be defined while the expression is parsed</summary>
      DefineNow = StaticVar,
      TempDef,
      /// <summary>The expression in parathesis (or quotes) following the token is evaluated and inserted in place of the token in the string</summary>
      EvalInsert,
      EvalDuringParse,
      /// <summary>Works like EvalInsert, except the expression is transformed instead of evaluated</summary>
      TransformInsert,
      /// <summary>The contents of the quoted file name following this token is inserted into the string</summary>
      FileInclude,
      CurrentSpace,
      DefSpace,
      EvalSpace,
      TokenTransform
   }

   /// <summary>Transform reset options</summary>
   /// <remarks>Use these options with Transform Reset.  The Default is All</remarks>
   public enum TransformResetOption : Int32 {
      /// <summary>Resets all (Input, Tokens, Rules)</summary>
      All,
      /// <summary>Resets the input to an empty string</summary>
      Input,
      /// <summary>Resets token definitions</summary>
      Tokens,
      /// <summary>Resets the FromTo rules</summary>
      Rules,
      /// <summary>Resets transform results</summary>
      Results,
      /// <summary>Removes passes</summary>
      Passes
   }


#pragma warning disable IDE1006 // Naming Styles -- uCalc intentionally begins with lower case u
#pragma warning disable IDE0090 // Use 'new(...)' -- the simplified "new" syntax is new in C# 9; must support older versions
#pragma warning disable IDE0034 // Simplify 'default' expression -- the simplified "default" syntax is new in C# 9; must support older versions
#pragma warning disable CS8625 // Cannot convert null literal to non-nullable reference type -- null optional parameters are intentional & controlled w/ conversion ops


   /// <summary>uCalc main class</summary>
   public partial class uCalc : IDisposable {      
      private bool disposed = false;
      public void Dispose() { Dispose(disposing: true); GC.SuppressFinalize(this); }
      protected virtual void Dispose(bool disposing) { if (disposed) return; Release(); disposed = true; }
#if __IOS__
#if DEBUG
      public const string uCalcDLL = "libuCalciOSsim.so";
#else
      public const string uCalcDLL = "libuCalciOSarmv7.so";
#endif
#elif WIN64 // Note: The WIN64 symbol must first be added to Project / Properties / Conditional compilation symbols
      public const string uCalcDLL = "uCalc64.dll";
      public const bool Is32Dll = false;
      public const bool Is64Dll = true;
#else
      public const string uCalcDLL = "uCalc32.dll";
      public const bool Is32Dll = true;
      public const bool Is64Dll = false;
#endif
      public delegate void OpCallback(uCalc.Callback OpHandle);
      public delegate void ErrorHandlerCallback(Handle_uCalc uCalcHandle);

      private static class Unicode {
         private static readonly UTF8Encoding Utf8 = new UTF8Encoding();
         public static byte[] Utf8GetBytes(string str) { return Utf8.GetBytes(str); }
         public static IntPtr Utf8GetLength(string str) { return (IntPtr)Utf8.GetBytes(str).Length; }
         public static string Utf16(IntPtr ReturnStr, IntPtr ReturnStrSize) {
            var CharBytes = new byte[(int)ReturnStrSize];
            Marshal.Copy(ReturnStr, CharBytes, 0, (int)ReturnStrSize);
            return Utf8.GetString(CharBytes);
         }
      }

      private static Matches AdjustMatchesForUTF16(Transforms UserTransform, Matches Matches) {
         var Offset = 0;
         var StartP = 0;

         for (var x = 0; x < Matches.Count(); x++) {
            var LengthOffset = Matches.Str(x).Length - Matches.Length(x);
            var OldLengthOffset = UserTransform.SubStr(Matches.Start(x), Matches.OldLength(x)).Length - Matches.OldLength(x);
            var Start = Matches.Start(x);
            var End = Matches.End(x);
            Offset += UserTransform.SubStr(StartP, Matches.Start(x) - StartP).Length - (Matches.Start(x) - StartP);
            Matches.Start(x, Matches.Start(x) + Offset);
            if (x < Matches.Count() - 1 && Matches.Start(x + 1) < End) StartP = Start;
            else {
               StartP = End;
               Offset += LengthOffset;
            }
            Matches.Length(x, Matches.Length(x) + LengthOffset);
            Matches.OldLength(x, Matches.OldLength(x) + OldLengthOffset);
         }
         return Matches;
      }

      private static Matches AdjustMatchesForUTF16(Rule ThisRule, Matches Matches) { return AdjustMatchesForUTF16(ThisRule.ParentTransform(), Matches); }

      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "AddErrorHandler_14")]
      private static extern Handle_Item AddErrorHandler_14(Handle_uCalc h, ErrorHandlerCallback ErrorHandlerAddress, IntPtr Position);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Alias_16")]
      private static extern Handle_Item Alias_16(Handle_uCalc h, byte[] NewSymbol, IntPtr NewSymbol_Size, Handle_Item ExistingItem);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Alias_17")]
      private static extern Handle_Item Alias_17(Handle_uCalc h, byte[] NewSymbol, IntPtr NewSymbol_Size, byte[] ExistingSymbol, IntPtr ExistingSymbol_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "AliasForDefine_18")]
      private static extern void AliasForDefine_18(Handle_uCalc h, byte[] NewCommandName, IntPtr NewCommandName_Size, byte[] ExistingCommandName, IntPtr ExistingCommandName_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Clone_555")]
      private static extern Handle_uCalc Clone_555(Handle_uCalc h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DataTypeOf_22")]
      private static extern Handle_DataType DataTypeOf_22(Handle_uCalc h, BuiltInType Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DataTypeOf_23")]
      private static extern Handle_DataType DataTypeOf_23(Handle_uCalc h, byte[] expression, IntPtr expression_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Default_25")] // static
      private static extern Handle_uCalc Default_25();
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DefaultClear_569")] // static
      private static extern void DefaultClear_569();
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DefaultCount_567")] // static
      private static extern IntPtr DefaultCount_567();
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DefaultDataType_28")]
      private static extern Handle_DataType DefaultDataType_28(Handle_uCalc h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DefaultDataType_29")]
      private static extern void DefaultDataType_29(Handle_uCalc h, BuiltInType DataTypeIndex);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DefaultDataType_30")]
      private static extern void DefaultDataType_30(Handle_uCalc h, Handle_DataType dType);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DefaultDataType_31")]
      private static extern void DefaultDataType_31(Handle_uCalc h, byte[] dTypeStr, IntPtr dTypeStr_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DefaultItem_32")]
      private static extern Handle_Item DefaultItem_32(Handle_uCalc h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DefaultPop_568")] // static
      private static extern void DefaultPop_568();
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DefaultPush_26")] // static
      private static extern void DefaultPush_26( Handle_uCalc uCalcObj);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DefaultPushNew_580")] // static
      private static extern Handle_uCalc DefaultPushNew_580( Handle_uCalc InheritsFrom);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Define_33")]
      private static extern Handle_Item Define_33(Handle_uCalc h, byte[] expression, IntPtr expression_Size, IntPtr Address, Handle_DataType dType);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DefineConstant_34")]
      private static extern Handle_Item DefineConstant_34(Handle_uCalc h, byte[] Definition, IntPtr Definition_Size, IntPtr VariableAddress, Handle_DataType dType);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DefineFunction_35")]
      private static extern Handle_Item DefineFunction_35(Handle_uCalc h, byte[] Definition, IntPtr Definition_Size, OpCallback FunctionAddr, Handle_DataType dType);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DefineOperator_36")]
      private static extern Handle_Item DefineOperator_36(Handle_uCalc h, byte[] Definition, IntPtr Definition_Size, IntPtr PrecedenceLevel, Grouping Direction, OpCallback FunctionAddr, Handle_DataType dType);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DefineVariable_37")]
      private static extern Handle_Item DefineVariable_37(Handle_uCalc h, byte[] Definition, IntPtr Definition_Size, IntPtr VariableAddress, Handle_DataType dType);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DiagMem_597")] // static
      private static extern IntPtr DiagMem_597( IntPtr uCalcIndex, BuiltInType Target, DiagMemCount Count, DiagMemOption Option);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ErrorExpression_39")]
      private static extern IntPtr ErrorExpression_39(Handle_uCalc h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ErrorExpression_40")]
      private static extern void ErrorExpression_40(Handle_uCalc h, byte[] expression, IntPtr expression_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ErrorLocation_41")]
      private static extern IntPtr ErrorLocation_41(Handle_uCalc h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ErrorMessage_42")]
      private static extern IntPtr ErrorMessage_42(Handle_uCalc h, out IntPtr ReturnSize, ErrorMsg Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ErrorNumber_43")]
      private static extern ErrorMsg ErrorNumber_43(Handle_uCalc h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ErrorResponse_44")]
      private static extern void ErrorResponse_44(Handle_uCalc h, ErrorHandlerResponse Response);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ErrorSymbol_45")]
      private static extern IntPtr ErrorSymbol_45(Handle_uCalc h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Eval_46")]
      private static extern double Eval_46(Handle_uCalc h, byte[] expression, IntPtr expression_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "EvalStr_47")]
      private static extern IntPtr EvalStr_47(Handle_uCalc h, out IntPtr ReturnSize, byte[] expression, IntPtr expression_Size, Int64 ReturnFormatted);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ExpressionTransforms_48")]
      private static extern Handle_Transforms ExpressionTransforms_48(Handle_uCalc h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "FloatingPointErrorsToCatch_49")]
      private static extern Int32 FloatingPointErrorsToCatch_49(Handle_uCalc h, Int32 Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Format_51")]
      private static extern Handle_Item Format_51(Handle_uCalc h, OpCallback FunctionAddr, Handle_DataType dType);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Format_52")]
      private static extern Handle_Item Format_52(Handle_uCalc h, OpCallback FunctionAddr, byte[] Properties, IntPtr Properties_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Format_53")]
      private static extern Handle_Item Format_53(Handle_uCalc h, byte[] Definition, IntPtr Definition_Size, Handle_DataType dType);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "FormatRemove_54")]
      private static extern void FormatRemove_54(Handle_uCalc h, Handle_DataType dType);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IsDefault_57")]
      private static extern Int64 IsDefault_57(Handle_uCalc h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IsDefault_58")]
      private static extern void IsDefault_58(Handle_uCalc h, Int64 Condition);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ItemOf_60")]
      private static extern Handle_Item ItemOf_60(Handle_uCalc h, Int64 Property_, IntPtr nth, byte[] DataTypeName, IntPtr DataTypeName_Size, Int64 IncludeOverloads);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ItemOf_61")]
      private static extern Handle_Item ItemOf_61(Handle_uCalc h, byte[] ItemName, IntPtr ItemName_Size, Int64 Properties, IntPtr nthOverload);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "License_62")] // static
      private static extern void License_62( LicenseType Type, IntPtr L1, IntPtr L2, IntPtr L3, IntPtr L4, IntPtr L5);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MemoryIndex_602")]
      private static extern IntPtr MemoryIndex_602(Handle_uCalc h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MemoryObjectAddress_")] // static
      private static extern IntPtr MemoryObjectAddress_( MemoryTable ObjectType, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "NamespaceSymbol_63")]
      private static extern Handle_Item NamespaceSymbol_63(Handle_uCalc h, IntPtr nth);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "NamespaceSymbolCount_64")]
      private static extern IntPtr NamespaceSymbolCount_64(Handle_uCalc h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "New_584")] // static
      private static extern Handle_uCalc New_584( Handle_uCalc InheritsFrom);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "NewDefault_20")] // static
      private static extern Handle_uCalc NewDefault_20( Handle_uCalc InheritsFrom);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "NewExpression_581")]
      private static extern Handle_Expression NewExpression_581(Handle_uCalc h, byte[] Expr, IntPtr Expr_Size, BuiltInType DataTypeIndex);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "NewExpression_582")]
      private static extern Handle_Expression NewExpression_582(Handle_uCalc h, byte[] Expr, IntPtr Expr_Size, Handle_DataType ReturnType);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "NewExpression_583")]
      private static extern Handle_Expression NewExpression_583(Handle_uCalc h, byte[] Expr, IntPtr Expr_Size, byte[] ReturnType, IntPtr ReturnType_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "NewString_65")]
      private static extern Handle_String NewString_65(Handle_uCalc h, byte[] InitialString, IntPtr InitialString_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "NewTransform_66")]
      private static extern Handle_Transforms NewTransform_66(Handle_uCalc h, Handle_Tokens TokensToInherit);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Parse_68")]
      private static extern Handle_Expression Parse_68(Handle_uCalc h, byte[] Expr, IntPtr Expr_Size, BuiltInType DataTypeIndex);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Parse_69")]
      private static extern Handle_Expression Parse_69(Handle_uCalc h, byte[] Expr, IntPtr Expr_Size, Handle_DataType ReturnType);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Parse_70")]
      private static extern Handle_Expression Parse_70(Handle_uCalc h, byte[] Expr, IntPtr Expr_Size, byte[] ReturnType, IntPtr ReturnType_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Precedence_71")]
      private static extern IntPtr Precedence_71(Handle_uCalc h, byte[] Operator1, IntPtr Operator1_Size, byte[] Operator2, IntPtr Operator2_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Properties_72")] // static
      private static extern Int64 Properties_72( ItemIs p1, ItemIs p2, ItemIs p3, ItemIs p4);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "RaiseErrorOnDivideByZero_73")]
      private static extern void RaiseErrorOnDivideByZero_73(Handle_uCalc h, Int64 TrueOrFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "RaiseErrorOnInexact_74")]
      private static extern void RaiseErrorOnInexact_74(Handle_uCalc h, Int64 TrueOrFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "RaiseErrorOnInvalid_75")]
      private static extern void RaiseErrorOnInvalid_75(Handle_uCalc h, Int64 TrueOrFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "RaiseErrorOnOverflow_76")]
      private static extern void RaiseErrorOnOverflow_76(Handle_uCalc h, Int64 TrueOrFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "RaiseErrorOnUnderflow_77")]
      private static extern void RaiseErrorOnUnderflow_77(Handle_uCalc h, Int64 TrueOrFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReferenceCount_570")]
      private static extern IntPtr ReferenceCount_570(Handle_uCalc h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReferenceIncr_556")]
      private static extern void ReferenceIncr_556(Handle_uCalc h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Release_78")]
      private static extern void Release_78(Handle_uCalc h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ResetAll_566")] // static
      private static extern void ResetAll_566();
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "SetAsDefault_585")]
      private static extern Handle_uCalc SetAsDefault_585(Handle_uCalc h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueAt_81")]
      private static extern IntPtr ValueAt_81(Handle_uCalc h, out IntPtr ReturnSize, IntPtr ValuePtr, Handle_DataType dType, Int64 FormattedOutput);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueAt_82")]
      private static extern IntPtr ValueAt_82(Handle_uCalc h, out IntPtr ReturnSize, IntPtr ValuePtr, byte[] ValueDataType, IntPtr ValueDataType_Size, Int64 FormattedOutput);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Arg_93")]
      private static extern double Arg_93(Handle_Callback h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Arg1_94")]
      private static extern double Arg1_94(Handle_Callback h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Arg2_95")]
      private static extern double Arg2_95(Handle_Callback h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ArgAddr_96")]
      private static extern IntPtr ArgAddr_96(Handle_Callback h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ArgBool_97")]
      private static extern Int64 ArgBool_97(Handle_Callback h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ArgCount_98")]
      private static extern IntPtr ArgCount_98(Handle_Callback h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ArgDbl_99")]
      private static extern double ArgDbl_99(Handle_Callback h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ArgExpr_100")]
      private static extern Handle_Expression ArgExpr_100(Handle_Callback h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ArgInt16_614")]
      private static extern Int16 ArgInt16_614(Handle_Callback h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ArgInt32_101")]
      private static extern Int32 ArgInt32_101(Handle_Callback h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ArgInt64_102")]
      private static extern Int64 ArgInt64_102(Handle_Callback h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ArgItem_103")]
      private static extern Handle_Item ArgItem_103(Handle_Callback h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ArgPtr_104")]
      private static extern IntPtr ArgPtr_104(Handle_Callback h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ArgStr_105")]
      private static extern IntPtr ArgStr_105(Handle_Callback h, out IntPtr ReturnSize, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ErrorRaise_106")]
      private static extern ErrorHandlerResponse ErrorRaise_106(Handle_Callback h, ErrorMsg ErrNumber);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ErrorRaiseMessage_107")]
      private static extern ErrorHandlerResponse ErrorRaiseMessage_107(Handle_Callback h, byte[] ErrMessage, IntPtr ErrMessage_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ErrorSet_108")]
      private static extern void ErrorSet_108(Handle_Callback h, ErrorMsg ErrNumber);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Item_110")]
      private static extern Handle_Item Item_110(Handle_Callback h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Return_112")]
      private static extern void Return_112(Handle_Callback h, double Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReturnBool_113")]
      private static extern void ReturnBool_113(Handle_Callback h, Int64 Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReturnDbl_114")]
      private static extern void ReturnDbl_114(Handle_Callback h, double Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReturnInt16_615")]
      private static extern void ReturnInt16_615(Handle_Callback h, Int16 Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReturnInt32_115")]
      private static extern void ReturnInt32_115(Handle_Callback h, Int32 Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReturnInt64_116")]
      private static extern void ReturnInt64_116(Handle_Callback h, Int64 Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReturnPtr_117")]
      private static extern void ReturnPtr_117(Handle_Callback h, IntPtr Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReturnStr_118")]
      private static extern void ReturnStr_118(Handle_Callback h, byte[] Value, IntPtr Value_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "uCalc_119")]
      private static extern Handle_uCalc uCalc_119(Handle_Callback h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ByteSize_121")]
      private static extern IntPtr ByteSize_121(Handle_DataType h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Index_123")]
      private static extern BuiltInType Index_123(Handle_DataType h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Item_124")]
      private static extern Handle_Item Item_124(Handle_DataType h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MemoryIndex_605")]
      private static extern IntPtr MemoryIndex_605(Handle_DataType h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Name_125")]
      private static extern IntPtr Name_125(Handle_DataType h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Release_126")]
      private static extern void Release_126(Handle_DataType h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Reset_128")]
      private static extern void Reset_128(Handle_DataType h, IntPtr ValuePtr);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Reset_129")]
      private static extern void Reset_129(Handle_DataType h, IntPtr ValuePtr, IntPtr Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "SetArray_130")]
      private static extern void SetArray_130(Handle_DataType h, IntPtr Destination, IntPtr Source, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "SetAsDefault_586")]
      private static extern Handle_DataType SetAsDefault_586(Handle_DataType h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "SetScalar_131")]
      private static extern void SetScalar_131(Handle_DataType h, IntPtr Destination, IntPtr Source);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "SwapScalarValues_132")]
      private static extern void SwapScalarValues_132(Handle_DataType h, IntPtr ValuePtrA, IntPtr ValuePtrB);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ToString_134")]
      private static extern IntPtr ToString_134(Handle_DataType h, out IntPtr ReturnSize, IntPtr ValuePtr, Int64 FormatOutput);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ToString_135")]
      private static extern IntPtr ToString_135(Handle_DataType h, out IntPtr ReturnSize, byte[] Value, IntPtr Value_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Clone_503")]
      private static extern Handle_Expression Clone_503(Handle_Expression h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DataType_137")]
      private static extern Handle_DataType DataType_137(Handle_Expression h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Evaluate_138")]
      private static extern double Evaluate_138(Handle_Expression h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "EvaluateBool_139")]
      private static extern Int64 EvaluateBool_139(Handle_Expression h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "EvaluateDbl_140")]
      private static extern double EvaluateDbl_140(Handle_Expression h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "EvaluateInt32_141")]
      private static extern Int32 EvaluateInt32_141(Handle_Expression h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "EvaluateStr_142")]
      private static extern IntPtr EvaluateStr_142(Handle_Expression h, out IntPtr ReturnSize, Int64 ReturnFormatted);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "EvaluateVoid_143")]
      private static extern IntPtr EvaluateVoid_143(Handle_Expression h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Execute_144")]
      private static extern void Execute_144(Handle_Expression h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MemoryIndex_606")]
      private static extern IntPtr MemoryIndex_606(Handle_Expression h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReferenceCount_578")]
      private static extern IntPtr ReferenceCount_578(Handle_Expression h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReferenceIncr_557")]
      private static extern void ReferenceIncr_557(Handle_Expression h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Release_146")]
      private static extern void Release_146(Handle_Expression h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "uCalc_463")]
      private static extern Handle_uCalc uCalc_463(Handle_Expression h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "BracketMatch_149")]
      private static extern Handle_Item BracketMatch_149(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "BracketMatch_150")]
      private static extern Handle_Item BracketMatch_150(Handle_Item h, Handle_Item BracketToken);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Comment_520")]
      private static extern IntPtr Comment_520(Handle_Item h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Comment_522")]
      private static extern Handle_Item Comment_522(Handle_Item h, byte[] CommentStr, IntPtr CommentStr_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Count_151")]
      private static extern IntPtr Count_151(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DataType_524")]
      private static extern Handle_DataType DataType_524(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DataType_152")]
      private static extern Handle_Item DataType_152(Handle_Item h, Handle_DataType dType);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "FunctionAddress_153")]
      private static extern OpCallback FunctionAddress_153(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "FunctionAddress_596")]
      private static extern void FunctionAddress_596(Handle_Item h, OpCallback Address);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IsProperty_156")]
      private static extern Int64 IsProperty_156(Handle_Item h, ItemIs ItemProperty);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IsProperty_157")]
      private static extern Handle_Item IsProperty_157(Handle_Item h, ItemIs ItemProperty, Int64 TrueOrFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MemoryIndex_607")]
      private static extern IntPtr MemoryIndex_607(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Name_525")]
      private static extern IntPtr Name_525(Handle_Item h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "NextOverload_526")]
      private static extern Handle_Item NextOverload_526(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Precedence_160")]
      private static extern IntPtr Precedence_160(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Precedence_161")]
      private static extern void Precedence_161(Handle_Item h, IntPtr Level);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReferenceCount_577")]
      private static extern IntPtr ReferenceCount_577(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReferenceIncr_562")]
      private static extern void ReferenceIncr_562(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Regex_163")]
      private static extern IntPtr Regex_163(Handle_Item h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Regex_164")]
      private static extern Handle_Item Regex_164(Handle_Item h, byte[] Pattern, IntPtr Pattern_Size, RegExGrammar Grammar);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Release_165")]
      private static extern void Release_165(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Rename_166")]
      private static extern Handle_Item Rename_166(Handle_Item h, byte[] NewName, IntPtr NewName_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "SetAsDefault_587")]
      private static extern void SetAsDefault_587(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Text_527")]
      private static extern IntPtr Text_527(Handle_Item h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "uCalc_528")]
      private static extern Handle_uCalc uCalc_528(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Value_168")]
      private static extern double Value_168(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Value_169")]
      private static extern void Value_169(Handle_Item h, double Val);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Value_170")]
      private static extern void Value_170(Handle_Item h, byte[] Val, IntPtr Val_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Value_592")]
      private static extern void Value_592(Handle_Item h, Handle_Item OtherItem);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueAddr_171")]
      private static extern IntPtr ValueAddr_171(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueAddr_595")]
      private static extern void ValueAddr_595(Handle_Item h, IntPtr Address);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueBool_173")]
      private static extern Int64 ValueBool_173(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueBool_174")]
      private static extern void ValueBool_174(Handle_Item h, Int64 Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueByPtr_175")]
      private static extern void ValueByPtr_175(Handle_Item h, IntPtr Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueByte_177")]
      private static extern Byte ValueByte_177(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueByte_178")]
      private static extern void ValueByte_178(Handle_Item h, Byte Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueDbl_180")]
      private static extern double ValueDbl_180(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueDbl_181")]
      private static extern void ValueDbl_181(Handle_Item h, double Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueInt16_183")]
      private static extern Int16 ValueInt16_183(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueInt16_184")]
      private static extern void ValueInt16_184(Handle_Item h, Int16 Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueInt32_186")]
      private static extern Int32 ValueInt32_186(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueInt32_187")]
      private static extern void ValueInt32_187(Handle_Item h, Int32 Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueInt64_189")]
      private static extern Int64 ValueInt64_189(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueInt64_190")]
      private static extern void ValueInt64_190(Handle_Item h, Int64 Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValuePtr_192")]
      private static extern IntPtr ValuePtr_192(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValuePtr_193")]
      private static extern void ValuePtr_193(Handle_Item h, IntPtr Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueSng_195")]
      private static extern float ValueSng_195(Handle_Item h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueSng_196")]
      private static extern void ValueSng_196(Handle_Item h, float Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueStr_198")]
      private static extern IntPtr ValueStr_198(Handle_Item h, out IntPtr ReturnSize, Int64 FormatOutput);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ValueStr_199")]
      private static extern void ValueStr_199(Handle_Item h, byte[] Value, IntPtr Value_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Count_201")]
      private static extern IntPtr Count_201(Handle_Matches h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "End_202")]
      private static extern IntPtr End_202(Handle_Matches h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Filter_203")]
      private static extern Handle_Matches Filter_203(Handle_Matches h, Handle_Rule NestedRule);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IndexOf_205")]
      private static extern IntPtr IndexOf_205(Handle_Matches h, IntPtr StartPos, IntPtr NearestIndex);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IsProperty_207")]
      private static extern Int64 IsProperty_207(Handle_Matches h, ItemIs ItemProperty);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IsProperty_208")]
      private static extern Handle_Matches IsProperty_208(Handle_Matches h, ItemIs ItemProperty, Int64 TrueOrFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Length_210")]
      private static extern IntPtr Length_210(Handle_Matches h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Length_211")]
      private static extern void Length_211(Handle_Matches h, IntPtr Index, IntPtr value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MemoryIndex_608")]
      private static extern IntPtr MemoryIndex_608(Handle_Matches h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "NewStr_212")]
      private static extern IntPtr NewStr_212(Handle_Matches h, out IntPtr ReturnSize, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Offset_213")]
      private static extern IntPtr Offset_213(Handle_Matches h, IntPtr StartingPos);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "OldEnd_214")]
      private static extern IntPtr OldEnd_214(Handle_Matches h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "OldLength_216")]
      private static extern IntPtr OldLength_216(Handle_Matches h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "OldLength_217")]
      private static extern void OldLength_217(Handle_Matches h, IntPtr Index, IntPtr value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Release_218")]
      private static extern void Release_218(Handle_Matches h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Remove_219")]
      private static extern void Remove_219(Handle_Matches h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Reset_220")]
      private static extern void Reset_220(Handle_Matches h, MatchesOption Options);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Rule_221")]
      private static extern Handle_Rule Rule_221(Handle_Matches h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Start_223")]
      private static extern IntPtr Start_223(Handle_Matches h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Start_224")]
      private static extern void Start_224(Handle_Matches h, IntPtr Index, IntPtr value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Str_226")]
      private static extern IntPtr Str_226(Handle_Matches h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Str_227")]
      private static extern IntPtr Str_227(Handle_Matches h, out IntPtr ReturnSize, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "uCalc_529")]
      private static extern Handle_uCalc uCalc_529(Handle_Matches h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Active_230")]
      private static extern Int64 Active_230(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Active_231")]
      private static extern Handle_Rule Active_231(Handle_Rule h, Int64 TrueFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "BracketSensitive_233")]
      private static extern Int64 BracketSensitive_233(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "BracketSensitive_234")]
      private static extern Handle_Rule BracketSensitive_234(Handle_Rule h, Int64 TrueFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "CaseSensitive_236")]
      private static extern Int64 CaseSensitive_236(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "CaseSensitive_237")]
      private static extern Handle_Rule CaseSensitive_237(Handle_Rule h, Int64 TrueFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Comment_530")]
      private static extern IntPtr Comment_530(Handle_Rule h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Comment_535")]
      private static extern Handle_Rule Comment_535(Handle_Rule h, byte[] CommentStr, IntPtr CommentStr_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "FindPart_239")]
      private static extern IntPtr FindPart_239(Handle_Rule h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "FindPart_240")]
      private static extern Handle_Rule FindPart_240(Handle_Rule h, byte[] Pattern, IntPtr Pattern_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Focusable_242")]
      private static extern Int64 Focusable_242(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Focusable_243")]
      private static extern Handle_Rule Focusable_243(Handle_Rule h, Int64 TrueFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "FromPart_245")]
      private static extern IntPtr FromPart_245(Handle_Rule h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "FromPart_246")]
      private static extern Handle_Rule FromPart_246(Handle_Rule h, byte[] Pattern, IntPtr Pattern_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "FromTo_247")]
      private static extern Handle_Rule FromTo_247(Handle_Rule h, byte[] FromStr, IntPtr FromStr_Size, byte[] ToStr, IntPtr ToStr_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "HasLocalTransform_249")]
      private static extern Int64 HasLocalTransform_249(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IgnoreInnerPatterns_251")]
      private static extern Int64 IgnoreInnerPatterns_251(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IgnoreInnerPatterns_252")]
      private static extern Handle_Rule IgnoreInnerPatterns_252(Handle_Rule h, Int64 TrueFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Index_254")]
      private static extern IntPtr Index_254(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Index_255")]
      private static extern Handle_Rule Index_255(Handle_Rule h, IntPtr Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IsChildRule_256")]
      private static extern Int64 IsChildRule_256(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IsProperty_258")]
      private static extern Int64 IsProperty_258(Handle_Rule h, ItemIs ItemProperty);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IsProperty_259")]
      private static extern Handle_Rule IsProperty_259(Handle_Rule h, ItemIs ItemProperty, Int64 TrueOrFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "LocalTransform_260")]
      private static extern Handle_Transforms LocalTransform_260(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Matches_261")]
      private static extern Handle_Matches Matches_261(Handle_Rule h, MatchesOption Options);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MatchesCount_262")]
      private static extern IntPtr MatchesCount_262(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Maximum_264")]
      private static extern IntPtr Maximum_264(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Maximum_265")]
      private static extern Handle_Rule Maximum_265(Handle_Rule h, IntPtr Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MaximumAND_267")]
      private static extern IntPtr MaximumAND_267(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MaximumAND_268")]
      private static extern Handle_Rule MaximumAND_268(Handle_Rule h, IntPtr Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MemoryIndex_609")]
      private static extern IntPtr MemoryIndex_609(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Minimum_270")]
      private static extern IntPtr Minimum_270(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Minimum_271")]
      private static extern Handle_Rule Minimum_271(Handle_Rule h, IntPtr Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MinimumAND_273")]
      private static extern IntPtr MinimumAND_273(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MinimumAND_274")]
      private static extern Handle_Rule MinimumAND_274(Handle_Rule h, IntPtr Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Name_531")]
      private static extern IntPtr Name_531(Handle_Rule h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "NextOverload_532")]
      private static extern Handle_Rule NextOverload_532(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ParentTransform_275")]
      private static extern Handle_Transforms ParentTransform_275(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Precedence_277")]
      private static extern IntPtr Precedence_277(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Precedence_278")]
      private static extern Handle_Rule Precedence_278(Handle_Rule h, IntPtr Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Property_279")]
      private static extern Handle_Rule Property_279(Handle_Rule h, ItemIs SelectedProperty, Int64 TrueFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "QuoteSensitive_281")]
      private static extern Int64 QuoteSensitive_281(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "QuoteSensitive_282")]
      private static extern Handle_Rule QuoteSensitive_282(Handle_Rule h, Int64 TrueFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Release_283")]
      private static extern void Release_283(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReplacementText_284")]
      private static extern IntPtr ReplacementText_284(Handle_Rule h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "RewindOnChange_286")]
      private static extern Int64 RewindOnChange_286(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "RewindOnChange_287")]
      private static extern Handle_Rule RewindOnChange_287(Handle_Rule h, Int64 TrueFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "RightToLeft_289")]
      private static extern Int64 RightToLeft_289(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "RightToLeft_290")]
      private static extern Handle_Rule RightToLeft_290(Handle_Rule h, Int64 TrueFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "SkipOver_292")]
      private static extern Int64 SkipOver_292(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "SkipOver_293")]
      private static extern Handle_Rule SkipOver_293(Handle_Rule h, Int64 TrueFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "StartAfter_295")]
      private static extern IntPtr StartAfter_295(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "StartAfter_296")]
      private static extern Handle_Rule StartAfter_296(Handle_Rule h, IntPtr Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "StatementSensitive_298")]
      private static extern Int64 StatementSensitive_298(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "StatementSensitive_299")]
      private static extern Handle_Rule StatementSensitive_299(Handle_Rule h, Int64 TrueFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "StopAfter_301")]
      private static extern IntPtr StopAfter_301(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "StopAfter_302")]
      private static extern Handle_Rule StopAfter_302(Handle_Rule h, IntPtr Value);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Text_533")]
      private static extern IntPtr Text_533(Handle_Rule h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ToPart_304")]
      private static extern IntPtr ToPart_304(Handle_Rule h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ToPart_305")]
      private static extern Handle_Rule ToPart_305(Handle_Rule h, byte[] Pattern, IntPtr Pattern_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "uCalc_534")]
      private static extern Handle_uCalc uCalc_534(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "WhitespaceSensitive_307")]
      private static extern Int64 WhitespaceSensitive_307(Handle_Rule h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "WhitespaceSensitive_308")]
      private static extern Handle_Rule WhitespaceSensitive_308(Handle_Rule h, Int64 TrueFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "After_310")]
      private static extern Handle_String After_310(Handle_String h, byte[] Pattern, IntPtr Pattern_Size, IntPtr Nth);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Apply_311")]
      private static extern Handle_String Apply_311(Handle_String h, byte[] expression, IntPtr expression_Size, byte[] Variable, IntPtr Variable_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "At_312")]
      private static extern Handle_String At_312(Handle_String h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "AtChar_313")]
      private static extern Handle_String AtChar_313(Handle_String h, IntPtr CharacterPosition);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "AtToken_314")]
      private static extern Handle_String AtToken_314(Handle_String h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Before_315")]
      private static extern Handle_String Before_315(Handle_String h, byte[] Pattern, IntPtr Pattern_Size, IntPtr Nth);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Between_316")]
      private static extern Handle_String Between_316(Handle_String h, byte[] StartPattern, IntPtr StartPattern_Size, byte[] StopPattern, IntPtr StopPattern_Size, IntPtr StartNth, IntPtr StopNth);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "BracketedText_317")]
      private static extern Handle_String BracketedText_317(Handle_String h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "BracketedTextInside_318")]
      private static extern Handle_String BracketedTextInside_318(Handle_String h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "BracketMatch_319")]
      private static extern Handle_String BracketMatch_319(Handle_String h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "BracketMatchTokenIndex_320")]
      private static extern IntPtr BracketMatchTokenIndex_320(Handle_String h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "CharCount_321")]
      private static extern IntPtr CharCount_321(Handle_String h, IntPtr Nth);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "CharPos_322")]
      private static extern IntPtr CharPos_322(Handle_String h, IntPtr Nth);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ChildStringOption_324")]
      private static extern ChildStringOptions ChildStringOption_324(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ChildStringOption_325")]
      private static extern Handle_String ChildStringOption_325(Handle_String h, ChildStringOptions Option);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ClearDerivedStrings_326")]
      private static extern Handle_String ClearDerivedStrings_326(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Clone_496")]
      private static extern Handle_String Clone_496(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Comment_536")]
      private static extern IntPtr Comment_536(Handle_String h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Comment_538")]
      private static extern Handle_String Comment_538(Handle_String h, byte[] CommentStr, IntPtr CommentStr_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Compare_327")]
      private static extern Comparison Compare_327(Handle_String h, byte[] OtherString, IntPtr OtherString_Size, IntPtr IndexA, IntPtr IndexB, IntPtr Length);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Data_498")]
      private static extern IntPtr Data_498(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Data_500")]
      private static extern Handle_String Data_500(Handle_String h, IntPtr StringAddress, IntPtr Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "DataSize_499")]
      private static extern IntPtr DataSize_499(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Find_328")]
      private static extern Handle_String Find_328(Handle_String h, byte[] Pattern, IntPtr Pattern_Size, IntPtr Nth);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "FindOtherThan_329")]
      private static extern Handle_String FindOtherThan_329(Handle_String h, byte[] Pattern, IntPtr Pattern_Size, IntPtr Nth);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IndexBase_332")]
      private static extern IntPtr IndexBase_332(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IndexBase_333")]
      private static extern Handle_String IndexBase_333(Handle_String h, IntPtr BaseValue);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "List_334")]
      private static extern Handle_String List_334(Handle_String h, StringListType ListType);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ListFormat_335")]
      private static extern Handle_String ListFormat_335(Handle_String h, byte[] Separator, IntPtr Separator_Size, byte[] Prefix, IntPtr Prefix_Size, byte[] Postfix, IntPtr Postfix_Size, byte[] UserFunc, IntPtr UserFunc_Size, byte[] Element, IntPtr Element_Size, byte[] Index, IntPtr Index_Size, byte[] Count, IntPtr Count_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ListFunction_336")]
      private static extern Handle_String ListFunction_336(Handle_String h, byte[] UserFunc, IntPtr UserFunc_Size, byte[] Element, IntPtr Element_Size, byte[] Index, IntPtr Index_Size, byte[] MatchCount, IntPtr MatchCount_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ListOfChars_337")]
      private static extern Handle_String ListOfChars_337(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ListOfTokens_338")]
      private static extern Handle_String ListOfTokens_338(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ListPostfix_339")]
      private static extern Handle_String ListPostfix_339(Handle_String h, byte[] Postfix, IntPtr Postfix_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ListPrefix_340")]
      private static extern Handle_String ListPrefix_340(Handle_String h, byte[] Prefix, IntPtr Prefix_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ListSeparator_341")]
      private static extern Handle_String ListSeparator_341(Handle_String h, byte[] Separator, IntPtr Separator_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Match_342")]
      private static extern Handle_String Match_342(Handle_String h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MatchCount_343")]
      private static extern IntPtr MatchCount_343(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MatchIndexAtChar_344")]
      private static extern IntPtr MatchIndexAtChar_344(Handle_String h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MatchIndexAtToken_345")]
      private static extern IntPtr MatchIndexAtToken_345(Handle_String h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MemoryIndex_610")]
      private static extern IntPtr MemoryIndex_610(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "NonMatches_346")]
      private static extern Handle_String NonMatches_346(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Parent_347")]
      private static extern Handle_String Parent_347(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Populate_348")]
      private static extern Handle_String Populate_348(Handle_String h, double first, double last, byte[] expression, IntPtr expression_Size, double Step, byte[] Condition, IntPtr Condition_Size, byte[] VarIndex, IntPtr VarIndex_Size, byte[] VarFirst, IntPtr VarFirst_Size, byte[] VarLast, IntPtr VarLast_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReferenceCount_574")]
      private static extern IntPtr ReferenceCount_574(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReferenceIncr_559")]
      private static extern void ReferenceIncr_559(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Release_349")]
      private static extern void Release_349(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Remove_350")]
      private static extern Handle_String Remove_350(Handle_String h, byte[] From_, IntPtr From__Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Replace_351")]
      private static extern Handle_String Replace_351(Handle_String h, byte[] From_, IntPtr From__Size, byte[] To_, IntPtr To__Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Root_352")]
      private static extern Handle_String Root_352(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Skip_353")]
      private static extern Handle_String Skip_353(Handle_String h, byte[] Pattern, IntPtr Pattern_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "StartingFrom_354")]
      private static extern Handle_String StartingFrom_354(Handle_String h, byte[] Pattern, IntPtr Pattern_Size, IntPtr Nth);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "StartingFromEndingWith_355")]
      private static extern Handle_String StartingFromEndingWith_355(Handle_String h, byte[] StartPattern, IntPtr StartPattern_Size, byte[] EndPattern, IntPtr EndPattern_Size, IntPtr StartNth, IntPtr EndNth);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Str_357")]
      private static extern IntPtr Str_357(Handle_String h, out IntPtr ReturnSize, IntPtr Nth, IntPtr TokenCount);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Str_358")]
      private static extern Handle_String Str_358(Handle_String h, byte[] StringValue, IntPtr StringValue_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "SubString_359")]
      private static extern Handle_String SubString_359(Handle_String h, IntPtr StartPos, IntPtr Length);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "TokenCharLen_360")]
      private static extern IntPtr TokenCharLen_360(Handle_String h, IntPtr StartIndex, IntPtr Count);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "TokenCharPos_361")]
      private static extern IntPtr TokenCharPos_361(Handle_String h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "TokenCount_362")]
      private static extern IntPtr TokenCount_362(Handle_String h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "TokenIndexAtChar_363")]
      private static extern IntPtr TokenIndexAtChar_363(Handle_String h, IntPtr CharacterPosition);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "TokenPos_364")]
      private static extern IntPtr TokenPos_364(Handle_String h, IntPtr Nth);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ToLower_365")]
      private static extern Handle_String ToLower_365(Handle_String h, byte[] Pattern, IntPtr Pattern_Size, IntPtr Nth);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ToUpper_366")]
      private static extern Handle_String ToUpper_366(Handle_String h, byte[] Pattern, IntPtr Pattern_Size, IntPtr Nth);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Transform_367")]
      private static extern Handle_String Transform_367(Handle_String h, Handle_Transforms tt);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "TypeOfToken_368")]
      private static extern TokenType TypeOfToken_368(Handle_String h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "uCalc_539")]
      private static extern Handle_uCalc uCalc_539(Handle_String h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "UpTo_369")]
      private static extern Handle_String UpTo_369(Handle_String h, byte[] Pattern, IntPtr Pattern_Size, IntPtr Nth);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Add_372")]
      private static extern Handle_Tokens Add_372(Handle_Tokens h, Handle_Item TokenItem);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Add_373")]
      private static extern Handle_Item Add_373(Handle_Tokens h, byte[] TokenDef, IntPtr TokenDef_Size, TokenType Type, byte[] CloseBracket, IntPtr CloseBracket_Size, IntPtr SubMatch, RegExGrammar Grammar, IntPtr InsertionPoint);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Add_374")]
      private static extern Handle_Tokens Add_374(Handle_Tokens h, Handle_Tokens TokenCollection);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Clear_375")]
      private static extern void Clear_375(Handle_Tokens h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Comment_540")]
      private static extern IntPtr Comment_540(Handle_Tokens h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Comment_547")]
      private static extern Handle_Tokens Comment_547(Handle_Tokens h, byte[] CommentStr, IntPtr CommentStr_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ContextSwitch_377")]
      private static extern Handle_Tokens ContextSwitch_377(Handle_Tokens h, Handle_Tokens OtherTokenGroup, Handle_Item StartToken, Int32 InsertionPointA, byte[] EndToken, IntPtr EndToken_Size, Int32 InsertionPointB);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ContextSwitch_378")]
      private static extern Handle_Tokens ContextSwitch_378(Handle_Tokens h, Handle_Tokens OtherTokenGroup, byte[] StartToken, IntPtr StartToken_Size, Int32 InsertionPointA, byte[] EndToken, IntPtr EndToken_Size, Int32 InsertionPointB);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ContextSwitch_379")]
      private static extern Handle_Tokens ContextSwitch_379(Handle_Tokens h, Handle_Tokens OtherTokenGroup, byte[] StartToken, IntPtr StartToken_Size, byte[] EndToken, IntPtr EndToken_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Count_380")]
      private static extern IntPtr Count_380(Handle_Tokens h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Define_381")]
      private static extern Handle_Item Define_381(Handle_Tokens h, byte[] TokenDef, IntPtr TokenDef_Size, TokenType Type, IntPtr SubMatch, RegExGrammar Grammar);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IndexOf_383")]
      private static extern Int32 IndexOf_383(Handle_Tokens h, Handle_Item GivenToken);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Insert_384")]
      private static extern Handle_Item Insert_384(Handle_Tokens h, Int32 InsertionPoint, byte[] TokenDef, IntPtr TokenDef_Size, TokenType Type, byte[] CloseBracket, IntPtr CloseBracket_Size, IntPtr SubMatch, RegExGrammar Grammar);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IsProperty_386")]
      private static extern Int64 IsProperty_386(Handle_Tokens h, ItemIs ItemProperty);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IsProperty_387")]
      private static extern Handle_Tokens IsProperty_387(Handle_Tokens h, ItemIs ItemProperty, Int64 TrueOrFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MemoryIndex_611")]
      private static extern IntPtr MemoryIndex_611(Handle_Tokens h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Release_388")]
      private static extern void Release_388(Handle_Tokens h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Remove_389")]
      private static extern void Remove_389(Handle_Tokens h, Handle_Item TokenItem);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Token_390")]
      private static extern Handle_Item Token_390(Handle_Tokens h, IntPtr Index);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "uCalc_544")]
      private static extern Handle_uCalc uCalc_544(Handle_Tokens h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Clone_504")]
      private static extern Handle_Transforms Clone_504(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Comment_542")]
      private static extern IntPtr Comment_542(Handle_Transforms h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Comment_548")]
      private static extern Handle_Transforms Comment_548(Handle_Transforms h, byte[] CommentStr, IntPtr CommentStr_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Filter_393")]
      private static extern Handle_Transforms Filter_393(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Filter_394")]
      private static extern Handle_Transforms Filter_394(Handle_Transforms h, byte[] expression, IntPtr expression_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Find_395")]
      private static extern Handle_Transforms Find_395(Handle_Transforms h, byte[] Str, IntPtr Str_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "FromTo_396")]
      private static extern Handle_Rule FromTo_396(Handle_Transforms h, byte[] FromStr, IntPtr FromStr_Size, byte[] ToStr, IntPtr ToStr_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IndexBase_399")]
      private static extern IntPtr IndexBase_399(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IndexBase_400")]
      private static extern Handle_Transforms IndexBase_400(Handle_Transforms h, IntPtr BaseValue);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IsProperty_402")]
      private static extern Int64 IsProperty_402(Handle_Transforms h, ItemIs ItemProperty);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "IsProperty_403")]
      private static extern Handle_Transforms IsProperty_403(Handle_Transforms h, ItemIs ItemProperty, Int64 TrueOrFalse);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Matches_404")]
      private static extern Handle_Matches Matches_404(Handle_Transforms h, MatchesOption Options);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "MemoryIndex_612")]
      private static extern IntPtr MemoryIndex_612(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "NewPass_405")]
      private static extern Handle_Transforms NewPass_405(Handle_Transforms h, Int64 Share);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "NewRule_406")]
      private static extern Handle_Rule NewRule_406(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Pass_407")]
      private static extern Handle_Transforms Pass_407(Handle_Transforms h, IntPtr Index, Int64 Share);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "PassCount_408")]
      private static extern IntPtr PassCount_408(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Range_409")]
      private static extern Handle_Transforms Range_409(Handle_Transforms h, IntPtr Start, IntPtr Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReferenceCount_572")]
      private static extern IntPtr ReferenceCount_572(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "ReferenceIncr_558")]
      private static extern void ReferenceIncr_558(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Release_410")]
      private static extern void Release_410(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Reset_411")]
      private static extern Handle_Transforms Reset_411(Handle_Transforms h, TransformResetOption ResetOption);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "RulesDefault_412")]
      private static extern Handle_Rule RulesDefault_412(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "RulesOverall_413")]
      private static extern Handle_Rule RulesOverall_413(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Str_415")]
      private static extern IntPtr Str_415(Handle_Transforms h, out IntPtr ReturnSize);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Str_416")]
      private static extern Handle_Transforms Str_416(Handle_Transforms h, byte[] InputStr, IntPtr InputStr_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "StrData_417")]
      private static extern byte[] StrData_417(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "StrLength_418")]
      private static extern IntPtr StrLength_418(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "SubStr_419")]
      private static extern IntPtr SubStr_419(Handle_Transforms h, out IntPtr ReturnSize, IntPtr Start, IntPtr Count);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Tokens_420")]
      private static extern Handle_Tokens Tokens_420(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Transform_422")]
      private static extern Handle_Transforms Transform_422(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "Transform_423")]
      private static extern Handle_Transforms Transform_423(Handle_Transforms h, byte[] expression, IntPtr expression_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "TransformWithSteps_424")]
      private static extern Handle_String TransformWithSteps_424(Handle_Transforms h, byte[] expression, IntPtr expression_Size);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "uCalc_546")]
      private static extern Handle_uCalc uCalc_546(Handle_Transforms h);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "uCalc_425")]
      private static extern Handle_Transforms uCalc_425(Handle_Transforms h, Handle_uCalc uh);
      [DllImport(uCalcDLL, CallingConvention = CallingConvention.StdCall, EntryPoint = "WasModified_426")]
      private static extern Int64 WasModified_426(Handle_Transforms h);


      /// <summary>Class for retrieving arguments passed to a callback, returning a value, and more</summary>
      public partial struct Callback { // Blittable Sequantial layout value type
         public readonly Handle_Callback h;
         public IntPtr Handle() { return (IntPtr)h; }

         /// <summary>Returns the value of an argument passed to a uCalc.Callback function</summary>
         /// <returns>Returns a Double</returns>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>
         /// <para>When a callback function is defined with DefineFunction or DefineOperator, that callback function will receive arguments
         /// that uCalc passes to it through Arg().  Arg() handles values of the default Double floating point data type.  For other data
         /// types, use ArgInt32, ArgStr, etc instead.</para>
         /// 
         /// <para>Note that Index is 1-based.  So if there are 3 arguments, the Index of the first will be 1 (not 0), and the second and
         /// third will be 2 and 3 respectively.</para>
         /// 
         /// <para>If you are defining a binary operator, 1 is the index for the left operand, and 2 is for the right operand. For a unary
         /// operator, the index is 1.</para>
         /// </remarks>
         public double Arg(int Index) { return Arg_93(h, (IntPtr)Index); }

         /// <summary>Returns the first argument of a callback</summary>
         /// <returns>Returns a Double</returns>
         /// <remarks>Similar to Arg[1], but slightly more efficient.</remarks>
         public double Arg1() { return Arg1_94(h); }

         /// <summary>Returns the second argument of a callback</summary>
         /// <remarks>Similar to Arg[2], but slightly more efficient.</remarks>
         public double Arg2() { return Arg2_95(h); }

         /// <summary>Returns the address of the argument value that is being passed</summary>
         /// <returns>Returns a pointer</returns>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>In addition to Arg() which returns arguments that are of the default Double floating point numeric type, there are
         /// specialized functions for other datatypes such as ArgInt32, ArgInt64, etc.  However, if the argument is of a data type that
         /// does not have a specialized function, you can retrieve the address of the argument value as a pointer, and use that to retrieve
         /// the value.</remarks>
         public IntPtr ArgAddr(int Index) { return ArgAddr_96(h, (IntPtr)Index); }

         /// <summary>Returns the Boolean integer argument passed to a uCalc.Callback function</summary>
         /// <returns>Returns a Boolean</returns>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>If a function parameter (or operator operand) was explicitely defined as Int32, then the callback function can recieve
         /// it through ArgBool.  If the argument is of a different but compatible numeric type, a conversion takes place first before
         /// being passed.</remarks>
         public bool ArgBool(int Index) { return ArgBool_97(h, (IntPtr)Index) != 0; }

         /// <summary>Returns the number of arguments being passed to a uCalc.Callback function</summary>
         /// <returns>0 if no arg is passed; or a positive integer representing the number of args being passed.</returns>
         /// <remarks>This is useful for functions that have a variable number of arguments</remarks>
         public int ArgCount() { return (int)ArgCount_98(h); }

         /// <summary>Returns the value of an argument passed to a uCalc.Callback function</summary>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>
         /// <para>When a callback function is defined with DefineFunction or DefineOperator, that callback function will receive arguments
         /// that uCalc passes to it through the Arg().  Arg() handles values of the default Double floating point data type.  For other
         /// data types, use ArgInt32, ArgStr, etc instead.</para>
         /// 
         /// <para>Note that Index is 1-based.  So if there are 3 arguments, the Index of the first will be 1 (not 0), and the second and
         /// third will be 2 and 3 respectively.</para>
         /// 
         /// <para>If you are defining a binary operator, 1 is the index for the left operand, and 2 is for the right operand. For a unary
         /// operator, the index is 1.</para>
         /// </remarks>
         public double ArgDbl(int Index) { return ArgDbl_99(h, (IntPtr)Index); }

         /// <summary>Returns the object instance of the expression that&apos;s passed as an argument to a uCalc.Callback function</summary>
         /// <returns>Returns a parsed expression object with which you can call functions like Evaluate or Execute</returns>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>uCalc allows you to not only pass arguments by value or by reference.  But it also lets you pass an argument by expression.
         ///  When an expression is passed ByExpression, the expresison is not evaluated before being passed. Instead a handle to the parsed
         /// expression is passed, allowing the callback to perform the evaluation.  This is very useful for things like &quot;if&quot;
         /// and &quot;while&quot; loop constructs etc.</remarks>
         public Expression ArgExpr(int Index) { return ArgExpr_100(h, (IntPtr)Index); }

         public Int16 ArgInt16(int Index) { return ArgInt16_614(h, (IntPtr)Index); }

         /// <summary>Returns the Int32 integer argument passed to a uCalc.Callback function</summary>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>If a function parameter (or operator operand) was explicitely defined as Int32, then the callback function can recieve
         /// it through ArgInt32.  If the argument is of a different but compatible numeric type, a conversion takes place first before
         /// being passed.</remarks>
         public Int32 ArgInt32(int Index) { return ArgInt32_101(h, (IntPtr)Index); }

         /// <summary>Returns the Int64 integer argument being passed to a uCalc.Callback function</summary>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>If a function parameter (or operator operand) was explicitely defined as Int64, then the callback function can recieve
         /// it through ArgInt64.  If the argument is of a different but compatible numeric type, a conversion takes place first before
         /// being passed.</remarks>
         public Int64 ArgInt64(int Index) { return ArgInt64_102(h, (IntPtr)Index); }

         /// <summary>Returns an object instance of the argument being passed to a uCalc.Callback function</summary>
         /// <returns>Returns a uCalc.Item; The returned object can be used (where relevent) just like any other uCalc.Item</returns>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>With this object, you can do such things as determine the argument&apos;s data type, determine which instance of
         /// uCalc it belongs to, retrieve the pointer of the value of the argument, etc.</remarks>
         public Item ArgItem(int Index) { return ArgItem_103(h, (IntPtr)Index); }

         /// <summary>Returns a pointer value being passed as an argument to a uCalc.Callback function</summary>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>If a function parameter (or operator operand) was explicitely defined as a Pointer, then the callback function can
         /// recieve it through ArgPtr.  Note that ArgPtr returns any address value that is being sent, whereas ArgAddr returns the address
         /// of the value being passed.</remarks>
         public IntPtr ArgPtr(int Index) { return ArgPtr_104(h, (IntPtr)Index); }

         /// <summary>Returns the value being passed as argument in to a uCalc.Callback function in the form of a string</summary>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>Returns data of any type to a string. In addition to returning an argument that was passed as a string, ArgStr has
         /// the special property of returning an argument that was passed as any data type.  If the argument was not a string, ArgStr
         /// first converts it to a string before returning its value.</remarks>
         public string ArgStr(int Index) {
            var ReturnStr = ArgStr_105(h, out var ReturnStrSize, (IntPtr)Index);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Raises an error in a uCalc.Callback function</summary>
         /// <returns>Returns ErrorHandlerResponse.Abort, ErrorHandlerResponse.Resume, or ErrorHandlerResponse.ReRaise.</returns>
         /// <param name="ErrNumber">Index number of error to raise</param>
         /// <remarks>Your callback can raise an error by calling this funciton with one of the values of the ErrorMsg enum.  End-user
         /// function definitions can also raise an error by calling Error(). The return value is determined by error handlers that you
         /// may have added.  ErrorHandlerResponse.Abort (value of 0) is returned by default if there are no error handlers.</remarks>
         public ErrorHandlerResponse ErrorRaise(ErrorMsg ErrNumber) { return ErrorRaise_106(h, ErrNumber); }

         /// <summary>Raises an error with a user-defined message in a uCalc.Callback function</summary>
         /// <returns>Returns ErrorHandlerResponse.Abort, ErrorHandlerResponse.Resume, or ErrorHandlerResponse.ReRaise.</returns>
         /// <param name="ErrMessage">Text of custom error message to return</param>
         /// <remarks>Your callback can raise a custom error message (one not listed as a ErrorMsg enum) by simply passing the text of
         /// the message to this function.</remarks>
         public ErrorHandlerResponse ErrorRaiseMessage(string ErrMessage) { return ErrorRaiseMessage_107(h, Unicode.Utf8GetBytes(ErrMessage), Unicode.Utf8GetLength(ErrMessage)); }

         /// <summary>Sets a numerical value for an error</summary>
         /// <param name="ErrNumber">Index number of error to raise</param>
         /// <remarks>The index corresponds with an ErrorMsg enum.</remarks>
         public void ErrorSet(ErrorMsg ErrNumber) { ErrorSet_108(h, ErrNumber); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Returns the function or operator Item of the current expression part</summary>
         /// <remarks>
         /// <para>In a callback, you can not only get the arguments that were passed, but also the associated function or operator that
         /// sent the arguments, using this function.</para>
         /// 
         /// <para>If several functions and/or operators are mapped to the same callback function name, you can use this to let you determine
         /// which one is currently being called, based on properties you can check.</para>
         /// </remarks>
         public Item Item() { return Item_110(h); }

         /// <summary>Sets the return value (floating point) of a uCalc.Callback function</summary>
         /// <param name="Value">Floating point value that your callback returns</param>
         /// <remarks>When you define a callback function with DefineFunction and you want to return a floating point number, do it with
         /// Return.</remarks>
         public void Return(double Value) { Return_112(h, Value); }

         /// <summary>Sets the return value of a uCalc.Callback function as an Boolean integer</summary>
         /// <param name="Value">Boolean value that your callback returns</param>
         /// <remarks>When you define a callback function with DefineFunction and you want to return a Boolean, do it with ReturnBool.</remarks>
         public void ReturnBool(bool Value) { ReturnBool_113(h, Convert.ToInt64(Value)); }

         /// <summary>Sets the return value (default floating point) of a uCalc.Callback function</summary>
         /// <param name="Value">Floating point value that your callback returns</param>
         /// <remarks>
         /// <para>When you define a callback function with DefineFunction and you want to return a floating point number, do it with Return.</para>
         /// 
         /// <para>ReturnDbl() is identical to Return(); you may want to use this if &quot;Return&quot; is a reserved keyword in the compiler
         /// you&apos;re using.</para>
         /// </remarks>
         public void ReturnDbl(double Value) { ReturnDbl_114(h, Value); }

         public void ReturnInt16(Int16 Value) { ReturnInt16_615(h, Value); }

         /// <summary>Sets the return value of a uCalc.Callback function as an Int32 integer</summary>
         /// <param name="Value">Integer (32-bit) that your callback returns</param>
         /// <remarks>When you define a callback function with DefineFunction and you want to return a Int32, do it with ReturnInt32.</remarks>
         public void ReturnInt32(Int32 Value) { ReturnInt32_115(h, Value); }

         /// <summary>Sets the return value of a uCalc.Callback function as an Int64 integer</summary>
         /// <param name="Value">Integer (64-bit) that your callback returns</param>
         /// <remarks>When you define a callback function with DefineFunction and you want to return a Int64, do it with ReturnInt64.</remarks>
         public void ReturnInt64(Int64 Value) { ReturnInt64_116(h, Value); }

         /// <summary>Sets the return value of a uCalc.Callback function as a Pointer</summary>
         /// <param name="Value">Pointer value that your callback returns</param>
         /// <remarks>When you define a callback function with DefineFunction that returns a pointer value, do it with ReturnPtr.</remarks>
         public void ReturnPtr(IntPtr Value) { ReturnPtr_117(h, Value); }

         /// <summary>Sets the return value of a uCalc.Callback function as a String</summary>
         /// <param name="Value">String value that your callback returns</param>
         /// <remarks>When you define a callback function with DefineFunction that returns a string, do it with ReturnStr.</remarks>
         public void ReturnStr(string Value) { ReturnStr_118(h, Unicode.Utf8GetBytes(Value), Unicode.Utf8GetLength(Value)); }

         /// <summary>Returns the uCalc instance the current expression part belongs to</summary>
         /// <remarks>A callback is designed to work with all instances of uCalc.  However, at times you may need to know which particular
         /// instance of uCalc the expression part being passed belongs to.</remarks>
         public uCalc uCalc() { return uCalc_119(h); }

      } // End of Callback struct


      /// <summary>Class for working with uCalc data types</summary>
      public partial class DataType {
         public static readonly Handle_DataType Empty = (Handle_DataType)MemoryObjectAddress_(MemoryTable.DataType, IntPtr.Zero);
         public Handle_DataType h = Empty;
         public int MemIndex { get => MemoryIndex(); }
         public DataType(Handle_DataType PassedHandle) { h = PassedHandle; }
         public IntPtr Handle() { return (IntPtr)h; }
         public bool IsEmpty() { return h == Empty; }
         public bool NotEmpty() { return h != Empty; }
         public static implicit operator DataType(Handle_DataType o) { return new DataType(o); }
         public static implicit operator Handle_DataType(DataType o) { return o == null ? Empty : o.h; }
         public DataType() { }
         public DataType(string Definition) { h = Default().Define("DataType: " + Definition).DataType().h; }
         public DataType(uCalc uc, string Definition) { h = uc.Define("DataType: " + Definition).DataType().h; }
         public override string ToString() { return Name(); }
         public static implicit operator string(DataType d) { return d.Name(); }

         /// <summary>Returns the size (in bytes) of one unit of data for this data type</summary>
         /// <returns>Returns an integer</returns>
         public int ByteSize() { return (int)ByteSize_121(h); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Returns the index of the data type</summary>
         /// <returns>Returns a BuiltInType enum</returns>
         /// <remarks>The return value is a BuiltInType enum value that represents the data type of the current object.  For instance,
         /// if the data type is a string, the return value would be BuiltInType.String.</remarks>
         public BuiltInType Index() { return Index_123(h); }

         /// <summary>Returns the uCalc.Item handle of the data type</summary>
         /// <remarks>Each data type object can also be represented as a uCalc Item object by using this return value.</remarks>
         public Item Item() { return Item_124(h); }

         public int MemoryIndex() { return (int)MemoryIndex_605(h); }

         /// <summary>Returns the name of the data type</summary>
         /// <returns>Returns a string</returns>
         public string Name() {
            var ReturnStr = Name_125(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Releases a data type</summary>
         /// <remarks>This releases a Data type from memory.</remarks>
         public void Release() { Release_126(h); h = Empty; }

         /// <summary>Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary>
         /// <param name="ValuePtr">Pointer of the uCalc scalar to be reset</param>
         /// <remarks>Resets a scalar variable to the default value for that variable&apos;s data type.</remarks>
         public void Reset(IntPtr ValuePtr) { Reset_128(h, ValuePtr); }

         /// <summary>Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary>
         /// <param name="ValuePtr">Pointer of the uCalc array to be reset</param>
         /// <param name="Size">Number of elements to reset</param>
         /// <remarks>Resets all the elements of the array to the default value for that variable&apos;s data type.  This should only be
         /// used for arrays that were explicitly allocated with uCalc&apos;s AllocArray.</remarks>
         public void Reset(IntPtr ValuePtr, int Size) { Reset_129(h, ValuePtr, (IntPtr)Size); }

         /// <summary>Sets the value of an array at a given memory location that was allocated by AllocArray</summary>
         /// <param name="Destination">Pointer of array (base address) of which the element value is to be set</param>
         /// <param name="Source">Pointer to a value of the same data type that will be copied to the element</param>
         /// <param name="Index">Index of the array element who&apos;s value needs to be set</param>
         /// <remarks>Sets the value of an array at a given memory location that was allocated by AllocArray</remarks>
         public void SetArray(IntPtr Destination, IntPtr Source, int Index) { SetArray_130(h, Destination, Source, (IntPtr)Index); }

         /// <summary>Sets the current data type as the default (for the current uCalc object)</summary>
         public DataType SetAsDefault() { return SetAsDefault_586(h); }

         /// <summary>Sets the value of a scalar at a given memory location that was allocated by AllocScalar</summary>
         /// <param name="Destination">Pointer of scalar who&apos;s value is to be set</param>
         /// <param name="Source">Pointer to a value of the same data type that will be copied to the element</param>
         /// <remarks>Sets the value of a scalar at a given memory location that was allocated by AllocScalar</remarks>
         public void SetScalar(IntPtr Destination, IntPtr Source) { SetScalar_131(h, Destination, Source); }

         /// <summary>Swaps the pointers of two variables that were allocated with AllocScalar or AllocArray</summary>
         /// <param name="ValuePtrA">Pointer of variable 1</param>
         /// <param name="ValuePtrB">Pointer of variable 2</param>
         /// <remarks>Swaps the pointers of two variables that were allocated with AllocScalar or AllocArray</remarks>
         public void SwapScalarValues(IntPtr ValuePtrA, IntPtr ValuePtrB) { SwapScalarValues_132(h, ValuePtrA, ValuePtrB); }

         /// <summary>Retuns a string representing the value at the given address formatted for the item&apos;s data type</summary>
         /// <param name="ValuePtr">Pointer of the data containing the value you want to display</param>
         /// <remarks>If you have a pointer to data of the given type, you can display this data as a string with this function.  Special
         /// formatting associated with a data type is used, if such formatting was defined.  For instance, if you&apos;re dealing with
         /// a Boolean, then you might get: True, Yes, On, etc, depending on how the formatting was configured with Format.</remarks>
         public string ToString(IntPtr ValuePtr, bool FormatOutput = false) {
            var ReturnStr = ToString_134(h, out var ReturnStrSize, ValuePtr, Convert.ToInt64(FormatOutput));
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Retuns a formatted string of the given data type representing the value that was passed as text</summary>
         /// <param name="Value">String representing data of any type that is to be represented as data of the given type</param>
         /// <remarks>If you pass a string, it will be converted to the given data type, and return using the formatting for that data
         /// type.</remarks>
         public string ToString(string Value) {
            var ReturnStr = ToString_135(h, out var ReturnStrSize, Unicode.Utf8GetBytes(Value), Unicode.Utf8GetLength(Value));
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

      } // End of DataType class


      /// <summary>Class for evaluating an already parsed expression or executing parsed code</summary>
      public partial class Expression : IDisposable {      
      private bool disposed = false;
      public void Dispose() { Dispose(disposing: true); GC.SuppressFinalize(this); }
      protected virtual void Dispose(bool disposing) { if (disposed) return; Release(); disposed = true; }
         public static readonly Handle_Expression Empty = (Handle_Expression)MemoryObjectAddress_(MemoryTable.Expression, IntPtr.Zero);
         public Handle_Expression h = Empty;
         public int MemIndex { get => MemoryIndex(); }
         public Expression(Handle_Expression PassedHandle) { h = PassedHandle; }
         public IntPtr Handle() { return (IntPtr)h; }
         public bool IsEmpty() { return h == Empty; }
         public bool NotEmpty() { return h != Empty; }
         public static implicit operator Expression(Handle_Expression o) { return new Expression(o); }
         public static implicit operator Handle_Expression(Expression o) { return o == null ? Empty : o.h; }
         public Expression(string Expr = "", DataType ReturnType = null) { h = Default().NewExpressionH(Expr, ReturnType); }
         public Expression(uCalc uc, string Expr = "", DataType ReturnType = null) { h = uc.NewExpressionH(Expr, ReturnType); }

         public static implicit operator string(Expression e) { return e.EvaluateStr(); }
         public static implicit operator double(Expression e) { return e.EvaluateDbl(); }
         public static implicit operator bool(Expression e) { return e.Evaluate() != 0; }
         public static implicit operator int(Expression e) { return (int)e.Evaluate(); }
         public override string ToString() { return EvaluateStr(); }
         public static implicit operator Expression(string s) { return new Expression(s); }

         /// <summary>Makes an identical but separate copy of an Expression object</summary>
         /// <returns>New Expression object</returns>
         public Expression Clone() { return Clone_503(h); }

         /// <summary>Returns the data type of a parsed expression</summary>
         /// <returns>DataType of the expression</returns>
         /// <remarks>Every parsed expression has a return type, which may be set explicity, or deduced implicitly based on the context;
         /// for instance comparison operations would return a Boolean, text within quotes would return as a string, and dividing two numeric
         /// values would return a Double precision floating point.  Sometimes it might not be clear what data type an expression has.
         ///  This function can return the data type.</remarks>
         public DataType DataType() { return DataType_137(h); }

         /// <summary>Rapidly evaluates a floating point expression that was parsed with Parse() or passed ByExpression</summary>
         /// <returns>Double precision numeric result of the evaluated expression</returns>
         /// <remarks>
         /// <para>Evaluate can be used instead of Eval for faster results in time-critical loops where the same expression is repeatedly
         /// re-evaluated, with the only change in each iteration typically being the value(s) of one or more variables.  Evaluate works
         /// in pair with Parse, the relatively slower part, which is called first before going into the time-critical loop.  The Expression
         /// object returned by Parse is what calls the Evaluate function.</para>
         /// 
         /// <para>Although Evaluate() is similar to EvaluateDbl(), it is slightly different.  Both return a double precision value, however,
         /// EvaluateDbl() can only return the value of a double precision expression, whereas Evaluate() can return the value of other
         /// numeric types (single precision, various integers, Boolean), in which case the value is first converted to Double precision
         /// before being returned.  Evaluate() is less prone to data type mixups.  On the other hand when a conversion does take place,
         /// there is a slight speed penalty.</para>
         /// 
         /// <para>Evaluate can also be called by an Expression object that was passed to a callback ByExpression.</para>
         /// 
         /// <para>If the expression evaluates to something other than Double precision floating point (such as various integers, complex,
         /// single precision, etc), the value is converted before being returned.  In some cases (such as string, complex, etc), the return
         /// result may or may not have a meaningful value.  However, an error is not raised.</para>
         /// </remarks>
         public double Evaluate() { return Evaluate_138(h); }

         /// <summary>Rapidly evaluates an expression that was parsed as Boolean with Parse or passed ByExpression</summary>
         /// <returns>The result of the evaluated Boolean expression</returns>
         /// <remarks>This is similar to Evaluate, but applies only to expressions that return a Boolean. Certain operators, such as comparison
         /// operators like &lt;, &gt;, ==, etc. return a Boolean.  If the final operation of an expression is Boolean, for instance in
         /// Parse(&quot;x + 1 &gt; 3&quot;), then you can use EvaluateBool.  In earlier versions Evaluate could not be used for types
         /// other than floating point, unless Double was specified in Parse so that a conversion could take place (from Bool in this case)
         /// to Double before being returned.  Now you can either use EvaluateBool (slightly more efficient) or else Evaluate as a catch
         /// all for any type including Boolean.</remarks>
         public bool EvaluateBool() { return EvaluateBool_139(h) != 0; }

         /// <summary>Rapidly evaluates a floating point expression that was parsed with Parse or passed ByExpression</summary>
         /// <returns>The result of the evaluated floating point expression</returns>
         /// <remarks>
         /// <para>EvaluateDbl is similar to Evaluate, but is intended strictly for expressions that return a Double precision value (determined
         /// either by the operators used in the expression, a default double precision type, or an explicit double precision).  It will
         /// not work with Integers or Single precision expressions for instance.</para>
         /// 
         /// <para>If the expression evaluates to a non-numeric value (such as a string) or non-scalar numeric value (such as a complex
         /// number), or if a numeric type other than the default was explicitely specified with Parse, the value returned will be invalid.
         ///  However, an error will not be raised.</para>
         /// </remarks>
         public double EvaluateDbl() { return EvaluateDbl_140(h); }

         /// <summary>Rapidly evaluates expression that was parsed as an integer with Parse or passed ByExpression</summary>
         /// <returns>Returns a 32-bit integer; the result of the evaluated expression</returns>
         /// <remarks>This is similar to Evaluate, but applies only to expressions that return a 32-bit integer.</remarks>
         public Int32 EvaluateInt32() { return EvaluateInt32_141(h); }

         /// <summary>Rapidly evaluates an expression of any data type that was parsed with Parse or passed ByExpression returning a string</summary>
         /// <returns>Returns a string which is the result of the evaluated expression</returns>
         /// <remarks>EvaluateStr is similar to Evaluate.  However, EvaluateStr returns a value as a string.  Unlike Evaluate, which can
         /// only return a floating point value, EvaluateStr can return a value of any data type.  Non-string values are converted to a
         /// string before being returned.</remarks>
         public string EvaluateStr(bool ReturnFormatted = true) {
            var ReturnStr = EvaluateStr_142(h, out var ReturnStrSize, Convert.ToInt64(ReturnFormatted));
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Rapidly evaluates an expression that was parsed with Parse or passed ByExpression and returns a Pointer to the result</summary>
         /// <returns>Returns a pointer to the result instead of the result itself</returns>
         /// <remarks>Use this when you are interested in a pointer to the result instead of the result itself.  For instance, given an
         /// integer result, you might use a pointer in order to retrieve the hi byte and lo byte values.</remarks>
         public IntPtr EvaluateVoid() { return EvaluateVoid_143(h); }

         /// <summary>Rapidly executes code that was parsed with Parse() or passed ByExpression for which a result is not required</summary>
         /// <remarks>This can be used in cases where the return value is not needed, such as statements that represent code to be executed
         /// rather than a math expression.</remarks>
         public void Execute() { Execute_144(h); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         public int MemoryIndex() { return (int)MemoryIndex_606(h); }

         /// <summary>Parses a new expression</summary>
         /// <returns>The current expression object</returns>
         /// <param name="Expr">Expression to be parsed</param>
         /// <param name="ReturnType">Data type of return value</param>
      public Expression Parse(string Expr, DataType ReturnType = null) { var u = uCalc(); Release(); h = u.NewExpressionH(Expr, ReturnType); return this; }

         /// <summary>Parses a new expression</summary>
         /// <returns>The current expression object</returns>
         /// <param name="Expr">Expression to be parsed</param>
         /// <param name="DataTypeIndex">Data type index of return value</param>
      public Expression Parse(string Expr, BuiltInType DataTypeIndex) { var u = uCalc(); Release(); h = u.NewExpressionH(Expr, u.DataTypeOf(DataTypeIndex)); return this; }

         /// <summary>Parses a new expression</summary>
         /// <returns>The current expression object</returns>
         /// <param name="Expr">Expression to be parsed</param>
         /// <param name="ReturnType">Data type of return value</param>
      public Expression Parse(string Expr, string ReturnType) { var u = uCalc(); Release(); h = u.NewExpressionH(Expr, u.DataTypeOf(ReturnType)); return this; }

         public int ReferenceCount() { return (int)ReferenceCount_578(h); }

         /// <summary>.</summary>
         /// <remarks>.</remarks>
         public void ReferenceIncr() { ReferenceIncr_557(h); }

         /// <summary>Releases an expression that was parsed with Parse</summary>
         /// <remarks>Whenever an expression is parsed with Parse, the parsed expression remains in memory until released. When the parsed
         /// expression is no longer needed it should be released with Release to preserve memory. Note: An expression parsed with Parse
         /// must explicitely be released with Release unless the uCalc object it is in is released.  A parsed expression will not be released
         /// automatically when it simply goes out of scope.</remarks>
         public void Release() { Release_146(h); h = Empty; }

         /// <summary>Returns the uCalc object the expression belongs to</summary>
         /// <returns>uCalc object</returns>
         public uCalc uCalc() { return uCalc_463(h); }

      } // End of Expression class


      /// <summary>Class for handling uCalc items</summary>
      public partial class Item : IDisposable {      
      private bool disposed = false;
      public void Dispose() { Dispose(disposing: true); GC.SuppressFinalize(this); }
      protected virtual void Dispose(bool disposing) { if (disposed) return; Release(); disposed = true; }
         public static readonly Handle_Item Empty = (Handle_Item)MemoryObjectAddress_(MemoryTable.Item, IntPtr.Zero);
         public Handle_Item h = Empty;
         public int MemIndex { get => MemoryIndex(); }
         public Item(Handle_Item PassedHandle) { h = PassedHandle; }
         public IntPtr Handle() { return (IntPtr)h; }
         public bool IsEmpty() { return h == Empty; }
         public bool NotEmpty() { return h != Empty; }
         public static implicit operator Item(Handle_Item o) { return new Item(o); }
         public static implicit operator Handle_Item(Item o) { return o == null ? Empty : o.h; }
         public Item() { }
         private OpCallback PinAddr(OpCallback Address) { return uCalc().PinAddr(Address); }
         //+++ Cannot overload assignment operator as in C++; implicit conversion op is static & deals with reference; use get/set property for Val instead
         public static implicit operator string(Item e) { return e.ValueStr(); }
         public static implicit operator double(Item e) { return e.Value(); }
         public static implicit operator bool(Item e) { return e.ValueBool(); }
         public static implicit operator int(Item e) { return e.ValueInt32(); }
         public override string ToString() { return ValueStr(); }

         /// <summary>Returns the bracket token that matches this token.</summary>
         /// <returns>Returns the bracket token that matches this token.</returns>
         /// <remarks>Bracket tokens are defined in pairs.  For instance, &quot;[&quot; typically would have &quot;]&quot; as a bracket
         /// match token.  &quot;(&quot; typically goes with bracket matching token &quot;)&quot;, &quot;{&quot; with &quot;}&quot; etc.
         ///  Use this to retrieve the matching bracket token item.</remarks>
         public Item BracketMatch() { return BracketMatch_149(h); }

         /// <summary>Associates another token with this one as the second part of a bracket pair</summary>
         /// <returns>Same token item that was passed as an argument</returns>
         /// <param name="BracketToken">Bracket token</param>
         /// <remarks>Bracket tokens are defined in pairs.  For instance, if you define &quot;[&quot; as a bracket token, you will want
         /// to define &quot;]&quot; as a bracket match token.  &quot;(&quot; typically goes with bracket matching token &quot;)&quot;,
         /// &quot;{&quot; with &quot;}&quot; etc.</remarks>
         public Item BracketMatch(Item BracketToken) { return BracketMatch_150(h, BracketToken); }

         /// <summary>Gets documentation/help/comment/tooltip type info associated with an item</summary>
         /// <returns>Comment associated with the item</returns>
         /// <remarks>Each item can be assigned a comment.  This comment can later be used at your discretion, for instance in a help file
         /// system, or as a tool tip.</remarks>
         public string Comment() {
            var ReturnStr = Comment_520(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets documentation/help/comment/tooltip type info for an item</summary>
         /// <param name="CommentStr">Text you want to set for the comment</param>
         /// <remarks>Each item can be assigned a comment.  This comment can later be used at your discretion, for instance in a help file
         /// system, or as a tool tip.</remarks>
         public Item Comment(string CommentStr) { return Comment_522(h, Unicode.Utf8GetBytes(CommentStr), Unicode.Utf8GetLength(CommentStr)); }

         /// <summary>Returns the number of elements in a uCalc Item</summary>
         /// <remarks>
         /// <para>Depending on the kind of object, this may represent the number of elements in an array, the number of parameters in
         /// a function, the number of operands in an operator, the number of parameters in a syntax construct, the number of parts in
         /// a transform, the number of parts in a compound token, or the value explicitlely set in the Count property when calling Define.</para>
         /// 
         /// <para>When determining the number of elements, optional parameters in a function count as an element.  If the function has
         /// a variable number of arguments, then count is -1.</para>
         /// 
         /// <para>The value returned for a syntax construct may not seem obvious, especially if it has optional and/or alternative parts
         /// or some other complexity.</para>
         /// </remarks>
         public int Count() { return (int)Count_151(h); }

         /// <summary>Data type of a uCalc Item</summary>
         /// <returns>DataType object</returns>
         /// <remarks>The returned data type object can be used to determine the type name, size, etc.</remarks>
         public DataType DataType() { return DataType_524(h); }

         /// <summary>Sets the data type of a uCalc Item</summary>
         /// <returns>Returns the current item</returns>
         /// <param name="dType">Data type you want to assign to this item</param>
         /// <remarks>
         /// <para>This function is intended for use with DefaultItem() or tokens.</para>
         /// 
         /// <para>If you want to define a default type other than Double precision for math parser operations, then use this on DafaultItem().</para>
         /// 
         /// <para>Some tokens, such as literal strings, or numeric values, generally require an underlying data type.</para>
         /// 
         /// <para>The data type for other items is usually defined at the same time the item itself is defined, instead of after the fact
         /// with this function.</para>
         /// </remarks>
         public Item DataType(DataType dType) { return DataType_152(h, dType); }

         /// <summary>Returns the callback address of a defined function (or operator)</summary>
         /// <returns>Returns the callback address of a defined function (or operator)</returns>
         /// <remarks>
         /// <para>The returned value is the actual pointer address of the function as used by uCalc internally, and not a delegate, even
         /// in .NET.  As such, in .NET this value can be used in the more general Define function but not functions like DefineFunction
         /// or DefineOperator.</para>
         /// 
         /// <para>+++ There are different delegate parts to account for</para>
         /// </remarks>
         public OpCallback FunctionAddress() { return FunctionAddress_153(h); }

         public void FunctionAddress(OpCallback Address) { FunctionAddress_596(h, PinAddr(Address)); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Determines if a uCalc Item has the given properties</summary>
         /// <returns>Returns Boolean True if the uCalc Item matches the given property, or False otherwise.</returns>
         /// <param name="ItemProperty">The property you want to check for</param>
         /// <remarks>This helps you determine whether the uCalc Item is a function, operator, variable, data type, literal value, etc,
         /// or has certain other special properties.  Note: Some items may have multiple properties.  For instance the Function property
         /// may be set to true for arrays.</remarks>
         public bool IsProperty(ItemIs ItemProperty) { return IsProperty_156(h, ItemProperty) != 0; }

         /// <summary>Sets a given property</summary>
         /// <returns>Returns a uCalc.Item</returns>
         /// <param name="ItemProperty">The property you want to set</param>
         /// <param name="TrueOrFalse">Value to set the property to</param>
         /// <remarks>Properties are selected from the ItemIs enum</remarks>
         public Item IsProperty(ItemIs ItemProperty, bool TrueOrFalse) { return IsProperty_157(h, ItemProperty, Convert.ToInt64(TrueOrFalse)); }

         public Item(string Definition, IntPtr Address = default(IntPtr)) { h = Default().DefineH(Definition, Address); }
         public Item(uCalc uc, string Definition, IntPtr Address = default(IntPtr)) { h = uc.DefineH(Definition, Address); }

         public int MemoryIndex() { return (int)MemoryIndex_607(h); }

         /// <summary>Returns the name of the uCalc Item</summary>
         /// <remarks>Items such as functions, operators, variables, etc generally are given a name when they are defined. That name can
         /// be retried with this function.</remarks>
         public string Name() {
            var ReturnStr = Name_525(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Next uCalc Item object that shares the same name</summary>
         /// <returns>Returns a uCalc Item object</returns>
         /// <remarks>If several functions, operators, parameters, etc are overloaded -- that is they are defined with different parameters
         /// but share the same name -- or if they (or variables) overshadow each other, this function returns the next uCalc.Item object
         /// that shares the same name.</remarks>
         public Item NextOverload() { return NextOverload_526(h); }

         /// <summary>Returns the precedence level of the uCalc.Item</summary>
         /// <returns>Returns the precedence level of the uCalc.Item</returns>
         /// <remarks>Operators, and some syntax constructs are defined with a precedence level.  When defining a new operator you may
         /// sometimes want to give it the same level as a pre-existing operator, which can be determined with this function.</remarks>
         public int Precedence() { return (int)Precedence_160(h); }

         /// <summary>Sets the precedence level of the uCalc Item</summary>
         /// <param name="Level">Precedence level you want to set for this item</param>
         /// <remarks>
         /// <para>Operators, and some syntax constructs are defined with a precedence level.  The actual level is an arbitrary numeric
         /// value that derives its meaning based on whether this value is greater or less than other operators.</para>
         /// 
         /// <para>For use mainly with DefaultItem.  Precedence levels are usually set at the time an item is defined instead of using
         /// this function.</para>
         /// </remarks>
         public void Precedence(int Level) { Precedence_161(h, (IntPtr)Level); }

         public int ReferenceCount() { return (int)ReferenceCount_577(h); }

         public void ReferenceIncr() { ReferenceIncr_562(h); }

         /// <summary>Retrieves the regex pattern assigned to this token type</summary>
         /// <returns>String</returns>
         /// <remarks>Tokens are defined with a string that represents the regex for the token.  This function can return that string.</remarks>
         public string Regex() {
            var ReturnStr = Regex_163(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets a new regex pattern for this token</summary>
         /// <returns>Returns the current Item</returns>
         /// <param name="Pattern">Regex pattern</param>
         /// <param name="Grammar">Type of RegEx grammar</param>
         /// <remarks>Typically a regex pattern is assigned to a token item at the same time the token item is defined.  However, the regex
         /// pattern and/or type of regex grammar can later be changed.</remarks>
         public Item Regex(string Pattern, RegExGrammar Grammar = RegExGrammar.Default) { return Regex_164(h, Unicode.Utf8GetBytes(Pattern), Unicode.Utf8GetLength(Pattern), Grammar); }

         /// <summary>Releases an Item defined with Define or other related function</summary>
         /// <remarks>
         /// <para>When a function, operator, variable, or other item is defined with such functions as Define, DefineFunction, DefineOperator,
         /// DefineVariable, these items remain in memory, until they are released with this function.  Items should generally be released
         /// when they are no longer needed, in order to preserve memory.</para>
         /// 
         /// <para>Note: A uCalc Item object must explicitely be released with Release unless the entire uCalc object it is a part of is
         /// released.  An Item object will not be released automatically when it simply goes out of scope.</para>
         /// </remarks>
         public void Release() { Release_165(h); h = Empty; }

         /// <summary>Renames an item</summary>
         /// <returns>Returns this object</returns>
         /// <param name="NewName">The new name of the item</param>
         /// <remarks>Any defined item that has a name (such as a variable, constant, function, operator, etc) can be renamed with this
         /// function.</remarks>
         public Item Rename(string NewName) { return Rename_166(h, Unicode.Utf8GetBytes(NewName), Unicode.Utf8GetLength(NewName)); }

         /// <remarks>+++ set this function up</remarks>
         public void SetAsDefault() { SetAsDefault_587(h); }

         /// <summary>Returns the verbatim text representing the definition of an Item</summary>
         /// <remarks>Certain items may have a text property that can be retrieved with a call to this function.  The literal text used
         /// to define an item is usually stored here.</remarks>
         public string Text() {
            var ReturnStr = Text_527(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Returns the uCalc object this Item belongs to</summary>
         /// <returns>uCalc object</returns>
         /// <remarks>
         /// <para>If your transform inovlves calcuations, then they will be performed using the uCalc object returned by this function.
         ///  Each uCalc object can have its own sets of variables, functions, and other definitions.</para>
         /// 
         /// <para>A uCalc.Item is defined in the context of a uCalc object.  You can later use this function to determine the uCalc object
         /// the defined item belongs to.</para>
         /// </remarks>
         public uCalc uCalc() { return uCalc_528(h); }

         /// <summary>Returns the numeric value associated with a uCalc.Item</summary>
         /// <returns>A double precision value</returns>
         /// <remarks>Use this to retrieve the Double precision value associated with a variable.  Other functions are provided for items
         /// that have another data type such as 16-bit integer, 32-bit integer, string, etc.</remarks>
         public double Value() { return Value_168(h); }

         /// <summary>Sets the value of a floating point variable</summary>
         /// <param name="Val">Double precision floating point value that the variable will be set to</param>
         /// <remarks>When a floating point variable is defined with DefineVariable, you can later set a value for this variable by passing
         /// an argument to Value.  Note: If your compiler supports pointers, you can define a uCalc variable that is directly attached
         /// to the address of a variable in your source code, and not have to use Value.</remarks>
         public void Value(double Val) { Value_169(h, Val); }

         /// <summary>Sets the value of a variable of any data type by passing a string</summary>
         /// <param name="Val">String containing the value that will be converted and stored in the variable</param>
         /// <remarks>This lets you set the value of a variable defined with DefineVariable (or Define) regardless of the variable&apos;s
         /// data type. The value is passed as a string, and parsed/evaluated as an expression, and internally converted to the appropriate
         /// data type.</remarks>
         public void Value(string Val) { Value_170(h, Unicode.Utf8GetBytes(Val), Unicode.Utf8GetLength(Val)); }

         /// <summary>Copies the value from one item (especially variable) to another</summary>
         /// <param name="OtherItem">Other item to copy from</param>
         public void Value(Item OtherItem) { Value_592(h, OtherItem); }

         /// <summary>Returns the Pointer address of the value associated with a uCalc.Item</summary>
         /// <returns>Pointer (or Integer)</returns>
         /// <remarks>Use this to retrieve the Pointer address of the value associated with a variable (instead of the value itself). 
         /// Note: If the uCalc.Item is a Variable of type Pointer, then ValuePtr returns a pointer value that is stored in that variable,
         /// whereas ValueAddr returns the Pointer of the variable itself.  In other words ValueAddr returns the address in memory of the
         /// current uCalc.Item value, while ValuePtr returns any given address you may have stored as the value of the variable.</remarks>
         public IntPtr ValueAddr() { return ValueAddr_171(h); }

         public void ValueAddr(IntPtr Address) { ValueAddr_595(h, Address); }

         /// <summary>Returns the Boolean value associated with a uCalc.Item</summary>
         /// <returns>Returns a Boolean value</returns>
         /// <remarks>Use this to retrieve the Boolean value associated with a variable.</remarks>
         public bool ValueBool() { return ValueBool_173(h) != 0; }

         /// <summary>Sets the value of a Boolean variable</summary>
         /// <param name="Value">Boolean value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a Boolean variable defined with DefineVariable (or Define)</remarks>
         public void ValueBool(bool Value) { ValueBool_174(h, Convert.ToInt64(Value)); }

         /// <summary>Sets the value of a variable to a value who&apos;s Pointer is passed</summary>
         /// <param name="Value">Pointer to the value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a variable by passing the pointer to the value you want to set the variable to.</remarks>
         public void ValueByPtr(IntPtr Value) { ValueByPtr_175(h, Value); }

         /// <summary>Returns the byte value associated with a uCalc.Item</summary>
         /// <returns>Returns an 8-bit number</returns>
         /// <remarks>Use this to retrieve the Byte value associated with a variable.</remarks>
         public Byte ValueByte() { return ValueByte_177(h); }

         /// <summary>Sets the value of a Byte variable</summary>
         /// <param name="Value">Byte value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a Byte variable defined with DefineVariable (or Define)</remarks>
         public void ValueByte(Byte Value) { ValueByte_178(h, Value); }

         /// <summary>Returns the Double precision value associated with a uCalc.Item</summary>
         /// <returns>Returns a double precision value</returns>
         /// <remarks>Synonymous with Value</remarks>
         public double ValueDbl() { return ValueDbl_180(h); }

         /// <summary>Sets the value of a double precision floating point variable</summary>
         /// <param name="Value">Double precision floating point value that the variable will be set to</param>
         /// <remarks>ValueDbl is synonymous with Value</remarks>
         public void ValueDbl(double Value) { ValueDbl_181(h, Value); }

         /// <summary>Returns the 16-bit integer value associated with a uCalc.Item</summary>
         /// <returns>Returns a 16-bit integer</returns>
         /// <remarks>Use this to retrieve the Int16 value associated with a variable.</remarks>
         public Int16 ValueInt16() { return ValueInt16_183(h); }

         /// <summary>Sets the value of a 16-bit integer variable</summary>
         /// <param name="Value">16-bit integer value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a 16-bit integer variable defined with DefineVariable (or Define).  This works for
         /// both signed and unsigned 16-bit integers.</remarks>
         public void ValueInt16(Int16 Value) { ValueInt16_184(h, Value); }

         /// <summary>Returns the 32-bit integer value associated with a uCalc.Item</summary>
         /// <returns>Returns a 32-bit integer</returns>
         /// <remarks>Use this to retrieve the Int32 value associated with a variable.</remarks>
         public Int32 ValueInt32() { return ValueInt32_186(h); }

         /// <summary>Sets the value of a 32-bit integer variable</summary>
         /// <param name="Value">32-bit integer value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a 32-bit integer variable defined with DefineVariable (or Define).  This works for
         /// both signed and unsigned 32-bit integers.</remarks>
         public void ValueInt32(Int32 Value) { ValueInt32_187(h, Value); }

         /// <summary>Returns the 64-bit integer value associated with a uCalc.Item</summary>
         /// <returns>Returns a 64-bit value</returns>
         /// <remarks>Use this to retrieve the Int64 value associated with a variable.</remarks>
         public Int64 ValueInt64() { return ValueInt64_189(h); }

         /// <summary>Sets the value of a 64-bit integer variable</summary>
         /// <param name="Value">64-bit integer value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a 64-bit integer variable defined with DefineVariable (or Define).  This works for
         /// both signed and unsigned 64-bit integers.</remarks>
         public void ValueInt64(Int64 Value) { ValueInt64_190(h, Value); }

         /// <summary>Returns the Pointer value associated with a uCalc.Item</summary>
         /// <remarks>Use this to retrieve the Pointer value associated with a variable. Note: If the uCalc.Item is a Variable of type
         /// Pointer, then ValuePtr returns a pointer value that is stored in that variable, whereas ValueAddr returns the Pointer of the
         /// variable itself.  In other words ValueAddr returns the address in memory of the current uCalc.Item value, while ValuePtr returns
         /// any value you may have set as the address the variable points to.</remarks>
         public IntPtr ValuePtr() { return ValuePtr_192(h); }

         /// <summary>Sets the value of a pointer variable</summary>
         /// <param name="Value">Pointer that the variable will be set to</param>
         /// <remarks>This lets you set the value of a pointer variable defined with DefineVariable (or Define).  Note: This does not change
         /// the pointer of the variable.  It simply stores a pointer value in the variable.</remarks>
         public void ValuePtr(IntPtr Value) { ValuePtr_193(h, Value); }

         /// <summary>Returns the Single precision value associated with a uCalc.Item</summary>
         /// <returns>Returns a Single precision value</returns>
         /// <remarks>Use this to retrieve the Single precision value associated with a variable.</remarks>
         public float ValueSng() { return ValueSng_195(h); }

         /// <summary>Sets the value of a single precision floating point variable</summary>
         /// <param name="Value">Single precision value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a single precision variable defined with DefineVariable (or Define).</remarks>
         public void ValueSng(float Value) { ValueSng_196(h, Value); }

         /// <summary>Returns the String value associated with a uCalc.Item</summary>
         /// <remarks>
         /// <para>Use this to retrieve the String value associated with a variable.</para>
         /// 
         /// <para>ValueStr has a special property beyond the Value function for other data types in that it can return the value of an
         /// item regardless of what data type it has.  The value is returned as a string, after being converted if necessary.</para>
         /// </remarks>
         public string ValueStr(bool FormatOutput = false) {
            var ReturnStr = ValueStr_198(h, out var ReturnStrSize, Convert.ToInt64(FormatOutput));
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets the value of a string variable</summary>
         /// <param name="Value">String value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a string variable defined with DefineVariable (or Define).</remarks>
         public void ValueStr(string Value) { ValueStr_199(h, Unicode.Utf8GetBytes(Value), Unicode.Utf8GetLength(Value)); }

      } // End of Item class


      /// <summary>Class for returning a list of matches</summary>
      public partial class Matches {
         public static readonly Handle_Matches Empty = (Handle_Matches)MemoryObjectAddress_(MemoryTable.Matches, IntPtr.Zero);
         public Handle_Matches h = Empty;
         public int MemIndex { get => MemoryIndex(); }
         public Matches(Handle_Matches PassedHandle) { h = PassedHandle; }
         public IntPtr Handle() { return (IntPtr)h; }
         public bool IsEmpty() { return h == Empty; }
         public bool NotEmpty() { return h != Empty; }
         public static implicit operator Matches(Handle_Matches o) { return new Matches(o); }
         public static implicit operator Handle_Matches(Matches o) { return o == null ? Empty : o.h; }
         public Matches() { }

         /// <summary>Number of matches</summary>
         /// <returns>Integer</returns>
         /// <remarks>This returns the number of matches that were found after a Transform operation</remarks>
         public int Count() { return (int)Count_201(h); }

         /// <summary>Returns the ending character position of a match</summary>
         /// <returns>Integer</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>Returns the ending character position of a match</remarks>
         public int End(int Index) { return (int)End_202(h, (IntPtr)Index); }

         /// <summary>Filters a list of matches</summary>
         /// <returns>Returns the same Matches object</returns>
         /// <param name="NestedRule">Rule to restrict list of matches to</param>
         /// <remarks>Filters a list of matches so that only the ones associated with the given Rule will be in the new list.  In a transform
         /// with multiple rules, sometimes you may want to zero in on the just the matches of a particular rule.  This function lets you
         /// do that.</remarks>
         public Matches Filter(Rule NestedRule) { return Filter_203(h, NestedRule); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Returns nth match index given a starting character position</summary>
         /// <returns>Integer representing nth index</returns>
         /// <param name="StartPos">Starting character position of match you&apos;re interested in</param>
         /// <param name="NearestIndex">Hint of nearest index</param>
         /// <remarks>
         /// <para>This gives you the index of the match that starts at a given character position.  For instance, given this string: &quot;This
         /// is a test&quot;, which might contain the matches &quot;This&quot;, &quot;is&quot;, &quot;a&quot;, and &quot;test&quot;.  If
         /// you know that &quot;is&quot; starts at character position 5, and want to know which match it is, IndexOf(5) will let you know
         /// that &quot;is&quot; reprents match 1 (where match 0 is &quot;This&quot;).</para>
         /// 
         /// <para>This is useful for finding the relative nth index in a list of matches, when you have more than one match lists for
         /// the same set of data and one is a subset of the other.</para>
         /// </remarks>
         public int IndexOf(int StartPos, int NearestIndex = 0) { return (int)IndexOf_205(h, (IntPtr)StartPos, (IntPtr)NearestIndex); }

         /// <summary>Determines if an Item has the given property</summary>
         /// <returns>Returns True if the Item matches the given property, or False otherwise.</returns>
         /// <param name="ItemProperty">The property you want to check for</param>
         public bool IsProperty(ItemIs ItemProperty) { return IsProperty_207(h, ItemProperty) != 0; }

         /// <summary>Sets a given property</summary>
         /// <returns>Returns the same Matches object</returns>
         /// <param name="ItemProperty">The property you want to set</param>
         /// <param name="TrueOrFalse">Value to set the property to</param>
         /// <remarks>Sets a property for a Matches object</remarks>
         public Matches IsProperty(ItemIs ItemProperty, bool TrueOrFalse) { return IsProperty_208(h, ItemProperty, Convert.ToInt64(TrueOrFalse)); }

         /// <summary>Returns the length of a match</summary>
         /// <returns>Integer</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>Returns the length of a match</remarks>
         public int Length(int Index) { return (int)Length_210(h, (IntPtr)Index); }

         /// <summary>Adjusts the reported length of a match</summary>
         /// <param name="Index">Index of match</param>
         /// <param name="value">New length of a match</param>
         /// <remarks>This adjusts the reported length of a match.  Updating the length of the matches is necessary for instance if you
         /// are converting text from utf-8 (default mode uCalc uses internally) to utf-16.</remarks>
         public void Length(int Index, int value) { Length_211(h, (IntPtr)Index, (IntPtr)value); }

         public int MemoryIndex() { return (int)MemoryIndex_608(h); }

         /// <summary>Returns the new string a match was changed to</summary>
         /// <returns>String value</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>This returns the new string that a match was changed to.  For instance if you do a search/replace transform operation
         /// that changes all occurences of &quot;This&quot; in a string to &quot;That&quot;, then NewStr(n) would return &quot;That&quot;.</remarks>
         public string NewStr(int Index) {
            var ReturnStr = NewStr_212(h, out var ReturnStrSize, (IntPtr)Index);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets an offset for matches</summary>
         /// <returns>Returns the number of matches that were displaced</returns>
         /// <param name="StartingPos">Index of match</param>
         /// <remarks>This sets an offset that the matches&apos; starting and ending character positions will be relaitve to.</remarks>
         public int Offset(int StartingPos = 0) { return (int)Offset_213(h, (IntPtr)StartingPos); }

         /// <summary>Returns the old ending character position of a match before it was changed</summary>
         /// <returns>Returns an integer</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>In a search/replace transform type of operation where string matches are changed to new strings, this function returns
         /// what the ending character position of the match used to be before it was modified.</remarks>
         public int OldEnd(int Index) { return (int)OldEnd_214(h, (IntPtr)Index); }

         /// <summary>Returns the old length of a match before it was changed</summary>
         /// <returns>Returns an integer</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>In a search/replace transform type of operation where string matches are changed to new strings, this function returns
         /// what the length of the matching string used to be before it was modified.</remarks>
         public int OldLength(int Index) { return (int)OldLength_216(h, (IntPtr)Index); }

         /// <summary>Sets the OldLength value of a match to something else</summary>
         /// <param name="Index">Index of match</param>
         /// <param name="value">Sets OldLength to this value</param>
         /// <remarks>In a search/replace transform type of operation where string matches are changed to new strings, the one argument
         /// version of OldLength() returns what the length of the matching string used to be before it was modified.  However, in some
         /// cases, the reported OldLength() value may need to be adjusted due to Unicode conversions, such as from utf-8 (default mode
         /// uCalc uses internally) to utf-16.  This function lets you change the OldLength() value of matches.</remarks>
         public void OldLength(int Index, int value) { OldLength_217(h, (IntPtr)Index, (IntPtr)value); }

         /// <summary>Releases a Matches object</summary>
         /// <remarks>This releases a Matches object from memory.</remarks>
         public void Release() { Release_218(h); h = Empty; }

         /// <summary>Removes a match from the list of matches</summary>
         /// <param name="Index">Index of match</param>
         public void Remove(int Index) { Remove_219(h, (IntPtr)Index); }

         /// <summary>Resets the list of matches</summary>
         /// <param name="Options">Option for subset of matches to return</param>
         /// <remarks>This resets the list of matches.  The Options argument lets you filter the list in certain ways.</remarks>
         public void Reset(MatchesOption Options = MatchesOption.SameAsBefore) { Reset_220(h, Options); }

         /// <summary>Returns the rule associated with a given match</summary>
         /// <returns>Rule object</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>This returns the rule associated with a given match.</remarks>
         public Rule Rule(int Index) { return Rule_221(h, (IntPtr)Index); }

         /// <summary>Rerturns the starting character position of a match</summary>
         /// <returns>Integer</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>Rerturns the starting character position of a match</remarks>
         public int Start(int Index) { return (int)Start_223(h, (IntPtr)Index); }

         /// <summary>Changes the reported starting position of a match</summary>
         /// <param name="Index">Index of match</param>
         /// <param name="value">Value of starting position</param>
         /// <remarks>This changes the reported starting position of a match.  Useful in cases when a string is converted from Unicode
         /// utf-8 (default mode uCalc uses internally) to utf-16.</remarks>
         public void Start(int Index, int value) { Start_224(h, (IntPtr)Index, (IntPtr)value); }

         /// <summary>Returns a concatenated list of matches</summary>
         /// <returns>String</returns>
         /// <remarks>This returns all of the matches, concatenated as one string.</remarks>
         public string Str() {
            var ReturnStr = Str_226(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Returns the matching string</summary>
         /// <returns>Returns a string</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>This returns the text string of the nth match.</remarks>
         public string Str(int Index) {
            var ReturnStr = Str_227(h, out var ReturnStrSize, (IntPtr)Index);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Returns the uCalc object this Match belongs to</summary>
         /// <returns>uCalc object</returns>
         public uCalc uCalc() { return uCalc_529(h); }

      } // End of Matches class


      /// <summary>Class for rules</summary>
      public partial class Rule {
         public static readonly Handle_Rule Empty = (Handle_Rule)MemoryObjectAddress_(MemoryTable.Rule, IntPtr.Zero);
         public Handle_Rule h = Empty;
         public int MemIndex { get => MemoryIndex(); }
         public Rule(Handle_Rule PassedHandle) { h = PassedHandle; }
         public IntPtr Handle() { return (IntPtr)h; }
         public bool IsEmpty() { return h == Empty; }
         public bool NotEmpty() { return h != Empty; }
         public static implicit operator Rule(Handle_Rule o) { return new Rule(o); }
         public static implicit operator Handle_Rule(Rule o) { return o == null ? Empty : o.h; }
         public Rule() { }

         /// <summary>Determines whether a rule is currently active</summary>
         /// <returns>True or False</returns>
         /// <remarks>This lets you know if a rule is currently active or not.</remarks>
         public bool Active() { return Active_230(h) != 0; }

         /// <summary>Sets whether a rule is active</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that sets a rule as active or not</param>
         /// <remarks>This sets whether a rule is active or not. When you no longer need a rule, you can release it entirely from memory
         /// with Release().  However, at times you may want to only temporarily deactivate a rule, and then later reactivate it for further
         /// use.  That is when you use this function.</remarks>
         public Rule Active(bool TrueFalse) { return Active_231(h, Convert.ToInt64(TrueFalse)); }

         /// <summary>Lets you konw if the rule is Bracket sensitive</summary>
         /// <returns>True/False</returns>
         /// <remarks>Lets you konw if the rule is Bracket sensitive.</remarks>
         public bool BracketSensitive() { return BracketSensitive_233(h) != 0; }

         /// <summary>Sets bracket sensitivity</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that determines bracket sensitivity</param>
         /// <remarks>
         /// <para>This sets whether a rule is bracket sensitive or not.  When set to True, a pattern will not match if it crosses the
         /// boundaries of a bracket grouping.</para>
         /// 
         /// <para>If set to False, then clode block delimiters are treated as ordinary generic tokens, with no special property as delimiters.</para>
         /// 
         /// <para>A bracketed group is a pattern enclosed between tokens defined as a bracket pair.  By default square brackets, [], parenthesis,
         /// (), and curly braces, {}, are defined as bracket pairs.  However, any string pairs can be defined as bracket tokens.</para>
         /// 
         /// <para>So, if you define a pattern &quot;a {etc} b&quot;, it will match &quot;a x y z b c&quot;, &quot;a (x y z) b c&quot;,
         /// &quot;a (x) (y z) b c&quot;, but not &quot;a x y z (b c)&quot;, because the bracket grouping (b c) crosses the boundary of
         /// the &quot;a {etc} b&quot; pattern; the opening parenthesis is found in the would-be match, but the closing parenthesis is
         /// beyond it, whereas in the other examples, both opening and closing parenthesis (when present) are fully within &quot;a {etc}
         /// b&quot;.</para>
         /// 
         /// <para>Note: Brackets used in pattern definitions take precedence over bracket token pairs.  Therefore you may define a pattern
         /// that gives brackets a different behavior than the default.</para>
         /// </remarks>
         public Rule BracketSensitive(bool TrueFalse) { return BracketSensitive_234(h, Convert.ToInt64(TrueFalse)); }

         /// <summary>Lets you know if the rule is case-sensitive</summary>
         /// <returns>True or false</returns>
         /// <remarks>Lets you know if the rule is case-sensitive.</remarks>
         public bool CaseSensitive() { return CaseSensitive_236(h) != 0; }

         /// <summary>Sets whether a rule is case-sensitive</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that determines case sensitivity</param>
         /// <remarks>This sets whether a rule is case-sensitive or not.</remarks>
         public Rule CaseSensitive(bool TrueFalse) { return CaseSensitive_237(h, Convert.ToInt64(TrueFalse)); }

         /// <summary>Documentation/help/comment/tooltip type info associated with a rule</summary>
         /// <returns>Comment associated with the Rule</returns>
         /// <remarks>Each item can be assigned a comment.  This comment can later be used at your discretion, for instance in a help file
         /// system, or as a tool tip.</remarks>
         public string Comment() {
            var ReturnStr = Comment_530(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets documentation/help/comment/tooltip type info for a Rule</summary>
         /// <param name="CommentStr">Text you want to set for the comment</param>
         /// <remarks>Each Rule can be assigned a comment.  This comment can later be used at your discretion, for instance in a help file
         /// system, or as a tool tip.</remarks>
         public Rule Comment(string CommentStr) { return Comment_535(h, Unicode.Utf8GetBytes(CommentStr), Unicode.Utf8GetLength(CommentStr)); }

         /// <summary>Returns the Find part of a rule in a search or transform operation</summary>
         /// <returns>A string</returns>
         /// <remarks>This returns the Find part of a rule.</remarks>
         public string FindPart() {
            var ReturnStr = FindPart_239(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets the Find part of a rule</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Pattern">Pattern for the Find part of a rule</param>
         /// <remarks>This sets the Find part of a rule in a search.</remarks>
         public Rule FindPart(string Pattern) { return FindPart_240(h, Unicode.Utf8GetBytes(Pattern), Unicode.Utf8GetLength(Pattern)); }

         /// <summary>Lets you know weather a rule is focusable</summary>
         /// <returns>True or false</returns>
         /// <remarks>This lets you know weather a rule is focusable.  You may use this property to decide wheather a match shouild be
         /// highlighted for instance.</remarks>
         public bool Focusable() { return Focusable_242(h) != 0; }

         /// <summary>Sets the focusable property of a Rule to True or False</summary>
         /// <returns>Current object</returns>
         /// <param name="TrueFalse">Value that determines rule is focusable</param>
         /// <remarks>This sets the focusable property of a Rule to True or False.</remarks>
         public Rule Focusable(bool TrueFalse) { return Focusable_243(h, Convert.ToInt64(TrueFalse)); }

         /// <summary>Returns the Find part of a rule</summary>
         /// <returns>The current rule object</returns>
         /// <remarks>This returns the Find part of a rule.</remarks>
         public string FromPart() {
            var ReturnStr = FromPart_245(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets the From part of a rule</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Pattern">Pattern for the From part of a rule</param>
         /// <remarks>This sets the From part of a rule.</remarks>
         public Rule FromPart(string Pattern) { return FromPart_246(h, Unicode.Utf8GetBytes(Pattern), Unicode.Utf8GetLength(Pattern)); }

         /// <summary>Defines a transform rule</summary>
         /// <returns>Object for rule being defined</returns>
         /// <param name="FromStr">Pattern that describes the text to match</param>
         /// <param name="ToStr">New text that the match should be changed to</param>
         /// <remarks>Whenever a piece of text matches the pattern described in FromStr, it is replaced with the text found in ToStr.</remarks>
         public Rule FromTo(string FromStr, string ToStr = "") { return FromTo_247(h, Unicode.Utf8GetBytes(FromStr), Unicode.Utf8GetLength(FromStr), Unicode.Utf8GetBytes(ToStr), Unicode.Utf8GetLength(ToStr)); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Lets you know if a rule has a local transform</summary>
         /// <returns>True/False</returns>
         /// <remarks>This lets you know if a rule has a local transform.</remarks>
         public bool HasLocalTransform() { return HasLocalTransform_249(h) != 0; }

         /// <summary>Lets you know if SkipPatternMatching is set</summary>
         /// <returns>True/False</returns>
         /// <remarks>Lets you know if SkipPatternMatching is set.</remarks>
         public bool IgnoreInnerPatterns() { return IgnoreInnerPatterns_251(h) != 0; }

         /// <summary>Sets whether uCalc should ignore pattern matching for a rule</summary>
         /// <returns>This Rule</returns>
         /// <param name="TrueFalse">True or false</param>
         /// <remarks>Sets whether uCalc should ignore pattern matching for a rule.</remarks>
         public Rule IgnoreInnerPatterns(bool TrueFalse) { return IgnoreInnerPatterns_252(h, Convert.ToInt64(TrueFalse)); }

         /// <summary>Returns the index value associated with a rule</summary>
         /// <returns>Integer</returns>
         /// <remarks>Returns the index value associated with a rule.</remarks>
         public int Index() { return (int)Index_254(h); }

         /// <summary>Sets an index value associated with this rule</summary>
         /// <returns>This same rule</returns>
         /// <param name="Value">Index to associate with this rule</param>
         /// <remarks>This sets an index value to be associated with this rule.  Each rule should have a unique index.  For instance, if
         /// you want to highlight the matches from one rule in Red, and the matches from another rule in Blue, by obtaining Matches(n).Rule().Index(),
         /// you can determine which color to choose for match n.</remarks>
         public Rule Index(int Value) { return Index_255(h, (IntPtr)Value); }

         /// <summary>Lets you know if this rule is the child of another rule</summary>
         /// <returns>True/False</returns>
         /// <remarks>Lets you know if this rule is the child of another rule</remarks>
         public bool IsChildRule() { return IsChildRule_256(h) != 0; }

         /// <summary>Determines if a uCalc Item has the given properties</summary>
         /// <returns>Returns True if the uCalc Item matches the given property, or False otherwise.</returns>
         /// <param name="ItemProperty">The property you want to check for</param>
         /// <remarks>Determines if a uCalc Item has the given properties.</remarks>
         public bool IsProperty(ItemIs ItemProperty) { return IsProperty_258(h, ItemProperty) != 0; }

         /// <summary>Sets a given property</summary>
         /// <returns>Current rule</returns>
         /// <param name="ItemProperty">The property you want to set</param>
         /// <param name="TrueOrFalse">Value to set the property to</param>
         /// <remarks>Sets a given property for a rule.</remarks>
         public Rule IsProperty(ItemIs ItemProperty, bool TrueOrFalse) { return IsProperty_259(h, ItemProperty, Convert.ToInt64(TrueOrFalse)); }

         /// <summary>Returns a local transform object</summary>
         /// <returns>Local transform object</returns>
         /// <remarks>When a pattern is matched, you can zoom into the text and perform a local search just within the matched text, independantly
         /// from the rest of the text outside of the match.</remarks>
         public Transforms LocalTransform() { return LocalTransform_260(h); }

         /// <summary>Returns an object that contains the list of matches along with associated properties for each match</summary>
         /// <param name="Options">Option for subset of matches to return</param>
         /// <remarks>Returns an object that contains the list of matches along with associated properties for each match.  With this object,
         /// you can for instance, determine the location and length of each match, which rule the match applies to etc.</remarks>
         public Matches Matches(MatchesOption Options = MatchesOption.SameAsBefore) { return MatchesUTF16(Options); }
         public Matches MatchesUTF16(MatchesOption Options = MatchesOption.SameAsBefore) { return AdjustMatchesForUTF16(this, MatchesUTF8(Options)); }
         public Matches MatchesUTF8(MatchesOption Options = MatchesOption.SameAsBefore) { return Matches_261(h, Options); }

         /// <summary>Returns the number of matches associated with this particular rule</summary>
         /// <returns>Integer</returns>
         /// <remarks>Returns the number of matches associated with this particular rule</remarks>
         public int MatchesCount() { return (int)MatchesCount_262(h); }

         /// <summary>Lets you know the maximum number of matches that will be looked for with this rule</summary>
         /// <returns>Integer</returns>
         /// <remarks>Lets you know the maximum number of matches that will be looked for with this rule.</remarks>
         public int Maximum() { return (int)Maximum_264(h); }

         /// <summary>Sets the maximum number of matches allowed for this rule to count</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Value">Maximum number of matches of this rule allowed</param>
         /// <remarks>
         /// <para>This sets the maximum number of individual matches that this rule should find in order for any matches of this rule
         /// to count.  If more than this number of matches are found in the text, then this rule will be considered to have no matches
         /// at all.  For instance if a rule searches for matches of &quot;a&quot; in &quot;a a r d v a r k&quot;, and the maximum is set
         /// to 2, then no matches of &quot;a&quot; will be counted at all, because there are more than 2 occurences of &quot;a&quot; in
         /// the text.  On the other hand, if Max is set to 3 or 4 or any larger value, then all the matches would count.</para>
         /// 
         /// <para>The default Maximum value is -1, which is the equivalent of no maximum limit.</para>
         /// 
         /// <para>If you want to stop counting after a given number of matches of a rule, without invalidating the matches that were already
         /// found, use StopAfter instead.</para>
         /// </remarks>
         public Rule Maximum(int Value) { return Maximum_265(h, (IntPtr)Value); }

         /// <summary>Returns the maximum number of matches of this rule allowed for the whole search to count</summary>
         /// <returns>Integer</returns>
         /// <remarks>Returns the maximum number of matches of this rule allowed for the whole search to count.</remarks>
         public int MaximumAND() { return (int)MaximumAND_267(h); }

         /// <summary>Sets the maximum number of matches of this rule for the whole search to count</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Value">Maximum number of matches of this rule allowed</param>
         /// <remarks>
         /// <para>This sets the maximum number of individual matches that this rule should find in order for any matches of the transform
         /// to count.  If more than this number of matches are found in the text, then no matches from any rule in the local transform
         /// will count (whereas in a maximum, only matches from the current rule will be cancelled).  For instance if a rule searches
         /// for matches of &quot;a&quot; in &quot;a a r d v a r k&quot;, and the maximum AND is set to 2, then the entire transform (including
         /// other rules) will be invalidated and return no matches, because there are more than 2 occurences of &quot;a&quot; in the text.
         ///  On the other hand, if Max is set to 3 or 4 or any larger value, then all the matches would count.</para>
         /// 
         /// <para>This differs from Maximum in that if Maximum is exceeded, then only the Rule with the maximum will return 0 matches,
         /// whereas this one will cause the whole transform to return 0 matches.  This affects only the current transform and does not
         /// propagate to a parent transform or a transform in a separate pass.</para>
         /// 
         /// <para>The default Maximum value is -1, which is the equivalent of no maximum limit.</para>
         /// 
         /// <para>If you want to stop counting after a given number of matches of a rule, without invalidating any matches, use StopAfter
         /// instead.</para>
         /// </remarks>
         public Rule MaximumAND(int Value) { return MaximumAND_268(h, (IntPtr)Value); }

         public int MemoryIndex() { return (int)MemoryIndex_609(h); }

         /// <summary>Returns the minimum number of matches that must be found in this rule</summary>
         /// <returns>Integer</returns>
         /// <remarks>Returns the minimum number of matches that must be found in this rule</remarks>
         public int Minimum() { return (int)Minimum_270(h); }

         /// <summary>Sets the minimum number of matches that must be found in this rule</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Value">Minimum number of matches required of this rule</param>
         /// <remarks>
         /// <para>This sets the minimum number of individual matches that this rule should find.</para>
         /// 
         /// <para>This differs from a MinimumAND in that if this Minimum is not reached, then it is considered that there are no matches
         /// for this particular rule.  Other rules are not affected.  Whereas in a MinimumAND if this rule does not reach the required
         /// minimum, then the entire local search or transform is invalidated, and it&apos;s considered that no match of any kind is found.</para>
         /// 
         /// <para>If less than this number of matches are found in the text, then none of the matches in this rule will count.  For instance
         /// if a rule searches for matches of &quot;a&quot; in &quot;a a r d v a r k&quot;, and the minimum is set to 5, then no matches
         /// of &quot;a&quot; will be counted at all, because there are not at least 5 occurences of &quot;a&quot; in the text.  On the
         /// other hand, if Min is set to 3 or less, then all the matches count.</para>
         /// 
         /// <para>Default: 0.</para>
         /// </remarks>
         public Rule Minimum(int Value) { return Minimum_271(h, (IntPtr)Value); }

         /// <summary>Lets you know the minimum number of matches that must be found in this rule</summary>
         /// <returns>Integer</returns>
         /// <remarks>Lets you know the minimum number of matches that must be found in this rule.</remarks>
         public int MinimumAND() { return (int)MinimumAND_273(h); }

         /// <summary>Sets the minimum number of matches that must be found in this rule</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Value">Minimum number of matches required of this rule</param>
         /// <remarks>
         /// <para>This sets the minimum number of individual matches that this rule should find.</para>
         /// 
         /// <para>This differs from a Minimum in that if Minimum is not reached, then it is considered that there are no matches for that
         /// particular rule.  Other rules are not affected.  Whereas in a MinimumAND, if this rule does not reach the required minimum,
         /// then the entire search or transform is invalidated, and it&apos;s considered that no match of any kind is found.</para>
         /// 
         /// <para>If less than this number of matches are found in the text, then the total match count will be 0. For instance if a rule
         /// searches for matches of &quot;a&quot; in &quot;a a r d v a r k&quot;, and the minimum is set to 5, the the whole transform
         /// operation will return 0 matches, because there are not at least 5 occurences of &quot;a&quot; in the text.  On the other hand,
         /// if it is set to 3 or less, then all the matches count.</para>
         /// 
         /// <para>Default: 0.</para>
         /// </remarks>
         public Rule MinimumAND(int Value) { return MinimumAND_274(h, (IntPtr)Value); }

         /// <summary>Returns the name of a Rule</summary>
         /// <remarks>Typically the first text token that identifies the rule</remarks>
         public string Name() {
            var ReturnStr = Name_531(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Next uCalc Item object that shares the same name</summary>
         /// <returns>Returns a Rule object</returns>
         /// <remarks>If several Rules overloaded -- that is they are defined with different parameters but share the same name -- or if
         /// they overshadow each other, this function returns the next Rule that shares the same name.</remarks>
         public Rule NextOverload() { return NextOverload_532(h); }

         /// <summary>Returns the transform this rule belongs to</summary>
         /// <returns>Transform object</returns>
         /// <remarks>This returns the transform this rule belongs to.</remarks>
         public Transforms ParentTransform() { return ParentTransform_275(h); }

         /// <summary>Returns the precedence level for a rule</summary>
         /// <returns>The current rule object</returns>
         /// <remarks>Returns the precedence level for a rule.</remarks>
         public int Precedence() { return (int)Precedence_277(h); }

         /// <summary>Sets the precedence level for a rule</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Value">Precedence level</param>
         /// <remarks>
         /// <para>Patterns that start (and/or end) with a pattern variable instead of a token or identifier can (and probably should)
         /// be assigned a precedence level.  You can choose any arbitrary numerical value.  The meaning of the precedence level depends
         /// on whether the value is greater, equal, or less than adjacent patterns with a precedence level.  It works similarly to precedence
         /// levels for operators.  For instance, if you define:</para>
         /// 
         /// <para>FromTo(&quot;{a} + {b}&quot;, &quot;({a} + {b})&quot;).Precedence(10); FromTo(&quot;{a} * {b}&quot;, &quot;({a} * {b})&quot;).Precedence(20);</para>
         /// 
         /// <para>Then</para>
         /// 
         /// <para>&quot;1 + 2 * 3 + 4&quot; transforms into &quot;(1 + (2 * 3) + 4)&quot; &quot;1 * 2 + 3 * 4&quot; transforms into &quot;(1
         /// * 2) + (3 * 4)&quot;</para>
         /// </remarks>
         public Rule Precedence(int Value) { return Precedence_278(h, (IntPtr)Value); }

         /// <summary>Sets a rule property</summary>
         /// <returns>Current object</returns>
         /// <param name="SelectedProperty">Selected Property</param>
         /// <param name="TrueFalse">True/False</param>
         /// <remarks>Sets a rule property</remarks>
         public Rule Property(ItemIs SelectedProperty, bool TrueFalse) { return Property_279(h, SelectedProperty, Convert.ToInt64(TrueFalse)); }

         /// <summary>Lets you know if the rule is quote sensitive</summary>
         /// <returns>True/False</returns>
         /// <remarks>This lets you know if the rule is quote sensitive.  The other version of this function sets or unsets quote sensitivity.</remarks>
         public bool QuoteSensitive() { return QuoteSensitive_281(h) != 0; }

         /// <summary>Sets quote sensitivity</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that determines quote sensitivity</param>
         /// <remarks>
         /// <para>This sets whether a rule is quote-sensitive or not. Quote sensitivity is generally designed for use across all rules
         /// in a transform (or sub transform).  However, it can also be set in the context of an individual pattern.</para>
         /// 
         /// <para>When set to true, it means that a block of text within quotes is treated as one token.  As such, patterns are not searched
         /// for within the quotes.  The contents of the quotes are skipped over.  You may still get around this and parse within quotes
         /// when necessary, by capturing the quote character itself, then parsing subsequent text.  When doing this be mindful of capturing
         /// quote characters in pairs.</para>
         /// 
         /// <para>The actual character(s) used as quotes are configurable with Tokens().Add().</para>
         /// </remarks>
         public Rule QuoteSensitive(bool TrueFalse) { return QuoteSensitive_282(h, Convert.ToInt64(TrueFalse)); }

         /// <summary>Releases a transform rule.  Transform rules are defined with FromTo.</summary>
         /// <remarks>This releases a transform rule from memory.</remarks>
         public void Release() { Release_283(h); h = Empty; }

         /// <summary>Returns the replacement text of a FromTo rule</summary>
         /// <returns>String representing the second argument that was used in FromTo</returns>
         /// <remarks>This allows you to keep track of the replacement text of any given rule, even after it has been defined.</remarks>
         public string ReplacementText() {
            var ReturnStr = ReplacementText_284(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Lets you know if the parser should rewind and parse again after a change</summary>
         /// <returns>True/False</returns>
         /// <remarks>Lets you know if the parser should rewind and parse again after a change.  The other version of this function sets
         /// the value.</remarks>
         public bool RewindOnChange() { return RewindOnChange_286(h) != 0; }

         /// <summary>Determines whether to rewind back to the start of the modified match for further parsing</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that determines if the transfrom should rewind and parse again after a change</param>
         /// <remarks>
         /// <para>After a match is modified with the replacement text of a rule, if RewindOnChange is false (default), then uCalc will
         /// continue parsing from the position just after the insertion of the modified text. If this property is set to true, then the
         /// parser will rewind to the location where the match was found, and parse again, which will allow it to potentially find additional
         /// matches (of any rule, not just the current one) within the already modified text.</para>
         /// 
         /// <para>This property can be extremely useful in many situations.  However, special care should be taken so that an infinite
         /// reparsing session doesn&apos;t take place.  For instance, if a rule changes A to B, and another rule changes B to A, and this
         /// property is set ot true, and a Maximum value is not set, and there is an occurence of A, then A will change to B.  B will
         /// be parsed again and changed to A. The cycle will repeat itself indefinitely.</para>
         /// 
         /// <para>Default: False</para>
         /// </remarks>
         public Rule RewindOnChange(bool TrueFalse) { return RewindOnChange_287(h, Convert.ToInt64(TrueFalse)); }

         /// <summary>Lets you know the associativity property of a rule</summary>
         /// <returns>True/Flase</returns>
         /// <remarks>Lets you know the associativity property of a rule; true means it goes right to left; false means it&apos;s left
         /// to right (default)</remarks>
         public bool RightToLeft() { return RightToLeft_289(h) != 0; }

         /// <summary>Sets whether a rule is right-to-left associative</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that determines case sensitivity</param>
         /// <remarks>This sets whether a rule is is right-to-left associative.</remarks>
         public Rule RightToLeft(bool TrueFalse) { return RightToLeft_290(h, Convert.ToInt64(TrueFalse)); }

         /// <summary>Lets you know if matches for the current pattern will be skipped over</summary>
         /// <returns>True/False</returns>
         /// <remarks>Lets you know if matches for the current pattern will be skipped over.  The other version of SkipOver lets you set
         /// the value.</remarks>
         public bool SkipOver() { return SkipOver_292(h) != 0; }

         /// <summary>Determines whether a given match is skipped over</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that determines if a matched pattern is skipped over</param>
         /// <remarks>
         /// <para>If SkipOver is set to true, then text matches this pattern will be skipped over. This is useful for preventing uCalc
         /// from searching for matches of any other rules within certain text you want to skip over.</para>
         /// 
         /// <para>For instance if you skip over &quot;({etc})&quot;, then everything in parenthesis will be skipped over. So if you want
         /// to find occurences of &quot;a&quot; in &quot;a b c (a b c a b c) a b c&quot;, it will find all occurences of &quot;a&quot;,
         /// except those within parenthesis, because everything in parenthesis is being skipped over.</para>
         /// 
         /// <para>Default: False</para>
         /// </remarks>
         public Rule SkipOver(bool TrueFalse) { return SkipOver_293(h, Convert.ToInt64(TrueFalse)); }

         /// <summary>Lets you know the number of matches this rule must find before matches start counting</summary>
         /// <returns>The current rule object</returns>
         /// <remarks>Lets you know the number of matches this rule must find before matches start counting.  The other version of StartAfter()
         /// lets you set this value.</remarks>
         public int StartAfter() { return (int)StartAfter_295(h); }

         /// <summary>Sets the number of matches this rule must find before matches start counting</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Value">Number of matches that must be found before matches start counting</param>
         /// <remarks>
         /// <para>This sets the number of individual matches that this rule should find first before matches start counting. These first
         /// few matches are not counted in the results.  Onlny matches starting after this number of initial matches are counted.  If
         /// less than this number of matches are found in the text, no match will count for this rule. For instance if a rule searches
         /// for matches of &quot;a&quot; in &quot;a a r d v a r k&quot;, starting after 2, then only the 3rd &quot;a&quot; (the one between
         /// &quot;v&quot; and &quot;r&quot;) will count.  If StartAfter is set to 3, there will be no matches.  If StartAfter is 1, then
         /// then the 2nd and 3rd occurences of &quot;a&quot; will count as matches.</para>
         /// 
         /// <para>The default StartAfter value is 0.  This means all matches that are found will count.</para>
         /// </remarks>
         public Rule StartAfter(int Value) { return StartAfter_296(h, (IntPtr)Value); }

         /// <summary>Lets you know if a rule is statement sensitive</summary>
         /// <returns>True/False</returns>
         /// <remarks>This lets you know if a rule is statement sensitive.  If a rule is statement sensitive, it will not allow a match
         /// to cross statement boundaries.</remarks>
         public bool StatementSensitive() { return StatementSensitive_298(h) != 0; }

         /// <summary>Sets a rule as statement Sensitive</summary>
         /// <returns>Current Rule</returns>
         /// <param name="TrueFalse">Value that determines bracket sensitivity</param>
         /// <remarks>Sets a rule as statement Sensitive</remarks>
         public Rule StatementSensitive(bool TrueFalse) { return StatementSensitive_299(h, Convert.ToInt64(TrueFalse)); }

         /// <summary>Lets you know the number of matches after which this rule should stop finding matches</summary>
         /// <returns>The current rule object</returns>
         /// <remarks>Lets you know the number of matches after which this rule should stop finding matches.  The other version of this
         /// function lets you set the value.</remarks>
         public int StopAfter() { return (int)StopAfter_301(h); }

         /// <summary>Sets the number of matches after which this rule should stop finding matches</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Value">Number of matches after which it should stop finding matches</param>
         /// <remarks>
         /// <para>This sets the number of individual matches that this rule should find.  After this number of matches are found, all
         /// other matches for this rule are ignored.  For instance if a rule searches for matches of &quot;a&quot; in &quot;a a r d v
         /// a r k&quot;, and stops after 2, then only the first two letters here wil match; the third &quot;a&quot; (the one between &quot;v&quot;
         /// and &quot;r&quot;) will not count as a match.</para>
         /// 
         /// <para>The default StopAfter value is -1, which means it will not stop counting matches for this rule.</para>
         /// 
         /// <para>This function has some similarities with Maximum.  However, with Maximum, if the maximum value is not attained, none
         /// of the matches for the rule will count.</para>
         /// </remarks>
         public Rule StopAfter(int Value) { return StopAfter_302(h, (IntPtr)Value); }

         /// <summary>Returns the verbatim text representing the definition of a Rule</summary>
         public string Text() {
            var ReturnStr = Text_533(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Returns the To part of a rule</summary>
         /// <returns>The current rule object</returns>
         /// <remarks>This sets the To part of a rule.</remarks>
         public string ToPart() {
            var ReturnStr = ToPart_304(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets the To part of a rule</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Pattern">Pattern for the From part of a rule</param>
         /// <remarks>This sets the To part of a rule.</remarks>
         public Rule ToPart(string Pattern) { return ToPart_305(h, Unicode.Utf8GetBytes(Pattern), Unicode.Utf8GetLength(Pattern)); }

         /// <summary>Returns the uCalc object this Rule belongs to</summary>
         /// <returns>uCalc object</returns>
         public uCalc uCalc() { return uCalc_534(h); }

         /// <summary>Lets you konw if the rule is Whitespace sensitive</summary>
         /// <returns>True/False</returns>
         /// <remarks>Lets you know if WhitespaceSensitive is set.</remarks>
         public bool WhitespaceSensitive() { return WhitespaceSensitive_307(h) != 0; }

         /// <summary>Sets whether a rule is whitespace-sensitive</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that determines whitespace sensitivity</param>
         /// <remarks>This sets whether a rule is whitespace-sensitive or not.</remarks>
         public Rule WhitespaceSensitive(bool TrueFalse) { return WhitespaceSensitive_308(h, Convert.ToInt64(TrueFalse)); }

      } // End of Rule class


      /// <summary>uCalc string library class</summary>
      public partial class String : IDisposable {      
      private bool disposed = false;
      public void Dispose() { Dispose(disposing: true); GC.SuppressFinalize(this); }
      protected virtual void Dispose(bool disposing) { if (disposed) return; Release(); disposed = true; }
         public static readonly Handle_String Empty = (Handle_String)MemoryObjectAddress_(MemoryTable.String, IntPtr.Zero);
         public Handle_String h = Empty;
         public int MemIndex { get => MemoryIndex(); }
         public String(Handle_String PassedHandle) { h = PassedHandle; }
         public IntPtr Handle() { return (IntPtr)h; }
         public bool IsEmpty() { return h == Empty; }
         public bool NotEmpty() { return h != Empty; }
         public static implicit operator String(Handle_String o) { return new String(o); }
         public static implicit operator Handle_String(String o) { return o == null ? Empty : o.h; }
         public String(string s = "") { h = Default().NewStringH(s); }
         public String(uCalc uc, string s = "") { h = uc.NewStringH(s); }

         public static implicit operator string(String s) { return s.Str(); }
         public static implicit operator String(string s) { return new String(s); }
         public override string ToString() { return Str(); }

         /// <summary>Returns a substring of all text that comes after the occurence of a pattern</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Pattern">Pattern</param>
         /// <param name="Nth">Nth match</param>
         /// <remarks>Returns a substring of all text that comes after the occurence of a pattern.</remarks>
         public String After(string Pattern, int Nth = 1) { return After_310(h, Unicode.Utf8GetBytes(Pattern), Unicode.Utf8GetLength(Pattern), (IntPtr)Nth); }

         /// <summary>Applies a special format to the string</summary>
         /// <returns>Returns the current uCalc string object</returns>
         /// <param name="expression">Expression</param>
         /// <param name="Variable">Name of variable</param>
         /// <remarks>This applies a special format to the string.  The format consists of an expression that evaluates to a string, where
         /// the variable x contains the string.</remarks>
         public String Apply(string expression, string Variable = "") { return Apply_311(h, Unicode.Utf8GetBytes(expression), Unicode.Utf8GetLength(expression), Unicode.Utf8GetBytes(Variable), Unicode.Utf8GetLength(Variable)); }

         /// <summary>Returns the nth token</summary>
         /// <returns>String</returns>
         /// <param name="Index">Nth otken Index</param>
         /// <remarks>Returns the nth token.</remarks>
         public String At(int Index) { return At_312(h, (IntPtr)Index); }

         /// <summary>Returns the character at the given location</summary>
         /// <returns>String object</returns>
         /// <param name="CharacterPosition">Index of charactor location</param>
         /// <remarks>This returns the character at a given location.  The character is returned as a uCalc string.  If that string is
         /// modified, the modification is reflected in the overall string.</remarks>
         public String AtChar(int CharacterPosition) { return AtChar_313(h, (IntPtr)CharacterPosition); }

         /// <summary>Returns the text string of the nth token</summary>
         /// <returns>uCalc string</returns>
         /// <param name="Index">Index of token location</param>
         /// <remarks>This returns the text string of the nth token.  Modifications to the returned uCalc string are reflected in the overall
         /// string.</remarks>
         public String AtToken(int Index) { return AtToken_314(h, (IntPtr)Index); }

         /// <summary>Returns a substring of all text up to (but not including) a given pattern</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Pattern">Pattern</param>
         /// <param name="Nth">Nth match</param>
         /// <remarks>Returns a substring of all text up to (but not including) a given pattern</remarks>
         public String Before(string Pattern, int Nth = 1) { return Before_315(h, Unicode.Utf8GetBytes(Pattern), Unicode.Utf8GetLength(Pattern), (IntPtr)Nth); }

         /// <summary>Returns a substring of text in between (but not including) occurences of two given patterns</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="StartPattern">Pattern</param>
         /// <param name="StopPattern">Pattern 2</param>
         /// <param name="StartNth">Nth match of starting pattern</param>
         /// <param name="StopNth">nth match of ending pattern</param>
         /// <remarks>Returns a substring of text in between (but not including) occurences of two given patterns.</remarks>
         public String Between(string StartPattern, string StopPattern, int StartNth = 1, int StopNth = 1) { return Between_316(h, Unicode.Utf8GetBytes(StartPattern), Unicode.Utf8GetLength(StartPattern), Unicode.Utf8GetBytes(StopPattern), Unicode.Utf8GetLength(StopPattern), (IntPtr)StartNth, (IntPtr)StopNth); }

         /// <summary>Returns the text corresponding to a bracket followed by text up to the next matching closing bracket</summary>
         /// <returns>uCalc string object</returns>
         /// <param name="Index">nth Token Index</param>
         /// <remarks>This returns the text corresponding to a bracket followed by text up to the next matching closing bracket.</remarks>
         public String BracketedText(int Index) { return BracketedText_317(h, (IntPtr)Index); }

         /// <summary>Returns the text between a bracket and the the next matching closing bracket</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Index">nth Token Index</param>
         /// <remarks>This returns the text between a bracket and the the next matching closing bracket.  Brackets themselves are not included.</remarks>
         public String BracketedTextInside(int Index) { return BracketedTextInside_318(h, (IntPtr)Index); }

         /// <summary>Returns the matching bracket for the indicated token</summary>
         /// <returns>uCalc string object</returns>
         /// <param name="Index">nth Token index</param>
         /// <remarks>This returns the matching bracket for the indicated token.  Manipulations to the returned uCalc string will be reflected
         /// in the overall string.</remarks>
         public String BracketMatch(int Index) { return BracketMatch_319(h, (IntPtr)Index); }

         /// <summary>Returns the nth index of the closing bracket token</summary>
         /// <returns>Integer</returns>
         /// <param name="Index">nth token index</param>
         /// <remarks>Returns nth index of the closing bracket token.</remarks>
         public int BracketMatchTokenIndex(int Index) { return (int)BracketMatchTokenIndex_320(h, (IntPtr)Index); }

         /// <summary>Returns the number of characters of the nth match</summary>
         /// <returns>Integer</returns>
         /// <param name="Nth">Index of nth match</param>
         /// <remarks>This returns the number of characters of the nth match.</remarks>
         public int CharCount(int Nth = -1) { return (int)CharCount_321(h, (IntPtr)Nth); }

         /// <summary>Returns the character position of the nth match</summary>
         /// <returns>Integer that represents the character position</returns>
         /// <param name="Nth">Index of nth match</param>
         /// <remarks>This returns the character position of the nth match.</remarks>
         public int CharPos(int Nth = -1) { return (int)CharPos_322(h, (IntPtr)Nth); }

         /// <summary>Gets the option</summary>
         /// <returns>ChildStringOptions</returns>
         /// <remarks>Retrieves the option for  ChildStringOption.</remarks>
         public ChildStringOptions ChildStringOption() { return ChildStringOption_324(h); }

         /// <summary>Sets rules for when a child object is created</summary>
         /// <returns>String object</returns>
         /// <param name="Option">Option</param>
         /// <remarks>Many functions create a new string object derived from another given object.  This option determins the persistence
         /// of previously derived objects.  The right option depends on what you want to do.  If you&apos;re looking to conserve memory,
         /// then ClearAll may be the option you need.  However, with this option, you can no longer refer to previously derived strings,
         /// otherwise the behavior will be undefined (such as crashing).  You may delay object deletetion be removing only the ones that
         /// overlap in location with the newly derived string.  The decision to delete may be delayed based on when an object is being
         /// written to.   ChildStringOptions.ClearOverlapOnWrite, allows you to have multiple overlapping views of the same parent string
         /// without any conflict.  Only when modifying a derived string are overlapped derived strings removed.  Corresponding Detach
         /// options are similar, except that instead of actually erasing an overlapping object, it is disassassiated from the parent string.
         ///  It can continue to be used as an independent string without undefined behavior.  With this option, the independent strings
         /// are erased only when the parent string itself is erased.</remarks>
         public String ChildStringOption(ChildStringOptions Option) { return ChildStringOption_325(h, Option); }

         /// <summary>Clears derived strings from memory</summary>
         /// <returns>String object</returns>
         /// <remarks>Clears derived strings from memory.</remarks>
         public String ClearDerivedStrings() { return ClearDerivedStrings_326(h); }

         /// <summary>Makes an identical but separate copy of a uCalc String object</summary>
         /// <returns>New String object</returns>
         public String Clone() { return Clone_496(h); }

         /// <summary>returns documentation/help/comment/tooltip type info associated with an item</summary>
         /// <returns>Comment associated with the String</returns>
         public string Comment() {
            var ReturnStr = Comment_536(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <param name="CommentStr">Text you want to set for the comment</param>
         public String Comment(string CommentStr) { return Comment_538(h, Unicode.Utf8GetBytes(CommentStr), Unicode.Utf8GetLength(CommentStr)); }

         /// <summary>Compares two strings</summary>
         /// <returns>Returns bigger than, equal, or less than (Comparison enum)</returns>
         /// <param name="OtherString">The other string to compare with</param>
         /// <param name="IndexA">Starting point of first string</param>
         /// <param name="IndexB">Starting point of next string</param>
         /// <param name="Length">Length</param>
         /// <remarks>Compares two strings</remarks>
         public Comparison Compare(string OtherString, int IndexA = 0, int IndexB = 0, int Length = -1) { return Compare_327(h, Unicode.Utf8GetBytes(OtherString), Unicode.Utf8GetLength(OtherString), (IntPtr)IndexA, (IntPtr)IndexB, (IntPtr)Length); }

         /// <summary>Returns the raw string data address in memory of a uCalc string object</summary>
         public IntPtr Data() { return Data_498(h); }

         /// <summary>Attaches a uCalc string to a raw string address in memory</summary>
         /// <param name="StringAddress">Address in memory of raw string</param>
         /// <param name="Size">Number of characters</param>
         /// <remarks>This sets a uCalc string based on a memory address and a size.  This is faster than copying one string to another.
         ///  The original string address must remain valid when this new string is used, until this string is released or its value is
         /// changed, at which point it will no longer depend on the original raw address.</remarks>
         public String Data(IntPtr StringAddress, int Size) { return Data_500(h, StringAddress, (IntPtr)Size); }

         /// <summary>Returns the raw size in characters of the string object</summary>
         /// <remarks>Returns the memory size in characters of the string object.</remarks>
         public int DataSize() { return (int)DataSize_499(h); }

         /// <summary>Searches for occurences of a pattern in the string</summary>
         /// <returns>This uCalc string object</returns>
         /// <param name="Pattern">Pattern</param>
         /// <param name="Nth">Nth match</param>
         /// <remarks>Searches for occurences of a pattern in the string.  Once this operation is executed, you may retrieve any of the
         /// matches with Match(n).</remarks>
         public String Find(string Pattern, int Nth = 1) { return Find_328(h, Unicode.Utf8GetBytes(Pattern), Unicode.Utf8GetLength(Pattern), (IntPtr)Nth); }

         /// <summary>Finds occurences of all text other than the matching pattern</summary>
         /// <returns>This uCalc string object</returns>
         /// <param name="Pattern">Pattern</param>
         /// <param name="Nth">Nth match</param>
         /// <remarks>Finds occurences of all text other than the matching pattern.  This allows you to perform a transform on all text
         /// other than text that matches the given pattern.</remarks>
         public String FindOtherThan(string Pattern, int Nth = 1) { return FindOtherThan_329(h, Unicode.Utf8GetBytes(Pattern), Unicode.Utf8GetLength(Pattern), (IntPtr)Nth); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Gets the index base used in various functions</summary>
         /// <remarks>By default, uCalc String Library functions use a 0-based index.  So the first element is 0 (not 1), followed by 1,
         /// 2, 3, 4 etc. Use this to check to see which base is being used.</remarks>
         public int IndexBase() { return (int)IndexBase_332(h); }

         /// <summary>Sets the index base used in various functions</summary>
         /// <param name="BaseValue">value representing index of 1st element</param>
         /// <remarks>By default, uCalc String Library functions use a 0-based index.  So the first element is 0 (not 1), followed by 1,
         /// 2, 3, 4 etc. Use this if you prefer to start by 1 or another non-zero value.  This value is passed on to string objects derived
         /// from a parent string. Each string may have a different index base.</remarks>
         public String IndexBase(int BaseValue) { return IndexBase_333(h, (IntPtr)BaseValue); }

         /// <summary>Returns a list of matches, consisting of characters, tokens, elements, statements, or lines</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="ListType">Type of list</param>
         /// <remarks>Returns a list of matches, consisting of characters, tokens, elements, statements, or lines.</remarks>
         public String List(StringListType ListType = StringListType.Matches) { return List_334(h, ListType); }

         /// <summary>Sets a format for the List() function</summary>
         /// <param name="Separator">Separator</param>
         /// <param name="Prefix">Prefix</param>
         /// <param name="Postfix">Postfix</param>
         /// <param name="UserFunc">UserFunc</param>
         /// <param name="Element">Element</param>
         /// <param name="Index">Index</param>
         /// <param name="Count">Count</param>
         /// <remarks>
         /// <para>This sets a format that determines how the value of Str() is returned.  Separator will be inserted between each match;
         /// Prefix will be added before the string; Postfix is appended at the end of the string.  UserFunc determines how each match
         /// is displayed.  Element is the name of the variable to use in the function (x by default). Index is the name of the current
         /// match (Index by default).  Count is the number of matches (Count by default).</para>
         /// 
         /// <para>For instance: Given this list of matches: &quot;a&quot;, &quot;b&quot;, &quot;c&quot;, &quot;d&quot; ListOfChars().ListFormat(&quot;,
         /// &quot;, &quot;{&quot;, &quot;}&quot;, &quot;$&apos;{Index} of {Count}:{x}&apos;&quot;).Str() would return: {1 of 4:a, 2 of
         /// 4:b, 3 of 4:c, 4 of 4:d}</para>
         /// </remarks>
         public String ListFormat(string Separator, string Prefix = "", string Postfix = "", string UserFunc = "", string Element = "", string Index = "", string Count = "") { return ListFormat_335(h, Unicode.Utf8GetBytes(Separator), Unicode.Utf8GetLength(Separator), Unicode.Utf8GetBytes(Prefix), Unicode.Utf8GetLength(Prefix), Unicode.Utf8GetBytes(Postfix), Unicode.Utf8GetLength(Postfix), Unicode.Utf8GetBytes(UserFunc), Unicode.Utf8GetLength(UserFunc), Unicode.Utf8GetBytes(Element), Unicode.Utf8GetLength(Element), Unicode.Utf8GetBytes(Index), Unicode.Utf8GetLength(Index), Unicode.Utf8GetBytes(Count), Unicode.Utf8GetLength(Count)); }

         /// <summary>Sets the formatting function for displaying the elements of a list</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="UserFunc">UserFunc</param>
         /// <param name="Element">Element</param>
         /// <param name="Index">Index</param>
         /// <param name="MatchCount">MatchCount</param>
         /// <remarks>Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items within
         /// curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix. See ListFormat().</remarks>
         public String ListFunction(string UserFunc, string Element = "", string Index = "", string MatchCount = "") { return ListFunction_336(h, Unicode.Utf8GetBytes(UserFunc), Unicode.Utf8GetLength(UserFunc), Unicode.Utf8GetBytes(Element), Unicode.Utf8GetLength(Element), Unicode.Utf8GetBytes(Index), Unicode.Utf8GetLength(Index), Unicode.Utf8GetBytes(MatchCount), Unicode.Utf8GetLength(MatchCount)); }

         /// <summary>Returns list of characters</summary>
         /// <returns>uCalc String object</returns>
         /// <remarks>Returns a list of characters.  Each characters can be accessed as a uCalc string with Match(n) or other indexed function.</remarks>
         public String ListOfChars() { return ListOfChars_337(h); }

         /// <summary>Returns a list of tokens</summary>
         /// <returns>uCalc String object</returns>
         /// <remarks>Returns a list of tokens.  Each token can be accessed with Match(n) or other indexed function.</remarks>
         public String ListOfTokens() { return ListOfTokens_338(h); }

         /// <summary>Sets the postfix that is appended at the end of a list</summary>
         /// <returns>Current uCalc String object</returns>
         /// <param name="Postfix">Postfix that will be appended at the end of a list</param>
         /// <remarks>Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items within
         /// curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix.</remarks>
         public String ListPostfix(string Postfix) { return ListPostfix_339(h, Unicode.Utf8GetBytes(Postfix), Unicode.Utf8GetLength(Postfix)); }

         /// <summary>Sets the prefix that is inserted at the start of lists</summary>
         /// <returns>Current uCalc String object</returns>
         /// <param name="Prefix">Prefix that will be inserted at the start of a list</param>
         /// <remarks>Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items within
         /// curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix.</remarks>
         public String ListPrefix(string Prefix) { return ListPrefix_340(h, Unicode.Utf8GetBytes(Prefix), Unicode.Utf8GetLength(Prefix)); }

         /// <summary>Sets the separator that will delimite the elements of a list</summary>
         /// <returns>Current uCalc String object</returns>
         /// <param name="Separator">Separator that will delimite each element of a list</param>
         /// <remarks>Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items within
         /// curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix.</remarks>
         public String ListSeparator(string Separator) { return ListSeparator_341(h, Unicode.Utf8GetBytes(Separator), Unicode.Utf8GetLength(Separator)); }

         /// <summary>Returns the nth match</summary>
         /// <returns>uCalc string object</returns>
         /// <param name="Index">Index of nth match</param>
         /// <remarks>Returns the nth match.</remarks>
         public String Match(int Index) { return Match_342(h, (IntPtr)Index); }

         /// <summary>Returns the number of matches</summary>
         /// <returns>Integer</returns>
         /// <remarks>Returns the number of matches</remarks>
         public int MatchCount() { return (int)MatchCount_343(h); }

         /// <summary>Returns the index of the nth match where a character location is</summary>
         /// <returns>Integer</returns>
         /// <param name="Index">Index of the character who&apos;s match index you want</param>
         /// <remarks>Returns the index of the nth match where a character location is</remarks>
         public int MatchIndexAtChar(int Index) { return (int)MatchIndexAtChar_344(h, (IntPtr)Index); }

         /// <summary>Returns the index of the nth match where a token location is</summary>
         /// <returns>Integer</returns>
         /// <param name="Index">Index of the token who&apos;s match index you want</param>
         /// <remarks>Returns the index of the nth match where a token location is</remarks>
         public int MatchIndexAtToken(int Index) { return (int)MatchIndexAtToken_345(h, (IntPtr)Index); }

         public int MemoryIndex() { return (int)MemoryIndex_610(h); }

         /// <summary>Returns a string with a list of matches consisting of everything other than occurences of the Find pattern.</summary>
         /// <returns>uCalc String object</returns>
         /// <remarks>Returns a string with a list of matches consisting of everything other than occurences of the Find pattern.</remarks>
         public String NonMatches() { return NonMatches_346(h); }

         /// <summary>Returns the immediate parent of the current string</summary>
         /// <returns>uCalc string object that represents the parent of this string</returns>
         /// <remarks>Returns the immediate parent of the current string.</remarks>
         public String Parent() { return Parent_347(h); }

         /// <summary>Creates a string from a programmatically constructed list</summary>
         /// <returns>uCalc string object</returns>
         /// <param name="first">first</param>
         /// <param name="last">last</param>
         /// <param name="expression">expression</param>
         /// <param name="Step">Step</param>
         /// <param name="Condition">Condition</param>
         /// <param name="VarIndex">VarIndex</param>
         /// <param name="VarFirst">VarFirst</param>
         /// <param name="VarLast">VarLast</param>
         /// <remarks>For example: Populate(2, 5, &quot;$&apos;{Index}:{First}:{Last} etc.&apos;&quot;).Str(); would return: &quot;2:2:5
         /// etc.3:2:5 etc.4:2:5 etc.5:2:5 etc.&quot;</remarks>
         public String Populate(double first, double last, string expression, double Step = 1, string Condition = "", string VarIndex = "", string VarFirst = "", string VarLast = "") { return Populate_348(h, first, last, Unicode.Utf8GetBytes(expression), Unicode.Utf8GetLength(expression), Step, Unicode.Utf8GetBytes(Condition), Unicode.Utf8GetLength(Condition), Unicode.Utf8GetBytes(VarIndex), Unicode.Utf8GetLength(VarIndex), Unicode.Utf8GetBytes(VarFirst), Unicode.Utf8GetLength(VarFirst), Unicode.Utf8GetBytes(VarLast), Unicode.Utf8GetLength(VarLast)); }

         public int ReferenceCount() { return (int)ReferenceCount_574(h); }

         /// <summary>.</summary>
         /// <remarks>.</remarks>
         public void ReferenceIncr() { ReferenceIncr_559(h); }

         /// <summary>Releases a uCalc string</summary>
         /// <remarks>This releases a uCalc string from memory along with all the objects dependent on it.</remarks>
         public void Release() { Release_349(h); h = Empty; }

         /// <summary>Removes occurrences of a pattern in the string</summary>
         /// <returns>uCalc String</returns>
         /// <param name="From_">pattern of occurences to remove from a string</param>
         /// <remarks>Removes occurrences of a pattern in the string.</remarks>
         public String Remove(string From_) { return Remove_350(h, Unicode.Utf8GetBytes(From_), Unicode.Utf8GetLength(From_)); }

         /// <summary>Replaces occurences of a pattern with something else</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="From_">patter of occurence to match</param>
         /// <param name="To_">replacement of occurence</param>
         /// <remarks>Replaces occurences of a pattern with something else.</remarks>
         public String Replace(string From_, string To_) { return Replace_351(h, Unicode.Utf8GetBytes(From_), Unicode.Utf8GetLength(From_), Unicode.Utf8GetBytes(To_), Unicode.Utf8GetLength(To_)); }

         /// <summary>Returns the root string of the current string</summary>
         /// <returns>uCalc String object representing the root string</returns>
         /// <remarks>Returns the root string of the current string</remarks>
         public String Root() { return Root_352(h); }

         /// <summary>Skips</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Pattern">Pattern</param>
         /// <remarks>Skips</remarks>
         public String Skip(string Pattern) { return Skip_353(h, Unicode.Utf8GetBytes(Pattern), Unicode.Utf8GetLength(Pattern)); }

         /// <summary>Retruns a substring starting from the given occurence of a match</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Pattern">Pattern to match</param>
         /// <param name="Nth">Nth occurence</param>
         /// <remarks>Retruns the text starting from the given occurence of a match until the end.</remarks>
         public String StartingFrom(string Pattern, int Nth = 1) { return StartingFrom_354(h, Unicode.Utf8GetBytes(Pattern), Unicode.Utf8GetLength(Pattern), (IntPtr)Nth); }

         /// <summary>Returns a substring that starts with a given pattern and ends with another pattern</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="StartPattern">Start pattern</param>
         /// <param name="EndPattern">End pattern</param>
         /// <param name="StartNth">nth occurence for Start pattern</param>
         /// <param name="EndNth">nth occurence for End pattern</param>
         /// <remarks>Returns text that starts with a given pattern and ends with another pattern.</remarks>
         public String StartingFromEndingWith(string StartPattern, string EndPattern, int StartNth = 1, int EndNth = 1) { return StartingFromEndingWith_355(h, Unicode.Utf8GetBytes(StartPattern), Unicode.Utf8GetLength(StartPattern), Unicode.Utf8GetBytes(EndPattern), Unicode.Utf8GetLength(EndPattern), (IntPtr)StartNth, (IntPtr)EndNth); }

         /// <summary>Retrieves the string value of a uCalc String object</summary>
         /// <returns>String value of the uCalc String object</returns>
         /// <param name="Nth">Index of match</param>
         /// <param name="TokenCount">Number of tokens</param>
         /// <remarks>This retrieves the string value associated with a uCalc String object. If no argument is passed then the entire string
         /// is returned. Use Str(n) when you just want the string representation of a match. Use Match(n) when you want uCalc String Object
         /// that you can manipulate further.</remarks>
         public string Str(int Nth = -1, int TokenCount = 1) {
            var ReturnStr = Str_357(h, out var ReturnStrSize, (IntPtr)Nth, (IntPtr)TokenCount);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Assigns a string value to the uCalc String object</summary>
         /// <param name="StringValue">New string value to assign to the uCalc string object</param>
         /// <remarks>This assigns a new string to the uCalc String object.</remarks>
         public String Str(string StringValue) { return Str_358(h, Unicode.Utf8GetBytes(StringValue), Unicode.Utf8GetLength(StringValue)); }

         /// <summary>Returns substring</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="StartPos">Starting position</param>
         /// <param name="Length">Length of substring</param>
         /// <remarks>
         /// <para>Returns a substring of the current string.</para>
         /// 
         /// <para>Note: the StartPos and Length parameters are in terms of tokens, not characters.</para>
         /// </remarks>
         public String SubString(int StartPos, int Length = -1) { return SubString_359(h, (IntPtr)StartPos, (IntPtr)Length); }

         /// <summary>Returns the length (number of characters) of the nth token i a string</summary>
         /// <returns>Integer</returns>
         /// <param name="StartIndex">Index of nth token</param>
         /// <param name="Count">Number of consecutive tokens to count</param>
         /// <remarks>
         /// <para>Returns the length (number of characters) of the nth token in a string.</para>
         /// 
         /// <para>Note: When Count &gt; 1, white space characters between tokens are part of the length.</para>
         /// </remarks>
         public int TokenCharLen(int StartIndex, int Count = 1) { return (int)TokenCharLen_360(h, (IntPtr)StartIndex, (IntPtr)Count); }

         /// <summary>Returns the character position of the nth token</summary>
         /// <param name="Index">Index of nth match</param>
         /// <remarks>Returns the character position of the nth token</remarks>
         public int TokenCharPos(int Index) { return (int)TokenCharPos_361(h, (IntPtr)Index); }

         /// <summary>Returns the numbers of tokens in the string</summary>
         /// <returns>Integer</returns>
         /// <param name="Index">Index of nth token</param>
         /// <remarks>This returns the number of tokens that are found in the string.  If the Index argument is used, it returns the number
         /// of tokens in the nth match.</remarks>
         public int TokenCount(int Index = -1) { return (int)TokenCount_362(h, (IntPtr)Index); }

         /// <summary>Returns the index of the token that contains the indicated character</summary>
         /// <returns>Integer</returns>
         /// <param name="CharacterPosition">Character position</param>
         /// <remarks>Returns the index of the token that contains the indicated character.</remarks>
         public int TokenIndexAtChar(int CharacterPosition) { return (int)TokenIndexAtChar_363(h, (IntPtr)CharacterPosition); }

         /// <summary>Returns the nth token position</summary>
         /// <returns>Integer</returns>
         /// <param name="Nth">Index of nth match</param>
         /// <remarks>Returns the nth token position.</remarks>
         public int TokenPos(int Nth = -1) { return (int)TokenPos_364(h, (IntPtr)Nth); }

         /// <summary>Changes occurences of a pattern to lower case</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Pattern">Pattern</param>
         /// <param name="Nth">nth occrence of pattern</param>
         /// <remarks>Changes occurences of a pattern to lower case.</remarks>
         public String ToLower(string Pattern = "", int Nth = -1) { return ToLower_365(h, Unicode.Utf8GetBytes(Pattern), Unicode.Utf8GetLength(Pattern), (IntPtr)Nth); }

         /// <summary>Changes occurences of a pattern to upper case</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Pattern">Pattern</param>
         /// <param name="Nth">nth occrence of pattern</param>
         /// <remarks>Changes occurences of a pattern to upper case.</remarks>
         public String ToUpper(string Pattern = "", int Nth = -1) { return ToUpper_366(h, Unicode.Utf8GetBytes(Pattern), Unicode.Utf8GetLength(Pattern), (IntPtr)Nth); }

         /// <summary>Transforms a string</summary>
         /// <returns>Transformed string object</returns>
         /// <param name="tt">Transforms</param>
         /// <remarks>Transforms a string</remarks>
         public String Transform(Transforms tt) { return Transform_367(h, tt); }

         /// <summary>Returns the type of token the nth token is</summary>
         /// <returns>TokenType</returns>
         /// <param name="Index">Index of nth token</param>
         /// <remarks>Returns the type of token the nth token is.</remarks>
         public TokenType TypeOfToken(int Index) { return TypeOfToken_368(h, (IntPtr)Index); }

         /// <summary>Returns the uCalc object this String belongs to</summary>
         /// <returns>uCalc object</returns>
         public uCalc uCalc() { return uCalc_539(h); }

         /// <summary>Returns a substring with all text up to and including the matched pattern</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Pattern">Pattern to match</param>
         /// <param name="Nth">nth occurence</param>
         /// <remarks>Returns all text up to and including the matched pattern.  By default it goes up to the first occurence of a match,
         /// but you may elect to have it go up to the nth occurence.</remarks>
         public String UpTo(string Pattern, int Nth = 1) { return UpTo_369(h, Unicode.Utf8GetBytes(Pattern), Unicode.Utf8GetLength(Pattern), (IntPtr)Nth); }

      } // End of String class


      /// <summary>Class for collection of tokens</summary>
      public partial class Tokens {
         public static readonly Handle_Tokens Empty = (Handle_Tokens)MemoryObjectAddress_(MemoryTable.Tokens, IntPtr.Zero);
         public Handle_Tokens h = Empty;
         public int MemIndex { get => MemoryIndex(); }
         public Tokens(Handle_Tokens PassedHandle) { h = PassedHandle; }
         public IntPtr Handle() { return (IntPtr)h; }
         public bool IsEmpty() { return h == Empty; }
         public bool NotEmpty() { return h != Empty; }
         public static implicit operator Tokens(Handle_Tokens o) { return new Tokens(o); }
         public static implicit operator Handle_Tokens(Tokens o) { return o == null ? Empty : o.h; }
         public Tokens() { }

         /// <summary>Imports an individual token item</summary>
         /// <returns>Current token list</returns>
         /// <param name="TokenItem">Pre-existing token item</param>
         /// <remarks>Imports an individual token item into the token list.</remarks>
         public Tokens Add(Item TokenItem) { return Add_372(h, TokenItem); }

         /// <summary>Defines a new token</summary>
         /// <returns>Returns the newly created token item object</returns>
         /// <param name="TokenDef">Token definition</param>
         /// <param name="Type">The type of token</param>
         /// <param name="CloseBracket">String representing closing bracket, such as &quot;}&quot;, &quot;)&quot;, &quot;]&quot;, etc.</param>
         /// <param name="SubMatch">Submatch</param>
         /// <param name="Grammar">Type of RegEx grammar</param>
         /// <param name="InsertionPoint">Where to insert the token into the token list</param>
         /// <remarks>
         /// <para>uCalc comes with a default set of tokens.  These are general tokens for things like numbers, alphanumeric words, whitespace,
         /// quotes, parenthesis, etc.  However you may define your own tokens to the exact specifications that you need.</para>
         /// 
         /// <para>A token is defined using a Regular Expression, which is passed as the first argument.</para>
         /// 
         /// <para>A token type can be set to any of those listed under the TokenType enum (except for TokenType.None).</para>
         /// 
         /// <para>Bracket tokens (such as parenthesis, square brackets, or curly braces) are typically defined in pairs.  So if &quot;(&quot;,
         /// &quot;)&quot; are defined as a token pair, then CloseBracket would be &quot;)&quot;.</para>
         /// 
         /// <para>In cases where you want a submatch within the token, and not the whole token, use the SubMatch argument with a numerical
         /// value represent the numerical index used in the regular expression.  Useful for instance in matching quoted text without the
         /// quotes themselves.</para>
         /// </remarks>
         public Item Add(string TokenDef, TokenType Type = TokenType.Generic, string CloseBracket = "", int SubMatch = 0, RegExGrammar Grammar = RegExGrammar.Default, int InsertionPoint = -1) { return Add_373(h, Unicode.Utf8GetBytes(TokenDef), Unicode.Utf8GetLength(TokenDef), Type, Unicode.Utf8GetBytes(CloseBracket), Unicode.Utf8GetLength(CloseBracket), (IntPtr)SubMatch, Grammar, (IntPtr)InsertionPoint); }

         /// <summary>Imports a list of tokens from another token list</summary>
         /// <returns>Object containing list of tokens</returns>
         /// <param name="TokenCollection">Pre-existing token list</param>
         /// <remarks>Instead of defining a set of tokens each time, you may import a pre-existing token list.  This function imports from
         /// another token list object.</remarks>
         public Tokens Add(Tokens TokenCollection) { return Add_374(h, TokenCollection); }

         /// <summary>Clears the token list</summary>
         /// <remarks>Removes all tokens from the token list.  When a token list is cleared, it is typically prior to replacing it with
         /// a new token list.  A token list should generally not remain empty when doing a transform.  A token list should at minimum
         /// contain a catch-all pattern, which might capture one character.</remarks>
         public void Clear() { Clear_375(h); }

         /// <summary>Returns documentation/help/comment/tooltip type info associated with this object</summary>
         public string Comment() {
            var ReturnStr = Comment_540(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         public Tokens Comment(string CommentStr) { return Comment_547(h, Unicode.Utf8GetBytes(CommentStr), Unicode.Utf8GetLength(CommentStr)); }

         /// <summary>Sets a token that will cause a temporary switch to another token list with configurable insertion points</summary>
         /// <returns>Current token</returns>
         /// <param name="OtherTokenGroup">Other token list to switch to</param>
         /// <param name="StartToken">Existing token that will cause the context switch</param>
         /// <param name="InsertionPointA">// Where in the token list to insert the token</param>
         /// <param name="EndToken">Regex pattern that causes a return to the original token list</param>
         /// <param name="InsertionPointB">Where in the other token list (a copy of it) to insert the ending token</param>
         /// <remarks>
         /// <para>This causes a context switch to happen when a given token pattern is reached.  This is useful for instance if are parsing
         /// source code, and you want comments to be parsed based on a different set of tokens from the rest of the code.</para>
         /// 
         /// <para>This version of the function inserts a pre-existing token for the context switch, and lets you decide where in the token
         /// list to insert it.</para>
         /// </remarks>
         public Tokens ContextSwitch(Tokens OtherTokenGroup, Item StartToken, Int32 InsertionPointA = -1, string EndToken = "", Int32 InsertionPointB = -1) { return ContextSwitch_377(h, OtherTokenGroup, StartToken, InsertionPointA, Unicode.Utf8GetBytes(EndToken), Unicode.Utf8GetLength(EndToken), InsertionPointB); }

         /// <summary>Sets a token pattern that will cause a temporary switch to another token list with configurable insertion points</summary>
         /// <returns>Current token list</returns>
         /// <param name="OtherTokenGroup">Other token list to switch to</param>
         /// <param name="StartToken">Regex pattern that triggers the context switch</param>
         /// <param name="InsertionPointA">Where in the token list to insert the new token</param>
         /// <param name="EndToken">Regex pattern that causes a return to the original token list</param>
         /// <param name="InsertionPointB">Where in the other token list (a copy of it) to insert the ending token</param>
         /// <remarks>
         /// <para>This causes a context switch to happen when a given token pattern is reached.  This is useful for instance if are parsing
         /// source code, and you want comments to be parsed based on a different set of tokens from the rest of the code.</para>
         /// 
         /// <para>This version of the function lets you decide where in the token list to insert the context switching token.</para>
         /// </remarks>
         public Tokens ContextSwitch(Tokens OtherTokenGroup, string StartToken, Int32 InsertionPointA = -1, string EndToken = "", Int32 InsertionPointB = -1) { return ContextSwitch_378(h, OtherTokenGroup, Unicode.Utf8GetBytes(StartToken), Unicode.Utf8GetLength(StartToken), InsertionPointA, Unicode.Utf8GetBytes(EndToken), Unicode.Utf8GetLength(EndToken), InsertionPointB); }

         /// <summary>Sets a token pattern that will cause a temporary switch to another token list</summary>
         /// <returns>Current token list</returns>
         /// <param name="OtherTokenGroup">Other token list to switch to</param>
         /// <param name="StartToken">Regex pattern that triggers the context switch</param>
         /// <param name="EndToken">Regex pattern that causes a return to the original token list</param>
         /// <remarks>This causes a context switch to happen when a given token pattern is reached.  This is useful for instance if are
         /// parsing source code, and you want comments to be parsed based on a different set of tokens from the rest of the code.</remarks>
         public Tokens ContextSwitch(Tokens OtherTokenGroup, string StartToken, string EndToken) { return ContextSwitch_379(h, OtherTokenGroup, Unicode.Utf8GetBytes(StartToken), Unicode.Utf8GetLength(StartToken), Unicode.Utf8GetBytes(EndToken), Unicode.Utf8GetLength(EndToken)); }

         /// <summary>Returns the number of tokens in a token list</summary>
         /// <returns>Integer representing number of tokens</returns>
         /// <remarks>This returns the number of tokens in a token list.</remarks>
         public int Count() { return (int)Count_380(h); }

         /// <summary>Defines a token</summary>
         /// <returns>Current object</returns>
         /// <param name="TokenDef">Token definition</param>
         /// <param name="Type">The type of token</param>
         /// <param name="SubMatch">Submatch</param>
         /// <param name="Grammar">Type of RegEx grammar</param>
         /// <remarks>Defines a token.  Similar to Add.  Add is more complete, but calls an internal version of Define().</remarks>
         public Item Define(string TokenDef, TokenType Type = TokenType.Generic, int SubMatch = 0, RegExGrammar Grammar = RegExGrammar.Default) { return Define_381(h, Unicode.Utf8GetBytes(TokenDef), Unicode.Utf8GetLength(TokenDef), Type, (IntPtr)SubMatch, Grammar); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Returns a token&apos;s position in the token list</summary>
         /// <returns>Integer value</returns>
         /// <param name="GivenToken">Token Item</param>
         /// <remarks>Returns a token&apos;s position in the token list.</remarks>
         public Int32 IndexOf(Item GivenToken) { return IndexOf_383(h, GivenToken); }

         /// <summary>Defines a new token</summary>
         /// <returns>Token item object</returns>
         /// <param name="InsertionPoint">Where to insert the token into the token list</param>
         /// <param name="TokenDef">Token definition</param>
         /// <param name="Type">The type of token</param>
         /// <param name="CloseBracket">String representing closing bracket, such as &quot;}&quot;, &quot;)&quot;, &quot;]&quot;, etc.</param>
         /// <param name="SubMatch">Submatch</param>
         /// <param name="Grammar">Type of RegEx grammar</param>
         /// <remarks>This is similar to Add(), however add appends the token to the end of the token list by default, whereas Insert()
         /// lets you choose where in the list to insert the token definition with the first parameter (TokenListIndex).  Tokens most recently
         /// defined have precedence over tokens defined earlier.  However, this allows a new token definition to have less precedence
         /// than the default. 0 inserts at the beginning, 1 inserts at the 2nd location etc.  Negative numbers insert starting at the
         /// end of the list, with -1 at the last position (default), -2 inserts at the next to last location, etc.</remarks>
         public Item Insert(Int32 InsertionPoint, string TokenDef, TokenType Type = TokenType.Generic, string CloseBracket = "", int SubMatch = 0, RegExGrammar Grammar = RegExGrammar.Default) { return Insert_384(h, InsertionPoint, Unicode.Utf8GetBytes(TokenDef), Unicode.Utf8GetLength(TokenDef), Type, Unicode.Utf8GetBytes(CloseBracket), Unicode.Utf8GetLength(CloseBracket), (IntPtr)SubMatch, Grammar); }

         /// <summary>Determines if a token has the given property</summary>
         /// <returns>Returns True if the token matches the given property, or False otherwise.</returns>
         /// <param name="ItemProperty">The property you want to check for</param>
         /// <remarks>Determines if a token has the given property</remarks>
         public bool IsProperty(ItemIs ItemProperty) { return IsProperty_386(h, ItemProperty) != 0; }

         /// <summary>Sets a given property</summary>
         /// <returns>Current object</returns>
         /// <param name="ItemProperty">The property you want to set</param>
         /// <param name="TrueOrFalse">Value to set the property to</param>
         /// <remarks>Sets a given property</remarks>
         public Tokens IsProperty(ItemIs ItemProperty, bool TrueOrFalse) { return IsProperty_387(h, ItemProperty, Convert.ToInt64(TrueOrFalse)); }

         public int MemoryIndex() { return (int)MemoryIndex_611(h); }

         /// <summary>Releases a token list from memory.  Transform rules are defined with FromTo.</summary>
         /// <remarks>This releases a transform rule from memory.</remarks>
         public void Release() { Release_388(h); h = Empty; }

         /// <summary>Removes a token from the token list</summary>
         /// <param name="TokenItem">Token to be removed from the token list</param>
         /// <remarks>Removes the specified token from the token list.</remarks>
         public void Remove(Item TokenItem) { Remove_389(h, TokenItem); }

         /// <summary>Returns the nth token from the token list</summary>
         /// <returns>Token Item</returns>
         /// <param name="Index">nth Index</param>
         /// <remarks>Returns the nth token from the token list.</remarks>
         public Item Token(int Index) { return Token_390(h, (IntPtr)Index); }

         public uCalc uCalc() { return uCalc_544(h); }

      } // End of Tokens class


      /// <summary>uCalc transform class</summary>
      public partial class Transforms : IDisposable {      
      private bool disposed = false;
      public void Dispose() { Dispose(disposing: true); GC.SuppressFinalize(this); }
      protected virtual void Dispose(bool disposing) { if (disposed) return; Release(); disposed = true; }
         public static readonly Handle_Transforms Empty = (Handle_Transforms)MemoryObjectAddress_(MemoryTable.Transforms, IntPtr.Zero);
         public Handle_Transforms h = Empty;
         public int MemIndex { get => MemoryIndex(); }
         public Transforms(Handle_Transforms PassedHandle) { h = PassedHandle; }
         public IntPtr Handle() { return (IntPtr)h; }
         public bool IsEmpty() { return h == Empty; }
         public bool NotEmpty() { return h != Empty; }
         public static implicit operator Transforms(Handle_Transforms o) { return new Transforms(o); }
         public static implicit operator Handle_Transforms(Transforms o) { return o == null ? Empty : o.h; }
         public Transforms(Tokens TokensToInherit = null) { h = Default().NewTransformH(TokensToInherit); }
         public Transforms(uCalc uc, Tokens TokensToInherit = null) { h = uc.NewTransformH(TokensToInherit); }

         public static implicit operator string(Transforms t) { return t.Str(); }
//         public static implicit operator Transforms(string s) { return new Transforms(s); }
         public override string ToString() { return Str(); }

         /// <summary>Makes an identical but separate copy of a Transforms object</summary>
         /// <returns>New transforms object</returns>
         public Transforms Clone() { return Clone_504(h); }

         /// <summary>Returns documentation/help/comment/tooltip type info associated with this object</summary>
         public string Comment() {
            var ReturnStr = Comment_542(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         public Transforms Comment(string CommentStr) { return Comment_548(h, Unicode.Utf8GetBytes(CommentStr), Unicode.Utf8GetLength(CommentStr)); }

         /// <summary>Filters the text</summary>
         /// <returns>Current object</returns>
         /// <remarks>This performs a filter operation on text.  This is similar to a Transform() operation, however, the result is text
         /// consisisting of just the matches.  The operation is performed on text that was previously passed to Str().</remarks>
         public Transforms Filter() { return Filter_393(h); }

         /// <summary>Performs a Filter operation on the given text</summary>
         /// <returns>Current transform</returns>
         /// <param name="expression">Expression to be transformed</param>
         /// <remarks>This performs a filter operation on text.  This is similar to a Transform() operation, however, the result is text
         /// consisisting of just the matches.  The operation is performed on the string passed to this Filter() function.</remarks>
         public Transforms Filter(string expression) { return Filter_394(h, Unicode.Utf8GetBytes(expression), Unicode.Utf8GetLength(expression)); }

         /// <summary>Performs a text search operation</summary>
         /// <returns>The current Transforms object</returns>
         /// <param name="Str">Text to search through</param>
         /// <remarks>
         /// <para>Once all the rules and parameters are set, you can call this function in order to perform the text search.</para>
         /// 
         /// <para>You may either set the string to Str() prior to calling Find(), or you can pass a string argument directly to this function.</para>
         /// </remarks>
         public Transforms Find(string Str = "") { return Find_395(h, Unicode.Utf8GetBytes(Str), Unicode.Utf8GetLength(Str)); }

         /// <summary>Defines a transform rule</summary>
         /// <returns>Object for rule being defined</returns>
         /// <param name="FromStr">Pattern that describes the text to match</param>
         /// <param name="ToStr">New text that the match should be changed to</param>
         /// <remarks>This defines something like a search/replace operation.  Whenever a piece of text matches the pattern described in
         /// FromStr, it is replaced with the text found in ToStr.</remarks>
         public Rule FromTo(string FromStr, string ToStr = "") { return FromTo_396(h, Unicode.Utf8GetBytes(FromStr), Unicode.Utf8GetLength(FromStr), Unicode.Utf8GetBytes(ToStr), Unicode.Utf8GetLength(ToStr)); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Gets the index base used in various functions</summary>
         /// <remarks>By default, uCalc String Library functions use a 0-based index.  So the first element is 0 (not 1), followed by 2,
         /// 3, 4 etc. Use this to check to see which base is being used.</remarks>
         public int IndexBase() { return (int)IndexBase_399(h); }

         /// <summary>Sets the index base used in various functions</summary>
         /// <param name="BaseValue">value representing index of 1st element</param>
         /// <remarks>By default, uCalc String Library functions use a 0-based index.  So the first element is 0 (not 1), followed by 2,
         /// 3, 4 etc. Use this if you prefer to start by 1 or another non-zero value.  This value is passed on to string objects derived
         /// from a parent string. Each string may have a different index base.</remarks>
         public Transforms IndexBase(int BaseValue) { return IndexBase_400(h, (IntPtr)BaseValue); }

         /// <summary>Determines if a uCalc Item has the given properties</summary>
         /// <returns>Returns True if the uCalc Item matches the given property, or False otherwise.</returns>
         /// <param name="ItemProperty">The property you want to check for</param>
         /// <remarks>This helps you determine if an item has the given property</remarks>
         public bool IsProperty(ItemIs ItemProperty) { return IsProperty_402(h, ItemProperty) != 0; }

         /// <summary>Sets a given property</summary>
         /// <returns>Returns the same Transforms object</returns>
         /// <param name="ItemProperty">The property you want to set</param>
         /// <param name="TrueOrFalse">Value to set the property to</param>
         /// <remarks>This sets a given property</remarks>
         public Transforms IsProperty(ItemIs ItemProperty, bool TrueOrFalse) { return IsProperty_403(h, ItemProperty, Convert.ToInt64(TrueOrFalse)); }

         /// <summary>Returns the Matches object for the current Transform</summary>
         /// <returns>Matches object</returns>
         /// <param name="Options">Option for subset of matches to return</param>
         /// <remarks>This returns the Matches object for the current Transform.  The Matches object contains the list of matches, the
         /// location and lengh of each match, etc.</remarks>
         public Matches Matches(MatchesOption Options = MatchesOption.FocusableOnly) { return MatchesUTF16(Options); }
         public Matches MatchesUTF16(MatchesOption Options = MatchesOption.FocusableOnly) { return AdjustMatchesForUTF16(this, MatchesUTF8(Options)); }
         public Matches MatchesUTF8(MatchesOption Options = MatchesOption.FocusableOnly) { return Matches_404(h, Options); }

         public int MemoryIndex() { return (int)MemoryIndex_612(h); }

         /// <summary>Creates a new Pass object</summary>
         /// <returns>Transform object</returns>
         /// <param name="Share">Share</param>
         /// <remarks>It is possible to create a transform with multiple passes.  Each pass can have a different set of rules.  The transform
         /// of each pass is executed in sequential order.  Passes can later be referenced by index.  Calling NewPass the first time returns
         /// the same as Pass(0), the next call to NewPass refers to Pass(1), etc.</remarks>
         public Transforms NewPass(bool Share = false) { return NewPass_405(h, Convert.ToInt64(Share)); }

         /// <summary>Creates a new rule</summary>
         /// <returns>Returns the new rule object</returns>
         /// <remarks>This creates a new rule.  A rule typically consists of a string to find and/or modify, along with properties associted
         /// with that rule.</remarks>
         public Rule NewRule() { return NewRule_406(h); }

         /// <summary>Returns a Pass object</summary>
         /// <returns>Transform object</returns>
         /// <param name="Index">Index of pass to select</param>
         /// <param name="Share">Share</param>
         /// <remarks>
         /// <para>It is possible to create a transform with multiple passes.  Each pass can have a different set of rules.  Each pass
         /// is transformed in sequential order, starting with Pass(0).  A pass consists of a transform.</para>
         /// 
         /// <para>This function returns a pass object, which typically was defined with NewPass().</para>
         /// </remarks>
         public Transforms Pass(int Index, bool Share = false) { return Pass_407(h, (IntPtr)Index, Convert.ToInt64(Share)); }

         /// <summary>Returns the number of passes</summary>
         /// <returns>Integer</returns>
         /// <remarks>Returns the number of passes</remarks>
         public int PassCount() { return (int)PassCount_408(h); }

         /// <summary>Restricts the range of text where the search or search/replace operations will be performed</summary>
         /// <returns>The current transform</returns>
         /// <param name="Start">Starting position</param>
         /// <param name="Size">Size of string to operate on</param>
         /// <remarks>Generally transform operations are performed on the entire text that is passed to the transform.  Use this if you
         /// only want to operate on a subset of the text, starting at Start, with size Size.</remarks>
         public Transforms Range(int Start, int Size) { return Range_409(h, (IntPtr)Start, (IntPtr)Size); }

         public int ReferenceCount() { return (int)ReferenceCount_572(h); }

         /// <summary>.</summary>
         /// <remarks>.</remarks>
         public void ReferenceIncr() { ReferenceIncr_558(h); }

         /// <summary>Releases a transform</summary>
         /// <remarks>This releases a transform from memory along with all the objects defined under it.</remarks>
         public void Release() { Release_410(h); h = Empty; }

         /// <summary>Resets the transform object</summary>
         /// <returns>Current transform object</returns>
         /// <param name="ResetOption">Reset option</param>
         /// <remarks>This clears the input string, transform rules, token definitions, and match list.  It also resets properties to default
         /// settings.</remarks>
         public Transforms Reset(TransformResetOption ResetOption = TransformResetOption.All) { return Reset_411(h, ResetOption); }

         /// <summary>Returns the defaults settings for rules</summary>
         /// <returns>Default rule object</returns>
         /// <remarks>Each FromTo or Find rule has many configurable properties that can be set.  Instead of excplicitely setting all of
         /// these many properties with each rule definition, you may define default settings that will automatically be assigned to new
         /// rules.  Any of these default settings can be overridden by explicitely setting the desired property for a given rule.</remarks>
         public Rule RulesDefault() { return RulesDefault_412(h); }

         /// <summary>Returns the overall settings for a transform</summary>
         /// <returns>Overall rule object</returns>
         /// <remarks>This returns an object containing the properties for a transform or search object.  This object is a Rules object,
         /// and conatains the same properties listed for individual rules, but works in a slightly different context.  For instance, if
         /// you set the StopAfter property of FromTo(&quot;a&quot;, &quot;b&quot;) to 5, then it will only change the first five occurences
         /// of &quot;a&quot; to &quot;b&quot;, whereas if you set the RulesOverall StopAfter property to 5, and you have rules FromTo(&quot;a&quot;,
         /// &quot;b&quot;) and FromTo(&quot;x&quot;, &quot;y&quot;) then it will stop changing any occurence of either rule, after the
         /// combined total of changes reaches 5; &quot;a&quot; might change to &quot;b&quot; 2 times and &quot;x&quot; to &quot;y&quot;
         /// 3 times, or whichever combination does not total more than 5.  You might set QuoteSensitive to &quot;true&quot; overall, while
         /// you may set it to false within a given rule.  In certain combinations, overall rules may clash with individual rules.  Although
         /// all combinations are allowed, not all combinations make sense.  When setting the case sensitivity property of individual rules
         /// differently from the overall property, keep in mind that if the overall case sensitivity is false, then all symbols are stored
         /// in the symbol table in all upper (or lower) case.</remarks>
         public Rule RulesOverall() { return RulesOverall_413(h); }

         /// <summary>Returns string result of transformed text</summary>
         /// <returns>String result of transformed text</returns>
         /// <remarks>After performing a text transformation, you may obtain the transformed result with this function.</remarks>
         public string Str() {
            var ReturnStr = Str_415(h, out var ReturnStrSize);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets the text input that is to be transformed or searched</summary>
         /// <returns>Current transform object</returns>
         /// <param name="InputStr">Text that mast be parsed and modified or searched</param>
         /// <remarks>The text that you want to transform or search should be passed to this function.</remarks>
         public Transforms Str(string InputStr) { return Str_416(h, Unicode.Utf8GetBytes(InputStr), Unicode.Utf8GetLength(InputStr)); }

         /// <summary>Returns actual character base pointer of string</summary>
         /// <returns>Pointer to byte array</returns>
         /// <remarks>Note: Should never be used alone.  Always use in pair with StrLength</remarks>
         public byte[] StrData() { return StrData_417(h); }

         /// <summary>Returns the length of the text string being operated on</summary>
         /// <returns>Integer</returns>
         /// <remarks>This returns the length of the text string being operated on</remarks>
         public int StrLength() { return (int)StrLength_418(h); }

         /// <summary>Specifically intended for use with handling Unicode utf8 vs utf16 difference</summary>
         /// <returns>String</returns>
         /// <param name="Start">Starting character position</param>
         /// <param name="Count">Number of characters</param>
         /// <remarks>This returns a SubStr of the text</remarks>
         public string SubStr(int Start, int Count = -1) {
            var ReturnStr = SubStr_419(h, out var ReturnStrSize, (IntPtr)Start, (IntPtr)Count);
            return Unicode.Utf16(ReturnStr, ReturnStrSize);
         }

         /// <summary>Returns the Tokens object for the current transform</summary>
         /// <returns>Token list object</returns>
         /// <remarks>Returns the Tokens object, which consists of a list of tokens for the current transform. You can use this to add,
         /// modify, delete, or reset the tokens.</remarks>
         public Tokens Tokens() { return Tokens_420(h); }

         /// <summary>Performs the transformation on some text</summary>
         /// <returns>The current transform object</returns>
         /// <remarks>
         /// <para>Once all the rules and parameters are set, you can call this function in order to perform the text transformation.</para>
         /// 
         /// <para>Prior to calling Transform, you must set the text to be transformed, with Str</para>
         /// </remarks>
         public Transforms Transform() { return Transform_422(h); }

         /// <summary>Transforms an expression in one quick step</summary>
         /// <returns>Current object</returns>
         /// <param name="expression">Expression to be transformed</param>
         /// <remarks>When you would like to set text to be transformed, and transform the text, and return a result (with .Str()), all
         /// in one step, you can use this function, instead of calling Str, then Transform, then Str.</remarks>
         public Transforms Transform(string expression) { return Transform_423(h, Unicode.Utf8GetBytes(expression), Unicode.Utf8GetLength(expression)); }

         /// <summary>Returns each step leading to an expression expansion</summary>
         /// <returns>String list containg steps leading up to transform result</returns>
         /// <param name="expression">Expression to be transformed</param>
         /// <remarks>Transforms an expression and returns intermediate steps</remarks>
         public String TransformWithSteps(string expression) { return TransformWithSteps_424(h, Unicode.Utf8GetBytes(expression), Unicode.Utf8GetLength(expression)); }

         public uCalc uCalc() { return uCalc_546(h); }

         /// <summary>Sets the uCalc object that performs calculations related to the transform</summary>
         /// <returns>uCalc object associated with current transform</returns>
         /// <param name="uh">Handle of a uCalc object</param>
         /// <remarks>If your transform inovlves calcuations, then they will be performed using the uCalc object the transform was defined
         /// in.  However, you can change the uCalc object with this function. Each uCalc object can have its own sets of variables, functions,
         /// and other definitions. For instance when {@Eval: ...} is used in the pattern or ~Eval() in the text.</remarks>
         public Transforms uCalc(uCalc uh) { return uCalc_425(h, uh); }

         /// <summary>Indicates whether text was modified in the most recent Transform operation</summary>
         /// <returns>True if text was modified in the most recent transformation operation or false otherwise</returns>
         /// <remarks>If text was modified after a call to Transform, then this function returns True.  When transforming large amounts
         /// of text, transferring strings back and forth with Str() can be expensive.  Checking this property allows you to skip the unnecessary
         /// transfer of text when the text has not been modified.</remarks>
         public bool WasModified() { return WasModified_426(h) != 0; }

      } // End of Transforms class


public partial class Tokens {
   public override string ToString() { var Str = ""; for (var x = 0; x < Count(); x++) Str += Token(x).Regex() + "\r\n"; return Str; }
}
      public partial class Variable : Item {
         bool disposed = false;
         protected override void Dispose(bool disposing) { if (disposed) return; disposed = true; base.Dispose(disposing); }
         public Variable(Handle_Item PassedHandle) : base(PassedHandle) { }
         public Variable() : base() { }
         public Variable(string s, IntPtr Address = default(IntPtr)) : base("Variable: " + s, Address) { }
         public Variable(uCalc uc, string s, IntPtr Address = default(IntPtr)) : base(uc, "Variable: " + s, Address) { }
      }
         public static readonly Handle_uCalc Empty = (Handle_uCalc)MemoryObjectAddress_(MemoryTable.uCalc, IntPtr.Zero);
         public Handle_uCalc h = Empty;
         public int MemIndex { get => MemoryIndex(); }
         public uCalc(Handle_uCalc PassedHandle) { h = PassedHandle; }
         public IntPtr Handle() { return (IntPtr)h; }
         public bool IsEmpty() { return h == Empty; }
         public bool NotEmpty() { return h != Empty; }
         public static implicit operator uCalc(Handle_uCalc o) { return new uCalc(o); }
         public static implicit operator Handle_uCalc(uCalc o) { return o == null ? Empty : o.h; }

      /// <summary>Adds an error handler</summary>
      /// <returns>The error handler item, which can later be removed using this return object</returns>
      /// <param name="ErrorHandlerAddress">Address of the callback function containing the error handler code</param>
      /// <param name="Position">Position in which to insert the error handler</param>
      /// <remarks>
      /// <para>Use this to set a callback function that is called whenever there is an error.  Your callback can do things like alert
      /// the user of the error, abort the operation, correct the error and resume, keep a log of errors, etc.</para>
      /// 
      /// <para>By default, if there are multiple error handlers, they are called starting with the most recently defined one first,
      /// ending with the first one to be defined last.  In order to define a handler in a different order, specify the Position value
      /// of where it should be inserted; 0 so that it is called first, -1 so it is called last, or a positive integer indicating where
      /// it should be inserted.  If the Position value is out of bounds, it is adjusted into the maximum acceptable value.</para>
      /// 
      /// <para>If you set ErrorResponse to ErrorHandlerResponse.Resume, subsequent error handler callbacks will be bypassed and uCalc
      /// will resume operations as though no error had taken place.  Note that the condition that caused the error must be corrected
      /// before resuming, otherwise uCalc may get stuck in an infinite loop.</para>
      /// 
      /// <para>Some functions, such as ErrorExpression, ErrorLocation, and ErrorSymbol produce a meaningful result only during the
      /// parsing stage.  They do not cause other problems beyond meaningless data during the evaluation stage.</para>
      /// </remarks>
      public Item AddErrorHandler(ErrorHandlerCallback ErrorHandlerAddress, int Position = 0) { return AddErrorHandler_14(h, PinAddr(ErrorHandlerAddress), (IntPtr)Position); }

      /// <summary>Adds an alias to an existing symbol</summary>
      /// <returns>Newly created alias Item</returns>
      /// <param name="NewSymbol">New symbol that will be set as equivalent to an existing symbol</param>
      /// <param name="ExistingItem">Pre-existing item</param>
      /// <remarks>
      /// <para>This function creates a new symbol that is pegged to an existing symbol.  Whenever this new symbol is encountered by
      /// the parser, it will be the same as if it had encountered the pre-existing symbol this new symbol is an alias of.  This new
      /// symbol shares the same handle as the pre-existing symbol within the symbol table.  The new symbol actually has its own handle,
      /// but that handle can only be used to delete the item. The pre-existing handle is used for all other purposes.</para>
      /// 
      /// <para>This version of Alias uses an Item object as second argument.  It is a little more efficient than the one that uses
      /// a string (the string version calls the internal version of this function).</para>
      /// </remarks>
      public Item Alias(string NewSymbol, Item ExistingItem) { return Alias_16(h, Unicode.Utf8GetBytes(NewSymbol), Unicode.Utf8GetLength(NewSymbol), ExistingItem); }

      /// <summary>Adds an alias to an existing symbol</summary>
      /// <returns>Newly created alias Item</returns>
      /// <param name="NewSymbol">New symbol that will be set as equivalent to an existing symbol</param>
      /// <param name="ExistingSymbol">Pre-existing symbol</param>
      /// <remarks>
      /// <para>This function creates a new symbol that is pegged to an existing symbol.  Whenever this new symbol is encountered by
      /// the parser, it will be the same as if it had encountered the pre-existing symbol this new symbol is an alias of.  This new
      /// symbol shares the same handle as the pre-existing symbol within the symbol table.  The new symbol actually has its own handle,
      /// but that handle can only be used to delete the item. The pre-existing handle is used for all other purposes.</para>
      /// 
      /// <para>For simple situations where you just want two symbols to be considered equivalent by the parser, Alias is prefered as
      /// a more efficient option over ExpressionTransforms().FromTo(OldSymbol, NewSymbol), which would modify the expression in order
      /// for the symbols to be equal.</para>
      /// </remarks>
      public Item Alias(string NewSymbol, string ExistingSymbol) { return Alias_17(h, Unicode.Utf8GetBytes(NewSymbol), Unicode.Utf8GetLength(NewSymbol), Unicode.Utf8GetBytes(ExistingSymbol), Unicode.Utf8GetLength(ExistingSymbol)); }

      /// <summary>Adds an alias name to an existing command name for Define()</summary>
      /// <param name="NewCommandName">New command name that will be set as equivalent to an existing command name</param>
      /// <param name="ExistingCommandName">Pre-existing command name</param>
      /// <remarks>This creates an alternative command name for use with the Define() function.</remarks>
      public void AliasForDefine(string NewCommandName, string ExistingCommandName) { AliasForDefine_18(h, Unicode.Utf8GetBytes(NewCommandName), Unicode.Utf8GetLength(NewCommandName), Unicode.Utf8GetBytes(ExistingCommandName), Unicode.Utf8GetLength(ExistingCommandName)); }

      /// <summary>Makes an identical but separate copy of a uCalc object</summary>
      /// <returns>uCalc object</returns>
      public uCalc Clone() { return Clone_555(h); }

      /// <summary>Returns a data type object</summary>
      /// <returns>Data type object</returns>
      /// <param name="Index">Index associated with the data type</param>
      /// <remarks>Each instance of uCalc has its own separate handle &amp; data type object for any given data type.  For instance,
      /// Item(&quot;Int64&quot;) in one instance of uCalc is not the same object as Item(&quot;Int64&quot;) from another uCalc instance.
      ///  This allows the same data type to have a different name and different formatting in different instances.  However, they share
      /// the same data type index across instances.  Given the index of a data type, you can directly obtain the proper data type object
      /// for a particular uCalc instance, faster than obtaining it by name as text with Item().</remarks>
      public DataType DataTypeOf(BuiltInType Index) { return DataTypeOf_22(h, Index); }

      /// <summary>Returns the data type of the expression that was passed</summary>
      /// <returns>Data type object</returns>
      /// <param name="expression">Expression whose data type will be returned</param>
      /// <remarks>Returns the data type of the expression that was passed</remarks>
      public DataType DataTypeOf(string expression) { return DataTypeOf_23(h, Unicode.Utf8GetBytes(expression), Unicode.Utf8GetLength(expression)); }

      /// <summary>Returns the default uCalc instance</summary>
      /// <returns>uCalc object</returns>
      /// <remarks>Returns the default uCalc instance.  New uCalc instances inherit from this by default.  Expression, String, and Transforms
      /// objects that are created without specifying a uCalc space also use this default.</remarks>
      public static uCalc Default() { return Default_25(); }

      /// <summary>Clears the stack of default uCalc objects</summary>
      public static void DefaultClear() { DefaultClear_569(); }

      /// <summary>Returns the number of objects on the uCalc default stack</summary>
      public static int DefaultCount() { return (int)DefaultCount_567(); }

      /// <summary>Default data type</summary>
      /// <returns>Default data type</returns>
      /// <remarks>This returns the default data type.  The default type is Double unless explicitly changed to something else.</remarks>
      public DataType DefaultDataType() { return DefaultDataType_28(h); }

      /// <summary>Sets a new default data type</summary>
      /// <param name="DataTypeIndex">Enum of type you want to set as default</param>
      /// <remarks>
      /// <para>This sets a new default data type.  This new default will be used instead of Double for all definitions where a type
      /// is relevant, when an explicit type is not specified.</para>
      /// 
      /// <para>The default is set using a BuiltInType enum.</para>
      /// </remarks>
      public void DefaultDataType(BuiltInType DataTypeIndex) { DefaultDataType_29(h, DataTypeIndex); }

      /// <summary>Sets a new default data type</summary>
      /// <param name="dType">The type you want to set as default</param>
      /// <remarks>
      /// <para>This sets a new default data type.  This new default will be used instead of Double for all definitions where a type
      /// is relevant, when an explicit type is not specified.</para>
      /// 
      /// <para>The default is set using a data type object.</para>
      /// </remarks>
      public void DefaultDataType(DataType dType) { DefaultDataType_30(h, dType); }

      /// <summary>Sets a new default data type</summary>
      /// <param name="dTypeStr">Name of existing data type</param>
      /// <remarks>
      /// <para>This sets a new default data type.  This new default will be used instead of Double for all definitions where a type
      /// is relevant, when an explicit type is not specified.</para>
      /// 
      /// <para>The default is set using the name of a data type.</para>
      /// </remarks>
      public void DefaultDataType(string dTypeStr) { DefaultDataType_31(h, Unicode.Utf8GetBytes(dTypeStr), Unicode.Utf8GetLength(dTypeStr)); }

      /// <summary>Returns default settings</summary>
      /// <returns>Default Item</returns>
      /// <remarks>The Item returned contains the default settings for the current uCalc instance.  To define new settings, use Define(&quot;Default
      /// ~~ ...&quot;).</remarks>
      public Item DefaultItem() { return DefaultItem_32(h); }

      /// <summary>Removes a uCalc object from the top of the uCalc default stack</summary>
      /// <remarks>+++ This should not be necessary</remarks>
      public static void DefaultPop() { DefaultPop_568(); }

      /// <summary>Sets a uCalc instance as the default</summary>
      /// <param name="uCalcObj">uCalc object to set as default</param>
      /// <remarks>
      /// <para>Sets the new uCalc default object that future uCalc objects will inherit from by default when none is explicitly specified.</para>
      /// 
      /// <para>Automatically popped when it goes out of scope
      /// +++ May not be needed; use IsDefault(true) instead</para>
      /// 
      /// <para></para>
      /// </remarks>
      public static void DefaultPush(uCalc uCalcObj) { DefaultPush_26(uCalcObj); }

      /// <summary>+++ Find better name</summary>
      /// <returns>Newly constructed uCalc object that will serve as default</returns>
      /// <param name="InheritsFrom">Other uCalc object to inherit from</param>
      public static uCalc DefaultPushNew(uCalc InheritsFrom = null) { return DefaultPushNew_580(InheritsFrom); }

      /// <summary>Defines a uCalc.Item object (a constunction, function, operator, variable, etc)</summary>
      /// <returns>The defined item object</returns>
      /// <param name="expression">Text containing item to be defined.</param>
      /// <param name="Address">Address a function or variable is attached to</param>
      /// <param name="dType">Data type associated with the item</param>
      /// <remarks>
      /// <para>Functions, operators, variables, constants, tokens, syntax constructs, etc., can be defined with this method.  Although
      /// there are specialized methods that do the same things, such as DefineFunction, DefineOperator, DefineVariable, and DefineConstant,
      /// this method may in same cases offer more flexibility.</para>
      /// 
      /// <para>The expression must start with a keyword such as Function, Operator, Variable, Token, Syntax, etc., followed by a colon,
      /// before the expression.</para>
      /// 
      /// <para>The expression must be preceded by a keyword and colon, such as &quot;Variable:&quot;.  For a full list of keywords,
      /// see Definition Keywords.</para>
      /// 
      /// <para>Sections are separated by ~~ (two tilde signs).</para>
      /// </remarks>
public Item Define(string expression, IntPtr Address = default(IntPtr), DataType dType = null) { return Define_33(h, Unicode.Utf8GetBytes(expression), Unicode.Utf8GetLength(expression), Address, dType); }
public Handle_Item DefineH(string expression, IntPtr Address = default(IntPtr), DataType dType = null) { return Define_33(h, Unicode.Utf8GetBytes(expression), Unicode.Utf8GetLength(expression), Address, dType); }

      /// <summary>Defines a constant</summary>
      /// <returns>The defined item object</returns>
      /// <param name="Definition">Text containing constant to be defined</param>
      /// <param name="VariableAddress">Address the constant is attached to</param>
      /// <remarks>
      /// <para>A constant definition is similar to that of a variable defined with DefineVariable, except that the end-user is not
      /// allowed to modify the value of a constant. You are, however, allowed to perform lower-level operations such as renaming a
      /// constant, or deleting it and re-defining a new constant or variable with the same name but different value.  If you attach
      /// the constant to the address of a variable in your host program, the host program may change the value.  However, when the
      /// end-user attempts to assign a value to the constant, an error is raised.</para>
      /// 
      /// <para>DefineConstant(&quot;x&quot;) is the same as: Define(&quot;Lock ~~ Variable: x&quot;)</para>
      /// </remarks>
      public Item DefineConstant(string Definition, IntPtr VariableAddress = default(IntPtr), DataType dType = null) { return DefineConstant_34(h, Unicode.Utf8GetBytes(Definition), Unicode.Utf8GetLength(Definition), VariableAddress, dType); }

      /// <summary>Defines a function</summary>
      /// <returns>The defined item object</returns>
      /// <param name="Definition">Text containing function to be defined</param>
      /// <param name="FunctionAddr">Address of callback function this function is attached to</param>
      /// <remarks>
      /// <para>A function definition can either be self-contained, or it may be attached to the address of a callback function in your
      /// source code.</para>
      /// 
      /// <para>DefineFunction(&quot;f(x) = ...&quot;) is the same as: Define(&quot;Function: ...&quot;)</para>
      /// </remarks>
      public Item DefineFunction(string Definition, OpCallback FunctionAddr = null, DataType dType = null) { return DefineFunction_35(h, Unicode.Utf8GetBytes(Definition), Unicode.Utf8GetLength(Definition), PinAddr(FunctionAddr), dType); }

      /// <summary>Defines an operator</summary>
      /// <returns>The defined item object</returns>
      /// <param name="Definition">Text containing operator to be defined</param>
      /// <param name="PrecedenceLevel">Precedence level of the operator</param>
      /// <param name="Direction">Operator associativity; arithmetic operators are typically grouped left-to-right; special operators, such as the equal sign, =, assignment operator are right-to-left</param>
      /// <param name="FunctionAddr">Address of callback function this operator is attached to</param>
      /// <remarks>
      /// <para>Many details for defining an operator are similar to those of defining a function.  In addition, all operators must
      /// be defined with a precedence level.  You may use the Precedence function to obtain precedence levels of existing operators,
      /// and select a precedence level relative to those operators.  By default operators are grouped left to right.  But you may also
      /// set it to right to left.  This means that when adjacent operators have the same precedence level, the operands on the right
      /// will be evaluated first.</para>
      /// 
      /// <para>DefineOperator(&quot;...&quot;) is the same as: Define(&quot;Operator: ...&quot;)</para>
      /// </remarks>
      public Item DefineOperator(string Definition, int PrecedenceLevel, Grouping Direction = Grouping.LeftToRight, OpCallback FunctionAddr = null, DataType dType = null) { return DefineOperator_36(h, Unicode.Utf8GetBytes(Definition), Unicode.Utf8GetLength(Definition), (IntPtr)PrecedenceLevel, Direction, PinAddr(FunctionAddr), dType); }

      /// <summary>Defines a variable</summary>
      /// <returns>The defined item object</returns>
      /// <param name="Definition">Text containing variable to be defined</param>
      /// <param name="VariableAddress">Address of variable in host program this uCalc variable is attached to</param>
      /// <remarks>
      /// <para>A variable typically contains a variable name, optionally followed by a data type, optionally followed by an equal sign
      /// and an expression whose result is the starting value for the variable.</para>
      /// 
      /// <para>DefineVariable(&quot;...&quot;) is the same as: Define(&quot;Variable: x&quot;)</para>
      /// </remarks>
      public Item DefineVariable(string Definition, IntPtr VariableAddress = default(IntPtr), DataType dType = null) { return DefineVariable_37(h, Unicode.Utf8GetBytes(Definition), Unicode.Utf8GetLength(Definition), VariableAddress, dType); }

      /// <summary>Returns a snapshot of uCalc&apos;s memory footprint</summary>
      /// <returns>Returns either number of bytes or number of elements for the selection type</returns>
      /// <param name="uCalcIndex">Memory index of owning uCalc instance</param>
      /// <param name="Target">Class or type whose memory footprint you want to diagnose</param>
      /// <param name="Count">Selects whether to count in terms of bytes or number of elements</param>
      public static int DiagMem(int uCalcIndex, BuiltInType Target, DiagMemCount Count, DiagMemOption Option = DiagMemOption.Default) { return (int)DiagMem_597((IntPtr)uCalcIndex, Target, Count, Option); }

      /// <summary>Returns the expression associated with an error that was raised</summary>
      /// <returns>Text string</returns>
      public string ErrorExpression() {
         var ReturnStr = ErrorExpression_39(h, out var ReturnStrSize);
         return Unicode.Utf16(ReturnStr, ReturnStrSize);
      }

      /// <summary>Sets a new expression after an error is raised</summary>
      /// <param name="expression">Error expression</param>
      public void ErrorExpression(string expression) { ErrorExpression_40(h, Unicode.Utf8GetBytes(expression), Unicode.Utf8GetLength(expression)); }

      /// <summary>Returns the location of an error that was raised</summary>
      public int ErrorLocation() { return (int)ErrorLocation_41(h); }

      /// <summary>Returns a message associated with the error that was raised</summary>
      /// <returns>Text string</returns>
      /// <param name="Index">Index of error message to return</param>
      /// <remarks>
      /// <para>If an error was raised, this returns a message describing the recent error.  If the Index argument is specified, then
      /// the error message associated with this Index is instead returned.  A value must be chosen from ErrorMsg.</para>
      /// 
      /// <para>Note: Errors are cleared whenever a new expression is parsed or a new item is defined.</para>
      /// </remarks>
      public string ErrorMessage(ErrorMsg Index = ErrorMsg.None) {
         var ReturnStr = ErrorMessage_42(h, out var ReturnStrSize, Index);
         return Unicode.Utf16(ReturnStr, ReturnStrSize);
      }

      /// <summary>Returns the error number of the error that was raised</summary>
      /// <returns>ErrorMsg enum</returns>
      /// <remarks>
      /// <para>Returns the error enum of the most recently raised error.</para>
      /// 
      /// <para>Note: Errors are cleared whenever a new expression is parsed or a new item is defined.</para>
      /// </remarks>
      public ErrorMsg ErrorNumber() { return ErrorNumber_43(h); }

      /// <summary>Sets the error response for an error handler</summary>
      /// <param name="Response">Indicates what should happen now that an error is raised</param>
      /// <remarks>
      /// <para>When an error is raised, you have the option of aborting the rest of the operation (this is the default action), or
      /// you may correct the problem, and then resume the operation.  Or you may perform an action, and then raise the error again
      /// for a subsequent error handler to take care of.</para>
      /// 
      /// <para>This method can only be called from within a callback of an error handler routine defined with AddErrorHandler. The
      /// options to use can be found at ErrorHandlerResponse.</para>
      /// </remarks>
      public void ErrorResponse(ErrorHandlerResponse Response) { ErrorResponse_44(h, Response); }

      /// <summary>Returns the symbol associated with the error that is raised</summary>
      /// <returns>text string</returns>
      public string ErrorSymbol() {
         var ReturnStr = ErrorSymbol_45(h, out var ReturnStrSize);
         return Unicode.Utf16(ReturnStr, ReturnStrSize);
      }

      /// <summary>Parses and evaluates an expression in one step</summary>
      /// <param name="expression">Expression to be evaluated</param>
      /// <remarks>This takes a math expression and returns a numeric result.  This is a convenient one-step alternative to ExprObj
      /// = Parse(Expressions); Result = ExprObj.Evaluate();</remarks>
      public double Eval(string expression) { return Eval_46(h, Unicode.Utf8GetBytes(expression), Unicode.Utf8GetLength(expression)); }

      /// <summary>Parses and evaluates an expression of any data type in one step, returning the result as a string</summary>
      /// <returns>String containing result</returns>
      /// <param name="expression">Expression to be evaluated</param>
      /// <remarks>This is similar to Eval, however, it can return non-numeric results, such as strings.  Also, the expression can be
      /// of any type.  The result is converted to a string and then returned.</remarks>
      public string EvalStr(string expression, bool ReturnFormatted = true) {
         var ReturnStr = EvalStr_47(h, out var ReturnStrSize, Unicode.Utf8GetBytes(expression), Unicode.Utf8GetLength(expression), Convert.ToInt64(ReturnFormatted));
         return Unicode.Utf16(ReturnStr, ReturnStrSize);
      }

      /// <summary>Returns the transform object for parsing math expressions, associated with the current uCalc instance</summary>
      /// <returns>Transform object</returns>
      /// <remarks>
      /// <para>Whenever a math expression is parsed, this transform is first called.  Use this when you want transforms that apply
      /// to math expressions parsed with Parse, Eval, or EvalStr.  To create a new transform that does not affect math expressions
      /// sent to Parse, use NewTransform instead.</para>
      /// 
      /// <para>Independent transforms created with NewTransform can be released when no longer needed.  However, the object created
      /// with ExpressionTransforms should never be released explicitly, because the uCalc object requires it in order parse expressions.</para>
      /// 
      /// <para>Note: For simple situations when you just want one keyword to be the equivalent of a pre-existing one, it is more efficient
      /// to use Alias().  With Alias, nothing is transformed.  Instead, an aliased symbol simply shares the same handle as a pre-existing
      /// one in the symbol table.  When there are no expression transforms at all, the transform step is skipped altogether, slightly
      /// speeding up parsing for all expressions in a uCalc instance.</para>
      /// </remarks>
      public Transforms ExpressionTransforms() { return ExpressionTransforms_48(h); }

      /// <summary>Returns the bits representing floating point errors that are cought &amp; raised</summary>
      /// <returns>Integer</returns>
      /// <param name="Value">True or false</param>
      /// <remarks>Returns the bits for the floating point errors that are to be raised</remarks>
      public Int32 FloatingPointErrorsToCatch(Int32 Value = -1) { return FloatingPointErrorsToCatch_49(h, Value); }

      /// <summary>Allows a callback to decorate the output of EvalStr</summary>
      /// <param name="FunctionAddr">Address of callback function that formats the output</param>
      /// <param name="dType">Data type that will have its output formatted; omit this to affect all data types and use default ranking</param>
      public Item Format(OpCallback FunctionAddr, DataType dType) { return Format_51(h, PinAddr(FunctionAddr), dType); }

      /// <summary>Allows a callback to decorate the output of EvalStr</summary>
      /// <param name="FunctionAddr">Address of callback function that formats the output</param>
      /// <param name="Properties">Name of datatype that will have its output formatted and/or ranking among other output formats; omit this to affect all data types and use default ranking.</param>
      public Item Format(OpCallback FunctionAddr, string Properties = "") { return Format_52(h, PinAddr(FunctionAddr), Unicode.Utf8GetBytes(Properties), Unicode.Utf8GetLength(Properties)); }

      /// <summary>Formats the appearance of results returned by EvalStr or EvaluateStr</summary>
      /// <param name="Definition">Format definition for the output</param>
      /// <param name="dType">Data type that will have its output formatted; omit this to affect all data types and use default ranking.</param>
      /// <remarks>
      /// <para>This function lets you modify the output returned by EvalStr or EvaluateStr.  The definition may contain one or more
      /// of the following:</para>
      /// 
      /// <para>Datatype:  If you specify a data type, then the modification will affect only the evaluation result of expressions that
      /// evaluate to the specified data type.</para>
      /// 
      /// <para>InsertAt:  By default, if you define multiple outputs, then the one defined most recently is applied first, followed
      /// by the other outputs in the reverse order in which they were defined, ending with the output that was defined first.  You
      /// may specify a different insertion point than the default with this parameter.</para>
      /// 
      /// <para>Expression:  This represents what the modified output should look like.  The expression consists of an identifier (any
      /// temporary variable name), followed by an equal sign, followed by a string expression representing the value that will be returned.
      ///  The variable holds the original value that would have been returned.</para>
      /// </remarks>
      public Item Format(string Definition, DataType dType = null) { return Format_53(h, Unicode.Utf8GetBytes(Definition), Unicode.Utf8GetLength(Definition), dType); }

      /// <summary>Removes any output formatting</summary>
      /// <param name="dType">Data type</param>
      /// <remarks>Removes formatting for EvalStr or EvaluateStr that was defined with Format.</remarks>
      public void FormatRemove(DataType dType = null) { FormatRemove_54(h, dType); }

      /// <summary>Returns the handle of an object</summary>
      /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
      /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

      /// <summary>Checks to see if this uCalc object instance is the default</summary>
      /// <returns>Boolean value</returns>
      /// <remarks>Definitions from this instance will automatically be inherited by new instances unless you specify otherwise.</remarks>
      public bool IsDefault() { return IsDefault_57(h) != 0; }

      /// <summary>Sets this uCalc object instance as the default</summary>
      /// <param name="Condition">True or False</param>
      /// <remarks>Definitions from this instance will automatically be inherited by new instances unless you specify otherwise. Set
      /// this to True.  When set to False, nothing happens.</remarks>
      public void IsDefault(bool Condition) { IsDefault_58(h, Convert.ToInt64(Condition)); }

      /// <summary>Returns the nth uCalc.Item object that matches a property</summary>
      /// <returns>Item object</returns>
      /// <param name="Property_">Properties of the uCalc.Item object you&apos;re after; use either 0 for all items or Properties()</param>
      /// <param name="nth">The nth object</param>
      /// <param name="DataTypeName">The data type of the object</param>
      /// <param name="IncludeOverloads">Include overloads</param>
      /// <remarks>This can be used to cycle through defined items. If there is no match, an item whose IsProperty ItemIs.NotFound value
      /// is true is returned.</remarks>
      public Item ItemOf(Int64 Property_, int nth, string DataTypeName = "", bool IncludeOverloads = false) { return ItemOf_60(h, Property_, (IntPtr)nth, Unicode.Utf8GetBytes(DataTypeName), Unicode.Utf8GetLength(DataTypeName), Convert.ToInt64(IncludeOverloads)); }

      /// <summary>Returns the uCalc.Item object associated with the given name</summary>
      /// <returns>Item object</returns>
      /// <param name="ItemName">Name of uCalc.Item that you want to get</param>
      /// <param name="Properties">Properties to help pinpoint the right Item in case several items share the same name</param>
      /// <param name="nthOverload">If more than one item matches the same name, the nth overload is selected</param>
      /// <remarks>
      /// <para>If there is no match, an item whose ItemIs.NotFound property is true is returned.</para>
      /// 
      /// <para>In case multiple items share the same name, you can narrow it down using properties.  For instance, the + operator has
      /// a binary and unary version.  You can specify Prefix if you wanted the unary one.</para>
      /// 
      /// <para>Note: A ItemIs member should not be passed directly as the second argument.  Instead Properties(), with one or more
      /// ItemIs members should be passed.  The default of 0 means that there are no property restrictions.</para>
      /// </remarks>
      public Item ItemOf(string ItemName, Int64 Properties = 0, int nthOverload = 0) { return ItemOf_61(h, Unicode.Utf8GetBytes(ItemName), Unicode.Utf8GetLength(ItemName), Properties, (IntPtr)nthOverload); }

      /// <summary>Sets the license code</summary>
      /// <param name="Type">Type of license</param>
      /// <param name="L1">Value 1</param>
      /// <param name="L2">Value 2</param>
      /// <param name="L3">Value 3</param>
      /// <param name="L4">Value 4</param>
      /// <param name="L5">Value 5</param>
      /// <remarks>The license code prevents the message box from showing.  A license can be purchased from www.uCalc.com</remarks>
      public static void License(LicenseType Type, int L1 = 0, int L2 = 0, int L3 = 0, int L4 = 0, int L5 = 0) { License_62(Type, (IntPtr)L1, (IntPtr)L2, (IntPtr)L3, (IntPtr)L4, (IntPtr)L5); }

      public int MemoryIndex() { return (int)MemoryIndex_602(h); }

      /// <summary>Returns the address in memory of a uCalc object; mainly for internal use</summary>
      /// <returns>Address of object at index</returns>
      /// <param name="Index">Index of object; 0 is reserved for Empty object (instead of null)</param>
      public static int MemoryObjectAddress(MemoryTable ObjectType, int Index) { return (int)MemoryObjectAddress_(ObjectType, (IntPtr)Index); }

      /// <summary>Namespace symbol</summary>
      /// <returns>Nth symbol item object</returns>
      /// <param name="nth">Nth symbol</param>
      public Item NamespaceSymbol(int nth) { return NamespaceSymbol_63(h, (IntPtr)nth); }

      /// <summary>Number of symbols</summary>
      public int NamespaceSymbolCount() { return (int)NamespaceSymbolCount_64(h); }

      /// <summary>Returns a new uCalc instance</summary>
public static uCalc New(uCalc InheritsFrom = null) { return New_584(InheritsFrom); }
public static Handle_uCalc NewH(uCalc InheritsFrom = null) { return New_584(InheritsFrom); }

      /// <summary>Creates a new uCalc instance and makes it the default</summary>
      /// <returns>uCalc object</returns>
      /// <param name="InheritsFrom">Other uCalc object to inherit from</param>
      /// <remarks>
      /// <para>Creates a new uCalc object and makes it the default.  When a new uCalc object is created without specifying another
      /// uCalc object to derive from, or when an Expression, String, or Transforms object is created without specifying which uCalc
      /// space it will belong to, this default uCalc object will be used.</para>
      /// 
      /// <para>New default uCalc objects are pushed onto a stack.  uCalc objects created with NewDefault() must explicitly be released
      /// with Release(), or popped with DefaultPop(), when no longer needed.  When a default uCalc object is released , this default
      /// is removed from the stack, and the uCalc instance now at the top of the stack becomes the new default.  If there&apos;s only
      /// one default on the stack, it is released, but a new default is generated and placed on the stack, so that there is always
      /// a default.</para>
      /// 
      /// <para>+++ Might not need this function</para>
      /// </remarks>
      public static uCalc NewDefault(uCalc InheritsFrom = null) { return NewDefault_20(InheritsFrom); }

public Expression NewExpression(string Expr, BuiltInType DataTypeIndex) { return NewExpression_581(h, Unicode.Utf8GetBytes(Expr), Unicode.Utf8GetLength(Expr), DataTypeIndex); }
public Handle_Expression NewExpressionH(string Expr, BuiltInType DataTypeIndex) { return NewExpression_581(h, Unicode.Utf8GetBytes(Expr), Unicode.Utf8GetLength(Expr), DataTypeIndex); }

public Expression NewExpression(string Expr, DataType ReturnType = null) { return NewExpression_582(h, Unicode.Utf8GetBytes(Expr), Unicode.Utf8GetLength(Expr), ReturnType); }
public Handle_Expression NewExpressionH(string Expr, DataType ReturnType = null) { return NewExpression_582(h, Unicode.Utf8GetBytes(Expr), Unicode.Utf8GetLength(Expr), ReturnType); }

public Expression NewExpression(string Expr, string ReturnType) { return NewExpression_583(h, Unicode.Utf8GetBytes(Expr), Unicode.Utf8GetLength(Expr), Unicode.Utf8GetBytes(ReturnType), Unicode.Utf8GetLength(ReturnType)); }
public Handle_Expression NewExpressionH(string Expr, string ReturnType) { return NewExpression_583(h, Unicode.Utf8GetBytes(Expr), Unicode.Utf8GetLength(Expr), Unicode.Utf8GetBytes(ReturnType), Unicode.Utf8GetLength(ReturnType)); }

      /// <summary>Returns a new uCalc String object</summary>
      /// <returns>New uCalc string object</returns>
      /// <param name="InitialString">String initial value</param>
      /// <remarks>Returns a new uCalc String object.
      /// Objects created with this function must explicitly be released with Release() (unless you release the parent uCalc object,
      /// in which case this object will also be released).</remarks>
public String NewString(string InitialString = "") { return NewString_65(h, Unicode.Utf8GetBytes(InitialString), Unicode.Utf8GetLength(InitialString)); }
public Handle_String NewStringH(string InitialString = "") { return NewString_65(h, Unicode.Utf8GetBytes(InitialString), Unicode.Utf8GetLength(InitialString)); }

      /// <summary>Returns a new transform object</summary>
      /// <returns>Transform object</returns>
      /// <param name="TokensToInherit">Existing token list to inherit from</param>
      /// <remarks>Returns a new object that can be used to transform text.Objects created with this function must explicitly be released
      /// with Release() (unless you release the parent uCalc object, in which case this object will also be released).</remarks>
public Transforms NewTransform(Tokens TokensToInherit = null) { return NewTransform_66(h, TokensToInherit); }
public Handle_Transforms NewTransformH(Tokens TokensToInherit = null) { return NewTransform_66(h, TokensToInherit); }

      /// <summary>Parses an expression that will later be evaluated with Evaluate or EvaluateStr</summary>
      /// <returns>Parsed expression</returns>
      /// <param name="Expr">Expression to be parsed</param>
      /// <param name="DataTypeIndex">Data type index of return value</param>
      /// <remarks>
      /// <para>Parse() is the same as NewExpression.
      /// Note:
      /// C++
      ///  auto MyExpr = uc.Parse(...) for MyExpr to live beyond the scope of where it&apos;s defined
      ///  uCalc.Expression MyExpr = uc.Parse(...) for Expr to be released automatically upon going out of scope</para>
      /// 
      /// <para>.NET (C#)
      /// var MyExpr = uc.Parse(...) for MyExpr to live beyond the scope of where it&apos;s defined
      /// using uCalc.Expression MyExpr = uc.Parse(...) for Expr to be released automatically upon going out of scope</para>
      /// </remarks>
      public Expression Parse(string Expr, BuiltInType DataTypeIndex) { return Parse_68(h, Unicode.Utf8GetBytes(Expr), Unicode.Utf8GetLength(Expr), DataTypeIndex); }

      /// <summary>Parses an expression that will later be evaluated with Evaluate or EvaluateStr</summary>
      /// <returns>Parsed expression</returns>
      /// <param name="Expr">Expression to be parsed</param>
      /// <param name="ReturnType">Data type of return value</param>
      public Expression Parse(string Expr, DataType ReturnType = null) { return Parse_69(h, Unicode.Utf8GetBytes(Expr), Unicode.Utf8GetLength(Expr), ReturnType); }

      /// <summary>Parses an expression that will later be evaluated with Evaluate or EvaluateStr</summary>
      /// <returns>Parsed expression</returns>
      /// <param name="Expr">Expression to be parsed</param>
      /// <param name="ReturnType">Data type of return value</param>
      public Expression Parse(string Expr, string ReturnType) { return Parse_70(h, Unicode.Utf8GetBytes(Expr), Unicode.Utf8GetLength(Expr), Unicode.Utf8GetBytes(ReturnType), Unicode.Utf8GetLength(ReturnType)); }

      /// <summary>Returns the precedence level of an operator or the level between two operators</summary>
      /// <param name="Operator1">Operator with lower (or equal) precedence</param>
      /// <param name="Operator2">Operator with greater (or equal precedence)</param>
      /// <remarks>This returns a precedence level based on levels of existing operators; useful when defining a new operator (or transform)
      /// with a level relative to levels of existing operators (or transforms).  If one argument is passed, then the precedence level
      /// of that item is returned.  This is useful for defining a new item at the same level as the specified item.  If two arguments
      /// are passed, then the average (mean) precedence level of the two items is returned.  This is useful for defining a level that&apos;s
      /// in between that of two existing items.</remarks>
      public int Precedence(string Operator1, string Operator2 = "") { return (int)Precedence_71(h, Unicode.Utf8GetBytes(Operator1), Unicode.Utf8GetLength(Operator1), Unicode.Utf8GetBytes(Operator2), Unicode.Utf8GetLength(Operator2)); }

      /// <summary>Combines properties into a numeric value that can be passed to Item</summary>
      /// <returns>Integer representation of combined properties</returns>
      /// <param name="p1">First property</param>
      /// <param name="p2">Second property</param>
      /// <param name="p3">Third property</param>
      /// <param name="p4">Fourth property</param>
      /// <remarks>ItemIs properties are numbered sequentially (0, 1, 2, 3, 4, ...).  However the Properties argument of the Item function
      /// requires properties in the 2^x sequence (1, 2, 4, 8, 16, ...).  So you must pass one or more ItemIs to Properties before passing
      /// it to Item.</remarks>
      public static Int64 Properties(ItemIs p1, ItemIs p2 = ItemIs.SelectAny, ItemIs p3 = ItemIs.SelectAny, ItemIs p4 = ItemIs.SelectAny) { return Properties_72(p1, p2, p3, p4); }

      /// <summary>Causes division by 0 errors to be raised</summary>
      /// <param name="TrueOrFalse">True or false</param>
      /// <remarks>This causes an error to be raised instead of silently returning a special value when a division by 0 or pole error
      /// occurs.</remarks>
      public void RaiseErrorOnDivideByZero(bool TrueOrFalse = true) { RaiseErrorOnDivideByZero_73(h, Convert.ToInt64(TrueOrFalse)); }

      /// <summary>Causes inexact results to raise an error</summary>
      /// <param name="TrueOrFalse">True or false</param>
      /// <remarks>This causes an error to be raised instead returning an approximate value when an exact value cannot be represented.</remarks>
      public void RaiseErrorOnInexact(bool TrueOrFalse = true) { RaiseErrorOnInexact_74(h, Convert.ToInt64(TrueOrFalse)); }

      /// <summary>Causes invalid value to raise an error</summary>
      /// <param name="TrueOrFalse">True or false</param>
      /// <remarks>This causes an error to be raised instead returning NaN when an invalid value is encountered.</remarks>
      public void RaiseErrorOnInvalid(bool TrueOrFalse = true) { RaiseErrorOnInvalid_75(h, Convert.ToInt64(TrueOrFalse)); }

      /// <summary>Causes overflow to raise an error</summary>
      /// <param name="TrueOrFalse">True or false</param>
      /// <remarks>This causes an error to be raised instead returning infinity when there&apos;s an overflow.</remarks>
      public void RaiseErrorOnOverflow(bool TrueOrFalse = true) { RaiseErrorOnOverflow_76(h, Convert.ToInt64(TrueOrFalse)); }

      /// <summary>Causes underflow to raise an error</summary>
      /// <param name="TrueOrFalse">True or false</param>
      /// <remarks>This causes an error to be raised for underflow instead of just returning 0.</remarks>
      public void RaiseErrorOnUnderflow(bool TrueOrFalse = true) { RaiseErrorOnUnderflow_77(h, Convert.ToInt64(TrueOrFalse)); }

      public int ReferenceCount() { return (int)ReferenceCount_570(h); }

      /// <summary>.</summary>
      /// <remarks>.</remarks>
      public void ReferenceIncr() { ReferenceIncr_556(h); }

      /// <summary>Releases a uCalc instance</summary>
      /// <remarks>
      /// <para>When a uCalc instance is no longer needed, it can be released with this function.  This releases all items associated
      /// with the uCalc instance.</para>
      /// 
      /// <para>Note: The uCalc object named uc, which is instantiated at startup (except in C#) should not be explicitly released.
      /// In C#, if there is only 1 uCalc object left, do not explicitly release it.</para>
      /// 
      /// <para>Note: A uCalc object must explicitly be released with Release for it to be released.  A uCalc object is not automatically
      /// released from memory when the object goes out of scope.</para>
      /// </remarks>
      public void Release() { Release_78(h); h = Empty; }

      /// <summary>Resets uCalc back to the way it was at startup</summary>
      /// <remarks>Releases all uCalc objects and resets the entire uCalc library back to the way it was at startup.</remarks>
      public static void ResetAll() { ResetAll_566(); }

      /// <summary>Sets the current uCalc object as the default</summary>
      public uCalc SetAsDefault() { return SetAsDefault_585(h); }

      /// <summary>Returns the value located at the specified pointer address</summary>
      /// <returns>String representation of the value at the given address</returns>
      /// <param name="ValuePtr">Pointer address of the value</param>
      /// <param name="dType">Data type of the value; helps determine in which form to return the value at the given location</param>
      /// <remarks>This returns the value located at the given address.  The value is first converted to a string.</remarks>
      public string ValueAt(IntPtr ValuePtr, DataType dType = null, bool FormattedOutput = false) {
         var ReturnStr = ValueAt_81(h, out var ReturnStrSize, ValuePtr, dType, Convert.ToInt64(FormattedOutput));
         return Unicode.Utf16(ReturnStr, ReturnStrSize);
      }

      /// <summary>Returns the value located at the specified pointer address</summary>
      /// <returns>String representation of the value at the given address</returns>
      /// <param name="ValuePtr">Pointer address containing the data</param>
      /// <param name="ValueDataType">Data type of the data you need, which helps to determine in which form to return the value at the given location</param>
      /// <remarks>This returns the value located at the given address.  The value is first converted to a string.</remarks>
      public string ValueAt(IntPtr ValuePtr, string ValueDataType, bool FormattedOutput = false) {
         var ReturnStr = ValueAt_82(h, out var ReturnStrSize, ValuePtr, Unicode.Utf8GetBytes(ValueDataType), Unicode.Utf8GetLength(ValueDataType), Convert.ToInt64(FormattedOutput));
         return Unicode.Utf16(ReturnStr, ReturnStrSize);
      }

      public void Reset(uCalc u = null) { Release(); h = NewH(u); }
      public uCalc() { h = NewH(); }
      public uCalc(LicenseType Type, int L1 = 0, int L2 = 0, int L3 = 0, int L4 = 0, int L5 = 0) { License(Type, L1, L2, L3, L4, L5); h = NewH(); }

      readonly HashSet<OpCallback> CallbackAddresses = new HashSet<OpCallback>();
      readonly HashSet<ErrorHandlerCallback> ErrorHandlerAddresses = new HashSet<ErrorHandlerCallback>();
      public OpCallback PinAddr(OpCallback Address) { CallbackAddresses.Add(Address); return Address; }
      public OpCallback UnPinAddr(OpCallback Address) { CallbackAddresses.Remove(Address); return Address; }
      public ErrorHandlerCallback PinAddr(ErrorHandlerCallback Address) { ErrorHandlerAddresses.Add(Address); return Address; }
      public ErrorHandlerCallback UnPinAddr(ErrorHandlerCallback Address) { ErrorHandlerAddresses.Remove(Address); return Address; }
   } // End of uCalc class


}