#pragma once

// #define uCalcForWindows 1
// #define uCalcForLinux 1
// #define uCalcForAndroid 1
// #define uCalcForMac 1
// #define uCalcForiOS 1

#include <iostream>
#include <cstdint>
#include <string>
#include <cfenv>

#if defined(_WIN64)
#   pragma comment(lib, "uCalc64.lib")
#elif defined(_WIN32)
#   pragma comment(lib, "uCalc32.lib")
#else
#  include <dlfcn.h>
#endif

namespace uCalcSoftware {
#if defined(uCalcForLinux) || defined(uCalcForMac) || defined(uCalcForiOS)
#define StdCall
#define DllImport
#elif defined(uCalcForAndroid)
#define uCalcLib "./libuCalc.so"
#define LoadLibraryA(lib) dlopen(lib, RTLD_LAZY)
#define GetProcAddress(lib_handle, func) dlsym(lib_handle, func)
#define HINSTANCE void *
#else // uCalcForWindows (implied)
#define StdCall __stdcall
#define DllImport __declspec(dllimport)

#if defined(_WIN64)
#define uCalcLib "uCalc64.dll"
#else // defined(_WIN32)
#define uCalcLib "uCalc32.dll"
#endif
#endif

   using BoolInt = int64_t;

   enum class Handle_Callback : size_t { Ignore };
   enum class Handle_DataType : size_t { Ignore };
   enum class Handle_Expression : size_t { Ignore };
   enum class Handle_Item : size_t { Ignore };
   enum class Handle_Matches : size_t { Ignore };
   enum class Handle_Rule : size_t { Ignore };
   enum class Handle_String : size_t { Ignore };
   enum class Handle_Tokens : size_t { Ignore };
   enum class Handle_Transforms : size_t { Ignore };
   enum class Handle_uCalc : size_t { Ignore };


   /// <summary>Data types that come with uCalc</summary>
   /// <remarks>These are built-in data types that come with uCalc</remarks>
   enum class BuiltInType : int32_t {
      Unspecified,
      Auto,
      /// <summary>Boolean</summary>
      Boolean,
      Size_t,
      /// <summary>Int8</summary>
      Integer_8,
      /// <summary>Byte</summary>
      Integer_8u,
      /// <summary>Int16</summary>
      Integer_16,
      /// <summary>Int16u</summary>
      Integer_16u,
      /// <summary>Int32</summary>
      Integer_32,
      /// <summary>Int32u</summary>
      Integer_32u,
      /// <summary>Int64</summary>
      Integer_64,
      /// <summary>Int64u</summary>
      Integer_64u,
      /// <summary>Int128</summary>
      Integer_128,
      /// <summary>Int128u</summary>
      Integer_128u,
      /// <summary>Single</summary>
      Float_Single,
      /// <summary>Double</summary>
      Float_Double,
      /// <summary>Extended</summary>
      Float_Extended,
      Complex,
      Decimal,
      BigNum,
      Date,
      String,
      StringView,
      /// <summary>LPCSTR</summary>
      String_LPCSTR,
      /// <summary>FixedStr</summary>
      String_Fixed,
      /// <summary>WideStr</summary>
      String_Wide,
      /// <summary>StackList</summary>
      List_Stack,
      /// <summary>TableList</summary>
      List_Table,
      /// <summary>SortedList</summary>
      List_Sorted,
      Null,
      Void,
      Any,
      Pointer,
      /// <summary>// +++ Maybe remove; doesn't appear to be used</summary>
      Omni,
      UserDefined,
      DataType,
      Namespace,
      IntPtr,
      ADDR,
      uCalcPtr,
      Item,
      ParsedExpr,
      uCalcCallback,
      Object,
      Intrinsic,
      SameTypeAs,
      DiagAll,
      DiagAllClasses,
      DiagAllTypes,
      DiaguCalc,
      DiagDataType,
      DiagItem,
      DiagDataFormat,
      DiagMatchList,
      DiagParsedExpression,
      DiagStringItem,
      DiagTable
   };

   enum class ChildStringOptions : int32_t {
      ClearAllOnNewObj,
      ClearOverlapOnRead,
      ClearOverlapOnWrite,
      DetachAllOnNewObj,
      DetachOverlapOnRead,
      DetachOverlapOnWrite
   };

   enum class Comparison : int32_t {
      LessThan = -1,
      Equal = 0,
      GreaterThan = 1
   };

   /// <summary>Data type constructors</summary>
   enum class DataTypeConstruction : int32_t {
      ByteSize,
      AllocArray,
      AllocScalar,
      AllocByPtr,
      ReleaseArray,
      ReleaseScalar,
      ResetScalar,
      ResetArray,
      SetArrayValue,
      SetScalarValue,
      SwapScalarValues
   };

   enum class DiagMemCount : int32_t {
      UnitSize,
      Bytes,
      Elements,
      Released
   };

   enum class DiagMemOption : int32_t {
      Default,
      Peak,
      Active,
      PeruCalcObj
   };

   /// <summary>Error hander responses</summary>
   enum class ErrorHandlerResponse : int32_t {
      /// <summary>Immediately stops parsing</summary>
      Abort,
      /// <summary>Continues parsing (after the error is resolved)</summary>
      Resume,
      /// <summary>Raises the same error again for the next error handler to take care of</summary>
      ReRaise
   };

   /// <summary>Error numbers that uCalc may return</summary>
   enum class ErrorMsg : int32_t {
      /// <summary>No error</summary>
      None,
      /// <summary>Inexact result</summary>
      FloatInexact = 1,
      /// <summary>Floating point underflow</summary>
      FloatUnderflow = 2,
      /// <summary>Floating point overflow</summary>
      FloatOverflow = 4,
      /// <summary>Division by 0</summary>
      FloatDivisionByZero = 8,
      /// <summary>Invalid floating point value</summary>
      FloatInvalid = 16,
      /// <summary>Dynamically defined user error</summary>
      Dynamically_Defined,
      /// <summary>Syntax error</summary>
      Syntax_Error,
      /// <summary>Undefined identifier</summary>
      Undefined_Identifier,
      /// <summary>File not found</summary>
      FileNotFound,
      /// <summary>Unrecognized token</summary>
      Unrecognized_Token,
      /// <summary>Unrecognized command</summary>
      Unrecognized_Command,
      /// <summary>Data type mismatch</summary>
      Datatype_Mismatch,
      /// <summary>Invalid number of arguments</summary>
      Invalid_Argument_Count,
      /// <summary>Invalid definition</summary>
      Invalid_Definition,
      /// <summary>Bracket delimiter error</summary>
      Bracket_Error,
      /// <summary>Undefined callback</summary>
      Undefined_Callback,
      /// <summary>Error message already defined</summary>
      ErrorMessageAlreadyDefined,
      /// <summary>This item cannot be modified</summary>
      ItemCannotBeModified,
      /// <summary>Unrecognized class member</summary>
      Unrecognized_Class_Member,
      /// <summary>Unrecognized namespace member</summary>
      Unrecognized_Namespace_Member,
      /// <summary>Unbalanced quote</summary>
      Unbalanced_Quote,
      /// <summary>Value cannot be assigned here</summary>
      ValueCannotBeAssigned,
      /// <summary>Reparsing has passed the specified limit</summary>
      ReParseOverflow,
      /// <summary>Array or list bound exceeded</summary>
      Array_Bounds_Exceeded,
      Customized1 = 100,
      Customized2,
      Customized3,
      Customized4,
      Customized5,
      Customized6,
      Customized7,
      Customized8,
      Customized9,
      Customized10,
      Customized11,
      Customized12,
      Customized13,
      Customized14,
      Customized15,
      Customized16,
      Customized17,
      Customized18,
      Customized19,
      Customized20
   };

   /// <summary>Operator grouping associativity</summary>
   enum class Grouping : int32_t {
      LeftToRight,
      RightToLeft
   };

   /// <summary>Properties that a uCalc item may have</summary>
   /// <remarks>
   /// <para>Items defined with uCalc typically have a combination of properties.  A variable defined with DefineVariable for instance
   /// will have the ItemIs[Variable] property set.  An operator will have the ItemIs[Operator] property set.  If it&apos;s an infix
   /// operator, in addition to having the ItemIs[Operator] property set, ItemIs[Infix] will also be set.</para>
   /// 
   /// <para>The IsProperty function can determine if a given uCalc.Item has a certain property.</para>
   /// 
   /// <para>You can use the IsProperty function to cycle through the list of all items that share certain properties, or to disambiguate
   /// between items that share the same name but have different properties (for instance the + operator with the Prefix property
   /// vs another one with the Infix property).</para>
   /// 
   /// <para>Some uCalc enum elements are keywords in VB.NET.  To avoid a naming conflict, enclose the member name in brackets. For
   /// instance: [Function], [Operator], [Optional], [ByVal], etc.</para>
   /// </remarks>
   enum class ItemIs : int32_t {
      SelectAny,
      SelectAll,
      RightToLeft,
      DataType,
      Array,
      Class,
      Namespace,
      Variable,
      Function,
      Operator,
      FunctionOrOperator,
      Prefix,
      Infix,
      Postfix,
      AddressOwner,
      /// <summary>// +++ This is the default; doesn't appear to be used explicitly</summary>
      ByVal,
      ByRef,
      ByCopy,
      ByExpr,
      /// <summary>// +++ Rename to ByItem for clarity; will match PassBy::ByItem</summary>
      ByHandle,
      Optional,
      VariableNumberOfArgs,
      Alias,
      LiteralItem,
      Locked,
      Exclude,
      Transform,
      Search,
      FinishedSearch,
      ReturnSelf,
      ErrorHandler,
      ApiCall,
      CaseSensitive,
      QuotedText,
      RewindOnChange,
      TransformArg,
      Active,
      WhitespaceCounts,
      IgnoreQuote,
      IgnoreStatementSep,
      IgnoreBracket,
      SkipEvalTransform,
      ExprUnit,
      EagerArg,
      SkipPatternMatching,
      SkipOver,
      Modified,
      UpdatePending,
      ParentUpdatePending,
      UpToEndOrSeparator,
      UpToCompleted,
      SkipReduce,
      PatternCheck,
      EvalStrOutput,
      NotFound,
      ListReturned,
      Focusable,
      DisconnectedParent,
      Child,
      Parent,
      Range,
      HasByExpr
   };

   /// <summary>Licensing options for uCalc products</summary>
   enum class LicenseType : int32_t {
      Demo,
      FMP_Free,
      FMP_Standard_32,
      FMP_Standard_64,
      FMP_32_and_64,
      LanguageBuilder,
      GeneralParser,
      Interpreter,
      ConsoleCalc,
      WindowsCalc,
      StringLib,
      Transform,
      TransformLib,
      Search,
      SearchLib,
      Library1,
      Library2,
      Beta,
      Debugger,
      uCalc_All,
      Developer,
      EndUser
   };

   enum class MatchesOption : int32_t {
      All,
      SameAsBefore,
      EmptyList,
      MainLevelOnly,
      InnerMostOnly,
      FocusableOnly,
      NoOffset
   };

   enum class MemoryTable : int32_t {
      Invalid,
      uCalc,
      Callback,
      DataType,
      Expression,
      Item,
      Matches,
      Rule,
      String,
      Tokens,
      Transforms,
      Format,
      Table,
      uCalcBase = uCalc
   };

   /// <summary>RegEx grammar option for defining tokens; based on C++ standard library regex flags</summary>
   enum class RegExGrammar : int32_t {
      /// <summary>Default grammar; ECMAScript is the default</summary>
      Default,
      /// <summary>JavaScript-like regex grammar</summary>
      ECMAScript,
      /// <summary>Basic POSIX grammar (BRE)</summary>
      basic,
      /// <summary>Extended POSIX grammar (ERE)</summary>
      extended,
      /// <summary>awk POSIX grammar</summary>
      awk,
      /// <summary>grep POSIX grammar</summary>
      grep,
      /// <summary>egrep POXIX grammar</summary>
      egrep
   };

   enum class StringListType : int32_t {
      None,
      Chars,
      Tokens,
      Elements,
      Statements,
      Lines,
      Matches
   };

   /// <summary>Types of tokens</summary>
   enum class TokenType : int32_t {
      None,
      /// <summary>A token defined without any specified property is generic (default)</summary>
      Generic,
      /// <summary>Represents the absence of any character (token not found)</summary>
      Empty,
      /// <summary>A token that behaves as alphanuric (works with word boundaries)</summary>
      AlphaNumeric,
      /// <summary>A token that serves as argument separator (typically a comma)</summary>
      ArgSeparator,
      /// <summary>A token that starts a bracketed group; typically strings like {, [, or (</summary>
      Bracket,
      /// <summary>The closing token that matches a Bracket token. Ex: }, ], or )</summary>
      BracketMatch,
      /// <summary>A token that separates an object from its member, typically a . (a dot)</summary>
      MemberAccess,
      /// <summary>A token that represents a literal value, typically a quoted string or number</summary>
      Literal,
      /// <summary>A token that represents a statement separator, typically a ; (semicolon)</summary>
      StatementSep,
      /// <summary>A token that represents whitespace, typically tab or space characters</summary>
      Whitespace,
      /// <summary>A token that is not delimited by word boundaries (+, -, *, etc), consecutive characters are reduced until they match a value that exists in the symbol table or until 1 character</summary>
      Reducible,
      /// <summary>Non-character that represents the end of a stream of text</summary>
      Eof,
      LocalVar,
      StaticVar,
      /// <summary>Item in parenthesis (or quotes) following this token will be defined while the expression is parsed</summary>
      DefineNow = StaticVar,
      TempDef,
      /// <summary>The expression in parathesis (or quotes) following the token is evaluated and inserted in place of the token in the string</summary>
      EvalInsert,
      EvalDuringParse,
      /// <summary>Works like EvalInsert, except the expression is transformed instead of evaluated</summary>
      TransformInsert,
      /// <summary>The contents of the quoted file name following this token is inserted into the string</summary>
      FileInclude,
      CurrentSpace,
      DefSpace,
      EvalSpace,
      TokenTransform
   };

   /// <summary>Transform reset options</summary>
   /// <remarks>Use these options with Transform Reset.  The Default is All</remarks>
   enum class TransformResetOption : int32_t {
      /// <summary>Resets all (Input, Tokens, Rules)</summary>
      All,
      /// <summary>Resets the input to an empty string</summary>
      Input,
      /// <summary>Resets token definitions</summary>
      Tokens,
      /// <summary>Resets the FromTo rules</summary>
      Rules,
      /// <summary>Resets transform results</summary>
      Results,
      /// <summary>Removes passes</summary>
      Passes
   };

   typedef void (StdCall * ErrorHandlerCallback)(Handle_uCalc);

namespace dll {
   extern "C" DllImport Handle_Item StdCall AddErrorHandler_14(Handle_uCalc, ErrorHandlerCallback, size_t);
   extern "C" DllImport Handle_Item StdCall Alias_16(Handle_uCalc, const char *, size_t, Handle_Item);
   extern "C" DllImport Handle_Item StdCall Alias_17(Handle_uCalc, const char *, size_t, const char *, size_t);
   extern "C" DllImport void StdCall AliasForDefine_18(Handle_uCalc, const char *, size_t, const char *, size_t);
   extern "C" DllImport Handle_uCalc StdCall Clone_555(Handle_uCalc);
   extern "C" DllImport Handle_DataType StdCall DataTypeOf_22(Handle_uCalc, BuiltInType);
   extern "C" DllImport Handle_DataType StdCall DataTypeOf_23(Handle_uCalc, const char *, size_t);
   extern "C" DllImport Handle_uCalc StdCall Default_25(); // static
   extern "C" DllImport void StdCall DefaultClear_569(); // static
   extern "C" DllImport size_t StdCall DefaultCount_567(); // static
   extern "C" DllImport Handle_DataType StdCall DefaultDataType_28(Handle_uCalc);
   extern "C" DllImport void StdCall DefaultDataType_29(Handle_uCalc, BuiltInType);
   extern "C" DllImport void StdCall DefaultDataType_30(Handle_uCalc, Handle_DataType);
   extern "C" DllImport void StdCall DefaultDataType_31(Handle_uCalc, const char *, size_t);
   extern "C" DllImport Handle_Item StdCall DefaultItem_32(Handle_uCalc);
   extern "C" DllImport void StdCall DefaultPop_568(); // static
   extern "C" DllImport void StdCall DefaultPush_26( Handle_uCalc); // static
   extern "C" DllImport Handle_uCalc StdCall DefaultPushNew_580( Handle_uCalc); // static
   extern "C" DllImport Handle_Item StdCall Define_33(Handle_uCalc, const char *, size_t, void *, Handle_DataType);
   extern "C" DllImport Handle_Item StdCall DefineConstant_34(Handle_uCalc, const char *, size_t, void *, Handle_DataType);
   extern "C" DllImport Handle_Item StdCall DefineFunction_35(Handle_uCalc, const char *, size_t, void *, Handle_DataType);
   extern "C" DllImport Handle_Item StdCall DefineOperator_36(Handle_uCalc, const char *, size_t, size_t, Grouping, void *, Handle_DataType);
   extern "C" DllImport Handle_Item StdCall DefineVariable_37(Handle_uCalc, const char *, size_t, void *, Handle_DataType);
   extern "C" DllImport size_t StdCall DiagMem_597( size_t, BuiltInType, DiagMemCount, DiagMemOption); // static
   extern "C" DllImport const char * StdCall ErrorExpression_39(Handle_uCalc, size_t &);
   extern "C" DllImport void StdCall ErrorExpression_40(Handle_uCalc, const char *, size_t);
   extern "C" DllImport size_t StdCall ErrorLocation_41(Handle_uCalc);
   extern "C" DllImport const char * StdCall ErrorMessage_42(Handle_uCalc, size_t &, ErrorMsg);
   extern "C" DllImport ErrorMsg StdCall ErrorNumber_43(Handle_uCalc);
   extern "C" DllImport void StdCall ErrorResponse_44(Handle_uCalc, ErrorHandlerResponse);
   extern "C" DllImport const char * StdCall ErrorSymbol_45(Handle_uCalc, size_t &);
   extern "C" DllImport double StdCall Eval_46(Handle_uCalc, const char *, size_t);
   extern "C" DllImport const char * StdCall EvalStr_47(Handle_uCalc, size_t &, const char *, size_t, BoolInt);
   extern "C" DllImport Handle_Transforms StdCall ExpressionTransforms_48(Handle_uCalc);
   extern "C" DllImport int32_t StdCall FloatingPointErrorsToCatch_49(Handle_uCalc, int32_t);
   extern "C" DllImport Handle_Item StdCall Format_51(Handle_uCalc, void *, Handle_DataType);
   extern "C" DllImport Handle_Item StdCall Format_52(Handle_uCalc, void *, const char *, size_t);
   extern "C" DllImport Handle_Item StdCall Format_53(Handle_uCalc, const char *, size_t, Handle_DataType);
   extern "C" DllImport void StdCall FormatRemove_54(Handle_uCalc, Handle_DataType);
   extern "C" DllImport BoolInt StdCall IsDefault_57(Handle_uCalc);
   extern "C" DllImport void StdCall IsDefault_58(Handle_uCalc, BoolInt);
   extern "C" DllImport Handle_Item StdCall ItemOf_60(Handle_uCalc, int64_t, size_t, const char *, size_t, BoolInt);
   extern "C" DllImport Handle_Item StdCall ItemOf_61(Handle_uCalc, const char *, size_t, int64_t, size_t);
   extern "C" DllImport void StdCall License_62( LicenseType, size_t, size_t, size_t, size_t, size_t); // static
   extern "C" DllImport size_t StdCall MemoryIndex_602(Handle_uCalc);
   extern "C" DllImport size_t StdCall MemoryObjectAddress_( MemoryTable, size_t); // static
   extern "C" DllImport Handle_Item StdCall NamespaceSymbol_63(Handle_uCalc, size_t);
   extern "C" DllImport size_t StdCall NamespaceSymbolCount_64(Handle_uCalc);
   extern "C" DllImport Handle_uCalc StdCall New_584( Handle_uCalc); // static
   extern "C" DllImport Handle_uCalc StdCall NewDefault_20( Handle_uCalc); // static
   extern "C" DllImport Handle_Expression StdCall NewExpression_581(Handle_uCalc, const char *, size_t, BuiltInType);
   extern "C" DllImport Handle_Expression StdCall NewExpression_582(Handle_uCalc, const char *, size_t, Handle_DataType);
   extern "C" DllImport Handle_Expression StdCall NewExpression_583(Handle_uCalc, const char *, size_t, const char *, size_t);
   extern "C" DllImport Handle_String StdCall NewString_65(Handle_uCalc, const char *, size_t);
   extern "C" DllImport Handle_Transforms StdCall NewTransform_66(Handle_uCalc, Handle_Tokens);
   extern "C" DllImport Handle_Expression StdCall Parse_68(Handle_uCalc, const char *, size_t, BuiltInType);
   extern "C" DllImport Handle_Expression StdCall Parse_69(Handle_uCalc, const char *, size_t, Handle_DataType);
   extern "C" DllImport Handle_Expression StdCall Parse_70(Handle_uCalc, const char *, size_t, const char *, size_t);
   extern "C" DllImport size_t StdCall Precedence_71(Handle_uCalc, const char *, size_t, const char *, size_t);
   extern "C" DllImport int64_t StdCall Properties_72( ItemIs, ItemIs, ItemIs, ItemIs); // static
   extern "C" DllImport void StdCall RaiseErrorOnDivideByZero_73(Handle_uCalc, BoolInt);
   extern "C" DllImport void StdCall RaiseErrorOnInexact_74(Handle_uCalc, BoolInt);
   extern "C" DllImport void StdCall RaiseErrorOnInvalid_75(Handle_uCalc, BoolInt);
   extern "C" DllImport void StdCall RaiseErrorOnOverflow_76(Handle_uCalc, BoolInt);
   extern "C" DllImport void StdCall RaiseErrorOnUnderflow_77(Handle_uCalc, BoolInt);
   extern "C" DllImport size_t StdCall ReferenceCount_570(Handle_uCalc);
   extern "C" DllImport void StdCall ReferenceIncr_556(Handle_uCalc);
   extern "C" DllImport void StdCall Release_78(Handle_uCalc);
   extern "C" DllImport void StdCall ResetAll_566(); // static
   extern "C" DllImport Handle_uCalc StdCall SetAsDefault_585(Handle_uCalc);
   extern "C" DllImport const char * StdCall ValueAt_81(Handle_uCalc, size_t &, void *, Handle_DataType, BoolInt);
   extern "C" DllImport const char * StdCall ValueAt_82(Handle_uCalc, size_t &, void *, const char *, size_t, BoolInt);
   extern "C" DllImport double StdCall Arg_93(Handle_Callback, size_t);
   extern "C" DllImport double StdCall Arg1_94(Handle_Callback);
   extern "C" DllImport double StdCall Arg2_95(Handle_Callback);
   extern "C" DllImport void * StdCall ArgAddr_96(Handle_Callback, size_t);
   extern "C" DllImport BoolInt StdCall ArgBool_97(Handle_Callback, size_t);
   extern "C" DllImport size_t StdCall ArgCount_98(Handle_Callback);
   extern "C" DllImport double StdCall ArgDbl_99(Handle_Callback, size_t);
   extern "C" DllImport Handle_Expression StdCall ArgExpr_100(Handle_Callback, size_t);
   extern "C" DllImport int16_t StdCall ArgInt16_614(Handle_Callback, size_t);
   extern "C" DllImport int32_t StdCall ArgInt32_101(Handle_Callback, size_t);
   extern "C" DllImport int64_t StdCall ArgInt64_102(Handle_Callback, size_t);
   extern "C" DllImport Handle_Item StdCall ArgItem_103(Handle_Callback, size_t);
   extern "C" DllImport void * StdCall ArgPtr_104(Handle_Callback, size_t);
   extern "C" DllImport const char * StdCall ArgStr_105(Handle_Callback, size_t &, size_t);
   extern "C" DllImport ErrorHandlerResponse StdCall ErrorRaise_106(Handle_Callback, ErrorMsg);
   extern "C" DllImport ErrorHandlerResponse StdCall ErrorRaiseMessage_107(Handle_Callback, const char *, size_t);
   extern "C" DllImport void StdCall ErrorSet_108(Handle_Callback, ErrorMsg);
   extern "C" DllImport Handle_Item StdCall Item_110(Handle_Callback);
   extern "C" DllImport void StdCall Return_112(Handle_Callback, double);
   extern "C" DllImport void StdCall ReturnBool_113(Handle_Callback, BoolInt);
   extern "C" DllImport void StdCall ReturnDbl_114(Handle_Callback, double);
   extern "C" DllImport void StdCall ReturnInt16_615(Handle_Callback, int16_t);
   extern "C" DllImport void StdCall ReturnInt32_115(Handle_Callback, int32_t);
   extern "C" DllImport void StdCall ReturnInt64_116(Handle_Callback, int64_t);
   extern "C" DllImport void StdCall ReturnPtr_117(Handle_Callback, void *);
   extern "C" DllImport void StdCall ReturnStr_118(Handle_Callback, const char *, size_t);
   extern "C" DllImport Handle_uCalc StdCall uCalc_119(Handle_Callback);
   extern "C" DllImport size_t StdCall ByteSize_121(Handle_DataType);
   extern "C" DllImport BuiltInType StdCall Index_123(Handle_DataType);
   extern "C" DllImport Handle_Item StdCall Item_124(Handle_DataType);
   extern "C" DllImport size_t StdCall MemoryIndex_605(Handle_DataType);
   extern "C" DllImport const char * StdCall Name_125(Handle_DataType, size_t &);
   extern "C" DllImport void StdCall Release_126(Handle_DataType);
   extern "C" DllImport void StdCall Reset_128(Handle_DataType, void *);
   extern "C" DllImport void StdCall Reset_129(Handle_DataType, void *, size_t);
   extern "C" DllImport void StdCall SetArray_130(Handle_DataType, void *, void *, size_t);
   extern "C" DllImport Handle_DataType StdCall SetAsDefault_586(Handle_DataType);
   extern "C" DllImport void StdCall SetScalar_131(Handle_DataType, void *, void *);
   extern "C" DllImport void StdCall SwapScalarValues_132(Handle_DataType, void *, void *);
   extern "C" DllImport const char * StdCall ToString_134(Handle_DataType, size_t &, void *, BoolInt);
   extern "C" DllImport const char * StdCall ToString_135(Handle_DataType, size_t &, const char *, size_t);
   extern "C" DllImport Handle_Expression StdCall Clone_503(Handle_Expression);
   extern "C" DllImport Handle_DataType StdCall DataType_137(Handle_Expression);
   extern "C" DllImport double StdCall Evaluate_138(Handle_Expression);
   extern "C" DllImport BoolInt StdCall EvaluateBool_139(Handle_Expression);
   extern "C" DllImport double StdCall EvaluateDbl_140(Handle_Expression);
   extern "C" DllImport int32_t StdCall EvaluateInt32_141(Handle_Expression);
   extern "C" DllImport const char * StdCall EvaluateStr_142(Handle_Expression, size_t &, BoolInt);
   extern "C" DllImport void * StdCall EvaluateVoid_143(Handle_Expression);
   extern "C" DllImport void StdCall Execute_144(Handle_Expression);
   extern "C" DllImport size_t StdCall MemoryIndex_606(Handle_Expression);
   extern "C" DllImport size_t StdCall ReferenceCount_578(Handle_Expression);
   extern "C" DllImport void StdCall ReferenceIncr_557(Handle_Expression);
   extern "C" DllImport void StdCall Release_146(Handle_Expression);
   extern "C" DllImport Handle_uCalc StdCall uCalc_463(Handle_Expression);
   extern "C" DllImport Handle_Item StdCall BracketMatch_149(Handle_Item);
   extern "C" DllImport Handle_Item StdCall BracketMatch_150(Handle_Item, Handle_Item);
   extern "C" DllImport const char * StdCall Comment_520(Handle_Item, size_t &);
   extern "C" DllImport Handle_Item StdCall Comment_522(Handle_Item, const char *, size_t);
   extern "C" DllImport size_t StdCall Count_151(Handle_Item);
   extern "C" DllImport Handle_DataType StdCall DataType_524(Handle_Item);
   extern "C" DllImport Handle_Item StdCall DataType_152(Handle_Item, Handle_DataType);
   extern "C" DllImport void * StdCall FunctionAddress_153(Handle_Item);
   extern "C" DllImport void StdCall FunctionAddress_596(Handle_Item, void *);
   extern "C" DllImport BoolInt StdCall IsProperty_156(Handle_Item, ItemIs);
   extern "C" DllImport Handle_Item StdCall IsProperty_157(Handle_Item, ItemIs, BoolInt);
   extern "C" DllImport size_t StdCall MemoryIndex_607(Handle_Item);
   extern "C" DllImport const char * StdCall Name_525(Handle_Item, size_t &);
   extern "C" DllImport Handle_Item StdCall NextOverload_526(Handle_Item);
   extern "C" DllImport size_t StdCall Precedence_160(Handle_Item);
   extern "C" DllImport void StdCall Precedence_161(Handle_Item, size_t);
   extern "C" DllImport size_t StdCall ReferenceCount_577(Handle_Item);
   extern "C" DllImport void StdCall ReferenceIncr_562(Handle_Item);
   extern "C" DllImport const char * StdCall Regex_163(Handle_Item, size_t &);
   extern "C" DllImport Handle_Item StdCall Regex_164(Handle_Item, const char *, size_t, RegExGrammar);
   extern "C" DllImport void StdCall Release_165(Handle_Item);
   extern "C" DllImport Handle_Item StdCall Rename_166(Handle_Item, const char *, size_t);
   extern "C" DllImport void StdCall SetAsDefault_587(Handle_Item);
   extern "C" DllImport const char * StdCall Text_527(Handle_Item, size_t &);
   extern "C" DllImport Handle_uCalc StdCall uCalc_528(Handle_Item);
   extern "C" DllImport double StdCall Value_168(Handle_Item);
   extern "C" DllImport void StdCall Value_169(Handle_Item, double);
   extern "C" DllImport void StdCall Value_170(Handle_Item, const char *, size_t);
   extern "C" DllImport void StdCall Value_592(Handle_Item, Handle_Item);
   extern "C" DllImport void * StdCall ValueAddr_171(Handle_Item);
   extern "C" DllImport void StdCall ValueAddr_595(Handle_Item, void *);
   extern "C" DllImport BoolInt StdCall ValueBool_173(Handle_Item);
   extern "C" DllImport void StdCall ValueBool_174(Handle_Item, BoolInt);
   extern "C" DllImport void StdCall ValueByPtr_175(Handle_Item, void *);
   extern "C" DllImport uint8_t StdCall ValueByte_177(Handle_Item);
   extern "C" DllImport void StdCall ValueByte_178(Handle_Item, uint8_t);
   extern "C" DllImport double StdCall ValueDbl_180(Handle_Item);
   extern "C" DllImport void StdCall ValueDbl_181(Handle_Item, double);
   extern "C" DllImport int16_t StdCall ValueInt16_183(Handle_Item);
   extern "C" DllImport void StdCall ValueInt16_184(Handle_Item, int16_t);
   extern "C" DllImport int32_t StdCall ValueInt32_186(Handle_Item);
   extern "C" DllImport void StdCall ValueInt32_187(Handle_Item, int32_t);
   extern "C" DllImport int64_t StdCall ValueInt64_189(Handle_Item);
   extern "C" DllImport void StdCall ValueInt64_190(Handle_Item, int64_t);
   extern "C" DllImport void * StdCall ValuePtr_192(Handle_Item);
   extern "C" DllImport void StdCall ValuePtr_193(Handle_Item, void *);
   extern "C" DllImport float StdCall ValueSng_195(Handle_Item);
   extern "C" DllImport void StdCall ValueSng_196(Handle_Item, float);
   extern "C" DllImport const char * StdCall ValueStr_198(Handle_Item, size_t &, BoolInt);
   extern "C" DllImport void StdCall ValueStr_199(Handle_Item, const char *, size_t);
   extern "C" DllImport size_t StdCall Count_201(Handle_Matches);
   extern "C" DllImport size_t StdCall End_202(Handle_Matches, size_t);
   extern "C" DllImport Handle_Matches StdCall Filter_203(Handle_Matches, Handle_Rule);
   extern "C" DllImport size_t StdCall IndexOf_205(Handle_Matches, size_t, size_t);
   extern "C" DllImport BoolInt StdCall IsProperty_207(Handle_Matches, ItemIs);
   extern "C" DllImport Handle_Matches StdCall IsProperty_208(Handle_Matches, ItemIs, BoolInt);
   extern "C" DllImport size_t StdCall Length_210(Handle_Matches, size_t);
   extern "C" DllImport void StdCall Length_211(Handle_Matches, size_t, size_t);
   extern "C" DllImport size_t StdCall MemoryIndex_608(Handle_Matches);
   extern "C" DllImport const char * StdCall NewStr_212(Handle_Matches, size_t &, size_t);
   extern "C" DllImport size_t StdCall Offset_213(Handle_Matches, size_t);
   extern "C" DllImport size_t StdCall OldEnd_214(Handle_Matches, size_t);
   extern "C" DllImport size_t StdCall OldLength_216(Handle_Matches, size_t);
   extern "C" DllImport void StdCall OldLength_217(Handle_Matches, size_t, size_t);
   extern "C" DllImport void StdCall Release_218(Handle_Matches);
   extern "C" DllImport void StdCall Remove_219(Handle_Matches, size_t);
   extern "C" DllImport void StdCall Reset_220(Handle_Matches, MatchesOption);
   extern "C" DllImport Handle_Rule StdCall Rule_221(Handle_Matches, size_t);
   extern "C" DllImport size_t StdCall Start_223(Handle_Matches, size_t);
   extern "C" DllImport void StdCall Start_224(Handle_Matches, size_t, size_t);
   extern "C" DllImport const char * StdCall Str_226(Handle_Matches, size_t &);
   extern "C" DllImport const char * StdCall Str_227(Handle_Matches, size_t &, size_t);
   extern "C" DllImport Handle_uCalc StdCall uCalc_529(Handle_Matches);
   extern "C" DllImport BoolInt StdCall Active_230(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall Active_231(Handle_Rule, BoolInt);
   extern "C" DllImport BoolInt StdCall BracketSensitive_233(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall BracketSensitive_234(Handle_Rule, BoolInt);
   extern "C" DllImport BoolInt StdCall CaseSensitive_236(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall CaseSensitive_237(Handle_Rule, BoolInt);
   extern "C" DllImport const char * StdCall Comment_530(Handle_Rule, size_t &);
   extern "C" DllImport Handle_Rule StdCall Comment_535(Handle_Rule, const char *, size_t);
   extern "C" DllImport const char * StdCall FindPart_239(Handle_Rule, size_t &);
   extern "C" DllImport Handle_Rule StdCall FindPart_240(Handle_Rule, const char *, size_t);
   extern "C" DllImport BoolInt StdCall Focusable_242(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall Focusable_243(Handle_Rule, BoolInt);
   extern "C" DllImport const char * StdCall FromPart_245(Handle_Rule, size_t &);
   extern "C" DllImport Handle_Rule StdCall FromPart_246(Handle_Rule, const char *, size_t);
   extern "C" DllImport Handle_Rule StdCall FromTo_247(Handle_Rule, const char *, size_t, const char *, size_t);
   extern "C" DllImport BoolInt StdCall HasLocalTransform_249(Handle_Rule);
   extern "C" DllImport BoolInt StdCall IgnoreInnerPatterns_251(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall IgnoreInnerPatterns_252(Handle_Rule, BoolInt);
   extern "C" DllImport size_t StdCall Index_254(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall Index_255(Handle_Rule, size_t);
   extern "C" DllImport BoolInt StdCall IsChildRule_256(Handle_Rule);
   extern "C" DllImport BoolInt StdCall IsProperty_258(Handle_Rule, ItemIs);
   extern "C" DllImport Handle_Rule StdCall IsProperty_259(Handle_Rule, ItemIs, BoolInt);
   extern "C" DllImport Handle_Transforms StdCall LocalTransform_260(Handle_Rule);
   extern "C" DllImport Handle_Matches StdCall Matches_261(Handle_Rule, MatchesOption);
   extern "C" DllImport size_t StdCall MatchesCount_262(Handle_Rule);
   extern "C" DllImport size_t StdCall Maximum_264(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall Maximum_265(Handle_Rule, size_t);
   extern "C" DllImport size_t StdCall MaximumAND_267(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall MaximumAND_268(Handle_Rule, size_t);
   extern "C" DllImport size_t StdCall MemoryIndex_609(Handle_Rule);
   extern "C" DllImport size_t StdCall Minimum_270(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall Minimum_271(Handle_Rule, size_t);
   extern "C" DllImport size_t StdCall MinimumAND_273(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall MinimumAND_274(Handle_Rule, size_t);
   extern "C" DllImport const char * StdCall Name_531(Handle_Rule, size_t &);
   extern "C" DllImport Handle_Rule StdCall NextOverload_532(Handle_Rule);
   extern "C" DllImport Handle_Transforms StdCall ParentTransform_275(Handle_Rule);
   extern "C" DllImport size_t StdCall Precedence_277(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall Precedence_278(Handle_Rule, size_t);
   extern "C" DllImport Handle_Rule StdCall Property_279(Handle_Rule, ItemIs, BoolInt);
   extern "C" DllImport BoolInt StdCall QuoteSensitive_281(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall QuoteSensitive_282(Handle_Rule, BoolInt);
   extern "C" DllImport void StdCall Release_283(Handle_Rule);
   extern "C" DllImport const char * StdCall ReplacementText_284(Handle_Rule, size_t &);
   extern "C" DllImport BoolInt StdCall RewindOnChange_286(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall RewindOnChange_287(Handle_Rule, BoolInt);
   extern "C" DllImport BoolInt StdCall RightToLeft_289(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall RightToLeft_290(Handle_Rule, BoolInt);
   extern "C" DllImport BoolInt StdCall SkipOver_292(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall SkipOver_293(Handle_Rule, BoolInt);
   extern "C" DllImport size_t StdCall StartAfter_295(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall StartAfter_296(Handle_Rule, size_t);
   extern "C" DllImport BoolInt StdCall StatementSensitive_298(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall StatementSensitive_299(Handle_Rule, BoolInt);
   extern "C" DllImport size_t StdCall StopAfter_301(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall StopAfter_302(Handle_Rule, size_t);
   extern "C" DllImport const char * StdCall Text_533(Handle_Rule, size_t &);
   extern "C" DllImport const char * StdCall ToPart_304(Handle_Rule, size_t &);
   extern "C" DllImport Handle_Rule StdCall ToPart_305(Handle_Rule, const char *, size_t);
   extern "C" DllImport Handle_uCalc StdCall uCalc_534(Handle_Rule);
   extern "C" DllImport BoolInt StdCall WhitespaceSensitive_307(Handle_Rule);
   extern "C" DllImport Handle_Rule StdCall WhitespaceSensitive_308(Handle_Rule, BoolInt);
   extern "C" DllImport Handle_String StdCall After_310(Handle_String, const char *, size_t, size_t);
   extern "C" DllImport Handle_String StdCall Apply_311(Handle_String, const char *, size_t, const char *, size_t);
   extern "C" DllImport Handle_String StdCall At_312(Handle_String, size_t);
   extern "C" DllImport Handle_String StdCall AtChar_313(Handle_String, size_t);
   extern "C" DllImport Handle_String StdCall AtToken_314(Handle_String, size_t);
   extern "C" DllImport Handle_String StdCall Before_315(Handle_String, const char *, size_t, size_t);
   extern "C" DllImport Handle_String StdCall Between_316(Handle_String, const char *, size_t, const char *, size_t, size_t, size_t);
   extern "C" DllImport Handle_String StdCall BracketedText_317(Handle_String, size_t);
   extern "C" DllImport Handle_String StdCall BracketedTextInside_318(Handle_String, size_t);
   extern "C" DllImport Handle_String StdCall BracketMatch_319(Handle_String, size_t);
   extern "C" DllImport size_t StdCall BracketMatchTokenIndex_320(Handle_String, size_t);
   extern "C" DllImport size_t StdCall CharCount_321(Handle_String, size_t);
   extern "C" DllImport size_t StdCall CharPos_322(Handle_String, size_t);
   extern "C" DllImport ChildStringOptions StdCall ChildStringOption_324(Handle_String);
   extern "C" DllImport Handle_String StdCall ChildStringOption_325(Handle_String, ChildStringOptions);
   extern "C" DllImport Handle_String StdCall ClearDerivedStrings_326(Handle_String);
   extern "C" DllImport Handle_String StdCall Clone_496(Handle_String);
   extern "C" DllImport const char * StdCall Comment_536(Handle_String, size_t &);
   extern "C" DllImport Handle_String StdCall Comment_538(Handle_String, const char *, size_t);
   extern "C" DllImport Comparison StdCall Compare_327(Handle_String, const char *, size_t, size_t, size_t, size_t);
   extern "C" DllImport void * StdCall Data_498(Handle_String);
   extern "C" DllImport Handle_String StdCall Data_500(Handle_String, void *, size_t);
   extern "C" DllImport size_t StdCall DataSize_499(Handle_String);
   extern "C" DllImport Handle_String StdCall Find_328(Handle_String, const char *, size_t, size_t);
   extern "C" DllImport Handle_String StdCall FindOtherThan_329(Handle_String, const char *, size_t, size_t);
   extern "C" DllImport size_t StdCall IndexBase_332(Handle_String);
   extern "C" DllImport Handle_String StdCall IndexBase_333(Handle_String, size_t);
   extern "C" DllImport Handle_String StdCall List_334(Handle_String, StringListType);
   extern "C" DllImport Handle_String StdCall ListFormat_335(Handle_String, const char *, size_t, const char *, size_t, const char *, size_t, const char *, size_t, const char *, size_t, const char *, size_t, const char *, size_t);
   extern "C" DllImport Handle_String StdCall ListFunction_336(Handle_String, const char *, size_t, const char *, size_t, const char *, size_t, const char *, size_t);
   extern "C" DllImport Handle_String StdCall ListOfChars_337(Handle_String);
   extern "C" DllImport Handle_String StdCall ListOfTokens_338(Handle_String);
   extern "C" DllImport Handle_String StdCall ListPostfix_339(Handle_String, const char *, size_t);
   extern "C" DllImport Handle_String StdCall ListPrefix_340(Handle_String, const char *, size_t);
   extern "C" DllImport Handle_String StdCall ListSeparator_341(Handle_String, const char *, size_t);
   extern "C" DllImport Handle_String StdCall Match_342(Handle_String, size_t);
   extern "C" DllImport size_t StdCall MatchCount_343(Handle_String);
   extern "C" DllImport size_t StdCall MatchIndexAtChar_344(Handle_String, size_t);
   extern "C" DllImport size_t StdCall MatchIndexAtToken_345(Handle_String, size_t);
   extern "C" DllImport size_t StdCall MemoryIndex_610(Handle_String);
   extern "C" DllImport Handle_String StdCall NonMatches_346(Handle_String);
   extern "C" DllImport Handle_String StdCall Parent_347(Handle_String);
   extern "C" DllImport Handle_String StdCall Populate_348(Handle_String, double, double, const char *, size_t, double, const char *, size_t, const char *, size_t, const char *, size_t, const char *, size_t);
   extern "C" DllImport size_t StdCall ReferenceCount_574(Handle_String);
   extern "C" DllImport void StdCall ReferenceIncr_559(Handle_String);
   extern "C" DllImport void StdCall Release_349(Handle_String);
   extern "C" DllImport Handle_String StdCall Remove_350(Handle_String, const char *, size_t);
   extern "C" DllImport Handle_String StdCall Replace_351(Handle_String, const char *, size_t, const char *, size_t);
   extern "C" DllImport Handle_String StdCall Root_352(Handle_String);
   extern "C" DllImport Handle_String StdCall Skip_353(Handle_String, const char *, size_t);
   extern "C" DllImport Handle_String StdCall StartingFrom_354(Handle_String, const char *, size_t, size_t);
   extern "C" DllImport Handle_String StdCall StartingFromEndingWith_355(Handle_String, const char *, size_t, const char *, size_t, size_t, size_t);
   extern "C" DllImport const char * StdCall Str_357(Handle_String, size_t &, size_t, size_t);
   extern "C" DllImport Handle_String StdCall Str_358(Handle_String, const char *, size_t);
   extern "C" DllImport Handle_String StdCall SubString_359(Handle_String, size_t, size_t);
   extern "C" DllImport size_t StdCall TokenCharLen_360(Handle_String, size_t, size_t);
   extern "C" DllImport size_t StdCall TokenCharPos_361(Handle_String, size_t);
   extern "C" DllImport size_t StdCall TokenCount_362(Handle_String, size_t);
   extern "C" DllImport size_t StdCall TokenIndexAtChar_363(Handle_String, size_t);
   extern "C" DllImport size_t StdCall TokenPos_364(Handle_String, size_t);
   extern "C" DllImport Handle_String StdCall ToLower_365(Handle_String, const char *, size_t, size_t);
   extern "C" DllImport Handle_String StdCall ToUpper_366(Handle_String, const char *, size_t, size_t);
   extern "C" DllImport Handle_String StdCall Transform_367(Handle_String, Handle_Transforms);
   extern "C" DllImport TokenType StdCall TypeOfToken_368(Handle_String, size_t);
   extern "C" DllImport Handle_uCalc StdCall uCalc_539(Handle_String);
   extern "C" DllImport Handle_String StdCall UpTo_369(Handle_String, const char *, size_t, size_t);
   extern "C" DllImport Handle_Tokens StdCall Add_372(Handle_Tokens, Handle_Item);
   extern "C" DllImport Handle_Item StdCall Add_373(Handle_Tokens, const char *, size_t, TokenType, const char *, size_t, size_t, RegExGrammar, size_t);
   extern "C" DllImport Handle_Tokens StdCall Add_374(Handle_Tokens, Handle_Tokens);
   extern "C" DllImport void StdCall Clear_375(Handle_Tokens);
   extern "C" DllImport const char * StdCall Comment_540(Handle_Tokens, size_t &);
   extern "C" DllImport Handle_Tokens StdCall Comment_547(Handle_Tokens, const char *, size_t);
   extern "C" DllImport Handle_Tokens StdCall ContextSwitch_377(Handle_Tokens, Handle_Tokens, Handle_Item, int32_t, const char *, size_t, int32_t);
   extern "C" DllImport Handle_Tokens StdCall ContextSwitch_378(Handle_Tokens, Handle_Tokens, const char *, size_t, int32_t, const char *, size_t, int32_t);
   extern "C" DllImport Handle_Tokens StdCall ContextSwitch_379(Handle_Tokens, Handle_Tokens, const char *, size_t, const char *, size_t);
   extern "C" DllImport size_t StdCall Count_380(Handle_Tokens);
   extern "C" DllImport Handle_Item StdCall Define_381(Handle_Tokens, const char *, size_t, TokenType, size_t, RegExGrammar);
   extern "C" DllImport int32_t StdCall IndexOf_383(Handle_Tokens, Handle_Item);
   extern "C" DllImport Handle_Item StdCall Insert_384(Handle_Tokens, int32_t, const char *, size_t, TokenType, const char *, size_t, size_t, RegExGrammar);
   extern "C" DllImport BoolInt StdCall IsProperty_386(Handle_Tokens, ItemIs);
   extern "C" DllImport Handle_Tokens StdCall IsProperty_387(Handle_Tokens, ItemIs, BoolInt);
   extern "C" DllImport size_t StdCall MemoryIndex_611(Handle_Tokens);
   extern "C" DllImport void StdCall Release_388(Handle_Tokens);
   extern "C" DllImport void StdCall Remove_389(Handle_Tokens, Handle_Item);
   extern "C" DllImport Handle_Item StdCall Token_390(Handle_Tokens, size_t);
   extern "C" DllImport Handle_uCalc StdCall uCalc_544(Handle_Tokens);
   extern "C" DllImport Handle_Transforms StdCall Clone_504(Handle_Transforms);
   extern "C" DllImport const char * StdCall Comment_542(Handle_Transforms, size_t &);
   extern "C" DllImport Handle_Transforms StdCall Comment_548(Handle_Transforms, const char *, size_t);
   extern "C" DllImport Handle_Transforms StdCall Filter_393(Handle_Transforms);
   extern "C" DllImport Handle_Transforms StdCall Filter_394(Handle_Transforms, const char *, size_t);
   extern "C" DllImport Handle_Transforms StdCall Find_395(Handle_Transforms, const char *, size_t);
   extern "C" DllImport Handle_Rule StdCall FromTo_396(Handle_Transforms, const char *, size_t, const char *, size_t);
   extern "C" DllImport size_t StdCall IndexBase_399(Handle_Transforms);
   extern "C" DllImport Handle_Transforms StdCall IndexBase_400(Handle_Transforms, size_t);
   extern "C" DllImport BoolInt StdCall IsProperty_402(Handle_Transforms, ItemIs);
   extern "C" DllImport Handle_Transforms StdCall IsProperty_403(Handle_Transforms, ItemIs, BoolInt);
   extern "C" DllImport Handle_Matches StdCall Matches_404(Handle_Transforms, MatchesOption);
   extern "C" DllImport size_t StdCall MemoryIndex_612(Handle_Transforms);
   extern "C" DllImport Handle_Transforms StdCall NewPass_405(Handle_Transforms, BoolInt);
   extern "C" DllImport Handle_Rule StdCall NewRule_406(Handle_Transforms);
   extern "C" DllImport Handle_Transforms StdCall Pass_407(Handle_Transforms, size_t, BoolInt);
   extern "C" DllImport size_t StdCall PassCount_408(Handle_Transforms);
   extern "C" DllImport Handle_Transforms StdCall Range_409(Handle_Transforms, size_t, size_t);
   extern "C" DllImport size_t StdCall ReferenceCount_572(Handle_Transforms);
   extern "C" DllImport void StdCall ReferenceIncr_558(Handle_Transforms);
   extern "C" DllImport void StdCall Release_410(Handle_Transforms);
   extern "C" DllImport Handle_Transforms StdCall Reset_411(Handle_Transforms, TransformResetOption);
   extern "C" DllImport Handle_Rule StdCall RulesDefault_412(Handle_Transforms);
   extern "C" DllImport Handle_Rule StdCall RulesOverall_413(Handle_Transforms);
   extern "C" DllImport const char * StdCall Str_415(Handle_Transforms, size_t &);
   extern "C" DllImport Handle_Transforms StdCall Str_416(Handle_Transforms, const char *, size_t);
   extern "C" DllImport const char * StdCall StrData_417(Handle_Transforms);
   extern "C" DllImport size_t StdCall StrLength_418(Handle_Transforms);
   extern "C" DllImport const char * StdCall SubStr_419(Handle_Transforms, size_t &, size_t, size_t);
   extern "C" DllImport Handle_Tokens StdCall Tokens_420(Handle_Transforms);
   extern "C" DllImport Handle_Transforms StdCall Transform_422(Handle_Transforms);
   extern "C" DllImport Handle_Transforms StdCall Transform_423(Handle_Transforms, const char *, size_t);
   extern "C" DllImport Handle_String StdCall TransformWithSteps_424(Handle_Transforms, const char *, size_t);
   extern "C" DllImport Handle_uCalc StdCall uCalc_546(Handle_Transforms);
   extern "C" DllImport Handle_Transforms StdCall uCalc_425(Handle_Transforms, Handle_uCalc);
   extern "C" DllImport BoolInt StdCall WasModified_426(Handle_Transforms);

}

   /// <summary>uCalc main class</summary>
struct uCalcBase {
#if defined(_WIN64)
      static const bool Is32Dll = false;
      static const bool Is64Dll = true;
#elif defined(_WIN32)
      static const bool Is32Dll = true;
      static const bool Is64Dll = false;
#endif
      using string = std::string;
      using ostream = std::ostream;

      struct Callback;
      struct DataType;
      struct Expression;
      struct Item;
      struct Matches;
      struct Rule;
      struct String;
      struct Tokens;
      struct Transforms;


      /// <summary>Class for retrieving arguments passed to a callback, returning a value, and more</summary>
      struct Callback { // Standard-layout class; pointer to object of {{cllass}} type same as pointer to first non-static member (h)
         const Handle_Callback h = Handle_Callback::Ignore;
         uintptr_t Handle() { return (uintptr_t)h; }

         /// <summary>Returns the value of an argument passed to a uCalc.Callback function</summary>
         /// <returns>Returns a Double</returns>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>
         /// <para>When a callback function is defined with DefineFunction or DefineOperator, that callback function will receive arguments
         /// that uCalc passes to it through Arg().  Arg() handles values of the default Double floating point data type.  For other data
         /// types, use ArgInt32, ArgStr, etc instead.</para>
         /// 
         /// <para>Note that Index is 1-based.  So if there are 3 arguments, the Index of the first will be 1 (not 0), and the second and
         /// third will be 2 and 3 respectively.</para>
         /// 
         /// <para>If you are defining a binary operator, 1 is the index for the left operand, and 2 is for the right operand. For a unary
         /// operator, the index is 1.</para>
         /// </remarks>
double Arg(size_t Index) { return *(double *)((int64_t *)h + Index-1); }

         /// <summary>Returns the first argument of a callback</summary>
         /// <returns>Returns a Double</returns>
         /// <remarks>Similar to Arg[1], but slightly more efficient.</remarks>
double Arg1() { return *(double*)((int64_t*)h); }

         /// <summary>Returns the second argument of a callback</summary>
         /// <remarks>Similar to Arg[2], but slightly more efficient.</remarks>
double Arg2() { return *(double*)((int64_t*)h + 1); }

         /// <summary>Returns the address of the argument value that is being passed</summary>
         /// <returns>Returns a pointer</returns>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>In addition to Arg() which returns arguments that are of the default Double floating point numeric type, there are
         /// specialized functions for other datatypes such as ArgInt32, ArgInt64, etc.  However, if the argument is of a data type that
         /// does not have a specialized function, you can retrieve the address of the argument value as a pointer, and use that to retrieve
         /// the value.</remarks>
void * ArgAddr(size_t Index) { return (uintptr_t *)((int64_t*)h + Index-1); }

         /// <summary>Returns the Boolean integer argument passed to a uCalc.Callback function</summary>
         /// <returns>Returns a Boolean</returns>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>If a function parameter (or operator operand) was explicitely defined as Int32, then the callback function can recieve
         /// it through ArgBool.  If the argument is of a different but compatible numeric type, a conversion takes place first before
         /// being passed.</remarks>
bool ArgBool(size_t Index) { return *(BoolInt*)((int64_t*)h + Index - 1); }

         /// <summary>Returns the number of arguments being passed to a uCalc.Callback function</summary>
         /// <returns>0 if no arg is passed; or a positive integer representing the number of args being passed.</returns>
         /// <remarks>This is useful for functions that have a variable number of arguments</remarks>
         size_t ArgCount() { return dll::ArgCount_98(h); }

         /// <summary>Returns the value of an argument passed to a uCalc.Callback function</summary>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>
         /// <para>When a callback function is defined with DefineFunction or DefineOperator, that callback function will receive arguments
         /// that uCalc passes to it through the Arg().  Arg() handles values of the default Double floating point data type.  For other
         /// data types, use ArgInt32, ArgStr, etc instead.</para>
         /// 
         /// <para>Note that Index is 1-based.  So if there are 3 arguments, the Index of the first will be 1 (not 0), and the second and
         /// third will be 2 and 3 respectively.</para>
         /// 
         /// <para>If you are defining a binary operator, 1 is the index for the left operand, and 2 is for the right operand. For a unary
         /// operator, the index is 1.</para>
         /// </remarks>
double ArgDbl(size_t Index) { return *(double*)((int64_t*)h + Index - 1); }

         /// <summary>Returns the object instance of the expression that&apos;s passed as an argument to a uCalc.Callback function</summary>
         /// <returns>Returns a parsed expression object with which you can call functions like Evaluate or Execute</returns>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>uCalc allows you to not only pass arguments by value or by reference.  But it also lets you pass an argument by expression.
         ///  When an expression is passed ByExpression, the expresison is not evaluated before being passed. Instead a handle to the parsed
         /// expression is passed, allowing the callback to perform the evaluation.  This is very useful for things like &quot;if&quot;
         /// and &quot;while&quot; loop constructs etc.</remarks>
Expression ArgExpr(size_t Index) { return *(Handle_Expression *)((int64_t *)h + Index-1); }

int16_t ArgInt16(size_t Index) { return *(int16_t*)((int64_t*)h + Index - 1); }

         /// <summary>Returns the Int32 integer argument passed to a uCalc.Callback function</summary>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>If a function parameter (or operator operand) was explicitely defined as Int32, then the callback function can recieve
         /// it through ArgInt32.  If the argument is of a different but compatible numeric type, a conversion takes place first before
         /// being passed.</remarks>
int32_t ArgInt32(size_t Index) { return *(int32_t*)((int64_t*)h + Index - 1); }

         /// <summary>Returns the Int64 integer argument being passed to a uCalc.Callback function</summary>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>If a function parameter (or operator operand) was explicitely defined as Int64, then the callback function can recieve
         /// it through ArgInt64.  If the argument is of a different but compatible numeric type, a conversion takes place first before
         /// being passed.</remarks>
int64_t ArgInt64(size_t Index) { return *(int64_t*)((int64_t*)h + Index - 1); }

         /// <summary>Returns an object instance of the argument being passed to a uCalc.Callback function</summary>
         /// <returns>Returns a uCalc.Item; The returned object can be used (where relevent) just like any other uCalc.Item</returns>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>With this object, you can do such things as determine the argument&apos;s data type, determine which instance of
         /// uCalc it belongs to, retrieve the pointer of the value of the argument, etc.</remarks>
Item ArgItem(size_t Index) { return *(Handle_Item*)((int64_t*)h + Index - 1); }

         /// <summary>Returns a pointer value being passed as an argument to a uCalc.Callback function</summary>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>If a function parameter (or operator operand) was explicitely defined as a Pointer, then the callback function can
         /// recieve it through ArgPtr.  Note that ArgPtr returns any address value that is being sent, whereas ArgAddr returns the address
         /// of the value being passed.</remarks>
void * ArgPtr(size_t Index) { return (void *)*(uintptr_t *)((int64_t*)h + Index - 1); }

         /// <summary>Returns the value being passed as argument in to a uCalc.Callback function in the form of a string</summary>
         /// <param name="Index">Index of the given argument.  Note that Index is 1-based; the Index for the first arg is 1 (not 0)</param>
         /// <remarks>Returns data of any type to a string. In addition to returning an argument that was passed as a string, ArgStr has
         /// the special property of returning an argument that was passed as any data type.  If the argument was not a string, ArgStr
         /// first converts it to a string before returning its value.</remarks>
         string ArgStr(size_t Index) {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::ArgStr_105(h, ReturnStrSize, Index);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Raises an error in a uCalc.Callback function</summary>
         /// <returns>Returns ErrorHandlerResponse::Abort, ErrorHandlerResponse::Resume, or ErrorHandlerResponse::ReRaise.</returns>
         /// <param name="ErrNumber">Index number of error to raise</param>
         /// <remarks>Your callback can raise an error by calling this funciton with one of the values of the ErrorMsg enum.  End-user
         /// function definitions can also raise an error by calling Error(). The return value is determined by error handlers that you
         /// may have added.  ErrorHandlerResponse::Abort (value of 0) is returned by default if there are no error handlers.</remarks>
         ErrorHandlerResponse ErrorRaise(ErrorMsg ErrNumber) { return dll::ErrorRaise_106(h, ErrNumber); }

         /// <summary>Raises an error with a user-defined message in a uCalc.Callback function</summary>
         /// <returns>Returns ErrorHandlerResponse::Abort, ErrorHandlerResponse::Resume, or ErrorHandlerResponse::ReRaise.</returns>
         /// <param name="ErrMessage">Text of custom error message to return</param>
         /// <remarks>Your callback can raise a custom error message (one not listed as a ErrorMsg enum) by simply passing the text of
         /// the message to this function.</remarks>
         ErrorHandlerResponse ErrorRaiseMessage(const string & ErrMessage) { return dll::ErrorRaiseMessage_107(h, ErrMessage.data(), ErrMessage.size()); }

         /// <summary>Sets a numerical value for an error</summary>
         /// <param name="ErrNumber">Index number of error to raise</param>
         /// <remarks>The index corresponds with an ErrorMsg enum.</remarks>
         void ErrorSet(ErrorMsg ErrNumber) { dll::ErrorSet_108(h, ErrNumber); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Returns the function or operator Item of the current expression part</summary>
         /// <remarks>
         /// <para>In a callback, you can not only get the arguments that were passed, but also the associated function or operator that
         /// sent the arguments, using this function.</para>
         /// 
         /// <para>If several functions and/or operators are mapped to the same callback function name, you can use this to let you determine
         /// which one is currently being called, based on properties you can check.</para>
         /// </remarks>
         Item Item() { return dll::Item_110(h); }

         /// <summary>Sets the return value (floating point) of a uCalc.Callback function</summary>
         /// <param name="Value">Floating point value that your callback returns</param>
         /// <remarks>When you define a callback function with DefineFunction and you want to return a floating point number, do it with
         /// Return.</remarks>
void Return(double Value) { *(double *)h = Value; }

         /// <summary>Sets the return value of a uCalc.Callback function as an Boolean integer</summary>
         /// <param name="Value">Boolean value that your callback returns</param>
         /// <remarks>When you define a callback function with DefineFunction and you want to return a Boolean, do it with ReturnBool.</remarks>
void ReturnBool(bool Value) { *(BoolInt *)h = Value; }

         /// <summary>Sets the return value (default floating point) of a uCalc.Callback function</summary>
         /// <param name="Value">Floating point value that your callback returns</param>
         /// <remarks>
         /// <para>When you define a callback function with DefineFunction and you want to return a floating point number, do it with Return.</para>
         /// 
         /// <para>ReturnDbl() is identical to Return(); you may want to use this if &quot;Return&quot; is a reserved keyword in the compiler
         /// you&apos;re using.</para>
         /// </remarks>
void ReturnDbl(double Value) { *(double *)h = Value; }

void ReturnInt16(int16_t Value) { *(int16_t *)h = Value; }

         /// <summary>Sets the return value of a uCalc.Callback function as an Int32 integer</summary>
         /// <param name="Value">Integer (32-bit) that your callback returns</param>
         /// <remarks>When you define a callback function with DefineFunction and you want to return a Int32, do it with ReturnInt32.</remarks>
void ReturnInt32(int32_t Value) { *(int32_t *)h = Value; }

         /// <summary>Sets the return value of a uCalc.Callback function as an Int64 integer</summary>
         /// <param name="Value">Integer (64-bit) that your callback returns</param>
         /// <remarks>When you define a callback function with DefineFunction and you want to return a Int64, do it with ReturnInt64.</remarks>
void ReturnInt64(int64_t Value) { *(int64_t *)h = Value; }

         /// <summary>Sets the return value of a uCalc.Callback function as a Pointer</summary>
         /// <param name="Value">Pointer value that your callback returns</param>
         /// <remarks>When you define a callback function with DefineFunction that returns a pointer value, do it with ReturnPtr.</remarks>
void ReturnPtr(void * Value) { *(uintptr_t *)h = (uintptr_t)Value; }

         /// <summary>Sets the return value of a uCalc.Callback function as a String</summary>
         /// <param name="Value">String value that your callback returns</param>
         /// <remarks>When you define a callback function with DefineFunction that returns a string, do it with ReturnStr.</remarks>
         void ReturnStr(const string & Value) { dll::ReturnStr_118(h, Value.data(), Value.size()); }

         /// <summary>Returns the uCalc instance the current expression part belongs to</summary>
         /// <remarks>A callback is designed to work with all instances of uCalc.  However, at times you may need to know which particular
         /// instance of uCalc the expression part being passed belongs to.</remarks>
         uCalcBase uCalc() { return dll::uCalc_119(h); }

      }; // End of Callback class

      typedef void (StdCall * OpCallback)(Callback);


      /// <summary>Class for working with uCalc data types</summary>
      struct DataType {
         inline static Handle_DataType Empty = (Handle_DataType)dll::MemoryObjectAddress_(MemoryTable::DataType, 0);
         Handle_DataType h = Empty;

         DataType(Handle_DataType PassedHandle) : h(PassedHandle) { }
         uintptr_t Handle() { return (uintptr_t)h; }
         bool IsEmpty() { return h == Empty; }
         bool NotEmpty() { return h != Empty; }
         DataType() { }
         DataType(uCalcBase uc, const string & Definition) : h(uc.Define("DataType: " + Definition).DataType().h) { }
         DataType(const string & Definition) : DataType(Default(), Definition) { }
         operator string() { return Name(); }

         /// <summary>Returns the size (in bytes) of one unit of data for this data type</summary>
         /// <returns>Returns an integer</returns>
         size_t ByteSize() { return dll::ByteSize_121(h); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Returns the index of the data type</summary>
         /// <returns>Returns a BuiltInType enum</returns>
         /// <remarks>The return value is a BuiltInType enum value that represents the data type of the current object.  For instance,
         /// if the data type is a string, the return value would be BuiltInType::String.</remarks>
         BuiltInType Index() { return dll::Index_123(h); }

         /// <summary>Returns the uCalc.Item handle of the data type</summary>
         /// <remarks>Each data type object can also be represented as a uCalc Item object by using this return value.</remarks>
         Item Item() { return dll::Item_124(h); }

         size_t MemoryIndex() { return dll::MemoryIndex_605(h); }

         /// <summary>Returns the name of the data type</summary>
         /// <returns>Returns a string</returns>
         string Name() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::Name_125(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Releases a data type</summary>
         /// <remarks>This releases a Data type from memory.</remarks>
         void Release() { dll::Release_126(h); h = Empty; }

         /// <summary>Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary>
         /// <param name="ValuePtr">Pointer of the uCalc scalar to be reset</param>
         /// <remarks>Resets a scalar variable to the default value for that variable&apos;s data type.</remarks>
         void Reset(void * ValuePtr) { dll::Reset_128(h, ValuePtr); }

         /// <summary>Resets the value of data that was allocated with SetScalar to the default value of the given data type</summary>
         /// <param name="ValuePtr">Pointer of the uCalc array to be reset</param>
         /// <param name="Size">Number of elements to reset</param>
         /// <remarks>Resets all the elements of the array to the default value for that variable&apos;s data type.  This should only be
         /// used for arrays that were explicitly allocated with uCalc&apos;s AllocArray.</remarks>
         void Reset(void * ValuePtr, size_t Size) { dll::Reset_129(h, ValuePtr, Size); }

         /// <summary>Sets the value of an array at a given memory location that was allocated by AllocArray</summary>
         /// <param name="Destination">Pointer of array (base address) of which the element value is to be set</param>
         /// <param name="Source">Pointer to a value of the same data type that will be copied to the element</param>
         /// <param name="Index">Index of the array element who&apos;s value needs to be set</param>
         /// <remarks>Sets the value of an array at a given memory location that was allocated by AllocArray</remarks>
         void SetArray(void * Destination, void * Source, size_t Index) { dll::SetArray_130(h, Destination, Source, Index); }

         /// <summary>Sets the current data type as the default (for the current uCalc object)</summary>
         DataType SetAsDefault() { return dll::SetAsDefault_586(h); }

         /// <summary>Sets the value of a scalar at a given memory location that was allocated by AllocScalar</summary>
         /// <param name="Destination">Pointer of scalar who&apos;s value is to be set</param>
         /// <param name="Source">Pointer to a value of the same data type that will be copied to the element</param>
         /// <remarks>Sets the value of a scalar at a given memory location that was allocated by AllocScalar</remarks>
         void SetScalar(void * Destination, void * Source) { dll::SetScalar_131(h, Destination, Source); }

         /// <summary>Swaps the pointers of two variables that were allocated with AllocScalar or AllocArray</summary>
         /// <param name="ValuePtrA">Pointer of variable 1</param>
         /// <param name="ValuePtrB">Pointer of variable 2</param>
         /// <remarks>Swaps the pointers of two variables that were allocated with AllocScalar or AllocArray</remarks>
         void SwapScalarValues(void * ValuePtrA, void * ValuePtrB) { dll::SwapScalarValues_132(h, ValuePtrA, ValuePtrB); }

         /// <summary>Retuns a string representing the value at the given address formatted for the item&apos;s data type</summary>
         /// <param name="ValuePtr">Pointer of the data containing the value you want to display</param>
         /// <remarks>If you have a pointer to data of the given type, you can display this data as a string with this function.  Special
         /// formatting associated with a data type is used, if such formatting was defined.  For instance, if you&apos;re dealing with
         /// a Boolean, then you might get: True, Yes, On, etc, depending on how the formatting was configured with Format.</remarks>
         string ToString(void * ValuePtr, bool FormatOutput = false) {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::ToString_134(h, ReturnStrSize, ValuePtr, FormatOutput);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Retuns a formatted string of the given data type representing the value that was passed as text</summary>
         /// <param name="Value">String representing data of any type that is to be represented as data of the given type</param>
         /// <remarks>If you pass a string, it will be converted to the given data type, and return using the formatting for that data
         /// type.</remarks>
         string ToString(const string & Value) {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::ToString_135(h, ReturnStrSize, Value.data(), Value.size());
            return string(ReturnStr, ReturnStrSize);
         }
         string ToString(const char * Value) { return ToString(string(Value)); }

      }; // End of DataType class


      /// <summary>Class for evaluating an already parsed expression or executing parsed code</summary>
      struct Expression {
         inline static Handle_Expression Empty = (Handle_Expression)dll::MemoryObjectAddress_(MemoryTable::Expression, 0);
         Handle_Expression h = Empty;

         Expression(Handle_Expression PassedHandle) : h(PassedHandle) { }
         uintptr_t Handle() { return (uintptr_t)h; }
         bool IsEmpty() { return h == Empty; }
         bool NotEmpty() { return h != Empty; }
         Expression(uCalcBase uc, const string & Expr = "", uCalcBase::DataType ReturnType = uCalcBase::DataType()) : h(uc.NewExpressionH(Expr, ReturnType)) { }
         Expression(const string & Expr = "", uCalcBase::DataType ReturnType = uCalcBase::DataType()) : Expression(Default(), Expr, ReturnType) { }
         Expression(const char * Expr, uCalcBase::DataType ReturnType = uCalcBase::DataType()) : Expression(string(Expr), ReturnType) { }

         operator string() { return EvaluateStr(); }
         operator double() { return EvaluateDbl(); }
         operator bool() { return Evaluate() != 0; }
         operator int32_t() { return (int32_t)Evaluate(); }
         Expression & operator=(const string & s) { auto u = uCalc(); Release(); h = u.NewExpressionH(s); return *this; }
//         Expression & operator=(const char * s) { auto u = uCalc(); Release(); h = u.NewExpressionH(s); return *this; }


         /// <summary>Makes an identical but separate copy of an Expression object</summary>
         /// <returns>New Expression object</returns>
         Expression Clone() { return dll::Clone_503(h); }

         /// <summary>Returns the data type of a parsed expression</summary>
         /// <returns>DataType of the expression</returns>
         /// <remarks>Every parsed expression has a return type, which may be set explicity, or deduced implicitly based on the context;
         /// for instance comparison operations would return a Boolean, text within quotes would return as a string, and dividing two numeric
         /// values would return a Double precision floating point.  Sometimes it might not be clear what data type an expression has.
         ///  This function can return the data type.</remarks>
         DataType DataType() { return dll::DataType_137(h); }

         /// <summary>Rapidly evaluates a floating point expression that was parsed with Parse() or passed ByExpression</summary>
         /// <returns>Double precision numeric result of the evaluated expression</returns>
         /// <remarks>
         /// <para>Evaluate can be used instead of Eval for faster results in time-critical loops where the same expression is repeatedly
         /// re-evaluated, with the only change in each iteration typically being the value(s) of one or more variables.  Evaluate works
         /// in pair with Parse, the relatively slower part, which is called first before going into the time-critical loop.  The Expression
         /// object returned by Parse is what calls the Evaluate function.</para>
         /// 
         /// <para>Although Evaluate() is similar to EvaluateDbl(), it is slightly different.  Both return a double precision value, however,
         /// EvaluateDbl() can only return the value of a double precision expression, whereas Evaluate() can return the value of other
         /// numeric types (single precision, various integers, Boolean), in which case the value is first converted to Double precision
         /// before being returned.  Evaluate() is less prone to data type mixups.  On the other hand when a conversion does take place,
         /// there is a slight speed penalty.</para>
         /// 
         /// <para>Evaluate can also be called by an Expression object that was passed to a callback ByExpression.</para>
         /// 
         /// <para>If the expression evaluates to something other than Double precision floating point (such as various integers, complex,
         /// single precision, etc), the value is converted before being returned.  In some cases (such as string, complex, etc), the return
         /// result may or may not have a meaningful value.  However, an error is not raised.</para>
         /// </remarks>
         double Evaluate() { return dll::Evaluate_138(h); }

         /// <summary>Rapidly evaluates an expression that was parsed as Boolean with Parse or passed ByExpression</summary>
         /// <returns>The result of the evaluated Boolean expression</returns>
         /// <remarks>This is similar to Evaluate, but applies only to expressions that return a Boolean. Certain operators, such as comparison
         /// operators like &lt;, &gt;, ==, etc. return a Boolean.  If the final operation of an expression is Boolean, for instance in
         /// Parse(&quot;x + 1 &gt; 3&quot;), then you can use EvaluateBool.  In earlier versions Evaluate could not be used for types
         /// other than floating point, unless Double was specified in Parse so that a conversion could take place (from Bool in this case)
         /// to Double before being returned.  Now you can either use EvaluateBool (slightly more efficient) or else Evaluate as a catch
         /// all for any type including Boolean.</remarks>
         bool EvaluateBool() { return dll::EvaluateBool_139(h) != 0; }

         /// <summary>Rapidly evaluates a floating point expression that was parsed with Parse or passed ByExpression</summary>
         /// <returns>The result of the evaluated floating point expression</returns>
         /// <remarks>
         /// <para>EvaluateDbl is similar to Evaluate, but is intended strictly for expressions that return a Double precision value (determined
         /// either by the operators used in the expression, a default double precision type, or an explicit double precision).  It will
         /// not work with Integers or Single precision expressions for instance.</para>
         /// 
         /// <para>If the expression evaluates to a non-numeric value (such as a string) or non-scalar numeric value (such as a complex
         /// number), or if a numeric type other than the default was explicitely specified with Parse, the value returned will be invalid.
         ///  However, an error will not be raised.</para>
         /// </remarks>
         double EvaluateDbl() { return dll::EvaluateDbl_140(h); }

         /// <summary>Rapidly evaluates expression that was parsed as an integer with Parse or passed ByExpression</summary>
         /// <returns>Returns a 32-bit integer; the result of the evaluated expression</returns>
         /// <remarks>This is similar to Evaluate, but applies only to expressions that return a 32-bit integer.</remarks>
         int32_t EvaluateInt32() { return dll::EvaluateInt32_141(h); }

         /// <summary>Rapidly evaluates an expression of any data type that was parsed with Parse or passed ByExpression returning a string</summary>
         /// <returns>Returns a string which is the result of the evaluated expression</returns>
         /// <remarks>EvaluateStr is similar to Evaluate.  However, EvaluateStr returns a value as a string.  Unlike Evaluate, which can
         /// only return a floating point value, EvaluateStr can return a value of any data type.  Non-string values are converted to a
         /// string before being returned.</remarks>
         string EvaluateStr(bool ReturnFormatted = true) {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::EvaluateStr_142(h, ReturnStrSize, ReturnFormatted);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Rapidly evaluates an expression that was parsed with Parse or passed ByExpression and returns a Pointer to the result</summary>
         /// <returns>Returns a pointer to the result instead of the result itself</returns>
         /// <remarks>Use this when you are interested in a pointer to the result instead of the result itself.  For instance, given an
         /// integer result, you might use a pointer in order to retrieve the hi byte and lo byte values.</remarks>
         void * EvaluateVoid() { return dll::EvaluateVoid_143(h); }

         /// <summary>Rapidly executes code that was parsed with Parse() or passed ByExpression for which a result is not required</summary>
         /// <remarks>This can be used in cases where the return value is not needed, such as statements that represent code to be executed
         /// rather than a math expression.</remarks>
         void Execute() { dll::Execute_144(h); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         size_t MemoryIndex() { return dll::MemoryIndex_606(h); }

         /// <summary>Parses a new expression</summary>
         /// <returns>The current expression object</returns>
         /// <param name="Expr">Expression to be parsed</param>
         /// <param name="ReturnType">Data type of return value</param>
      Expression Parse(const string & Expr, uCalcBase::DataType ReturnType = uCalcBase::DataType()) { auto u = uCalc(); Release(); h = u.NewExpressionH(Expr, ReturnType); return *this; }

         /// <summary>Parses a new expression</summary>
         /// <returns>The current expression object</returns>
         /// <param name="Expr">Expression to be parsed</param>
         /// <param name="DataTypeIndex">Data type index of return value</param>
      Expression Parse(const string & Expr, BuiltInType DataTypeIndex) { auto u = uCalc(); Release(); h = u.NewExpressionH(Expr, u.DataTypeOf(DataTypeIndex)); return *this; }

         /// <summary>Parses a new expression</summary>
         /// <returns>The current expression object</returns>
         /// <param name="Expr">Expression to be parsed</param>
         /// <param name="ReturnType">Data type of return value</param>
      Expression Parse(const string & Expr, const string & ReturnType) { auto u = uCalc(); Release(); h = u.NewExpressionH(Expr, u.DataTypeOf(ReturnType)); return *this; }

         size_t ReferenceCount() { return dll::ReferenceCount_578(h); }

         /// <summary>.</summary>
         /// <remarks>.</remarks>
         void ReferenceIncr() { dll::ReferenceIncr_557(h); }

         /// <summary>Releases an expression that was parsed with Parse</summary>
         /// <remarks>Whenever an expression is parsed with Parse, the parsed expression remains in memory until released. When the parsed
         /// expression is no longer needed it should be released with Release to preserve memory. Note: An expression parsed with Parse
         /// must explicitely be released with Release unless the uCalc object it is in is released.  A parsed expression will not be released
         /// automatically when it simply goes out of scope.</remarks>
         void Release() { dll::Release_146(h); h = Empty; }

         /// <summary>Returns the uCalc object the expression belongs to</summary>
         /// <returns>uCalc object</returns>
         uCalcBase uCalc() { return dll::uCalc_463(h); }

      }; // End of Expression class


      /// <summary>Class for handling uCalc items</summary>
      struct Item {
         inline static Handle_Item Empty = (Handle_Item)dll::MemoryObjectAddress_(MemoryTable::Item, 0);
         Handle_Item h = Empty;

         Item(Handle_Item PassedHandle) : h(PassedHandle) { }
         uintptr_t Handle() { return (uintptr_t)h; }
         bool IsEmpty() { return h == Empty; }
         bool NotEmpty() { return h != Empty; }
         Item() { }

         /// <summary>Returns the bracket token that matches this token.</summary>
         /// <returns>Returns the bracket token that matches this token.</returns>
         /// <remarks>Bracket tokens are defined in pairs.  For instance, &quot;[&quot; typically would have &quot;]&quot; as a bracket
         /// match token.  &quot;(&quot; typically goes with bracket matching token &quot;)&quot;, &quot;{&quot; with &quot;}&quot; etc.
         ///  Use this to retrieve the matching bracket token item.</remarks>
         Item BracketMatch() { return dll::BracketMatch_149(h); }

         /// <summary>Associates another token with this one as the second part of a bracket pair</summary>
         /// <returns>Same token item that was passed as an argument</returns>
         /// <param name="BracketToken">Bracket token</param>
         /// <remarks>Bracket tokens are defined in pairs.  For instance, if you define &quot;[&quot; as a bracket token, you will want
         /// to define &quot;]&quot; as a bracket match token.  &quot;(&quot; typically goes with bracket matching token &quot;)&quot;,
         /// &quot;{&quot; with &quot;}&quot; etc.</remarks>
         Item BracketMatch(Item BracketToken) { return dll::BracketMatch_150(h, BracketToken.h); }

         /// <summary>Gets documentation/help/comment/tooltip type info associated with an item</summary>
         /// <returns>Comment associated with the item</returns>
         /// <remarks>Each item can be assigned a comment.  This comment can later be used at your discretion, for instance in a help file
         /// system, or as a tool tip.</remarks>
         string Comment() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::Comment_520(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets documentation/help/comment/tooltip type info for an item</summary>
         /// <param name="CommentStr">Text you want to set for the comment</param>
         /// <remarks>Each item can be assigned a comment.  This comment can later be used at your discretion, for instance in a help file
         /// system, or as a tool tip.</remarks>
         Item Comment(const string & CommentStr) { return dll::Comment_522(h, CommentStr.data(), CommentStr.size()); }

         /// <summary>Returns the number of elements in a uCalc Item</summary>
         /// <remarks>
         /// <para>Depending on the kind of object, this may represent the number of elements in an array, the number of parameters in
         /// a function, the number of operands in an operator, the number of parameters in a syntax construct, the number of parts in
         /// a transform, the number of parts in a compound token, or the value explicitlely set in the Count property when calling Define.</para>
         /// 
         /// <para>When determining the number of elements, optional parameters in a function count as an element.  If the function has
         /// a variable number of arguments, then count is -1.</para>
         /// 
         /// <para>The value returned for a syntax construct may not seem obvious, especially if it has optional and/or alternative parts
         /// or some other complexity.</para>
         /// </remarks>
         size_t Count() { return dll::Count_151(h); }

         /// <summary>Data type of a uCalc Item</summary>
         /// <returns>DataType object</returns>
         /// <remarks>The returned data type object can be used to determine the type name, size, etc.</remarks>
         DataType DataType() { return dll::DataType_524(h); }

         /// <summary>Sets the data type of a uCalc Item</summary>
         /// <returns>Returns the current item</returns>
         /// <param name="dType">Data type you want to assign to this item</param>
         /// <remarks>
         /// <para>This function is intended for use with DefaultItem() or tokens.</para>
         /// 
         /// <para>If you want to define a default type other than Double precision for math parser operations, then use this on DafaultItem().</para>
         /// 
         /// <para>Some tokens, such as literal strings, or numeric values, generally require an underlying data type.</para>
         /// 
         /// <para>The data type for other items is usually defined at the same time the item itself is defined, instead of after the fact
         /// with this function.</para>
         /// </remarks>
         Item DataType(uCalcBase::DataType dType) { return dll::DataType_152(h, dType.h); }

         /// <summary>Returns the callback address of a defined function (or operator)</summary>
         /// <returns>Returns the callback address of a defined function (or operator)</returns>
         /// <remarks>
         /// <para>The returned value is the actual pointer address of the function as used by uCalc internally, and not a delegate, even
         /// in .NET.  As such, in .NET this value can be used in the more general Define function but not functions like DefineFunction
         /// or DefineOperator.</para>
         /// 
         /// <para>+++ There are different delegate parts to account for</para>
         /// </remarks>
         OpCallback FunctionAddress() { return (OpCallback)dll::FunctionAddress_153(h); }

         void FunctionAddress(OpCallback Address) { dll::FunctionAddress_596(h, Address); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Determines if a uCalc Item has the given properties</summary>
         /// <returns>Returns Boolean True if the uCalc Item matches the given property, or False otherwise.</returns>
         /// <param name="ItemProperty">The property you want to check for</param>
         /// <remarks>This helps you determine whether the uCalc Item is a function, operator, variable, data type, literal value, etc,
         /// or has certain other special properties.  Note: Some items may have multiple properties.  For instance the Function property
         /// may be set to true for arrays.</remarks>
         bool IsProperty(ItemIs ItemProperty) { return dll::IsProperty_156(h, ItemProperty) != 0; }

         /// <summary>Sets a given property</summary>
         /// <returns>Returns a uCalc.Item</returns>
         /// <param name="ItemProperty">The property you want to set</param>
         /// <param name="TrueOrFalse">Value to set the property to</param>
         /// <remarks>Properties are selected from the ItemIs enum</remarks>
         Item IsProperty(ItemIs ItemProperty, bool TrueOrFalse) { return dll::IsProperty_157(h, ItemProperty, TrueOrFalse); }

         Item(const string & Definition, void * Address = nullptr) { h = Default().DefineH(Definition, Address); }
         Item(uCalcBase uc, const string & Definition, void * Address = nullptr) { h = uc.DefineH(Definition, Address); }

         size_t MemoryIndex() { return dll::MemoryIndex_607(h); }

         /// <summary>Returns the name of the uCalc Item</summary>
         /// <remarks>Items such as functions, operators, variables, etc generally are given a name when they are defined. That name can
         /// be retried with this function.</remarks>
         string Name() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::Name_525(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Next uCalc Item object that shares the same name</summary>
         /// <returns>Returns a uCalc Item object</returns>
         /// <remarks>If several functions, operators, parameters, etc are overloaded -- that is they are defined with different parameters
         /// but share the same name -- or if they (or variables) overshadow each other, this function returns the next uCalc.Item object
         /// that shares the same name.</remarks>
         Item NextOverload() { return dll::NextOverload_526(h); }

         /// <summary>Returns the precedence level of the uCalc.Item</summary>
         /// <returns>Returns the precedence level of the uCalc.Item</returns>
         /// <remarks>Operators, and some syntax constructs are defined with a precedence level.  When defining a new operator you may
         /// sometimes want to give it the same level as a pre-existing operator, which can be determined with this function.</remarks>
         size_t Precedence() { return dll::Precedence_160(h); }

         /// <summary>Sets the precedence level of the uCalc Item</summary>
         /// <param name="Level">Precedence level you want to set for this item</param>
         /// <remarks>
         /// <para>Operators, and some syntax constructs are defined with a precedence level.  The actual level is an arbitrary numeric
         /// value that derives its meaning based on whether this value is greater or less than other operators.</para>
         /// 
         /// <para>For use mainly with DefaultItem.  Precedence levels are usually set at the time an item is defined instead of using
         /// this function.</para>
         /// </remarks>
         void Precedence(size_t Level) { dll::Precedence_161(h, Level); }

         size_t ReferenceCount() { return dll::ReferenceCount_577(h); }

         void ReferenceIncr() { dll::ReferenceIncr_562(h); }

         /// <summary>Retrieves the regex pattern assigned to this token type</summary>
         /// <returns>String</returns>
         /// <remarks>Tokens are defined with a string that represents the regex for the token.  This function can return that string.</remarks>
         string Regex() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::Regex_163(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets a new regex pattern for this token</summary>
         /// <returns>Returns the current Item</returns>
         /// <param name="Pattern">Regex pattern</param>
         /// <param name="Grammar">Type of RegEx grammar</param>
         /// <remarks>Typically a regex pattern is assigned to a token item at the same time the token item is defined.  However, the regex
         /// pattern and/or type of regex grammar can later be changed.</remarks>
         Item Regex(const string & Pattern, RegExGrammar Grammar = RegExGrammar::Default) { return dll::Regex_164(h, Pattern.data(), Pattern.size(), Grammar); }

         /// <summary>Releases an Item defined with Define or other related function</summary>
         /// <remarks>
         /// <para>When a function, operator, variable, or other item is defined with such functions as Define, DefineFunction, DefineOperator,
         /// DefineVariable, these items remain in memory, until they are released with this function.  Items should generally be released
         /// when they are no longer needed, in order to preserve memory.</para>
         /// 
         /// <para>Note: A uCalc Item object must explicitely be released with Release unless the entire uCalc object it is a part of is
         /// released.  An Item object will not be released automatically when it simply goes out of scope.</para>
         /// </remarks>
         void Release() { dll::Release_165(h); h = Empty; }

         /// <summary>Renames an item</summary>
         /// <returns>Returns this object</returns>
         /// <param name="NewName">The new name of the item</param>
         /// <remarks>Any defined item that has a name (such as a variable, constant, function, operator, etc) can be renamed with this
         /// function.</remarks>
         Item Rename(const string & NewName) { return dll::Rename_166(h, NewName.data(), NewName.size()); }

         /// <remarks>+++ set this function up</remarks>
         void SetAsDefault() { dll::SetAsDefault_587(h); }

         /// <summary>Returns the verbatim text representing the definition of an Item</summary>
         /// <remarks>Certain items may have a text property that can be retrieved with a call to this function.  The literal text used
         /// to define an item is usually stored here.</remarks>
         string Text() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::Text_527(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Returns the uCalc object this Item belongs to</summary>
         /// <returns>uCalc object</returns>
         /// <remarks>
         /// <para>If your transform inovlves calcuations, then they will be performed using the uCalc object returned by this function.
         ///  Each uCalc object can have its own sets of variables, functions, and other definitions.</para>
         /// 
         /// <para>A uCalc.Item is defined in the context of a uCalc object.  You can later use this function to determine the uCalc object
         /// the defined item belongs to.</para>
         /// </remarks>
         uCalcBase uCalc() { return dll::uCalc_528(h); }

         /// <summary>Returns the numeric value associated with a uCalc.Item</summary>
         /// <returns>A double precision value</returns>
         /// <remarks>Use this to retrieve the Double precision value associated with a variable.  Other functions are provided for items
         /// that have another data type such as 16-bit integer, 32-bit integer, string, etc.</remarks>
         double Value() { return dll::Value_168(h); }

         /// <summary>Sets the value of a floating point variable</summary>
         /// <param name="Val">Double precision floating point value that the variable will be set to</param>
         /// <remarks>When a floating point variable is defined with DefineVariable, you can later set a value for this variable by passing
         /// an argument to Value.  Note: If your compiler supports pointers, you can define a uCalc variable that is directly attached
         /// to the address of a variable in your source code, and not have to use Value.</remarks>
         void Value(double Val) { dll::Value_169(h, Val); }

         /// <summary>Sets the value of a variable of any data type by passing a string</summary>
         /// <param name="Val">String containing the value that will be converted and stored in the variable</param>
         /// <remarks>This lets you set the value of a variable defined with DefineVariable (or Define) regardless of the variable&apos;s
         /// data type. The value is passed as a string, and parsed/evaluated as an expression, and internally converted to the appropriate
         /// data type.</remarks>
void Value(const string & Val) {dll::Value_170(h, Val.data(), Val.size());}
void Value(const char * Val) { Value(string(Val)); }

         /// <summary>Copies the value from one item (especially variable) to another</summary>
         /// <param name="OtherItem">Other item to copy from</param>
         void Value(Item OtherItem) { dll::Value_592(h, OtherItem.h); }

         /// <summary>Returns the Pointer address of the value associated with a uCalc.Item</summary>
         /// <returns>Pointer (or Integer)</returns>
         /// <remarks>Use this to retrieve the Pointer address of the value associated with a variable (instead of the value itself). 
         /// Note: If the uCalc.Item is a Variable of type Pointer, then ValuePtr returns a pointer value that is stored in that variable,
         /// whereas ValueAddr returns the Pointer of the variable itself.  In other words ValueAddr returns the address in memory of the
         /// current uCalc.Item value, while ValuePtr returns any given address you may have stored as the value of the variable.</remarks>
         void * ValueAddr() { return dll::ValueAddr_171(h); }

         void ValueAddr(void * Address) { dll::ValueAddr_595(h, Address); }

         /// <summary>Returns the Boolean value associated with a uCalc.Item</summary>
         /// <returns>Returns a Boolean value</returns>
         /// <remarks>Use this to retrieve the Boolean value associated with a variable.</remarks>
         bool ValueBool() { return dll::ValueBool_173(h) != 0; }

         /// <summary>Sets the value of a Boolean variable</summary>
         /// <param name="Value">Boolean value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a Boolean variable defined with DefineVariable (or Define)</remarks>
         void ValueBool(bool Value) { dll::ValueBool_174(h, Value); }

         /// <summary>Sets the value of a variable to a value who&apos;s Pointer is passed</summary>
         /// <param name="Value">Pointer to the value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a variable by passing the pointer to the value you want to set the variable to.</remarks>
         void ValueByPtr(void * Value) { dll::ValueByPtr_175(h, Value); }

         /// <summary>Returns the byte value associated with a uCalc.Item</summary>
         /// <returns>Returns an 8-bit number</returns>
         /// <remarks>Use this to retrieve the Byte value associated with a variable.</remarks>
         uint8_t ValueByte() { return dll::ValueByte_177(h); }

         /// <summary>Sets the value of a Byte variable</summary>
         /// <param name="Value">Byte value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a Byte variable defined with DefineVariable (or Define)</remarks>
         void ValueByte(uint8_t Value) { dll::ValueByte_178(h, Value); }

         /// <summary>Returns the Double precision value associated with a uCalc.Item</summary>
         /// <returns>Returns a double precision value</returns>
         /// <remarks>Synonymous with Value</remarks>
         double ValueDbl() { return dll::ValueDbl_180(h); }

         /// <summary>Sets the value of a double precision floating point variable</summary>
         /// <param name="Value">Double precision floating point value that the variable will be set to</param>
         /// <remarks>ValueDbl is synonymous with Value</remarks>
         void ValueDbl(double Value) { dll::ValueDbl_181(h, Value); }

         /// <summary>Returns the 16-bit integer value associated with a uCalc.Item</summary>
         /// <returns>Returns a 16-bit integer</returns>
         /// <remarks>Use this to retrieve the Int16 value associated with a variable.</remarks>
         int16_t ValueInt16() { return dll::ValueInt16_183(h); }

         /// <summary>Sets the value of a 16-bit integer variable</summary>
         /// <param name="Value">16-bit integer value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a 16-bit integer variable defined with DefineVariable (or Define).  This works for
         /// both signed and unsigned 16-bit integers.</remarks>
         void ValueInt16(int16_t Value) { dll::ValueInt16_184(h, Value); }

         /// <summary>Returns the 32-bit integer value associated with a uCalc.Item</summary>
         /// <returns>Returns a 32-bit integer</returns>
         /// <remarks>Use this to retrieve the Int32 value associated with a variable.</remarks>
         int32_t ValueInt32() { return dll::ValueInt32_186(h); }

         /// <summary>Sets the value of a 32-bit integer variable</summary>
         /// <param name="Value">32-bit integer value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a 32-bit integer variable defined with DefineVariable (or Define).  This works for
         /// both signed and unsigned 32-bit integers.</remarks>
         void ValueInt32(int32_t Value) { dll::ValueInt32_187(h, Value); }

         /// <summary>Returns the 64-bit integer value associated with a uCalc.Item</summary>
         /// <returns>Returns a 64-bit value</returns>
         /// <remarks>Use this to retrieve the Int64 value associated with a variable.</remarks>
         int64_t ValueInt64() { return dll::ValueInt64_189(h); }

         /// <summary>Sets the value of a 64-bit integer variable</summary>
         /// <param name="Value">64-bit integer value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a 64-bit integer variable defined with DefineVariable (or Define).  This works for
         /// both signed and unsigned 64-bit integers.</remarks>
         void ValueInt64(int64_t Value) { dll::ValueInt64_190(h, Value); }

         /// <summary>Returns the Pointer value associated with a uCalc.Item</summary>
         /// <remarks>Use this to retrieve the Pointer value associated with a variable. Note: If the uCalc.Item is a Variable of type
         /// Pointer, then ValuePtr returns a pointer value that is stored in that variable, whereas ValueAddr returns the Pointer of the
         /// variable itself.  In other words ValueAddr returns the address in memory of the current uCalc.Item value, while ValuePtr returns
         /// any value you may have set as the address the variable points to.</remarks>
         void * ValuePtr() { return dll::ValuePtr_192(h); }

         /// <summary>Sets the value of a pointer variable</summary>
         /// <param name="Value">Pointer that the variable will be set to</param>
         /// <remarks>This lets you set the value of a pointer variable defined with DefineVariable (or Define).  Note: This does not change
         /// the pointer of the variable.  It simply stores a pointer value in the variable.</remarks>
         void ValuePtr(void * Value) { dll::ValuePtr_193(h, Value); }

         /// <summary>Returns the Single precision value associated with a uCalc.Item</summary>
         /// <returns>Returns a Single precision value</returns>
         /// <remarks>Use this to retrieve the Single precision value associated with a variable.</remarks>
         float ValueSng() { return dll::ValueSng_195(h); }

         /// <summary>Sets the value of a single precision floating point variable</summary>
         /// <param name="Value">Single precision value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a single precision variable defined with DefineVariable (or Define).</remarks>
         void ValueSng(float Value) { dll::ValueSng_196(h, Value); }

         /// <summary>Returns the String value associated with a uCalc.Item</summary>
         /// <remarks>
         /// <para>Use this to retrieve the String value associated with a variable.</para>
         /// 
         /// <para>ValueStr has a special property beyond the Value function for other data types in that it can return the value of an
         /// item regardless of what data type it has.  The value is returned as a string, after being converted if necessary.</para>
         /// </remarks>
         string ValueStr(bool FormatOutput = false) {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::ValueStr_198(h, ReturnStrSize, FormatOutput);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets the value of a string variable</summary>
         /// <param name="Value">String value that the variable will be set to</param>
         /// <remarks>This lets you set the value of a string variable defined with DefineVariable (or Define).</remarks>
         void ValueStr(const string & Value) {dll::ValueStr_199(h, Value.data(), Value.size());}
         void ValueStr(const char * Value) { ValueStr(string(Value)); }


         //Item & operator=(Item && other) noexcept { h = other.h; return *this; }
         Item & operator=(const string & s) { if (DataType().Index() == BuiltInType::String) ValueStr(s); else Value(s); return *this; }
         Item & operator=(const char * s)   { if (DataType().Index() == BuiltInType::String) ValueStr(s); else Value(string(s)); return *this; }
         Item & operator=(const double & d) { if (DataType().Index() == BuiltInType::Integer_32) ValueInt32((int32_t)d); else Value(d); return *this; }
         Item & operator=(const bool & b) { ValueBool(b); return *this; }
         Item & operator=(const int32_t & i) { if (DataType().Index() == BuiltInType::Integer_32) ValueInt32(i); else Value(i); return *this; }

         operator string() { return ValueStr(); }
         operator double() { return Value(); }
         operator bool() { return ValueBool(); }
         operator int32_t() { return ValueInt32(); }
      }; // End of Item class


      /// <summary>Class for returning a list of matches</summary>
      struct Matches {
         inline static Handle_Matches Empty = (Handle_Matches)dll::MemoryObjectAddress_(MemoryTable::Matches, 0);
         Handle_Matches h = Empty;

         Matches(Handle_Matches PassedHandle) : h(PassedHandle) { }
         uintptr_t Handle() { return (uintptr_t)h; }
         bool IsEmpty() { return h == Empty; }
         bool NotEmpty() { return h != Empty; }
         Matches() { }

         /// <summary>Number of matches</summary>
         /// <returns>Integer</returns>
         /// <remarks>This returns the number of matches that were found after a Transform operation</remarks>
         size_t Count() { return dll::Count_201(h); }

         /// <summary>Returns the ending character position of a match</summary>
         /// <returns>Integer</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>Returns the ending character position of a match</remarks>
         size_t End(size_t Index) { return dll::End_202(h, Index); }

         /// <summary>Filters a list of matches</summary>
         /// <returns>Returns the same Matches object</returns>
         /// <param name="NestedRule">Rule to restrict list of matches to</param>
         /// <remarks>Filters a list of matches so that only the ones associated with the given Rule will be in the new list.  In a transform
         /// with multiple rules, sometimes you may want to zero in on the just the matches of a particular rule.  This function lets you
         /// do that.</remarks>
         Matches Filter(Rule NestedRule) { return dll::Filter_203(h, NestedRule.h); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Returns nth match index given a starting character position</summary>
         /// <returns>Integer representing nth index</returns>
         /// <param name="StartPos">Starting character position of match you&apos;re interested in</param>
         /// <param name="NearestIndex">Hint of nearest index</param>
         /// <remarks>
         /// <para>This gives you the index of the match that starts at a given character position.  For instance, given this string: &quot;This
         /// is a test&quot;, which might contain the matches &quot;This&quot;, &quot;is&quot;, &quot;a&quot;, and &quot;test&quot;.  If
         /// you know that &quot;is&quot; starts at character position 5, and want to know which match it is, IndexOf(5) will let you know
         /// that &quot;is&quot; reprents match 1 (where match 0 is &quot;This&quot;).</para>
         /// 
         /// <para>This is useful for finding the relative nth index in a list of matches, when you have more than one match lists for
         /// the same set of data and one is a subset of the other.</para>
         /// </remarks>
         size_t IndexOf(size_t StartPos, size_t NearestIndex = 0) { return dll::IndexOf_205(h, StartPos, NearestIndex); }

         /// <summary>Determines if an Item has the given property</summary>
         /// <returns>Returns True if the Item matches the given property, or False otherwise.</returns>
         /// <param name="ItemProperty">The property you want to check for</param>
         bool IsProperty(ItemIs ItemProperty) { return dll::IsProperty_207(h, ItemProperty) != 0; }

         /// <summary>Sets a given property</summary>
         /// <returns>Returns the same Matches object</returns>
         /// <param name="ItemProperty">The property you want to set</param>
         /// <param name="TrueOrFalse">Value to set the property to</param>
         /// <remarks>Sets a property for a Matches object</remarks>
         Matches IsProperty(ItemIs ItemProperty, bool TrueOrFalse) { return dll::IsProperty_208(h, ItemProperty, TrueOrFalse); }

         /// <summary>Returns the length of a match</summary>
         /// <returns>Integer</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>Returns the length of a match</remarks>
         size_t Length(size_t Index) { return dll::Length_210(h, Index); }

         /// <summary>Adjusts the reported length of a match</summary>
         /// <param name="Index">Index of match</param>
         /// <param name="value">New length of a match</param>
         /// <remarks>This adjusts the reported length of a match.  Updating the length of the matches is necessary for instance if you
         /// are converting text from utf-8 (default mode uCalc uses internally) to utf-16.</remarks>
         void Length(size_t Index, size_t value) { dll::Length_211(h, Index, value); }

         size_t MemoryIndex() { return dll::MemoryIndex_608(h); }

         /// <summary>Returns the new string a match was changed to</summary>
         /// <returns>String value</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>This returns the new string that a match was changed to.  For instance if you do a search/replace transform operation
         /// that changes all occurences of &quot;This&quot; in a string to &quot;That&quot;, then NewStr(n) would return &quot;That&quot;.</remarks>
         string NewStr(size_t Index) {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::NewStr_212(h, ReturnStrSize, Index);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets an offset for matches</summary>
         /// <returns>Returns the number of matches that were displaced</returns>
         /// <param name="StartingPos">Index of match</param>
         /// <remarks>This sets an offset that the matches&apos; starting and ending character positions will be relaitve to.</remarks>
         size_t Offset(size_t StartingPos = 0) { return dll::Offset_213(h, StartingPos); }

         /// <summary>Returns the old ending character position of a match before it was changed</summary>
         /// <returns>Returns an integer</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>In a search/replace transform type of operation where string matches are changed to new strings, this function returns
         /// what the ending character position of the match used to be before it was modified.</remarks>
         size_t OldEnd(size_t Index) { return dll::OldEnd_214(h, Index); }

         /// <summary>Returns the old length of a match before it was changed</summary>
         /// <returns>Returns an integer</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>In a search/replace transform type of operation where string matches are changed to new strings, this function returns
         /// what the length of the matching string used to be before it was modified.</remarks>
         size_t OldLength(size_t Index) { return dll::OldLength_216(h, Index); }

         /// <summary>Sets the OldLength value of a match to something else</summary>
         /// <param name="Index">Index of match</param>
         /// <param name="value">Sets OldLength to this value</param>
         /// <remarks>In a search/replace transform type of operation where string matches are changed to new strings, the one argument
         /// version of OldLength() returns what the length of the matching string used to be before it was modified.  However, in some
         /// cases, the reported OldLength() value may need to be adjusted due to Unicode conversions, such as from utf-8 (default mode
         /// uCalc uses internally) to utf-16.  This function lets you change the OldLength() value of matches.</remarks>
         void OldLength(size_t Index, size_t value) { dll::OldLength_217(h, Index, value); }

         /// <summary>Releases a Matches object</summary>
         /// <remarks>This releases a Matches object from memory.</remarks>
         void Release() { dll::Release_218(h); h = Empty; }

         /// <summary>Removes a match from the list of matches</summary>
         /// <param name="Index">Index of match</param>
         void Remove(size_t Index) { dll::Remove_219(h, Index); }

         /// <summary>Resets the list of matches</summary>
         /// <param name="Options">Option for subset of matches to return</param>
         /// <remarks>This resets the list of matches.  The Options argument lets you filter the list in certain ways.</remarks>
         void Reset(MatchesOption Options = MatchesOption::SameAsBefore) { dll::Reset_220(h, Options); }

         /// <summary>Returns the rule associated with a given match</summary>
         /// <returns>Rule object</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>This returns the rule associated with a given match.</remarks>
         Rule Rule(size_t Index) { return dll::Rule_221(h, Index); }

         /// <summary>Rerturns the starting character position of a match</summary>
         /// <returns>Integer</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>Rerturns the starting character position of a match</remarks>
         size_t Start(size_t Index) { return dll::Start_223(h, Index); }

         /// <summary>Changes the reported starting position of a match</summary>
         /// <param name="Index">Index of match</param>
         /// <param name="value">Value of starting position</param>
         /// <remarks>This changes the reported starting position of a match.  Useful in cases when a string is converted from Unicode
         /// utf-8 (default mode uCalc uses internally) to utf-16.</remarks>
         void Start(size_t Index, size_t value) { dll::Start_224(h, Index, value); }

         /// <summary>Returns a concatenated list of matches</summary>
         /// <returns>String</returns>
         /// <remarks>This returns all of the matches, concatenated as one string.</remarks>
         string Str() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::Str_226(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Returns the matching string</summary>
         /// <returns>Returns a string</returns>
         /// <param name="Index">Index of match</param>
         /// <remarks>This returns the text string of the nth match.</remarks>
         string Str(size_t Index) {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::Str_227(h, ReturnStrSize, Index);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Returns the uCalc object this Match belongs to</summary>
         /// <returns>uCalc object</returns>
         uCalcBase uCalc() { return dll::uCalc_529(h); }

      }; // End of Matches class


      /// <summary>Class for rules</summary>
      struct Rule {
         inline static Handle_Rule Empty = (Handle_Rule)dll::MemoryObjectAddress_(MemoryTable::Rule, 0);
         Handle_Rule h = Empty;

         Rule(Handle_Rule PassedHandle) : h(PassedHandle) { }
         uintptr_t Handle() { return (uintptr_t)h; }
         bool IsEmpty() { return h == Empty; }
         bool NotEmpty() { return h != Empty; }
         Rule() { }

         /// <summary>Determines whether a rule is currently active</summary>
         /// <returns>True or False</returns>
         /// <remarks>This lets you know if a rule is currently active or not.</remarks>
         bool Active() { return dll::Active_230(h) != 0; }

         /// <summary>Sets whether a rule is active</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that sets a rule as active or not</param>
         /// <remarks>This sets whether a rule is active or not. When you no longer need a rule, you can release it entirely from memory
         /// with Release().  However, at times you may want to only temporarily deactivate a rule, and then later reactivate it for further
         /// use.  That is when you use this function.</remarks>
         Rule Active(bool TrueFalse) { return dll::Active_231(h, TrueFalse); }

         /// <summary>Lets you konw if the rule is Bracket sensitive</summary>
         /// <returns>True/False</returns>
         /// <remarks>Lets you konw if the rule is Bracket sensitive.</remarks>
         bool BracketSensitive() { return dll::BracketSensitive_233(h) != 0; }

         /// <summary>Sets bracket sensitivity</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that determines bracket sensitivity</param>
         /// <remarks>
         /// <para>This sets whether a rule is bracket sensitive or not.  When set to True, a pattern will not match if it crosses the
         /// boundaries of a bracket grouping.</para>
         /// 
         /// <para>If set to False, then clode block delimiters are treated as ordinary generic tokens, with no special property as delimiters.</para>
         /// 
         /// <para>A bracketed group is a pattern enclosed between tokens defined as a bracket pair.  By default square brackets, [], parenthesis,
         /// (), and curly braces, {}, are defined as bracket pairs.  However, any string pairs can be defined as bracket tokens.</para>
         /// 
         /// <para>So, if you define a pattern &quot;a {etc} b&quot;, it will match &quot;a x y z b c&quot;, &quot;a (x y z) b c&quot;,
         /// &quot;a (x) (y z) b c&quot;, but not &quot;a x y z (b c)&quot;, because the bracket grouping (b c) crosses the boundary of
         /// the &quot;a {etc} b&quot; pattern; the opening parenthesis is found in the would-be match, but the closing parenthesis is
         /// beyond it, whereas in the other examples, both opening and closing parenthesis (when present) are fully within &quot;a {etc}
         /// b&quot;.</para>
         /// 
         /// <para>Note: Brackets used in pattern definitions take precedence over bracket token pairs.  Therefore you may define a pattern
         /// that gives brackets a different behavior than the default.</para>
         /// </remarks>
         Rule BracketSensitive(bool TrueFalse) { return dll::BracketSensitive_234(h, TrueFalse); }

         /// <summary>Lets you know if the rule is case-sensitive</summary>
         /// <returns>True or false</returns>
         /// <remarks>Lets you know if the rule is case-sensitive.</remarks>
         bool CaseSensitive() { return dll::CaseSensitive_236(h) != 0; }

         /// <summary>Sets whether a rule is case-sensitive</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that determines case sensitivity</param>
         /// <remarks>This sets whether a rule is case-sensitive or not.</remarks>
         Rule CaseSensitive(bool TrueFalse) { return dll::CaseSensitive_237(h, TrueFalse); }

         /// <summary>Documentation/help/comment/tooltip type info associated with a rule</summary>
         /// <returns>Comment associated with the Rule</returns>
         /// <remarks>Each item can be assigned a comment.  This comment can later be used at your discretion, for instance in a help file
         /// system, or as a tool tip.</remarks>
         string Comment() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::Comment_530(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets documentation/help/comment/tooltip type info for a Rule</summary>
         /// <param name="CommentStr">Text you want to set for the comment</param>
         /// <remarks>Each Rule can be assigned a comment.  This comment can later be used at your discretion, for instance in a help file
         /// system, or as a tool tip.</remarks>
         Rule Comment(const string & CommentStr) { return dll::Comment_535(h, CommentStr.data(), CommentStr.size()); }

         /// <summary>Returns the Find part of a rule in a search or transform operation</summary>
         /// <returns>A string</returns>
         /// <remarks>This returns the Find part of a rule.</remarks>
         string FindPart() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::FindPart_239(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets the Find part of a rule</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Pattern">Pattern for the Find part of a rule</param>
         /// <remarks>This sets the Find part of a rule in a search.</remarks>
         Rule FindPart(const string & Pattern) { return dll::FindPart_240(h, Pattern.data(), Pattern.size()); }

         /// <summary>Lets you know weather a rule is focusable</summary>
         /// <returns>True or false</returns>
         /// <remarks>This lets you know weather a rule is focusable.  You may use this property to decide wheather a match shouild be
         /// highlighted for instance.</remarks>
         bool Focusable() { return dll::Focusable_242(h) != 0; }

         /// <summary>Sets the focusable property of a Rule to True or False</summary>
         /// <returns>Current object</returns>
         /// <param name="TrueFalse">Value that determines rule is focusable</param>
         /// <remarks>This sets the focusable property of a Rule to True or False.</remarks>
         Rule Focusable(bool TrueFalse) { return dll::Focusable_243(h, TrueFalse); }

         /// <summary>Returns the Find part of a rule</summary>
         /// <returns>The current rule object</returns>
         /// <remarks>This returns the Find part of a rule.</remarks>
         string FromPart() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::FromPart_245(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets the From part of a rule</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Pattern">Pattern for the From part of a rule</param>
         /// <remarks>This sets the From part of a rule.</remarks>
         Rule FromPart(const string & Pattern) { return dll::FromPart_246(h, Pattern.data(), Pattern.size()); }

         /// <summary>Defines a transform rule</summary>
         /// <returns>Object for rule being defined</returns>
         /// <param name="FromStr">Pattern that describes the text to match</param>
         /// <param name="ToStr">New text that the match should be changed to</param>
         /// <remarks>Whenever a piece of text matches the pattern described in FromStr, it is replaced with the text found in ToStr.</remarks>
         Rule FromTo(const string & FromStr, const string & ToStr = "") { return dll::FromTo_247(h, FromStr.data(), FromStr.size(), ToStr.data(), ToStr.size()); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Lets you know if a rule has a local transform</summary>
         /// <returns>True/False</returns>
         /// <remarks>This lets you know if a rule has a local transform.</remarks>
         bool HasLocalTransform() { return dll::HasLocalTransform_249(h) != 0; }

         /// <summary>Lets you know if SkipPatternMatching is set</summary>
         /// <returns>True/False</returns>
         /// <remarks>Lets you know if SkipPatternMatching is set.</remarks>
         bool IgnoreInnerPatterns() { return dll::IgnoreInnerPatterns_251(h) != 0; }

         /// <summary>Sets whether uCalc should ignore pattern matching for a rule</summary>
         /// <returns>This Rule</returns>
         /// <param name="TrueFalse">True or false</param>
         /// <remarks>Sets whether uCalc should ignore pattern matching for a rule.</remarks>
         Rule IgnoreInnerPatterns(bool TrueFalse) { return dll::IgnoreInnerPatterns_252(h, TrueFalse); }

         /// <summary>Returns the index value associated with a rule</summary>
         /// <returns>Integer</returns>
         /// <remarks>Returns the index value associated with a rule.</remarks>
         size_t Index() { return dll::Index_254(h); }

         /// <summary>Sets an index value associated with this rule</summary>
         /// <returns>This same rule</returns>
         /// <param name="Value">Index to associate with this rule</param>
         /// <remarks>This sets an index value to be associated with this rule.  Each rule should have a unique index.  For instance, if
         /// you want to highlight the matches from one rule in Red, and the matches from another rule in Blue, by obtaining Matches(n).Rule().Index(),
         /// you can determine which color to choose for match n.</remarks>
         Rule Index(size_t Value) { return dll::Index_255(h, Value); }

         /// <summary>Lets you know if this rule is the child of another rule</summary>
         /// <returns>True/False</returns>
         /// <remarks>Lets you know if this rule is the child of another rule</remarks>
         bool IsChildRule() { return dll::IsChildRule_256(h) != 0; }

         /// <summary>Determines if a uCalc Item has the given properties</summary>
         /// <returns>Returns True if the uCalc Item matches the given property, or False otherwise.</returns>
         /// <param name="ItemProperty">The property you want to check for</param>
         /// <remarks>Determines if a uCalc Item has the given properties.</remarks>
         bool IsProperty(ItemIs ItemProperty) { return dll::IsProperty_258(h, ItemProperty) != 0; }

         /// <summary>Sets a given property</summary>
         /// <returns>Current rule</returns>
         /// <param name="ItemProperty">The property you want to set</param>
         /// <param name="TrueOrFalse">Value to set the property to</param>
         /// <remarks>Sets a given property for a rule.</remarks>
         Rule IsProperty(ItemIs ItemProperty, bool TrueOrFalse) { return dll::IsProperty_259(h, ItemProperty, TrueOrFalse); }

         /// <summary>Returns a local transform object</summary>
         /// <returns>Local transform object</returns>
         /// <remarks>When a pattern is matched, you can zoom into the text and perform a local search just within the matched text, independantly
         /// from the rest of the text outside of the match.</remarks>
         Transforms LocalTransform() { return dll::LocalTransform_260(h); }

         /// <summary>Returns an object that contains the list of matches along with associated properties for each match</summary>
         /// <param name="Options">Option for subset of matches to return</param>
         /// <remarks>Returns an object that contains the list of matches along with associated properties for each match.  With this object,
         /// you can for instance, determine the location and length of each match, which rule the match applies to etc.</remarks>
         Matches Matches(MatchesOption Options = MatchesOption::SameAsBefore) { return dll::Matches_261(h, Options); }

         /// <summary>Returns the number of matches associated with this particular rule</summary>
         /// <returns>Integer</returns>
         /// <remarks>Returns the number of matches associated with this particular rule</remarks>
         size_t MatchesCount() { return dll::MatchesCount_262(h); }

         /// <summary>Lets you know the maximum number of matches that will be looked for with this rule</summary>
         /// <returns>Integer</returns>
         /// <remarks>Lets you know the maximum number of matches that will be looked for with this rule.</remarks>
         size_t Maximum() { return dll::Maximum_264(h); }

         /// <summary>Sets the maximum number of matches allowed for this rule to count</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Value">Maximum number of matches of this rule allowed</param>
         /// <remarks>
         /// <para>This sets the maximum number of individual matches that this rule should find in order for any matches of this rule
         /// to count.  If more than this number of matches are found in the text, then this rule will be considered to have no matches
         /// at all.  For instance if a rule searches for matches of &quot;a&quot; in &quot;a a r d v a r k&quot;, and the maximum is set
         /// to 2, then no matches of &quot;a&quot; will be counted at all, because there are more than 2 occurences of &quot;a&quot; in
         /// the text.  On the other hand, if Max is set to 3 or 4 or any larger value, then all the matches would count.</para>
         /// 
         /// <para>The default Maximum value is -1, which is the equivalent of no maximum limit.</para>
         /// 
         /// <para>If you want to stop counting after a given number of matches of a rule, without invalidating the matches that were already
         /// found, use StopAfter instead.</para>
         /// </remarks>
         Rule Maximum(size_t Value) { return dll::Maximum_265(h, Value); }

         /// <summary>Returns the maximum number of matches of this rule allowed for the whole search to count</summary>
         /// <returns>Integer</returns>
         /// <remarks>Returns the maximum number of matches of this rule allowed for the whole search to count.</remarks>
         size_t MaximumAND() { return dll::MaximumAND_267(h); }

         /// <summary>Sets the maximum number of matches of this rule for the whole search to count</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Value">Maximum number of matches of this rule allowed</param>
         /// <remarks>
         /// <para>This sets the maximum number of individual matches that this rule should find in order for any matches of the transform
         /// to count.  If more than this number of matches are found in the text, then no matches from any rule in the local transform
         /// will count (whereas in a maximum, only matches from the current rule will be cancelled).  For instance if a rule searches
         /// for matches of &quot;a&quot; in &quot;a a r d v a r k&quot;, and the maximum AND is set to 2, then the entire transform (including
         /// other rules) will be invalidated and return no matches, because there are more than 2 occurences of &quot;a&quot; in the text.
         ///  On the other hand, if Max is set to 3 or 4 or any larger value, then all the matches would count.</para>
         /// 
         /// <para>This differs from Maximum in that if Maximum is exceeded, then only the Rule with the maximum will return 0 matches,
         /// whereas this one will cause the whole transform to return 0 matches.  This affects only the current transform and does not
         /// propagate to a parent transform or a transform in a separate pass.</para>
         /// 
         /// <para>The default Maximum value is -1, which is the equivalent of no maximum limit.</para>
         /// 
         /// <para>If you want to stop counting after a given number of matches of a rule, without invalidating any matches, use StopAfter
         /// instead.</para>
         /// </remarks>
         Rule MaximumAND(size_t Value) { return dll::MaximumAND_268(h, Value); }

         size_t MemoryIndex() { return dll::MemoryIndex_609(h); }

         /// <summary>Returns the minimum number of matches that must be found in this rule</summary>
         /// <returns>Integer</returns>
         /// <remarks>Returns the minimum number of matches that must be found in this rule</remarks>
         size_t Minimum() { return dll::Minimum_270(h); }

         /// <summary>Sets the minimum number of matches that must be found in this rule</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Value">Minimum number of matches required of this rule</param>
         /// <remarks>
         /// <para>This sets the minimum number of individual matches that this rule should find.</para>
         /// 
         /// <para>This differs from a MinimumAND in that if this Minimum is not reached, then it is considered that there are no matches
         /// for this particular rule.  Other rules are not affected.  Whereas in a MinimumAND if this rule does not reach the required
         /// minimum, then the entire local search or transform is invalidated, and it&apos;s considered that no match of any kind is found.</para>
         /// 
         /// <para>If less than this number of matches are found in the text, then none of the matches in this rule will count.  For instance
         /// if a rule searches for matches of &quot;a&quot; in &quot;a a r d v a r k&quot;, and the minimum is set to 5, then no matches
         /// of &quot;a&quot; will be counted at all, because there are not at least 5 occurences of &quot;a&quot; in the text.  On the
         /// other hand, if Min is set to 3 or less, then all the matches count.</para>
         /// 
         /// <para>Default: 0.</para>
         /// </remarks>
         Rule Minimum(size_t Value) { return dll::Minimum_271(h, Value); }

         /// <summary>Lets you know the minimum number of matches that must be found in this rule</summary>
         /// <returns>Integer</returns>
         /// <remarks>Lets you know the minimum number of matches that must be found in this rule.</remarks>
         size_t MinimumAND() { return dll::MinimumAND_273(h); }

         /// <summary>Sets the minimum number of matches that must be found in this rule</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Value">Minimum number of matches required of this rule</param>
         /// <remarks>
         /// <para>This sets the minimum number of individual matches that this rule should find.</para>
         /// 
         /// <para>This differs from a Minimum in that if Minimum is not reached, then it is considered that there are no matches for that
         /// particular rule.  Other rules are not affected.  Whereas in a MinimumAND, if this rule does not reach the required minimum,
         /// then the entire search or transform is invalidated, and it&apos;s considered that no match of any kind is found.</para>
         /// 
         /// <para>If less than this number of matches are found in the text, then the total match count will be 0. For instance if a rule
         /// searches for matches of &quot;a&quot; in &quot;a a r d v a r k&quot;, and the minimum is set to 5, the the whole transform
         /// operation will return 0 matches, because there are not at least 5 occurences of &quot;a&quot; in the text.  On the other hand,
         /// if it is set to 3 or less, then all the matches count.</para>
         /// 
         /// <para>Default: 0.</para>
         /// </remarks>
         Rule MinimumAND(size_t Value) { return dll::MinimumAND_274(h, Value); }

         /// <summary>Returns the name of a Rule</summary>
         /// <remarks>Typically the first text token that identifies the rule</remarks>
         string Name() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::Name_531(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Next uCalc Item object that shares the same name</summary>
         /// <returns>Returns a Rule object</returns>
         /// <remarks>If several Rules overloaded -- that is they are defined with different parameters but share the same name -- or if
         /// they overshadow each other, this function returns the next Rule that shares the same name.</remarks>
         Rule NextOverload() { return dll::NextOverload_532(h); }

         /// <summary>Returns the transform this rule belongs to</summary>
         /// <returns>Transform object</returns>
         /// <remarks>This returns the transform this rule belongs to.</remarks>
         Transforms ParentTransform() { return dll::ParentTransform_275(h); }

         /// <summary>Returns the precedence level for a rule</summary>
         /// <returns>The current rule object</returns>
         /// <remarks>Returns the precedence level for a rule.</remarks>
         size_t Precedence() { return dll::Precedence_277(h); }

         /// <summary>Sets the precedence level for a rule</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Value">Precedence level</param>
         /// <remarks>
         /// <para>Patterns that start (and/or end) with a pattern variable instead of a token or identifier can (and probably should)
         /// be assigned a precedence level.  You can choose any arbitrary numerical value.  The meaning of the precedence level depends
         /// on whether the value is greater, equal, or less than adjacent patterns with a precedence level.  It works similarly to precedence
         /// levels for operators.  For instance, if you define:</para>
         /// 
         /// <para>FromTo(&quot;{a} + {b}&quot;, &quot;({a} + {b})&quot;).Precedence(10); FromTo(&quot;{a} * {b}&quot;, &quot;({a} * {b})&quot;).Precedence(20);</para>
         /// 
         /// <para>Then</para>
         /// 
         /// <para>&quot;1 + 2 * 3 + 4&quot; transforms into &quot;(1 + (2 * 3) + 4)&quot; &quot;1 * 2 + 3 * 4&quot; transforms into &quot;(1
         /// * 2) + (3 * 4)&quot;</para>
         /// </remarks>
         Rule Precedence(size_t Value) { return dll::Precedence_278(h, Value); }

         /// <summary>Sets a rule property</summary>
         /// <returns>Current object</returns>
         /// <param name="SelectedProperty">Selected Property</param>
         /// <param name="TrueFalse">True/False</param>
         /// <remarks>Sets a rule property</remarks>
         Rule Property(ItemIs SelectedProperty, bool TrueFalse) { return dll::Property_279(h, SelectedProperty, TrueFalse); }

         /// <summary>Lets you know if the rule is quote sensitive</summary>
         /// <returns>True/False</returns>
         /// <remarks>This lets you know if the rule is quote sensitive.  The other version of this function sets or unsets quote sensitivity.</remarks>
         bool QuoteSensitive() { return dll::QuoteSensitive_281(h) != 0; }

         /// <summary>Sets quote sensitivity</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that determines quote sensitivity</param>
         /// <remarks>
         /// <para>This sets whether a rule is quote-sensitive or not. Quote sensitivity is generally designed for use across all rules
         /// in a transform (or sub transform).  However, it can also be set in the context of an individual pattern.</para>
         /// 
         /// <para>When set to true, it means that a block of text within quotes is treated as one token.  As such, patterns are not searched
         /// for within the quotes.  The contents of the quotes are skipped over.  You may still get around this and parse within quotes
         /// when necessary, by capturing the quote character itself, then parsing subsequent text.  When doing this be mindful of capturing
         /// quote characters in pairs.</para>
         /// 
         /// <para>The actual character(s) used as quotes are configurable with Tokens().Add().</para>
         /// </remarks>
         Rule QuoteSensitive(bool TrueFalse) { return dll::QuoteSensitive_282(h, TrueFalse); }

         /// <summary>Releases a transform rule.  Transform rules are defined with FromTo.</summary>
         /// <remarks>This releases a transform rule from memory.</remarks>
         void Release() { dll::Release_283(h); h = Empty; }

         /// <summary>Returns the replacement text of a FromTo rule</summary>
         /// <returns>String representing the second argument that was used in FromTo</returns>
         /// <remarks>This allows you to keep track of the replacement text of any given rule, even after it has been defined.</remarks>
         string ReplacementText() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::ReplacementText_284(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Lets you know if the parser should rewind and parse again after a change</summary>
         /// <returns>True/False</returns>
         /// <remarks>Lets you know if the parser should rewind and parse again after a change.  The other version of this function sets
         /// the value.</remarks>
         bool RewindOnChange() { return dll::RewindOnChange_286(h) != 0; }

         /// <summary>Determines whether to rewind back to the start of the modified match for further parsing</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that determines if the transfrom should rewind and parse again after a change</param>
         /// <remarks>
         /// <para>After a match is modified with the replacement text of a rule, if RewindOnChange is false (default), then uCalc will
         /// continue parsing from the position just after the insertion of the modified text. If this property is set to true, then the
         /// parser will rewind to the location where the match was found, and parse again, which will allow it to potentially find additional
         /// matches (of any rule, not just the current one) within the already modified text.</para>
         /// 
         /// <para>This property can be extremely useful in many situations.  However, special care should be taken so that an infinite
         /// reparsing session doesn&apos;t take place.  For instance, if a rule changes A to B, and another rule changes B to A, and this
         /// property is set ot true, and a Maximum value is not set, and there is an occurence of A, then A will change to B.  B will
         /// be parsed again and changed to A. The cycle will repeat itself indefinitely.</para>
         /// 
         /// <para>Default: False</para>
         /// </remarks>
         Rule RewindOnChange(bool TrueFalse) { return dll::RewindOnChange_287(h, TrueFalse); }

         /// <summary>Lets you know the associativity property of a rule</summary>
         /// <returns>True/Flase</returns>
         /// <remarks>Lets you know the associativity property of a rule; true means it goes right to left; false means it&apos;s left
         /// to right (default)</remarks>
         bool RightToLeft() { return dll::RightToLeft_289(h) != 0; }

         /// <summary>Sets whether a rule is right-to-left associative</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that determines case sensitivity</param>
         /// <remarks>This sets whether a rule is is right-to-left associative.</remarks>
         Rule RightToLeft(bool TrueFalse) { return dll::RightToLeft_290(h, TrueFalse); }

         /// <summary>Lets you know if matches for the current pattern will be skipped over</summary>
         /// <returns>True/False</returns>
         /// <remarks>Lets you know if matches for the current pattern will be skipped over.  The other version of SkipOver lets you set
         /// the value.</remarks>
         bool SkipOver() { return dll::SkipOver_292(h) != 0; }

         /// <summary>Determines whether a given match is skipped over</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that determines if a matched pattern is skipped over</param>
         /// <remarks>
         /// <para>If SkipOver is set to true, then text matches this pattern will be skipped over. This is useful for preventing uCalc
         /// from searching for matches of any other rules within certain text you want to skip over.</para>
         /// 
         /// <para>For instance if you skip over &quot;({etc})&quot;, then everything in parenthesis will be skipped over. So if you want
         /// to find occurences of &quot;a&quot; in &quot;a b c (a b c a b c) a b c&quot;, it will find all occurences of &quot;a&quot;,
         /// except those within parenthesis, because everything in parenthesis is being skipped over.</para>
         /// 
         /// <para>Default: False</para>
         /// </remarks>
         Rule SkipOver(bool TrueFalse) { return dll::SkipOver_293(h, TrueFalse); }

         /// <summary>Lets you know the number of matches this rule must find before matches start counting</summary>
         /// <returns>The current rule object</returns>
         /// <remarks>Lets you know the number of matches this rule must find before matches start counting.  The other version of StartAfter()
         /// lets you set this value.</remarks>
         size_t StartAfter() { return dll::StartAfter_295(h); }

         /// <summary>Sets the number of matches this rule must find before matches start counting</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Value">Number of matches that must be found before matches start counting</param>
         /// <remarks>
         /// <para>This sets the number of individual matches that this rule should find first before matches start counting. These first
         /// few matches are not counted in the results.  Onlny matches starting after this number of initial matches are counted.  If
         /// less than this number of matches are found in the text, no match will count for this rule. For instance if a rule searches
         /// for matches of &quot;a&quot; in &quot;a a r d v a r k&quot;, starting after 2, then only the 3rd &quot;a&quot; (the one between
         /// &quot;v&quot; and &quot;r&quot;) will count.  If StartAfter is set to 3, there will be no matches.  If StartAfter is 1, then
         /// then the 2nd and 3rd occurences of &quot;a&quot; will count as matches.</para>
         /// 
         /// <para>The default StartAfter value is 0.  This means all matches that are found will count.</para>
         /// </remarks>
         Rule StartAfter(size_t Value) { return dll::StartAfter_296(h, Value); }

         /// <summary>Lets you know if a rule is statement sensitive</summary>
         /// <returns>True/False</returns>
         /// <remarks>This lets you know if a rule is statement sensitive.  If a rule is statement sensitive, it will not allow a match
         /// to cross statement boundaries.</remarks>
         bool StatementSensitive() { return dll::StatementSensitive_298(h) != 0; }

         /// <summary>Sets a rule as statement Sensitive</summary>
         /// <returns>Current Rule</returns>
         /// <param name="TrueFalse">Value that determines bracket sensitivity</param>
         /// <remarks>Sets a rule as statement Sensitive</remarks>
         Rule StatementSensitive(bool TrueFalse) { return dll::StatementSensitive_299(h, TrueFalse); }

         /// <summary>Lets you know the number of matches after which this rule should stop finding matches</summary>
         /// <returns>The current rule object</returns>
         /// <remarks>Lets you know the number of matches after which this rule should stop finding matches.  The other version of this
         /// function lets you set the value.</remarks>
         size_t StopAfter() { return dll::StopAfter_301(h); }

         /// <summary>Sets the number of matches after which this rule should stop finding matches</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Value">Number of matches after which it should stop finding matches</param>
         /// <remarks>
         /// <para>This sets the number of individual matches that this rule should find.  After this number of matches are found, all
         /// other matches for this rule are ignored.  For instance if a rule searches for matches of &quot;a&quot; in &quot;a a r d v
         /// a r k&quot;, and stops after 2, then only the first two letters here wil match; the third &quot;a&quot; (the one between &quot;v&quot;
         /// and &quot;r&quot;) will not count as a match.</para>
         /// 
         /// <para>The default StopAfter value is -1, which means it will not stop counting matches for this rule.</para>
         /// 
         /// <para>This function has some similarities with Maximum.  However, with Maximum, if the maximum value is not attained, none
         /// of the matches for the rule will count.</para>
         /// </remarks>
         Rule StopAfter(size_t Value) { return dll::StopAfter_302(h, Value); }

         /// <summary>Returns the verbatim text representing the definition of a Rule</summary>
         string Text() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::Text_533(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Returns the To part of a rule</summary>
         /// <returns>The current rule object</returns>
         /// <remarks>This sets the To part of a rule.</remarks>
         string ToPart() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::ToPart_304(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets the To part of a rule</summary>
         /// <returns>The current rule object</returns>
         /// <param name="Pattern">Pattern for the From part of a rule</param>
         /// <remarks>This sets the To part of a rule.</remarks>
         Rule ToPart(const string & Pattern) { return dll::ToPart_305(h, Pattern.data(), Pattern.size()); }

         /// <summary>Returns the uCalc object this Rule belongs to</summary>
         /// <returns>uCalc object</returns>
         uCalcBase uCalc() { return dll::uCalc_534(h); }

         /// <summary>Lets you konw if the rule is Whitespace sensitive</summary>
         /// <returns>True/False</returns>
         /// <remarks>Lets you know if WhitespaceSensitive is set.</remarks>
         bool WhitespaceSensitive() { return dll::WhitespaceSensitive_307(h) != 0; }

         /// <summary>Sets whether a rule is whitespace-sensitive</summary>
         /// <returns>The current rule object</returns>
         /// <param name="TrueFalse">Value that determines whitespace sensitivity</param>
         /// <remarks>This sets whether a rule is whitespace-sensitive or not.</remarks>
         Rule WhitespaceSensitive(bool TrueFalse) { return dll::WhitespaceSensitive_308(h, TrueFalse); }

      }; // End of Rule class


      /// <summary>uCalc string library class</summary>
      struct String {
         inline static Handle_String Empty = (Handle_String)dll::MemoryObjectAddress_(MemoryTable::String, 0);
         Handle_String h = Empty;

         String(Handle_String PassedHandle) : h(PassedHandle) { }
         uintptr_t Handle() { return (uintptr_t)h; }
         bool IsEmpty() { return h == Empty; }
         bool NotEmpty() { return h != Empty; }
         String(uCalcBase uc, const string & s = "") : h(uc.NewStringH(s)) { }
         String(const string & s = "") : String(Default(), s) { }
         String(const char * s) : String(string(s)) { }

         operator string() { return Str(); }
         String & operator=(const string & s) { Str(s); return *this; }
         String & operator=(const char * s) { Str(s); return *this; }

         /// <summary>Returns a substring of all text that comes after the occurence of a pattern</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Pattern">Pattern</param>
         /// <param name="Nth">Nth match</param>
         /// <remarks>Returns a substring of all text that comes after the occurence of a pattern.</remarks>
         String After(const string & Pattern, size_t Nth = 1) { return dll::After_310(h, Pattern.data(), Pattern.size(), Nth); }

         /// <summary>Applies a special format to the string</summary>
         /// <returns>Returns the current uCalc string object</returns>
         /// <param name="expression">Expression</param>
         /// <param name="Variable">Name of variable</param>
         /// <remarks>This applies a special format to the string.  The format consists of an expression that evaluates to a string, where
         /// the variable x contains the string.</remarks>
         String Apply(const string & expression, const string & Variable = "") { return dll::Apply_311(h, expression.data(), expression.size(), Variable.data(), Variable.size()); }

         /// <summary>Returns the nth token</summary>
         /// <returns>String</returns>
         /// <param name="Index">Nth otken Index</param>
         /// <remarks>Returns the nth token.</remarks>
         String At(size_t Index) { return dll::At_312(h, Index); }

         /// <summary>Returns the character at the given location</summary>
         /// <returns>String object</returns>
         /// <param name="CharacterPosition">Index of charactor location</param>
         /// <remarks>This returns the character at a given location.  The character is returned as a uCalc string.  If that string is
         /// modified, the modification is reflected in the overall string.</remarks>
         String AtChar(size_t CharacterPosition) { return dll::AtChar_313(h, CharacterPosition); }

         /// <summary>Returns the text string of the nth token</summary>
         /// <returns>uCalc string</returns>
         /// <param name="Index">Index of token location</param>
         /// <remarks>This returns the text string of the nth token.  Modifications to the returned uCalc string are reflected in the overall
         /// string.</remarks>
         String AtToken(size_t Index) { return dll::AtToken_314(h, Index); }

         /// <summary>Returns a substring of all text up to (but not including) a given pattern</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Pattern">Pattern</param>
         /// <param name="Nth">Nth match</param>
         /// <remarks>Returns a substring of all text up to (but not including) a given pattern</remarks>
         String Before(const string & Pattern, size_t Nth = 1) { return dll::Before_315(h, Pattern.data(), Pattern.size(), Nth); }

         /// <summary>Returns a substring of text in between (but not including) occurences of two given patterns</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="StartPattern">Pattern</param>
         /// <param name="StopPattern">Pattern 2</param>
         /// <param name="StartNth">Nth match of starting pattern</param>
         /// <param name="StopNth">nth match of ending pattern</param>
         /// <remarks>Returns a substring of text in between (but not including) occurences of two given patterns.</remarks>
         String Between(const string & StartPattern, const string & StopPattern, size_t StartNth = 1, size_t StopNth = 1) { return dll::Between_316(h, StartPattern.data(), StartPattern.size(), StopPattern.data(), StopPattern.size(), StartNth, StopNth); }

         /// <summary>Returns the text corresponding to a bracket followed by text up to the next matching closing bracket</summary>
         /// <returns>uCalc string object</returns>
         /// <param name="Index">nth Token Index</param>
         /// <remarks>This returns the text corresponding to a bracket followed by text up to the next matching closing bracket.</remarks>
         String BracketedText(size_t Index) { return dll::BracketedText_317(h, Index); }

         /// <summary>Returns the text between a bracket and the the next matching closing bracket</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Index">nth Token Index</param>
         /// <remarks>This returns the text between a bracket and the the next matching closing bracket.  Brackets themselves are not included.</remarks>
         String BracketedTextInside(size_t Index) { return dll::BracketedTextInside_318(h, Index); }

         /// <summary>Returns the matching bracket for the indicated token</summary>
         /// <returns>uCalc string object</returns>
         /// <param name="Index">nth Token index</param>
         /// <remarks>This returns the matching bracket for the indicated token.  Manipulations to the returned uCalc string will be reflected
         /// in the overall string.</remarks>
         String BracketMatch(size_t Index) { return dll::BracketMatch_319(h, Index); }

         /// <summary>Returns the nth index of the closing bracket token</summary>
         /// <returns>Integer</returns>
         /// <param name="Index">nth token index</param>
         /// <remarks>Returns nth index of the closing bracket token.</remarks>
         size_t BracketMatchTokenIndex(size_t Index) { return dll::BracketMatchTokenIndex_320(h, Index); }

         /// <summary>Returns the number of characters of the nth match</summary>
         /// <returns>Integer</returns>
         /// <param name="Nth">Index of nth match</param>
         /// <remarks>This returns the number of characters of the nth match.</remarks>
         size_t CharCount(size_t Nth = -1) { return dll::CharCount_321(h, Nth); }

         /// <summary>Returns the character position of the nth match</summary>
         /// <returns>Integer that represents the character position</returns>
         /// <param name="Nth">Index of nth match</param>
         /// <remarks>This returns the character position of the nth match.</remarks>
         size_t CharPos(size_t Nth = -1) { return dll::CharPos_322(h, Nth); }

         /// <summary>Gets the option</summary>
         /// <returns>ChildStringOptions</returns>
         /// <remarks>Retrieves the option for  ChildStringOption.</remarks>
         ChildStringOptions ChildStringOption() { return dll::ChildStringOption_324(h); }

         /// <summary>Sets rules for when a child object is created</summary>
         /// <returns>String object</returns>
         /// <param name="Option">Option</param>
         /// <remarks>Many functions create a new string object derived from another given object.  This option determins the persistence
         /// of previously derived objects.  The right option depends on what you want to do.  If you&apos;re looking to conserve memory,
         /// then ClearAll may be the option you need.  However, with this option, you can no longer refer to previously derived strings,
         /// otherwise the behavior will be undefined (such as crashing).  You may delay object deletetion be removing only the ones that
         /// overlap in location with the newly derived string.  The decision to delete may be delayed based on when an object is being
         /// written to.   ChildStringOptions::ClearOverlapOnWrite, allows you to have multiple overlapping views of the same parent string
         /// without any conflict.  Only when modifying a derived string are overlapped derived strings removed.  Corresponding Detach
         /// options are similar, except that instead of actually erasing an overlapping object, it is disassassiated from the parent string.
         ///  It can continue to be used as an independent string without undefined behavior.  With this option, the independent strings
         /// are erased only when the parent string itself is erased.</remarks>
         String ChildStringOption(ChildStringOptions Option) { return dll::ChildStringOption_325(h, Option); }

         /// <summary>Clears derived strings from memory</summary>
         /// <returns>String object</returns>
         /// <remarks>Clears derived strings from memory.</remarks>
         String ClearDerivedStrings() { return dll::ClearDerivedStrings_326(h); }

         /// <summary>Makes an identical but separate copy of a uCalc String object</summary>
         /// <returns>New String object</returns>
         String Clone() { return dll::Clone_496(h); }

         /// <summary>returns documentation/help/comment/tooltip type info associated with an item</summary>
         /// <returns>Comment associated with the String</returns>
         string Comment() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::Comment_536(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <param name="CommentStr">Text you want to set for the comment</param>
         String Comment(const string & CommentStr) { return dll::Comment_538(h, CommentStr.data(), CommentStr.size()); }

         /// <summary>Compares two strings</summary>
         /// <returns>Returns bigger than, equal, or less than (Comparison enum)</returns>
         /// <param name="OtherString">The other string to compare with</param>
         /// <param name="IndexA">Starting point of first string</param>
         /// <param name="IndexB">Starting point of next string</param>
         /// <param name="Length">Length</param>
         /// <remarks>Compares two strings</remarks>
         Comparison Compare(const string & OtherString, size_t IndexA = 0, size_t IndexB = 0, size_t Length = -1) { return dll::Compare_327(h, OtherString.data(), OtherString.size(), IndexA, IndexB, Length); }

         /// <summary>Returns the raw string data address in memory of a uCalc string object</summary>
         void * Data() { return dll::Data_498(h); }

         /// <summary>Attaches a uCalc string to a raw string address in memory</summary>
         /// <param name="StringAddress">Address in memory of raw string</param>
         /// <param name="Size">Number of characters</param>
         /// <remarks>This sets a uCalc string based on a memory address and a size.  This is faster than copying one string to another.
         ///  The original string address must remain valid when this new string is used, until this string is released or its value is
         /// changed, at which point it will no longer depend on the original raw address.</remarks>
         String Data(void * StringAddress, size_t Size) { return dll::Data_500(h, StringAddress, Size); }

         /// <summary>Returns the raw size in characters of the string object</summary>
         /// <remarks>Returns the memory size in characters of the string object.</remarks>
         size_t DataSize() { return dll::DataSize_499(h); }

         /// <summary>Searches for occurences of a pattern in the string</summary>
         /// <returns>This uCalc string object</returns>
         /// <param name="Pattern">Pattern</param>
         /// <param name="Nth">Nth match</param>
         /// <remarks>Searches for occurences of a pattern in the string.  Once this operation is executed, you may retrieve any of the
         /// matches with Match(n).</remarks>
         String Find(const string & Pattern, size_t Nth = 1) { return dll::Find_328(h, Pattern.data(), Pattern.size(), Nth); }

         /// <summary>Finds occurences of all text other than the matching pattern</summary>
         /// <returns>This uCalc string object</returns>
         /// <param name="Pattern">Pattern</param>
         /// <param name="Nth">Nth match</param>
         /// <remarks>Finds occurences of all text other than the matching pattern.  This allows you to perform a transform on all text
         /// other than text that matches the given pattern.</remarks>
         String FindOtherThan(const string & Pattern, size_t Nth = 1) { return dll::FindOtherThan_329(h, Pattern.data(), Pattern.size(), Nth); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Gets the index base used in various functions</summary>
         /// <remarks>By default, uCalc String Library functions use a 0-based index.  So the first element is 0 (not 1), followed by 1,
         /// 2, 3, 4 etc. Use this to check to see which base is being used.</remarks>
         size_t IndexBase() { return dll::IndexBase_332(h); }

         /// <summary>Sets the index base used in various functions</summary>
         /// <param name="BaseValue">value representing index of 1st element</param>
         /// <remarks>By default, uCalc String Library functions use a 0-based index.  So the first element is 0 (not 1), followed by 1,
         /// 2, 3, 4 etc. Use this if you prefer to start by 1 or another non-zero value.  This value is passed on to string objects derived
         /// from a parent string. Each string may have a different index base.</remarks>
         String IndexBase(size_t BaseValue) { return dll::IndexBase_333(h, BaseValue); }

         /// <summary>Returns a list of matches, consisting of characters, tokens, elements, statements, or lines</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="ListType">Type of list</param>
         /// <remarks>Returns a list of matches, consisting of characters, tokens, elements, statements, or lines.</remarks>
         String List(StringListType ListType = StringListType::Matches) { return dll::List_334(h, ListType); }

         /// <summary>Sets a format for the List() function</summary>
         /// <param name="Separator">Separator</param>
         /// <param name="Prefix">Prefix</param>
         /// <param name="Postfix">Postfix</param>
         /// <param name="UserFunc">UserFunc</param>
         /// <param name="Element">Element</param>
         /// <param name="Index">Index</param>
         /// <param name="Count">Count</param>
         /// <remarks>
         /// <para>This sets a format that determines how the value of Str() is returned.  Separator will be inserted between each match;
         /// Prefix will be added before the string; Postfix is appended at the end of the string.  UserFunc determines how each match
         /// is displayed.  Element is the name of the variable to use in the function (x by default). Index is the name of the current
         /// match (Index by default).  Count is the number of matches (Count by default).</para>
         /// 
         /// <para>For instance: Given this list of matches: &quot;a&quot;, &quot;b&quot;, &quot;c&quot;, &quot;d&quot; ListOfChars().ListFormat(&quot;,
         /// &quot;, &quot;{&quot;, &quot;}&quot;, &quot;$&apos;{Index} of {Count}:{x}&apos;&quot;).Str() would return: {1 of 4:a, 2 of
         /// 4:b, 3 of 4:c, 4 of 4:d}</para>
         /// </remarks>
         String ListFormat(const string & Separator, const string & Prefix = "", const string & Postfix = "", const string & UserFunc = "", const string & Element = "", const string & Index = "", const string & Count = "") { return dll::ListFormat_335(h, Separator.data(), Separator.size(), Prefix.data(), Prefix.size(), Postfix.data(), Postfix.size(), UserFunc.data(), UserFunc.size(), Element.data(), Element.size(), Index.data(), Index.size(), Count.data(), Count.size()); }

         /// <summary>Sets the formatting function for displaying the elements of a list</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="UserFunc">UserFunc</param>
         /// <param name="Element">Element</param>
         /// <param name="Index">Index</param>
         /// <param name="MatchCount">MatchCount</param>
         /// <remarks>Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items within
         /// curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix. See ListFormat().</remarks>
         String ListFunction(const string & UserFunc, const string & Element = "", const string & Index = "", const string & MatchCount = "") { return dll::ListFunction_336(h, UserFunc.data(), UserFunc.size(), Element.data(), Element.size(), Index.data(), Index.size(), MatchCount.data(), MatchCount.size()); }

         /// <summary>Returns list of characters</summary>
         /// <returns>uCalc String object</returns>
         /// <remarks>Returns a list of characters.  Each characters can be accessed as a uCalc string with Match(n) or other indexed function.</remarks>
         String ListOfChars() { return dll::ListOfChars_337(h); }

         /// <summary>Returns a list of tokens</summary>
         /// <returns>uCalc String object</returns>
         /// <remarks>Returns a list of tokens.  Each token can be accessed with Match(n) or other indexed function.</remarks>
         String ListOfTokens() { return dll::ListOfTokens_338(h); }

         /// <summary>Sets the postfix that is appended at the end of a list</summary>
         /// <returns>Current uCalc String object</returns>
         /// <param name="Postfix">Postfix that will be appended at the end of a list</param>
         /// <remarks>Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items within
         /// curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix.</remarks>
         String ListPostfix(const string & Postfix) { return dll::ListPostfix_339(h, Postfix.data(), Postfix.size()); }

         /// <summary>Sets the prefix that is inserted at the start of lists</summary>
         /// <returns>Current uCalc String object</returns>
         /// <param name="Prefix">Prefix that will be inserted at the start of a list</param>
         /// <remarks>Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items within
         /// curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix.</remarks>
         String ListPrefix(const string & Prefix) { return dll::ListPrefix_340(h, Prefix.data(), Prefix.size()); }

         /// <summary>Sets the separator that will delimite the elements of a list</summary>
         /// <returns>Current uCalc String object</returns>
         /// <param name="Separator">Separator that will delimite each element of a list</param>
         /// <remarks>Text that is returned as a list can have a prefix, postfix, and separators between elements.  A list of items within
         /// curly braces might have an open curly brace as prefix, a comma as delimiter, and a closing curly brace as postfix.</remarks>
         String ListSeparator(const string & Separator) { return dll::ListSeparator_341(h, Separator.data(), Separator.size()); }

         /// <summary>Returns the nth match</summary>
         /// <returns>uCalc string object</returns>
         /// <param name="Index">Index of nth match</param>
         /// <remarks>Returns the nth match.</remarks>
         String Match(size_t Index) { return dll::Match_342(h, Index); }

         /// <summary>Returns the number of matches</summary>
         /// <returns>Integer</returns>
         /// <remarks>Returns the number of matches</remarks>
         size_t MatchCount() { return dll::MatchCount_343(h); }

         /// <summary>Returns the index of the nth match where a character location is</summary>
         /// <returns>Integer</returns>
         /// <param name="Index">Index of the character who&apos;s match index you want</param>
         /// <remarks>Returns the index of the nth match where a character location is</remarks>
         size_t MatchIndexAtChar(size_t Index) { return dll::MatchIndexAtChar_344(h, Index); }

         /// <summary>Returns the index of the nth match where a token location is</summary>
         /// <returns>Integer</returns>
         /// <param name="Index">Index of the token who&apos;s match index you want</param>
         /// <remarks>Returns the index of the nth match where a token location is</remarks>
         size_t MatchIndexAtToken(size_t Index) { return dll::MatchIndexAtToken_345(h, Index); }

         size_t MemoryIndex() { return dll::MemoryIndex_610(h); }

         /// <summary>Returns a string with a list of matches consisting of everything other than occurences of the Find pattern.</summary>
         /// <returns>uCalc String object</returns>
         /// <remarks>Returns a string with a list of matches consisting of everything other than occurences of the Find pattern.</remarks>
         String NonMatches() { return dll::NonMatches_346(h); }

         /// <summary>Returns the immediate parent of the current string</summary>
         /// <returns>uCalc string object that represents the parent of this string</returns>
         /// <remarks>Returns the immediate parent of the current string.</remarks>
         String Parent() { return dll::Parent_347(h); }

         /// <summary>Creates a string from a programmatically constructed list</summary>
         /// <returns>uCalc string object</returns>
         /// <param name="first">first</param>
         /// <param name="last">last</param>
         /// <param name="expression">expression</param>
         /// <param name="Step">Step</param>
         /// <param name="Condition">Condition</param>
         /// <param name="VarIndex">VarIndex</param>
         /// <param name="VarFirst">VarFirst</param>
         /// <param name="VarLast">VarLast</param>
         /// <remarks>For example: Populate(2, 5, &quot;$&apos;{Index}:{First}:{Last} etc.&apos;&quot;).Str(); would return: &quot;2:2:5
         /// etc.3:2:5 etc.4:2:5 etc.5:2:5 etc.&quot;</remarks>
         String Populate(double first, double last, const string & expression, double Step = 1, const string & Condition = "", const string & VarIndex = "", const string & VarFirst = "", const string & VarLast = "") { return dll::Populate_348(h, first, last, expression.data(), expression.size(), Step, Condition.data(), Condition.size(), VarIndex.data(), VarIndex.size(), VarFirst.data(), VarFirst.size(), VarLast.data(), VarLast.size()); }

         size_t ReferenceCount() { return dll::ReferenceCount_574(h); }

         /// <summary>.</summary>
         /// <remarks>.</remarks>
         void ReferenceIncr() { dll::ReferenceIncr_559(h); }

         /// <summary>Releases a uCalc string</summary>
         /// <remarks>This releases a uCalc string from memory along with all the objects dependent on it.</remarks>
         void Release() { dll::Release_349(h); h = Empty; }

         /// <summary>Removes occurrences of a pattern in the string</summary>
         /// <returns>uCalc String</returns>
         /// <param name="From_">pattern of occurences to remove from a string</param>
         /// <remarks>Removes occurrences of a pattern in the string.</remarks>
         String Remove(const string & From_) { return dll::Remove_350(h, From_.data(), From_.size()); }

         /// <summary>Replaces occurences of a pattern with something else</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="From_">patter of occurence to match</param>
         /// <param name="To_">replacement of occurence</param>
         /// <remarks>Replaces occurences of a pattern with something else.</remarks>
         String Replace(const string & From_, const string & To_) { return dll::Replace_351(h, From_.data(), From_.size(), To_.data(), To_.size()); }

         /// <summary>Returns the root string of the current string</summary>
         /// <returns>uCalc String object representing the root string</returns>
         /// <remarks>Returns the root string of the current string</remarks>
         String Root() { return dll::Root_352(h); }

         /// <summary>Skips</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Pattern">Pattern</param>
         /// <remarks>Skips</remarks>
         String Skip(const string & Pattern) { return dll::Skip_353(h, Pattern.data(), Pattern.size()); }

         /// <summary>Retruns a substring starting from the given occurence of a match</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Pattern">Pattern to match</param>
         /// <param name="Nth">Nth occurence</param>
         /// <remarks>Retruns the text starting from the given occurence of a match until the end.</remarks>
         String StartingFrom(const string & Pattern, size_t Nth = 1) { return dll::StartingFrom_354(h, Pattern.data(), Pattern.size(), Nth); }

         /// <summary>Returns a substring that starts with a given pattern and ends with another pattern</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="StartPattern">Start pattern</param>
         /// <param name="EndPattern">End pattern</param>
         /// <param name="StartNth">nth occurence for Start pattern</param>
         /// <param name="EndNth">nth occurence for End pattern</param>
         /// <remarks>Returns text that starts with a given pattern and ends with another pattern.</remarks>
         String StartingFromEndingWith(const string & StartPattern, const string & EndPattern, size_t StartNth = 1, size_t EndNth = 1) { return dll::StartingFromEndingWith_355(h, StartPattern.data(), StartPattern.size(), EndPattern.data(), EndPattern.size(), StartNth, EndNth); }

         /// <summary>Retrieves the string value of a uCalc String object</summary>
         /// <returns>String value of the uCalc String object</returns>
         /// <param name="Nth">Index of match</param>
         /// <param name="TokenCount">Number of tokens</param>
         /// <remarks>This retrieves the string value associated with a uCalc String object. If no argument is passed then the entire string
         /// is returned. Use Str(n) when you just want the string representation of a match. Use Match(n) when you want uCalc String Object
         /// that you can manipulate further.</remarks>
         string Str(size_t Nth = -1, size_t TokenCount = 1) {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::Str_357(h, ReturnStrSize, Nth, TokenCount);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Assigns a string value to the uCalc String object</summary>
         /// <param name="StringValue">New string value to assign to the uCalc string object</param>
         /// <remarks>This assigns a new string to the uCalc String object.</remarks>
         String Str(const string & StringValue) { return dll::Str_358(h, StringValue.data(), StringValue.size()); }

         /// <summary>Returns substring</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="StartPos">Starting position</param>
         /// <param name="Length">Length of substring</param>
         /// <remarks>
         /// <para>Returns a substring of the current string.</para>
         /// 
         /// <para>Note: the StartPos and Length parameters are in terms of tokens, not characters.</para>
         /// </remarks>
         String SubString(size_t StartPos, size_t Length = -1) { return dll::SubString_359(h, StartPos, Length); }

         /// <summary>Returns the length (number of characters) of the nth token i a string</summary>
         /// <returns>Integer</returns>
         /// <param name="StartIndex">Index of nth token</param>
         /// <param name="Count">Number of consecutive tokens to count</param>
         /// <remarks>
         /// <para>Returns the length (number of characters) of the nth token in a string.</para>
         /// 
         /// <para>Note: When Count &gt; 1, white space characters between tokens are part of the length.</para>
         /// </remarks>
         size_t TokenCharLen(size_t StartIndex, size_t Count = 1) { return dll::TokenCharLen_360(h, StartIndex, Count); }

         /// <summary>Returns the character position of the nth token</summary>
         /// <param name="Index">Index of nth match</param>
         /// <remarks>Returns the character position of the nth token</remarks>
         size_t TokenCharPos(size_t Index) { return dll::TokenCharPos_361(h, Index); }

         /// <summary>Returns the numbers of tokens in the string</summary>
         /// <returns>Integer</returns>
         /// <param name="Index">Index of nth token</param>
         /// <remarks>This returns the number of tokens that are found in the string.  If the Index argument is used, it returns the number
         /// of tokens in the nth match.</remarks>
         size_t TokenCount(size_t Index = -1) { return dll::TokenCount_362(h, Index); }

         /// <summary>Returns the index of the token that contains the indicated character</summary>
         /// <returns>Integer</returns>
         /// <param name="CharacterPosition">Character position</param>
         /// <remarks>Returns the index of the token that contains the indicated character.</remarks>
         size_t TokenIndexAtChar(size_t CharacterPosition) { return dll::TokenIndexAtChar_363(h, CharacterPosition); }

         /// <summary>Returns the nth token position</summary>
         /// <returns>Integer</returns>
         /// <param name="Nth">Index of nth match</param>
         /// <remarks>Returns the nth token position.</remarks>
         size_t TokenPos(size_t Nth = -1) { return dll::TokenPos_364(h, Nth); }

         /// <summary>Changes occurences of a pattern to lower case</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Pattern">Pattern</param>
         /// <param name="Nth">nth occrence of pattern</param>
         /// <remarks>Changes occurences of a pattern to lower case.</remarks>
         String ToLower(const string & Pattern = "", size_t Nth = -1) { return dll::ToLower_365(h, Pattern.data(), Pattern.size(), Nth); }

         /// <summary>Changes occurences of a pattern to upper case</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Pattern">Pattern</param>
         /// <param name="Nth">nth occrence of pattern</param>
         /// <remarks>Changes occurences of a pattern to upper case.</remarks>
         String ToUpper(const string & Pattern = "", size_t Nth = -1) { return dll::ToUpper_366(h, Pattern.data(), Pattern.size(), Nth); }

         /// <summary>Transforms a string</summary>
         /// <returns>Transformed string object</returns>
         /// <param name="tt">Transforms</param>
         /// <remarks>Transforms a string</remarks>
         String Transform(Transforms tt) { return dll::Transform_367(h, tt.h); }

         /// <summary>Returns the type of token the nth token is</summary>
         /// <returns>TokenType</returns>
         /// <param name="Index">Index of nth token</param>
         /// <remarks>Returns the type of token the nth token is.</remarks>
         TokenType TypeOfToken(size_t Index) { return dll::TypeOfToken_368(h, Index); }

         /// <summary>Returns the uCalc object this String belongs to</summary>
         /// <returns>uCalc object</returns>
         uCalcBase uCalc() { return dll::uCalc_539(h); }

         /// <summary>Returns a substring with all text up to and including the matched pattern</summary>
         /// <returns>uCalc String object</returns>
         /// <param name="Pattern">Pattern to match</param>
         /// <param name="Nth">nth occurence</param>
         /// <remarks>Returns all text up to and including the matched pattern.  By default it goes up to the first occurence of a match,
         /// but you may elect to have it go up to the nth occurence.</remarks>
         String UpTo(const string & Pattern, size_t Nth = 1) { return dll::UpTo_369(h, Pattern.data(), Pattern.size(), Nth); }

      }; // End of String class


      /// <summary>Class for collection of tokens</summary>
      struct Tokens {
         inline static Handle_Tokens Empty = (Handle_Tokens)dll::MemoryObjectAddress_(MemoryTable::Tokens, 0);
         Handle_Tokens h = Empty;

         Tokens(Handle_Tokens PassedHandle) : h(PassedHandle) { }
         uintptr_t Handle() { return (uintptr_t)h; }
         bool IsEmpty() { return h == Empty; }
         bool NotEmpty() { return h != Empty; }
         Tokens() { }

         /// <summary>Imports an individual token item</summary>
         /// <returns>Current token list</returns>
         /// <param name="TokenItem">Pre-existing token item</param>
         /// <remarks>Imports an individual token item into the token list.</remarks>
         Tokens Add(Item TokenItem) { return dll::Add_372(h, TokenItem.h); }

         /// <summary>Defines a new token</summary>
         /// <returns>Returns the newly created token item object</returns>
         /// <param name="TokenDef">Token definition</param>
         /// <param name="Type">The type of token</param>
         /// <param name="CloseBracket">String representing closing bracket, such as &quot;}&quot;, &quot;)&quot;, &quot;]&quot;, etc.</param>
         /// <param name="SubMatch">Submatch</param>
         /// <param name="Grammar">Type of RegEx grammar</param>
         /// <param name="InsertionPoint">Where to insert the token into the token list</param>
         /// <remarks>
         /// <para>uCalc comes with a default set of tokens.  These are general tokens for things like numbers, alphanumeric words, whitespace,
         /// quotes, parenthesis, etc.  However you may define your own tokens to the exact specifications that you need.</para>
         /// 
         /// <para>A token is defined using a Regular Expression, which is passed as the first argument.</para>
         /// 
         /// <para>A token type can be set to any of those listed under the TokenType enum (except for TokenType::None).</para>
         /// 
         /// <para>Bracket tokens (such as parenthesis, square brackets, or curly braces) are typically defined in pairs.  So if &quot;(&quot;,
         /// &quot;)&quot; are defined as a token pair, then CloseBracket would be &quot;)&quot;.</para>
         /// 
         /// <para>In cases where you want a submatch within the token, and not the whole token, use the SubMatch argument with a numerical
         /// value represent the numerical index used in the regular expression.  Useful for instance in matching quoted text without the
         /// quotes themselves.</para>
         /// </remarks>
         Item Add(const string & TokenDef, TokenType Type = TokenType::Generic, const string & CloseBracket = "", size_t SubMatch = 0, RegExGrammar Grammar = RegExGrammar::Default, size_t InsertionPoint = -1) { return dll::Add_373(h, TokenDef.data(), TokenDef.size(), Type, CloseBracket.data(), CloseBracket.size(), SubMatch, Grammar, InsertionPoint); }

         /// <summary>Imports a list of tokens from another token list</summary>
         /// <returns>Object containing list of tokens</returns>
         /// <param name="TokenCollection">Pre-existing token list</param>
         /// <remarks>Instead of defining a set of tokens each time, you may import a pre-existing token list.  This function imports from
         /// another token list object.</remarks>
         Tokens Add(Tokens TokenCollection) { return dll::Add_374(h, TokenCollection.h); }

         /// <summary>Clears the token list</summary>
         /// <remarks>Removes all tokens from the token list.  When a token list is cleared, it is typically prior to replacing it with
         /// a new token list.  A token list should generally not remain empty when doing a transform.  A token list should at minimum
         /// contain a catch-all pattern, which might capture one character.</remarks>
         void Clear() { dll::Clear_375(h); }

         /// <summary>Returns documentation/help/comment/tooltip type info associated with this object</summary>
         string Comment() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::Comment_540(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         Tokens Comment(const string & CommentStr) { return dll::Comment_547(h, CommentStr.data(), CommentStr.size()); }

         /// <summary>Sets a token that will cause a temporary switch to another token list with configurable insertion points</summary>
         /// <returns>Current token</returns>
         /// <param name="OtherTokenGroup">Other token list to switch to</param>
         /// <param name="StartToken">Existing token that will cause the context switch</param>
         /// <param name="InsertionPointA">// Where in the token list to insert the token</param>
         /// <param name="EndToken">Regex pattern that causes a return to the original token list</param>
         /// <param name="InsertionPointB">Where in the other token list (a copy of it) to insert the ending token</param>
         /// <remarks>
         /// <para>This causes a context switch to happen when a given token pattern is reached.  This is useful for instance if are parsing
         /// source code, and you want comments to be parsed based on a different set of tokens from the rest of the code.</para>
         /// 
         /// <para>This version of the function inserts a pre-existing token for the context switch, and lets you decide where in the token
         /// list to insert it.</para>
         /// </remarks>
         Tokens ContextSwitch(Tokens OtherTokenGroup, Item StartToken, int32_t InsertionPointA = -1, const string & EndToken = "", int32_t InsertionPointB = -1) { return dll::ContextSwitch_377(h, OtherTokenGroup.h, StartToken.h, InsertionPointA, EndToken.data(), EndToken.size(), InsertionPointB); }

         /// <summary>Sets a token pattern that will cause a temporary switch to another token list with configurable insertion points</summary>
         /// <returns>Current token list</returns>
         /// <param name="OtherTokenGroup">Other token list to switch to</param>
         /// <param name="StartToken">Regex pattern that triggers the context switch</param>
         /// <param name="InsertionPointA">Where in the token list to insert the new token</param>
         /// <param name="EndToken">Regex pattern that causes a return to the original token list</param>
         /// <param name="InsertionPointB">Where in the other token list (a copy of it) to insert the ending token</param>
         /// <remarks>
         /// <para>This causes a context switch to happen when a given token pattern is reached.  This is useful for instance if are parsing
         /// source code, and you want comments to be parsed based on a different set of tokens from the rest of the code.</para>
         /// 
         /// <para>This version of the function lets you decide where in the token list to insert the context switching token.</para>
         /// </remarks>
         Tokens ContextSwitch(Tokens OtherTokenGroup, const string & StartToken, int32_t InsertionPointA = -1, const string & EndToken = "", int32_t InsertionPointB = -1) { return dll::ContextSwitch_378(h, OtherTokenGroup.h, StartToken.data(), StartToken.size(), InsertionPointA, EndToken.data(), EndToken.size(), InsertionPointB); }

         /// <summary>Sets a token pattern that will cause a temporary switch to another token list</summary>
         /// <returns>Current token list</returns>
         /// <param name="OtherTokenGroup">Other token list to switch to</param>
         /// <param name="StartToken">Regex pattern that triggers the context switch</param>
         /// <param name="EndToken">Regex pattern that causes a return to the original token list</param>
         /// <remarks>This causes a context switch to happen when a given token pattern is reached.  This is useful for instance if are
         /// parsing source code, and you want comments to be parsed based on a different set of tokens from the rest of the code.</remarks>
         Tokens ContextSwitch(Tokens OtherTokenGroup, const string & StartToken, const string & EndToken) { return dll::ContextSwitch_379(h, OtherTokenGroup.h, StartToken.data(), StartToken.size(), EndToken.data(), EndToken.size()); }

         /// <summary>Returns the number of tokens in a token list</summary>
         /// <returns>Integer representing number of tokens</returns>
         /// <remarks>This returns the number of tokens in a token list.</remarks>
         size_t Count() { return dll::Count_380(h); }

         /// <summary>Defines a token</summary>
         /// <returns>Current object</returns>
         /// <param name="TokenDef">Token definition</param>
         /// <param name="Type">The type of token</param>
         /// <param name="SubMatch">Submatch</param>
         /// <param name="Grammar">Type of RegEx grammar</param>
         /// <remarks>Defines a token.  Similar to Add.  Add is more complete, but calls an internal version of Define().</remarks>
         Item Define(const string & TokenDef, TokenType Type = TokenType::Generic, size_t SubMatch = 0, RegExGrammar Grammar = RegExGrammar::Default) { return dll::Define_381(h, TokenDef.data(), TokenDef.size(), Type, SubMatch, Grammar); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Returns a token&apos;s position in the token list</summary>
         /// <returns>Integer value</returns>
         /// <param name="GivenToken">Token Item</param>
         /// <remarks>Returns a token&apos;s position in the token list.</remarks>
         int32_t IndexOf(Item GivenToken) { return dll::IndexOf_383(h, GivenToken.h); }

         /// <summary>Defines a new token</summary>
         /// <returns>Token item object</returns>
         /// <param name="InsertionPoint">Where to insert the token into the token list</param>
         /// <param name="TokenDef">Token definition</param>
         /// <param name="Type">The type of token</param>
         /// <param name="CloseBracket">String representing closing bracket, such as &quot;}&quot;, &quot;)&quot;, &quot;]&quot;, etc.</param>
         /// <param name="SubMatch">Submatch</param>
         /// <param name="Grammar">Type of RegEx grammar</param>
         /// <remarks>This is similar to Add(), however add appends the token to the end of the token list by default, whereas Insert()
         /// lets you choose where in the list to insert the token definition with the first parameter (TokenListIndex).  Tokens most recently
         /// defined have precedence over tokens defined earlier.  However, this allows a new token definition to have less precedence
         /// than the default. 0 inserts at the beginning, 1 inserts at the 2nd location etc.  Negative numbers insert starting at the
         /// end of the list, with -1 at the last position (default), -2 inserts at the next to last location, etc.</remarks>
         Item Insert(int32_t InsertionPoint, const string & TokenDef, TokenType Type = TokenType::Generic, const string & CloseBracket = "", size_t SubMatch = 0, RegExGrammar Grammar = RegExGrammar::Default) { return dll::Insert_384(h, InsertionPoint, TokenDef.data(), TokenDef.size(), Type, CloseBracket.data(), CloseBracket.size(), SubMatch, Grammar); }

         /// <summary>Determines if a token has the given property</summary>
         /// <returns>Returns True if the token matches the given property, or False otherwise.</returns>
         /// <param name="ItemProperty">The property you want to check for</param>
         /// <remarks>Determines if a token has the given property</remarks>
         bool IsProperty(ItemIs ItemProperty) { return dll::IsProperty_386(h, ItemProperty) != 0; }

         /// <summary>Sets a given property</summary>
         /// <returns>Current object</returns>
         /// <param name="ItemProperty">The property you want to set</param>
         /// <param name="TrueOrFalse">Value to set the property to</param>
         /// <remarks>Sets a given property</remarks>
         Tokens IsProperty(ItemIs ItemProperty, bool TrueOrFalse) { return dll::IsProperty_387(h, ItemProperty, TrueOrFalse); }

         size_t MemoryIndex() { return dll::MemoryIndex_611(h); }

         /// <summary>Releases a token list from memory.  Transform rules are defined with FromTo.</summary>
         /// <remarks>This releases a transform rule from memory.</remarks>
         void Release() { dll::Release_388(h); h = Empty; }

         /// <summary>Removes a token from the token list</summary>
         /// <param name="TokenItem">Token to be removed from the token list</param>
         /// <remarks>Removes the specified token from the token list.</remarks>
         void Remove(Item TokenItem) { dll::Remove_389(h, TokenItem.h); }

         /// <summary>Returns the nth token from the token list</summary>
         /// <returns>Token Item</returns>
         /// <param name="Index">nth Index</param>
         /// <remarks>Returns the nth token from the token list.</remarks>
         Item Token(size_t Index) { return dll::Token_390(h, Index); }

         uCalcBase uCalc() { return dll::uCalc_544(h); }

      }; // End of Tokens class


      /// <summary>uCalc transform class</summary>
      struct Transforms {
         inline static Handle_Transforms Empty = (Handle_Transforms)dll::MemoryObjectAddress_(MemoryTable::Transforms, 0);
         Handle_Transforms h = Empty;

         Transforms(Handle_Transforms PassedHandle) : h(PassedHandle) { }
         uintptr_t Handle() { return (uintptr_t)h; }
         bool IsEmpty() { return h == Empty; }
         bool NotEmpty() { return h != Empty; }
         Transforms(uCalcBase uc, uCalcBase::Tokens TokensToInherit = uCalcBase::Tokens()) : h(uc.NewTransformH(TokensToInherit)) { }
         Transforms(uCalcBase::Tokens TokensToInherit = uCalcBase::Tokens()) : Transforms(Default(), TokensToInherit) { }

         operator string() { return Str(); }
         Transforms & operator=(const string & s) { Str(s); return *this; }
         //Transforms & operator=(const char * s) { Str(s); return *this; }

         /// <summary>Makes an identical but separate copy of a Transforms object</summary>
         /// <returns>New transforms object</returns>
         Transforms Clone() { return dll::Clone_504(h); }

         /// <summary>Returns documentation/help/comment/tooltip type info associated with this object</summary>
         string Comment() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::Comment_542(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         Transforms Comment(const string & CommentStr) { return dll::Comment_548(h, CommentStr.data(), CommentStr.size()); }

         /// <summary>Filters the text</summary>
         /// <returns>Current object</returns>
         /// <remarks>This performs a filter operation on text.  This is similar to a Transform() operation, however, the result is text
         /// consisisting of just the matches.  The operation is performed on text that was previously passed to Str().</remarks>
         Transforms Filter() { return dll::Filter_393(h); }

         /// <summary>Performs a Filter operation on the given text</summary>
         /// <returns>Current transform</returns>
         /// <param name="expression">Expression to be transformed</param>
         /// <remarks>This performs a filter operation on text.  This is similar to a Transform() operation, however, the result is text
         /// consisisting of just the matches.  The operation is performed on the string passed to this Filter() function.</remarks>
         Transforms Filter(const string & expression) { return dll::Filter_394(h, expression.data(), expression.size()); }

         /// <summary>Performs a text search operation</summary>
         /// <returns>The current Transforms object</returns>
         /// <param name="Str">Text to search through</param>
         /// <remarks>
         /// <para>Once all the rules and parameters are set, you can call this function in order to perform the text search.</para>
         /// 
         /// <para>You may either set the string to Str() prior to calling Find(), or you can pass a string argument directly to this function.</para>
         /// </remarks>
         Transforms Find(const string & Str = "") { return dll::Find_395(h, Str.data(), Str.size()); }

         /// <summary>Defines a transform rule</summary>
         /// <returns>Object for rule being defined</returns>
         /// <param name="FromStr">Pattern that describes the text to match</param>
         /// <param name="ToStr">New text that the match should be changed to</param>
         /// <remarks>This defines something like a search/replace operation.  Whenever a piece of text matches the pattern described in
         /// FromStr, it is replaced with the text found in ToStr.</remarks>
         Rule FromTo(const string & FromStr, const string & ToStr = "") { return dll::FromTo_396(h, FromStr.data(), FromStr.size(), ToStr.data(), ToStr.size()); }

         /// <summary>Returns the handle of an object</summary>
         /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
         /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

         /// <summary>Gets the index base used in various functions</summary>
         /// <remarks>By default, uCalc String Library functions use a 0-based index.  So the first element is 0 (not 1), followed by 2,
         /// 3, 4 etc. Use this to check to see which base is being used.</remarks>
         size_t IndexBase() { return dll::IndexBase_399(h); }

         /// <summary>Sets the index base used in various functions</summary>
         /// <param name="BaseValue">value representing index of 1st element</param>
         /// <remarks>By default, uCalc String Library functions use a 0-based index.  So the first element is 0 (not 1), followed by 2,
         /// 3, 4 etc. Use this if you prefer to start by 1 or another non-zero value.  This value is passed on to string objects derived
         /// from a parent string. Each string may have a different index base.</remarks>
         Transforms IndexBase(size_t BaseValue) { return dll::IndexBase_400(h, BaseValue); }

         /// <summary>Determines if a uCalc Item has the given properties</summary>
         /// <returns>Returns True if the uCalc Item matches the given property, or False otherwise.</returns>
         /// <param name="ItemProperty">The property you want to check for</param>
         /// <remarks>This helps you determine if an item has the given property</remarks>
         bool IsProperty(ItemIs ItemProperty) { return dll::IsProperty_402(h, ItemProperty) != 0; }

         /// <summary>Sets a given property</summary>
         /// <returns>Returns the same Transforms object</returns>
         /// <param name="ItemProperty">The property you want to set</param>
         /// <param name="TrueOrFalse">Value to set the property to</param>
         /// <remarks>This sets a given property</remarks>
         Transforms IsProperty(ItemIs ItemProperty, bool TrueOrFalse) { return dll::IsProperty_403(h, ItemProperty, TrueOrFalse); }

         /// <summary>Returns the Matches object for the current Transform</summary>
         /// <returns>Matches object</returns>
         /// <param name="Options">Option for subset of matches to return</param>
         /// <remarks>This returns the Matches object for the current Transform.  The Matches object contains the list of matches, the
         /// location and lengh of each match, etc.</remarks>
         Matches Matches(MatchesOption Options = MatchesOption::FocusableOnly) { return dll::Matches_404(h, Options); }

         size_t MemoryIndex() { return dll::MemoryIndex_612(h); }

         /// <summary>Creates a new Pass object</summary>
         /// <returns>Transform object</returns>
         /// <param name="Share">Share</param>
         /// <remarks>It is possible to create a transform with multiple passes.  Each pass can have a different set of rules.  The transform
         /// of each pass is executed in sequential order.  Passes can later be referenced by index.  Calling NewPass the first time returns
         /// the same as Pass(0), the next call to NewPass refers to Pass(1), etc.</remarks>
         Transforms NewPass(bool Share = false) { return dll::NewPass_405(h, Share); }

         /// <summary>Creates a new rule</summary>
         /// <returns>Returns the new rule object</returns>
         /// <remarks>This creates a new rule.  A rule typically consists of a string to find and/or modify, along with properties associted
         /// with that rule.</remarks>
         Rule NewRule() { return dll::NewRule_406(h); }

         /// <summary>Returns a Pass object</summary>
         /// <returns>Transform object</returns>
         /// <param name="Index">Index of pass to select</param>
         /// <param name="Share">Share</param>
         /// <remarks>
         /// <para>It is possible to create a transform with multiple passes.  Each pass can have a different set of rules.  Each pass
         /// is transformed in sequential order, starting with Pass(0).  A pass consists of a transform.</para>
         /// 
         /// <para>This function returns a pass object, which typically was defined with NewPass().</para>
         /// </remarks>
         Transforms Pass(size_t Index, bool Share = false) { return dll::Pass_407(h, Index, Share); }

         /// <summary>Returns the number of passes</summary>
         /// <returns>Integer</returns>
         /// <remarks>Returns the number of passes</remarks>
         size_t PassCount() { return dll::PassCount_408(h); }

         /// <summary>Restricts the range of text where the search or search/replace operations will be performed</summary>
         /// <returns>The current transform</returns>
         /// <param name="Start">Starting position</param>
         /// <param name="Size">Size of string to operate on</param>
         /// <remarks>Generally transform operations are performed on the entire text that is passed to the transform.  Use this if you
         /// only want to operate on a subset of the text, starting at Start, with size Size.</remarks>
         Transforms Range(size_t Start, size_t Size) { return dll::Range_409(h, Start, Size); }

         size_t ReferenceCount() { return dll::ReferenceCount_572(h); }

         /// <summary>.</summary>
         /// <remarks>.</remarks>
         void ReferenceIncr() { dll::ReferenceIncr_558(h); }

         /// <summary>Releases a transform</summary>
         /// <remarks>This releases a transform from memory along with all the objects defined under it.</remarks>
         void Release() { dll::Release_410(h); h = Empty; }

         /// <summary>Resets the transform object</summary>
         /// <returns>Current transform object</returns>
         /// <param name="ResetOption">Reset option</param>
         /// <remarks>This clears the input string, transform rules, token definitions, and match list.  It also resets properties to default
         /// settings.</remarks>
         Transforms Reset(TransformResetOption ResetOption = TransformResetOption::All) { return dll::Reset_411(h, ResetOption); }

         /// <summary>Returns the defaults settings for rules</summary>
         /// <returns>Default rule object</returns>
         /// <remarks>Each FromTo or Find rule has many configurable properties that can be set.  Instead of excplicitely setting all of
         /// these many properties with each rule definition, you may define default settings that will automatically be assigned to new
         /// rules.  Any of these default settings can be overridden by explicitely setting the desired property for a given rule.</remarks>
         Rule RulesDefault() { return dll::RulesDefault_412(h); }

         /// <summary>Returns the overall settings for a transform</summary>
         /// <returns>Overall rule object</returns>
         /// <remarks>This returns an object containing the properties for a transform or search object.  This object is a Rules object,
         /// and conatains the same properties listed for individual rules, but works in a slightly different context.  For instance, if
         /// you set the StopAfter property of FromTo(&quot;a&quot;, &quot;b&quot;) to 5, then it will only change the first five occurences
         /// of &quot;a&quot; to &quot;b&quot;, whereas if you set the RulesOverall StopAfter property to 5, and you have rules FromTo(&quot;a&quot;,
         /// &quot;b&quot;) and FromTo(&quot;x&quot;, &quot;y&quot;) then it will stop changing any occurence of either rule, after the
         /// combined total of changes reaches 5; &quot;a&quot; might change to &quot;b&quot; 2 times and &quot;x&quot; to &quot;y&quot;
         /// 3 times, or whichever combination does not total more than 5.  You might set QuoteSensitive to &quot;true&quot; overall, while
         /// you may set it to false within a given rule.  In certain combinations, overall rules may clash with individual rules.  Although
         /// all combinations are allowed, not all combinations make sense.  When setting the case sensitivity property of individual rules
         /// differently from the overall property, keep in mind that if the overall case sensitivity is false, then all symbols are stored
         /// in the symbol table in all upper (or lower) case.</remarks>
         Rule RulesOverall() { return dll::RulesOverall_413(h); }

         /// <summary>Returns string result of transformed text</summary>
         /// <returns>String result of transformed text</returns>
         /// <remarks>After performing a text transformation, you may obtain the transformed result with this function.</remarks>
         string Str() {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::Str_415(h, ReturnStrSize);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Sets the text input that is to be transformed or searched</summary>
         /// <returns>Current transform object</returns>
         /// <param name="InputStr">Text that mast be parsed and modified or searched</param>
         /// <remarks>The text that you want to transform or search should be passed to this function.</remarks>
         Transforms Str(const string & InputStr) { return dll::Str_416(h, InputStr.data(), InputStr.size()); }

         /// <summary>Returns actual character base pointer of string</summary>
         /// <returns>Pointer to byte array</returns>
         /// <remarks>Note: Should never be used alone.  Always use in pair with StrLength</remarks>
         const char * StrData() { return dll::StrData_417(h); }

         /// <summary>Returns the length of the text string being operated on</summary>
         /// <returns>Integer</returns>
         /// <remarks>This returns the length of the text string being operated on</remarks>
         size_t StrLength() { return dll::StrLength_418(h); }

         /// <summary>Specifically intended for use with handling Unicode utf8 vs utf16 difference</summary>
         /// <returns>String</returns>
         /// <param name="Start">Starting character position</param>
         /// <param name="Count">Number of characters</param>
         /// <remarks>This returns a SubStr of the text</remarks>
         string SubStr(size_t Start, size_t Count = -1) {
            size_t ReturnStrSize;
            const char * ReturnStr = dll::SubStr_419(h, ReturnStrSize, Start, Count);
            return string(ReturnStr, ReturnStrSize);
         }

         /// <summary>Returns the Tokens object for the current transform</summary>
         /// <returns>Token list object</returns>
         /// <remarks>Returns the Tokens object, which consists of a list of tokens for the current transform. You can use this to add,
         /// modify, delete, or reset the tokens.</remarks>
         Tokens Tokens() { return dll::Tokens_420(h); }

         /// <summary>Performs the transformation on some text</summary>
         /// <returns>The current transform object</returns>
         /// <remarks>
         /// <para>Once all the rules and parameters are set, you can call this function in order to perform the text transformation.</para>
         /// 
         /// <para>Prior to calling Transform, you must set the text to be transformed, with Str</para>
         /// </remarks>
         Transforms Transform() { return dll::Transform_422(h); }

         /// <summary>Transforms an expression in one quick step</summary>
         /// <returns>Current object</returns>
         /// <param name="expression">Expression to be transformed</param>
         /// <remarks>When you would like to set text to be transformed, and transform the text, and return a result (with .Str()), all
         /// in one step, you can use this function, instead of calling Str, then Transform, then Str.</remarks>
         Transforms Transform(const string & expression) { return dll::Transform_423(h, expression.data(), expression.size()); }

         /// <summary>Returns each step leading to an expression expansion</summary>
         /// <returns>String list containg steps leading up to transform result</returns>
         /// <param name="expression">Expression to be transformed</param>
         /// <remarks>Transforms an expression and returns intermediate steps</remarks>
         String TransformWithSteps(const string & expression) { return dll::TransformWithSteps_424(h, expression.data(), expression.size()); }

         uCalcBase uCalc() { return dll::uCalc_546(h); }

         /// <summary>Sets the uCalc object that performs calculations related to the transform</summary>
         /// <returns>uCalc object associated with current transform</returns>
         /// <param name="uh">Handle of a uCalc object</param>
         /// <remarks>If your transform inovlves calcuations, then they will be performed using the uCalc object the transform was defined
         /// in.  However, you can change the uCalc object with this function. Each uCalc object can have its own sets of variables, functions,
         /// and other definitions. For instance when {@Eval: ...} is used in the pattern or ~Eval() in the text.</remarks>
         Transforms uCalc(uCalcBase uh) { return dll::uCalc_425(h, uh.h); }

         /// <summary>Indicates whether text was modified in the most recent Transform operation</summary>
         /// <returns>True if text was modified in the most recent transformation operation or false otherwise</returns>
         /// <remarks>If text was modified after a call to Transform, then this function returns True.  When transforming large amounts
         /// of text, transferring strings back and forth with Str() can be expensive.  Checking this property allows you to skip the unnecessary
         /// transfer of text when the text has not been modified.</remarks>
         bool WasModified() { return dll::WasModified_426(h) != 0; }

      }; // End of Transforms class


         inline static Handle_uCalc Empty = (Handle_uCalc)dll::MemoryObjectAddress_(MemoryTable::uCalcBase, 0);
         Handle_uCalc h = Empty;

         uCalcBase(Handle_uCalc PassedHandle) : h(PassedHandle) { }
         uintptr_t Handle() { return (uintptr_t)h; }
         bool IsEmpty() { return h == Empty; }
         bool NotEmpty() { return h != Empty; }

      /// <summary>Adds an error handler</summary>
      /// <returns>The error handler item, which can later be removed using this return object</returns>
      /// <param name="ErrorHandlerAddress">Address of the callback function containing the error handler code</param>
      /// <param name="Position">Position in which to insert the error handler</param>
      /// <remarks>
      /// <para>Use this to set a callback function that is called whenever there is an error.  Your callback can do things like alert
      /// the user of the error, abort the operation, correct the error and resume, keep a log of errors, etc.</para>
      /// 
      /// <para>By default, if there are multiple error handlers, they are called starting with the most recently defined one first,
      /// ending with the first one to be defined last.  In order to define a handler in a different order, specify the Position value
      /// of where it should be inserted; 0 so that it is called first, -1 so it is called last, or a positive integer indicating where
      /// it should be inserted.  If the Position value is out of bounds, it is adjusted into the maximum acceptable value.</para>
      /// 
      /// <para>If you set ErrorResponse to ErrorHandlerResponse::Resume, subsequent error handler callbacks will be bypassed and uCalc
      /// will resume operations as though no error had taken place.  Note that the condition that caused the error must be corrected
      /// before resuming, otherwise uCalc may get stuck in an infinite loop.</para>
      /// 
      /// <para>Some functions, such as ErrorExpression, ErrorLocation, and ErrorSymbol produce a meaningful result only during the
      /// parsing stage.  They do not cause other problems beyond meaningless data during the evaluation stage.</para>
      /// </remarks>
      Item AddErrorHandler(ErrorHandlerCallback ErrorHandlerAddress, size_t Position = 0) { return dll::AddErrorHandler_14(h, ErrorHandlerAddress, Position); }

      /// <summary>Adds an alias to an existing symbol</summary>
      /// <returns>Newly created alias Item</returns>
      /// <param name="NewSymbol">New symbol that will be set as equivalent to an existing symbol</param>
      /// <param name="ExistingItem">Pre-existing item</param>
      /// <remarks>
      /// <para>This function creates a new symbol that is pegged to an existing symbol.  Whenever this new symbol is encountered by
      /// the parser, it will be the same as if it had encountered the pre-existing symbol this new symbol is an alias of.  This new
      /// symbol shares the same handle as the pre-existing symbol within the symbol table.  The new symbol actually has its own handle,
      /// but that handle can only be used to delete the item. The pre-existing handle is used for all other purposes.</para>
      /// 
      /// <para>This version of Alias uses an Item object as second argument.  It is a little more efficient than the one that uses
      /// a string (the string version calls the internal version of this function).</para>
      /// </remarks>
      Item Alias(const string & NewSymbol, Item ExistingItem) { return dll::Alias_16(h, NewSymbol.data(), NewSymbol.size(), ExistingItem.h); }

      /// <summary>Adds an alias to an existing symbol</summary>
      /// <returns>Newly created alias Item</returns>
      /// <param name="NewSymbol">New symbol that will be set as equivalent to an existing symbol</param>
      /// <param name="ExistingSymbol">Pre-existing symbol</param>
      /// <remarks>
      /// <para>This function creates a new symbol that is pegged to an existing symbol.  Whenever this new symbol is encountered by
      /// the parser, it will be the same as if it had encountered the pre-existing symbol this new symbol is an alias of.  This new
      /// symbol shares the same handle as the pre-existing symbol within the symbol table.  The new symbol actually has its own handle,
      /// but that handle can only be used to delete the item. The pre-existing handle is used for all other purposes.</para>
      /// 
      /// <para>For simple situations where you just want two symbols to be considered equivalent by the parser, Alias is prefered as
      /// a more efficient option over ExpressionTransforms().FromTo(OldSymbol, NewSymbol), which would modify the expression in order
      /// for the symbols to be equal.</para>
      /// </remarks>
      Item Alias(const string & NewSymbol, const string & ExistingSymbol) { return dll::Alias_17(h, NewSymbol.data(), NewSymbol.size(), ExistingSymbol.data(), ExistingSymbol.size()); }

      /// <summary>Adds an alias name to an existing command name for Define()</summary>
      /// <param name="NewCommandName">New command name that will be set as equivalent to an existing command name</param>
      /// <param name="ExistingCommandName">Pre-existing command name</param>
      /// <remarks>This creates an alternative command name for use with the Define() function.</remarks>
      void AliasForDefine(const string & NewCommandName, const string & ExistingCommandName) { dll::AliasForDefine_18(h, NewCommandName.data(), NewCommandName.size(), ExistingCommandName.data(), ExistingCommandName.size()); }

      /// <summary>Makes an identical but separate copy of a uCalc object</summary>
      /// <returns>uCalc object</returns>
      uCalcBase Clone() { return dll::Clone_555(h); }

      /// <summary>Returns a data type object</summary>
      /// <returns>Data type object</returns>
      /// <param name="Index">Index associated with the data type</param>
      /// <remarks>Each instance of uCalc has its own separate handle &amp; data type object for any given data type.  For instance,
      /// Item(&quot;Int64&quot;) in one instance of uCalc is not the same object as Item(&quot;Int64&quot;) from another uCalc instance.
      ///  This allows the same data type to have a different name and different formatting in different instances.  However, they share
      /// the same data type index across instances.  Given the index of a data type, you can directly obtain the proper data type object
      /// for a particular uCalc instance, faster than obtaining it by name as text with Item().</remarks>
      DataType DataTypeOf(BuiltInType Index) { return dll::DataTypeOf_22(h, Index); }

      /// <summary>Returns the data type of the expression that was passed</summary>
      /// <returns>Data type object</returns>
      /// <param name="expression">Expression whose data type will be returned</param>
      /// <remarks>Returns the data type of the expression that was passed</remarks>
      DataType DataTypeOf(const string & expression) { return dll::DataTypeOf_23(h, expression.data(), expression.size()); }

      /// <summary>Returns the default uCalc instance</summary>
      /// <returns>uCalc object</returns>
      /// <remarks>Returns the default uCalc instance.  New uCalc instances inherit from this by default.  Expression, String, and Transforms
      /// objects that are created without specifying a uCalc space also use this default.</remarks>
      static uCalcBase Default() { return dll::Default_25(); }

      /// <summary>Clears the stack of default uCalc objects</summary>
      static void DefaultClear() { dll::DefaultClear_569(); }

      /// <summary>Returns the number of objects on the uCalc default stack</summary>
      static size_t DefaultCount() { return dll::DefaultCount_567(); }

      /// <summary>Default data type</summary>
      /// <returns>Default data type</returns>
      /// <remarks>This returns the default data type.  The default type is Double unless explicitly changed to something else.</remarks>
      DataType DefaultDataType() { return dll::DefaultDataType_28(h); }

      /// <summary>Sets a new default data type</summary>
      /// <param name="DataTypeIndex">Enum of type you want to set as default</param>
      /// <remarks>
      /// <para>This sets a new default data type.  This new default will be used instead of Double for all definitions where a type
      /// is relevant, when an explicit type is not specified.</para>
      /// 
      /// <para>The default is set using a BuiltInType enum.</para>
      /// </remarks>
      void DefaultDataType(BuiltInType DataTypeIndex) { dll::DefaultDataType_29(h, DataTypeIndex); }

      /// <summary>Sets a new default data type</summary>
      /// <param name="dType">The type you want to set as default</param>
      /// <remarks>
      /// <para>This sets a new default data type.  This new default will be used instead of Double for all definitions where a type
      /// is relevant, when an explicit type is not specified.</para>
      /// 
      /// <para>The default is set using a data type object.</para>
      /// </remarks>
      void DefaultDataType(DataType dType) { dll::DefaultDataType_30(h, dType.h); }

      /// <summary>Sets a new default data type</summary>
      /// <param name="dTypeStr">Name of existing data type</param>
      /// <remarks>
      /// <para>This sets a new default data type.  This new default will be used instead of Double for all definitions where a type
      /// is relevant, when an explicit type is not specified.</para>
      /// 
      /// <para>The default is set using the name of a data type.</para>
      /// </remarks>
      void DefaultDataType(const string & dTypeStr) { dll::DefaultDataType_31(h, dTypeStr.data(), dTypeStr.size()); }

      /// <summary>Returns default settings</summary>
      /// <returns>Default Item</returns>
      /// <remarks>The Item returned contains the default settings for the current uCalc instance.  To define new settings, use Define(&quot;Default
      /// ~~ ...&quot;).</remarks>
      Item DefaultItem() { return dll::DefaultItem_32(h); }

      /// <summary>Removes a uCalc object from the top of the uCalc default stack</summary>
      /// <remarks>+++ This should not be necessary</remarks>
      static void DefaultPop() { dll::DefaultPop_568(); }

      /// <summary>Sets a uCalc instance as the default</summary>
      /// <param name="uCalcObj">uCalc object to set as default</param>
      /// <remarks>
      /// <para>Sets the new uCalc default object that future uCalc objects will inherit from by default when none is explicitly specified.</para>
      /// 
      /// <para>Automatically popped when it goes out of scope
      /// +++ May not be needed; use IsDefault(true) instead</para>
      /// 
      /// <para></para>
      /// </remarks>
      static void DefaultPush(uCalcBase uCalcObj) { dll::DefaultPush_26(uCalcObj.h); }

      /// <summary>+++ Find better name</summary>
      /// <returns>Newly constructed uCalc object that will serve as default</returns>
      /// <param name="InheritsFrom">Other uCalc object to inherit from</param>
      static uCalcBase DefaultPushNew(uCalcBase InheritsFrom = uCalcBase::Empty) { return dll::DefaultPushNew_580(InheritsFrom.h); }

      /// <summary>Defines a uCalc.Item object (a constunction, function, operator, variable, etc)</summary>
      /// <returns>The defined item object</returns>
      /// <param name="expression">Text containing item to be defined.</param>
      /// <param name="Address">Address a function or variable is attached to</param>
      /// <param name="dType">Data type associated with the item</param>
      /// <remarks>
      /// <para>Functions, operators, variables, constants, tokens, syntax constructs, etc., can be defined with this method.  Although
      /// there are specialized methods that do the same things, such as DefineFunction, DefineOperator, DefineVariable, and DefineConstant,
      /// this method may in same cases offer more flexibility.</para>
      /// 
      /// <para>The expression must start with a keyword such as Function, Operator, Variable, Token, Syntax, etc., followed by a colon,
      /// before the expression.</para>
      /// 
      /// <para>The expression must be preceded by a keyword and colon, such as &quot;Variable:&quot;.  For a full list of keywords,
      /// see Definition Keywords.</para>
      /// 
      /// <para>Sections are separated by ~~ (two tilde signs).</para>
      /// </remarks>
Item Define(const string & expression, void * Address = 0, DataType dType = DataType::Empty) { return dll::Define_33(h, expression.data(), expression.size(), Address, dType.h); }
Handle_Item DefineH(const string & expression, void * Address = 0, DataType dType = DataType::Empty) { return dll::Define_33(h, expression.data(), expression.size(), Address, dType.h); }

      /// <summary>Defines a constant</summary>
      /// <returns>The defined item object</returns>
      /// <param name="Definition">Text containing constant to be defined</param>
      /// <param name="VariableAddress">Address the constant is attached to</param>
      /// <remarks>
      /// <para>A constant definition is similar to that of a variable defined with DefineVariable, except that the end-user is not
      /// allowed to modify the value of a constant. You are, however, allowed to perform lower-level operations such as renaming a
      /// constant, or deleting it and re-defining a new constant or variable with the same name but different value.  If you attach
      /// the constant to the address of a variable in your host program, the host program may change the value.  However, when the
      /// end-user attempts to assign a value to the constant, an error is raised.</para>
      /// 
      /// <para>DefineConstant(&quot;x&quot;) is the same as: Define(&quot;Lock ~~ Variable: x&quot;)</para>
      /// </remarks>
      Item DefineConstant(const string & Definition, void * VariableAddress = 0, DataType dType = DataType::Empty) { return dll::DefineConstant_34(h, Definition.data(), Definition.size(), VariableAddress, dType.h); }

      /// <summary>Defines a function</summary>
      /// <returns>The defined item object</returns>
      /// <param name="Definition">Text containing function to be defined</param>
      /// <param name="FunctionAddr">Address of callback function this function is attached to</param>
      /// <remarks>
      /// <para>A function definition can either be self-contained, or it may be attached to the address of a callback function in your
      /// source code.</para>
      /// 
      /// <para>DefineFunction(&quot;f(x) = ...&quot;) is the same as: Define(&quot;Function: ...&quot;)</para>
      /// </remarks>
      Item DefineFunction(const string & Definition, OpCallback FunctionAddr = 0, DataType dType = DataType::Empty) { return dll::DefineFunction_35(h, Definition.data(), Definition.size(), FunctionAddr, dType.h); }

      /// <summary>Defines an operator</summary>
      /// <returns>The defined item object</returns>
      /// <param name="Definition">Text containing operator to be defined</param>
      /// <param name="PrecedenceLevel">Precedence level of the operator</param>
      /// <param name="Direction">Operator associativity; arithmetic operators are typically grouped left-to-right; special operators, such as the equal sign, =, assignment operator are right-to-left</param>
      /// <param name="FunctionAddr">Address of callback function this operator is attached to</param>
      /// <remarks>
      /// <para>Many details for defining an operator are similar to those of defining a function.  In addition, all operators must
      /// be defined with a precedence level.  You may use the Precedence function to obtain precedence levels of existing operators,
      /// and select a precedence level relative to those operators.  By default operators are grouped left to right.  But you may also
      /// set it to right to left.  This means that when adjacent operators have the same precedence level, the operands on the right
      /// will be evaluated first.</para>
      /// 
      /// <para>DefineOperator(&quot;...&quot;) is the same as: Define(&quot;Operator: ...&quot;)</para>
      /// </remarks>
      Item DefineOperator(const string & Definition, size_t PrecedenceLevel, Grouping Direction = Grouping::LeftToRight, OpCallback FunctionAddr = 0, DataType dType = DataType::Empty) { return dll::DefineOperator_36(h, Definition.data(), Definition.size(), PrecedenceLevel, Direction, FunctionAddr, dType.h); }

      /// <summary>Defines a variable</summary>
      /// <returns>The defined item object</returns>
      /// <param name="Definition">Text containing variable to be defined</param>
      /// <param name="VariableAddress">Address of variable in host program this uCalc variable is attached to</param>
      /// <remarks>
      /// <para>A variable typically contains a variable name, optionally followed by a data type, optionally followed by an equal sign
      /// and an expression whose result is the starting value for the variable.</para>
      /// 
      /// <para>DefineVariable(&quot;...&quot;) is the same as: Define(&quot;Variable: x&quot;)</para>
      /// </remarks>
      Item DefineVariable(const string & Definition, void * VariableAddress = 0, DataType dType = DataType::Empty) { return dll::DefineVariable_37(h, Definition.data(), Definition.size(), VariableAddress, dType.h); }

      /// <summary>Returns a snapshot of uCalc&apos;s memory footprint</summary>
      /// <returns>Returns either number of bytes or number of elements for the selection type</returns>
      /// <param name="uCalcIndex">Memory index of owning uCalc instance</param>
      /// <param name="Target">Class or type whose memory footprint you want to diagnose</param>
      /// <param name="Count">Selects whether to count in terms of bytes or number of elements</param>
      static size_t DiagMem(size_t uCalcIndex, BuiltInType Target, DiagMemCount Count, DiagMemOption Option = DiagMemOption::Default) { return dll::DiagMem_597(uCalcIndex, Target, Count, Option); }

      /// <summary>Returns the expression associated with an error that was raised</summary>
      /// <returns>Text string</returns>
      string ErrorExpression() {
         size_t ReturnStrSize;
         const char * ReturnStr = dll::ErrorExpression_39(h, ReturnStrSize);
         return string(ReturnStr, ReturnStrSize);
      }

      /// <summary>Sets a new expression after an error is raised</summary>
      /// <param name="expression">Error expression</param>
      void ErrorExpression(const string & expression) { dll::ErrorExpression_40(h, expression.data(), expression.size()); }

      /// <summary>Returns the location of an error that was raised</summary>
      size_t ErrorLocation() { return dll::ErrorLocation_41(h); }

      /// <summary>Returns a message associated with the error that was raised</summary>
      /// <returns>Text string</returns>
      /// <param name="Index">Index of error message to return</param>
      /// <remarks>
      /// <para>If an error was raised, this returns a message describing the recent error.  If the Index argument is specified, then
      /// the error message associated with this Index is instead returned.  A value must be chosen from ErrorMsg.</para>
      /// 
      /// <para>Note: Errors are cleared whenever a new expression is parsed or a new item is defined.</para>
      /// </remarks>
      string ErrorMessage(ErrorMsg Index = ErrorMsg::None) {
         size_t ReturnStrSize;
         const char * ReturnStr = dll::ErrorMessage_42(h, ReturnStrSize, Index);
         return string(ReturnStr, ReturnStrSize);
      }

      /// <summary>Returns the error number of the error that was raised</summary>
      /// <returns>ErrorMsg enum</returns>
      /// <remarks>
      /// <para>Returns the error enum of the most recently raised error.</para>
      /// 
      /// <para>Note: Errors are cleared whenever a new expression is parsed or a new item is defined.</para>
      /// </remarks>
      ErrorMsg ErrorNumber() { return dll::ErrorNumber_43(h); }

      /// <summary>Sets the error response for an error handler</summary>
      /// <param name="Response">Indicates what should happen now that an error is raised</param>
      /// <remarks>
      /// <para>When an error is raised, you have the option of aborting the rest of the operation (this is the default action), or
      /// you may correct the problem, and then resume the operation.  Or you may perform an action, and then raise the error again
      /// for a subsequent error handler to take care of.</para>
      /// 
      /// <para>This method can only be called from within a callback of an error handler routine defined with AddErrorHandler. The
      /// options to use can be found at ErrorHandlerResponse.</para>
      /// </remarks>
      void ErrorResponse(ErrorHandlerResponse Response) { dll::ErrorResponse_44(h, Response); }

      /// <summary>Returns the symbol associated with the error that is raised</summary>
      /// <returns>text string</returns>
      string ErrorSymbol() {
         size_t ReturnStrSize;
         const char * ReturnStr = dll::ErrorSymbol_45(h, ReturnStrSize);
         return string(ReturnStr, ReturnStrSize);
      }

      /// <summary>Parses and evaluates an expression in one step</summary>
      /// <param name="expression">Expression to be evaluated</param>
      /// <remarks>This takes a math expression and returns a numeric result.  This is a convenient one-step alternative to ExprObj
      /// = Parse(Expressions); Result = ExprObj.Evaluate();</remarks>
      double Eval(const string & expression) { return dll::Eval_46(h, expression.data(), expression.size()); }

      /// <summary>Parses and evaluates an expression of any data type in one step, returning the result as a string</summary>
      /// <returns>String containing result</returns>
      /// <param name="expression">Expression to be evaluated</param>
      /// <remarks>This is similar to Eval, however, it can return non-numeric results, such as strings.  Also, the expression can be
      /// of any type.  The result is converted to a string and then returned.</remarks>
      string EvalStr(const string & expression, bool ReturnFormatted = true) {
         size_t ReturnStrSize;
         const char * ReturnStr = dll::EvalStr_47(h, ReturnStrSize, expression.data(), expression.size(), ReturnFormatted);
         return string(ReturnStr, ReturnStrSize);
      }

      /// <summary>Returns the transform object for parsing math expressions, associated with the current uCalc instance</summary>
      /// <returns>Transform object</returns>
      /// <remarks>
      /// <para>Whenever a math expression is parsed, this transform is first called.  Use this when you want transforms that apply
      /// to math expressions parsed with Parse, Eval, or EvalStr.  To create a new transform that does not affect math expressions
      /// sent to Parse, use NewTransform instead.</para>
      /// 
      /// <para>Independent transforms created with NewTransform can be released when no longer needed.  However, the object created
      /// with ExpressionTransforms should never be released explicitly, because the uCalc object requires it in order parse expressions.</para>
      /// 
      /// <para>Note: For simple situations when you just want one keyword to be the equivalent of a pre-existing one, it is more efficient
      /// to use Alias().  With Alias, nothing is transformed.  Instead, an aliased symbol simply shares the same handle as a pre-existing
      /// one in the symbol table.  When there are no expression transforms at all, the transform step is skipped altogether, slightly
      /// speeding up parsing for all expressions in a uCalc instance.</para>
      /// </remarks>
      Transforms ExpressionTransforms() { return dll::ExpressionTransforms_48(h); }

      /// <summary>Returns the bits representing floating point errors that are cought &amp; raised</summary>
      /// <returns>Integer</returns>
      /// <param name="Value">True or false</param>
      /// <remarks>Returns the bits for the floating point errors that are to be raised</remarks>
      int32_t FloatingPointErrorsToCatch(int32_t Value = -1) { return dll::FloatingPointErrorsToCatch_49(h, Value); }

      /// <summary>Allows a callback to decorate the output of EvalStr</summary>
      /// <param name="FunctionAddr">Address of callback function that formats the output</param>
      /// <param name="dType">Data type that will have its output formatted; omit this to affect all data types and use default ranking</param>
      Item Format(OpCallback FunctionAddr, DataType dType) { return dll::Format_51(h, FunctionAddr, dType.h); }

      /// <summary>Allows a callback to decorate the output of EvalStr</summary>
      /// <param name="FunctionAddr">Address of callback function that formats the output</param>
      /// <param name="Properties">Name of datatype that will have its output formatted and/or ranking among other output formats; omit this to affect all data types and use default ranking.</param>
      Item Format(OpCallback FunctionAddr, const string & Properties = "") { return dll::Format_52(h, FunctionAddr, Properties.data(), Properties.size()); }

      /// <summary>Formats the appearance of results returned by EvalStr or EvaluateStr</summary>
      /// <param name="Definition">Format definition for the output</param>
      /// <param name="dType">Data type that will have its output formatted; omit this to affect all data types and use default ranking.</param>
      /// <remarks>
      /// <para>This function lets you modify the output returned by EvalStr or EvaluateStr.  The definition may contain one or more
      /// of the following:</para>
      /// 
      /// <para>Datatype:  If you specify a data type, then the modification will affect only the evaluation result of expressions that
      /// evaluate to the specified data type.</para>
      /// 
      /// <para>InsertAt:  By default, if you define multiple outputs, then the one defined most recently is applied first, followed
      /// by the other outputs in the reverse order in which they were defined, ending with the output that was defined first.  You
      /// may specify a different insertion point than the default with this parameter.</para>
      /// 
      /// <para>Expression:  This represents what the modified output should look like.  The expression consists of an identifier (any
      /// temporary variable name), followed by an equal sign, followed by a string expression representing the value that will be returned.
      ///  The variable holds the original value that would have been returned.</para>
      /// </remarks>
      Item Format(const string & Definition, DataType dType = DataType::Empty) { return dll::Format_53(h, Definition.data(), Definition.size(), dType.h); }

      /// <summary>Removes any output formatting</summary>
      /// <param name="dType">Data type</param>
      /// <remarks>Removes formatting for EvalStr or EvaluateStr that was defined with Format.</remarks>
      void FormatRemove(DataType dType = DataType::Empty) { dll::FormatRemove_54(h, dType.h); }

      /// <summary>Returns the handle of an object</summary>
      /// <remarks>Mostly for internal use.  However, if you are iterating through a series of objects with NextOverload (when applicable),
      /// you may check Handle() to see when it becomes Empty to know when to stop.</remarks>

      /// <summary>Checks to see if this uCalc object instance is the default</summary>
      /// <returns>Boolean value</returns>
      /// <remarks>Definitions from this instance will automatically be inherited by new instances unless you specify otherwise.</remarks>
      bool IsDefault() { return dll::IsDefault_57(h) != 0; }

      /// <summary>Sets this uCalc object instance as the default</summary>
      /// <param name="Condition">True or False</param>
      /// <remarks>Definitions from this instance will automatically be inherited by new instances unless you specify otherwise. Set
      /// this to True.  When set to False, nothing happens.</remarks>
      void IsDefault(bool Condition) { dll::IsDefault_58(h, Condition); }

      /// <summary>Returns the nth uCalc.Item object that matches a property</summary>
      /// <returns>Item object</returns>
      /// <param name="Property_">Properties of the uCalc.Item object you&apos;re after; use either 0 for all items or Properties()</param>
      /// <param name="nth">The nth object</param>
      /// <param name="DataTypeName">The data type of the object</param>
      /// <param name="IncludeOverloads">Include overloads</param>
      /// <remarks>This can be used to cycle through defined items. If there is no match, an item whose IsProperty ItemIs.NotFound value
      /// is true is returned.</remarks>
      Item ItemOf(int64_t Property_, size_t nth, const string & DataTypeName = "", bool IncludeOverloads = false) { return dll::ItemOf_60(h, Property_, nth, DataTypeName.data(), DataTypeName.size(), IncludeOverloads); }

      /// <summary>Returns the uCalc.Item object associated with the given name</summary>
      /// <returns>Item object</returns>
      /// <param name="ItemName">Name of uCalc.Item that you want to get</param>
      /// <param name="Properties">Properties to help pinpoint the right Item in case several items share the same name</param>
      /// <param name="nthOverload">If more than one item matches the same name, the nth overload is selected</param>
      /// <remarks>
      /// <para>If there is no match, an item whose ItemIs.NotFound property is true is returned.</para>
      /// 
      /// <para>In case multiple items share the same name, you can narrow it down using properties.  For instance, the + operator has
      /// a binary and unary version.  You can specify Prefix if you wanted the unary one.</para>
      /// 
      /// <para>Note: A ItemIs member should not be passed directly as the second argument.  Instead Properties(), with one or more
      /// ItemIs members should be passed.  The default of 0 means that there are no property restrictions.</para>
      /// </remarks>
      Item ItemOf(const string & ItemName, int64_t Properties = 0, size_t nthOverload = 0) { return dll::ItemOf_61(h, ItemName.data(), ItemName.size(), Properties, nthOverload); }

      /// <summary>Sets the license code</summary>
      /// <param name="Type">Type of license</param>
      /// <param name="L1">Value 1</param>
      /// <param name="L2">Value 2</param>
      /// <param name="L3">Value 3</param>
      /// <param name="L4">Value 4</param>
      /// <param name="L5">Value 5</param>
      /// <remarks>The license code prevents the message box from showing.  A license can be purchased from www.uCalc.com</remarks>
      static void License(LicenseType Type, size_t L1 = 0, size_t L2 = 0, size_t L3 = 0, size_t L4 = 0, size_t L5 = 0) { dll::License_62(Type, L1, L2, L3, L4, L5); }

      size_t MemoryIndex() { return dll::MemoryIndex_602(h); }

      /// <summary>Returns the address in memory of a uCalc object; mainly for internal use</summary>
      /// <returns>Address of object at index</returns>
      /// <param name="Index">Index of object; 0 is reserved for Empty object (instead of null)</param>
      static size_t MemoryObjectAddress(MemoryTable ObjectType, size_t Index) { return dll::MemoryObjectAddress_(ObjectType, Index); }

      /// <summary>Namespace symbol</summary>
      /// <returns>Nth symbol item object</returns>
      /// <param name="nth">Nth symbol</param>
      Item NamespaceSymbol(size_t nth) { return dll::NamespaceSymbol_63(h, nth); }

      /// <summary>Number of symbols</summary>
      size_t NamespaceSymbolCount() { return dll::NamespaceSymbolCount_64(h); }

      /// <summary>Returns a new uCalc instance</summary>
static uCalcBase New(uCalcBase InheritsFrom = uCalcBase::Empty) { return dll::New_584(InheritsFrom.h); }
static Handle_uCalc NewH(uCalcBase InheritsFrom = uCalcBase::Empty) { return dll::New_584(InheritsFrom.h); }

      /// <summary>Creates a new uCalc instance and makes it the default</summary>
      /// <returns>uCalc object</returns>
      /// <param name="InheritsFrom">Other uCalc object to inherit from</param>
      /// <remarks>
      /// <para>Creates a new uCalc object and makes it the default.  When a new uCalc object is created without specifying another
      /// uCalc object to derive from, or when an Expression, String, or Transforms object is created without specifying which uCalc
      /// space it will belong to, this default uCalc object will be used.</para>
      /// 
      /// <para>New default uCalc objects are pushed onto a stack.  uCalc objects created with NewDefault() must explicitly be released
      /// with Release(), or popped with DefaultPop(), when no longer needed.  When a default uCalc object is released , this default
      /// is removed from the stack, and the uCalc instance now at the top of the stack becomes the new default.  If there&apos;s only
      /// one default on the stack, it is released, but a new default is generated and placed on the stack, so that there is always
      /// a default.</para>
      /// 
      /// <para>+++ Might not need this function</para>
      /// </remarks>
      static uCalcBase NewDefault(uCalcBase InheritsFrom = uCalcBase::Empty) { return dll::NewDefault_20(InheritsFrom.h); }

Expression NewExpression(const string & Expr, BuiltInType DataTypeIndex) { return dll::NewExpression_581(h, Expr.data(), Expr.size(), DataTypeIndex); }
Handle_Expression NewExpressionH(const string & Expr, BuiltInType DataTypeIndex) { return dll::NewExpression_581(h, Expr.data(), Expr.size(), DataTypeIndex); }

Expression NewExpression(const string & Expr, DataType ReturnType = DataType::Empty) { return dll::NewExpression_582(h, Expr.data(), Expr.size(), ReturnType.h); }
Handle_Expression NewExpressionH(const string & Expr, DataType ReturnType = DataType::Empty) { return dll::NewExpression_582(h, Expr.data(), Expr.size(), ReturnType.h); }

Expression NewExpression(const string & Expr, const string & ReturnType) { return dll::NewExpression_583(h, Expr.data(), Expr.size(), ReturnType.data(), ReturnType.size()); }
Handle_Expression NewExpressionH(const string & Expr, const string & ReturnType) { return dll::NewExpression_583(h, Expr.data(), Expr.size(), ReturnType.data(), ReturnType.size()); }

      /// <summary>Returns a new uCalc String object</summary>
      /// <returns>New uCalc string object</returns>
      /// <param name="InitialString">String initial value</param>
      /// <remarks>Returns a new uCalc String object.
      /// Objects created with this function must explicitly be released with Release() (unless you release the parent uCalc object,
      /// in which case this object will also be released).</remarks>
String NewString(const string & InitialString = "") { return dll::NewString_65(h, InitialString.data(), InitialString.size()); }
Handle_String NewStringH(const string & InitialString = "") { return dll::NewString_65(h, InitialString.data(), InitialString.size()); }

      /// <summary>Returns a new transform object</summary>
      /// <returns>Transform object</returns>
      /// <param name="TokensToInherit">Existing token list to inherit from</param>
      /// <remarks>Returns a new object that can be used to transform text.Objects created with this function must explicitly be released
      /// with Release() (unless you release the parent uCalc object, in which case this object will also be released).</remarks>
Transforms NewTransform(Tokens TokensToInherit = Tokens::Empty) { return dll::NewTransform_66(h, TokensToInherit.h); }
Handle_Transforms NewTransformH(Tokens TokensToInherit = Tokens::Empty) { return dll::NewTransform_66(h, TokensToInherit.h); }

      /// <summary>Parses an expression that will later be evaluated with Evaluate or EvaluateStr</summary>
      /// <returns>Parsed expression</returns>
      /// <param name="Expr">Expression to be parsed</param>
      /// <param name="DataTypeIndex">Data type index of return value</param>
      /// <remarks>
      /// <para>Parse() is the same as NewExpression.
      /// Note:
      /// C++
      ///  auto MyExpr = uc.Parse(...) for MyExpr to live beyond the scope of where it&apos;s defined
      ///  uCalc::Expression MyExpr = uc.Parse(...) for Expr to be released automatically upon going out of scope</para>
      /// 
      /// <para>.NET (C#)
      /// var MyExpr = uc.Parse(...) for MyExpr to live beyond the scope of where it&apos;s defined
      /// using uCalc.Expression MyExpr = uc.Parse(...) for Expr to be released automatically upon going out of scope</para>
      /// </remarks>
      Expression Parse(const string & Expr, BuiltInType DataTypeIndex) { return dll::Parse_68(h, Expr.data(), Expr.size(), DataTypeIndex); }

      /// <summary>Parses an expression that will later be evaluated with Evaluate or EvaluateStr</summary>
      /// <returns>Parsed expression</returns>
      /// <param name="Expr">Expression to be parsed</param>
      /// <param name="ReturnType">Data type of return value</param>
      Expression Parse(const string & Expr, DataType ReturnType = DataType::Empty) { return dll::Parse_69(h, Expr.data(), Expr.size(), ReturnType.h); }

      /// <summary>Parses an expression that will later be evaluated with Evaluate or EvaluateStr</summary>
      /// <returns>Parsed expression</returns>
      /// <param name="Expr">Expression to be parsed</param>
      /// <param name="ReturnType">Data type of return value</param>
      Expression Parse(const string & Expr, const string & ReturnType) { return dll::Parse_70(h, Expr.data(), Expr.size(), ReturnType.data(), ReturnType.size()); }

      /// <summary>Returns the precedence level of an operator or the level between two operators</summary>
      /// <param name="Operator1">Operator with lower (or equal) precedence</param>
      /// <param name="Operator2">Operator with greater (or equal precedence)</param>
      /// <remarks>This returns a precedence level based on levels of existing operators; useful when defining a new operator (or transform)
      /// with a level relative to levels of existing operators (or transforms).  If one argument is passed, then the precedence level
      /// of that item is returned.  This is useful for defining a new item at the same level as the specified item.  If two arguments
      /// are passed, then the average (mean) precedence level of the two items is returned.  This is useful for defining a level that&apos;s
      /// in between that of two existing items.</remarks>
      size_t Precedence(const string & Operator1, const string & Operator2 = "") { return dll::Precedence_71(h, Operator1.data(), Operator1.size(), Operator2.data(), Operator2.size()); }

      /// <summary>Combines properties into a numeric value that can be passed to Item</summary>
      /// <returns>Integer representation of combined properties</returns>
      /// <param name="p1">First property</param>
      /// <param name="p2">Second property</param>
      /// <param name="p3">Third property</param>
      /// <param name="p4">Fourth property</param>
      /// <remarks>ItemIs properties are numbered sequentially (0, 1, 2, 3, 4, ...).  However the Properties argument of the Item function
      /// requires properties in the 2^x sequence (1, 2, 4, 8, 16, ...).  So you must pass one or more ItemIs to Properties before passing
      /// it to Item.</remarks>
      static int64_t Properties(ItemIs p1, ItemIs p2 = ItemIs::SelectAny, ItemIs p3 = ItemIs::SelectAny, ItemIs p4 = ItemIs::SelectAny) { return dll::Properties_72(p1, p2, p3, p4); }

      /// <summary>Causes division by 0 errors to be raised</summary>
      /// <param name="TrueOrFalse">True or false</param>
      /// <remarks>This causes an error to be raised instead of silently returning a special value when a division by 0 or pole error
      /// occurs.</remarks>
      void RaiseErrorOnDivideByZero(bool TrueOrFalse = true) { dll::RaiseErrorOnDivideByZero_73(h, TrueOrFalse); }

      /// <summary>Causes inexact results to raise an error</summary>
      /// <param name="TrueOrFalse">True or false</param>
      /// <remarks>This causes an error to be raised instead returning an approximate value when an exact value cannot be represented.</remarks>
      void RaiseErrorOnInexact(bool TrueOrFalse = true) { dll::RaiseErrorOnInexact_74(h, TrueOrFalse); }

      /// <summary>Causes invalid value to raise an error</summary>
      /// <param name="TrueOrFalse">True or false</param>
      /// <remarks>This causes an error to be raised instead returning NaN when an invalid value is encountered.</remarks>
      void RaiseErrorOnInvalid(bool TrueOrFalse = true) { dll::RaiseErrorOnInvalid_75(h, TrueOrFalse); }

      /// <summary>Causes overflow to raise an error</summary>
      /// <param name="TrueOrFalse">True or false</param>
      /// <remarks>This causes an error to be raised instead returning infinity when there&apos;s an overflow.</remarks>
      void RaiseErrorOnOverflow(bool TrueOrFalse = true) { dll::RaiseErrorOnOverflow_76(h, TrueOrFalse); }

      /// <summary>Causes underflow to raise an error</summary>
      /// <param name="TrueOrFalse">True or false</param>
      /// <remarks>This causes an error to be raised for underflow instead of just returning 0.</remarks>
      void RaiseErrorOnUnderflow(bool TrueOrFalse = true) { dll::RaiseErrorOnUnderflow_77(h, TrueOrFalse); }

      size_t ReferenceCount() { return dll::ReferenceCount_570(h); }

      /// <summary>.</summary>
      /// <remarks>.</remarks>
      void ReferenceIncr() { dll::ReferenceIncr_556(h); }

      /// <summary>Releases a uCalc instance</summary>
      /// <remarks>
      /// <para>When a uCalc instance is no longer needed, it can be released with this function.  This releases all items associated
      /// with the uCalc instance.</para>
      /// 
      /// <para>Note: The uCalc object named uc, which is instantiated at startup (except in C#) should not be explicitly released.
      /// In C#, if there is only 1 uCalc object left, do not explicitly release it.</para>
      /// 
      /// <para>Note: A uCalc object must explicitly be released with Release for it to be released.  A uCalc object is not automatically
      /// released from memory when the object goes out of scope.</para>
      /// </remarks>
      void Release() { dll::Release_78(h); h = Empty; }

      /// <summary>Resets uCalc back to the way it was at startup</summary>
      /// <remarks>Releases all uCalc objects and resets the entire uCalc library back to the way it was at startup.</remarks>
      static void ResetAll() { dll::ResetAll_566(); }

      /// <summary>Sets the current uCalc object as the default</summary>
      uCalcBase SetAsDefault() { return dll::SetAsDefault_585(h); }

      /// <summary>Returns the value located at the specified pointer address</summary>
      /// <returns>String representation of the value at the given address</returns>
      /// <param name="ValuePtr">Pointer address of the value</param>
      /// <param name="dType">Data type of the value; helps determine in which form to return the value at the given location</param>
      /// <remarks>This returns the value located at the given address.  The value is first converted to a string.</remarks>
      string ValueAt(void * ValuePtr, DataType dType = DataType::Empty, bool FormattedOutput = false) {
         size_t ReturnStrSize;
         const char * ReturnStr = dll::ValueAt_81(h, ReturnStrSize, ValuePtr, dType.h, FormattedOutput);
         return string(ReturnStr, ReturnStrSize);
      }

      /// <summary>Returns the value located at the specified pointer address</summary>
      /// <returns>String representation of the value at the given address</returns>
      /// <param name="ValuePtr">Pointer address containing the data</param>
      /// <param name="ValueDataType">Data type of the data you need, which helps to determine in which form to return the value at the given location</param>
      /// <remarks>This returns the value located at the given address.  The value is first converted to a string.</remarks>
      string ValueAt(void * ValuePtr, const string & ValueDataType, bool FormattedOutput = false) {
         size_t ReturnStrSize;
         const char * ReturnStr = dll::ValueAt_82(h, ReturnStrSize, ValuePtr, ValueDataType.data(), ValueDataType.size(), FormattedOutput);
         return string(ReturnStr, ReturnStrSize);
      }

      void Reset(uCalcBase u = Empty) { Release(); h = NewH(u); }
      uCalcBase() { h = NewH(); }
      uCalcBase(LicenseType Type, size_t L1 = 0, size_t L2 = 0, size_t L3 = 0, size_t L4 = 0, size_t L5 = 0) { License(Type, L1, L2, L3, L4, L5); h = NewH(); }

      // Handles ostream << overload
      friend ostream & operator << (ostream & os, String obj) { os << obj.Str(); return os; }
      friend ostream & operator << (ostream & os, Transforms obj) { os << obj.Str(); return os; }
      friend ostream & operator << (ostream & os, DataType obj) { os << obj.Name(); return os; }
      friend ostream & operator << (ostream & os, Expression obj) { os << obj.EvaluateStr(); return os; }
      friend ostream & operator << (ostream & os, Item obj) { os << obj.ValueStr(); return os; }
      friend ostream & operator << (ostream & os, Matches obj) { os << obj.Str(); return os; }
      friend ostream & operator << (ostream & os, Rule obj) { os << obj.FromPart() << std::endl << obj.ToPart(); return os; }
      friend ostream & operator << (ostream & os, Tokens obj) { for (size_t x = 0; x < obj.Count(); x++) os << obj.Token(x).Regex() << std::endl; return os; }
      friend ostream & operator << (ostream & os, uCalcBase obj) { os << obj; return os; }
   }; // End of uCalcBase class

struct uCalc : public uCalcBase {
  struct Callback;
  struct DataType;
  struct Expression;
  struct Item;
  struct Matches;
  struct Rule;
  struct String;
  struct Tokens;
  struct Transforms;

  struct Callback : public uCalcBase::Callback {
    using uCalcBase::Callback::Callback;
  };

  struct DataType : public uCalcBase::DataType {
    using uCalcBase::DataType::DataType;
  };

  struct Expression : public uCalcBase::Expression {
    using uCalcBase::Expression::Expression;
    Expression(Handle_Expression PassedHandle) : uCalcBase::Expression(PassedHandle) { ReferenceIncr(); }
    Expression() : uCalcBase::Expression() { }
    Expression(const Expression & other) : uCalcBase::Expression(other.h) { ReferenceIncr(); }
    Expression(const uCalcBase::Expression & other) : uCalcBase::Expression(other.h) { ReferenceIncr(); }
    Expression(Expression && other) noexcept : uCalcBase::Expression(other.h) { other.h = Empty; }
    Expression(uCalcBase::Expression && other) noexcept : uCalcBase::Expression(other.h) { other.h = Empty; }
    Expression & operator=(Expression & other) { Release(); h = other.Clone().h; return *this; }
    Expression & operator=(uCalcBase::Expression & other) { Release(); h = other.Clone().h; return *this; }
    Expression & operator=(Expression && other) noexcept { Release(); h = other.h; other.h = Empty; return *this; }
    Expression & operator=(uCalcBase::Expression && other) noexcept { Release(); h = other.h; other.h = Empty; return *this; }
    ~Expression() { Release(); }

    Expression & operator=(const string & s) { auto u = uCalc(); Release(); h = u.NewExpressionH(s); return *this; }
    Expression & operator=(const char * s) { auto u = uCalc(); Release(); h = u.NewExpressionH(s); return *this; }
  };

  struct Item : public uCalcBase::Item {
    using uCalcBase::Item::Item;
    Item(Handle_Item PassedHandle) : uCalcBase::Item(PassedHandle) { ReferenceIncr(); }
    Item() : uCalcBase::Item() { } // +++ Make this deleted
    Item(const Item & other) : uCalcBase::Item(other.h) { ReferenceIncr(); }
    Item(const uCalcBase::Item & other) : uCalcBase::Item(other.h) { ReferenceIncr(); }
    Item(Item && other) noexcept : uCalcBase::Item(other.h) { other.h = Empty; }
    Item(uCalcBase::Item && other) noexcept : uCalcBase::Item(other.h) { other.h = Empty; }
    Item & operator=(Item & other) { Value(other); return *this; }
    Item & operator=(uCalcBase::Item & other) { Value(other); return *this; }
    Item & operator=(Item && other) noexcept { Release(); h = other.h; other.h = Empty; return *this; }
    Item & operator=(uCalcBase::Item && other) noexcept { Release(); h = other.h; other.h = Empty; return *this; }
    ~Item() { Release(); }

    Item(const string & s) : uCalcBase::Item(s) { }
    //Item(const char * s) : uCalcBase::Item(string(s)) { }

    Item & operator=(const string & s) { uCalcBase::Item::operator=(s); return *this; }
    Item & operator=(const char * s) { uCalcBase::Item::operator=(s); return *this; }
  };

  struct Matches : public uCalcBase::Matches {
    using uCalcBase::Matches::Matches;
  };
  struct Rule : public uCalcBase::Rule {
    using uCalcBase::Rule::Rule; 
  };
  struct String : public uCalcBase::String {
    using uCalcBase::String::String;
    String(Handle_String PassedHandle) : uCalcBase::String(PassedHandle) { ReferenceIncr(); }
    String() : uCalcBase::String() { }
    String(const String & other) : uCalcBase::String(other.h) { ReferenceIncr(); }
    String(const uCalcBase::String & other) : uCalcBase::String(other.h) {ReferenceIncr(); }
    String(String && other) noexcept : uCalcBase::String(other.h) { other.h = Empty; }
    String(uCalcBase::String && other) noexcept : uCalcBase::String(other.h) { other.h = Empty; }
    String & operator=(String & other) { Release(); h = other.Clone().h; return *this; }
    String & operator=(uCalcBase::String & other) { Release(); h = other.Clone().h; return *this; }
    String & operator=(String && other) noexcept { Release(); h = other.h; other.h = Empty; return *this; }
    String & operator=(uCalcBase::String && other) noexcept { Release(); h = other.h; other.h = Empty; return *this; }
    ~String() { Release(); }

    String & operator=(const string & s) { Str(s); return *this; }
    String & operator=(const char * s) { Str(s); return *this; }
  };

  struct Variable : public uCalcBase::Item {
    using uCalcBase::Item::Item;
    Variable(Handle_Item PassedHandle) : uCalcBase::Item(PassedHandle) { ReferenceIncr(); }
    Variable() : uCalcBase::Item() { } // +++ Make this deleted
    Variable(const Variable & other) : uCalcBase::Item(other.h) { ReferenceIncr(); }
    Variable(const uCalcBase::Item & other) : uCalcBase::Item(other.h) {ReferenceIncr(); } // +++ Only if other is in fact a variable item, otherwise raise error
    Variable(Variable && other) noexcept : uCalcBase::Item(other.h) { other.h = Empty; }
    Variable(uCalcBase::Item && other) noexcept : uCalcBase::Item(other.h) { other.h = Empty; } // +++ Only if other is in fact a variable item, otherwise raise error
    Variable & operator=(Variable & other) { Value(other); return *this; }
    Variable & operator=(uCalcBase::Item & other) { Value(other); return *this; } // +++ Only if other is in fact a variable item, otherwise raise error
    Variable & operator=(Variable && other) noexcept { Release(); h = other.h; other.h = Empty; return *this; }
    Variable & operator=(uCalcBase::Item && other) noexcept { Release(); h = other.h; other.h = Empty; return *this; } // +++ Only if other is in fact a variable item, otherwise raise error
    ~Variable() { Release(); }

    Variable(const string & s, void * Address = nullptr) : uCalcBase::Item("Variable: " + s, Address) { }
    Variable(const char * s, void * Address = nullptr) : uCalcBase::Item("Variable: " + string(s), Address) { }
    Variable(uCalcBase uc, const string & s, void * Address = nullptr) : uCalcBase::Item(uc, "Variable: " + s, Address) { }
    Variable(uCalcBase uc, const char * s, void * Address = nullptr) : uCalcBase::Item(uc, "Variable: " + string(s), Address) { }

    Variable & operator=(const string & s) { uCalcBase::Item::operator=(s); return *this; } // { Str(s); return *this; }
    Variable & operator=(const char * s) { uCalcBase::Item::operator=(s); return *this; }
  };
  
  struct Tokens : public uCalcBase::Tokens {
    using uCalcBase::Tokens::Token;
  };

  struct Transforms : public uCalcBase::Transforms {
    using uCalcBase::Transforms::Transform;
         
    Transforms() : uCalcBase::Transforms() { }
    Transforms(Handle_Transforms PassedHandle) : uCalcBase::Transforms(PassedHandle) { ReferenceIncr(); }
    Transforms(const Transforms & other) : uCalcBase::Transforms(other.h) { ReferenceIncr(); }
    Transforms(const uCalcBase::Transforms & other) : uCalcBase::Transforms(other.h) { ReferenceIncr(); }
    Transforms(Transforms && other) noexcept : uCalcBase::Transforms(other.h) { other.h = Empty; }
    Transforms(uCalcBase::Transforms && other) noexcept : uCalcBase::Transforms(other.h) { other.h = Empty; }
    Transforms & operator=(Transforms & other) { Release(); h = other.Clone().h; return *this; }
    Transforms & operator=(uCalcBase::Transforms & other) { Release(); h = other.Clone().h; return *this; }
    Transforms & operator=(Transforms && other) noexcept { Release(); h = other.h; other.h = Empty; return *this; }
    Transforms & operator=(uCalcBase::Transforms && other) noexcept { Release(); h = other.h; other.h = Empty; return *this; }
    ~Transforms() { Release(); }

    Transforms & operator=(const string & s) { Str(s); return *this; }
    Transforms & operator=(const char * s) { Str(s); return *this; }
  };

      
  using uCalcBase::uCalcBase;
  uCalc(Handle_uCalc PassedHandle) : uCalcBase(PassedHandle) { ReferenceIncr(); }
  uCalc() : uCalcBase() { }
  uCalc(const uCalc & other) : uCalcBase(other.h) { ReferenceIncr(); }
  uCalc(const uCalcBase & other) : uCalcBase(other.h) { ReferenceIncr(); }
  uCalc(uCalc && other) noexcept : uCalcBase(other.h) { other.h = Empty; }
  uCalc(uCalcBase && other) noexcept : uCalcBase(other.h) { other.h = Empty; }
  uCalc & operator=(uCalc & other) { Release(); h = other.Clone().h; return *this; }
  uCalc & operator=(uCalcBase & other) { Release(); h = other.Clone().h; return *this; }
  uCalc & operator=(uCalc && other) noexcept { Release(); h = other.h; other.h = Empty; return *this; }
  uCalc & operator=(uCalcBase && other) noexcept { Release(); h = other.h; other.h = Empty; return *this; }
  ~uCalc() { Release(); }
};


} // Namespace End

using namespace uCalcSoftware;